/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.bridge;

import static org.wso2.carbon.bridge.FrameworkLauncherConstants.DYNAMIC_IMPORT_PACKAGE;

import java.io.File;
import java.util.jar.Manifest;
import java.util.jar.Attributes;

/**
 * Creates regular OSGi bundles out of regular jar files
 */
public class DefaultBundleCreator implements BundleCreator{
    private final static String JARS_DIR =
            "repository" + File.separator + "components" + File.separator + "lib";

    public void create(File targetDir) {
        File dir = BridgeUtils.getBundleDirectory(JARS_DIR);
        File[] files = dir.listFiles(new BridgeUtils.JarFileFilter());
        if (files != null) {
            for (File file : files) {
                try {
                    Manifest mf = new Manifest();
                    Attributes attribs = mf.getMainAttributes();
                    attribs.putValue(DYNAMIC_IMPORT_PACKAGE, "*");
                    BridgeUtils.createBundle(file, targetDir, mf, "");
                } catch (Throwable e) {
                    System.err.println("Cannot create bundle from jar file " +
                                       file.getAbsolutePath());
                    e.printStackTrace();
                }
            }
        }
    }
}
