package org.wso2.carbon.core.services.internal;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.service.component.ComponentContext;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.user.core.UserRealm;

/**
 * @scr.component name="core.services.dscomponent" immediate="true"
 * @scr.reference name="registry.service" interface="org.wso2.carbon.registry.core.service.RegistryService"
 * cardinality="1..1" policy="dynamic"  bind="setRegistryService" unbind="unsetRegistryService"
 * @scr.reference name="user.realm.delegating" interface="org.wso2.carbon.user.core.UserRealm"
 * cardinality="1..1" policy="dynamic" target="(RealmGenre=Delegating)"
 * bind="setUserRealmDelegating" unbind="unsetUserRealmDelegating"
 * @scr.reference name="user.realm.default" interface="org.wso2.carbon.user.core.UserRealm"
 * cardinality="1..1" policy="dynamic" target="(RealmGenre=Default)" bind="setUserRealmDefault"
 * unbind="unsetUserRealmDefault"
 */
public class CarbonServicesServiceComponent {

    private static UserRealm userRealmDefaultService;
    private static UserRealm userRealmDelegatingService;
    private static RegistryService registryServiceInstance;

    private static final Log log = LogFactory.getLog(CarbonServicesServiceComponent.class);

    protected void activate(ComponentContext ctxt) {
        try {
            log.debug("Carbon Core Services bundle is activated ");
        } catch (Throwable e) {
            log.error("Failed to activate Carbon Core Services bundle ", e);
        }
    }

    protected void deactivate(ComponentContext ctxt) {
        log.debug("Carbon Core Services bundle is deactivated ");
    }
   
    protected void setRegistryService(RegistryService registryService) {
        registryServiceInstance = registryService;
    }

    protected void unsetRegistryService(RegistryService registryService) {
        registryServiceInstance = null;
    }

    protected void setUserRealmDelegating(UserRealm userRealmDelegating) {
        userRealmDelegatingService = userRealmDelegating;
    }

    protected void unsetUserRealmDelegating(UserRealm userRealmDelegating) {
        userRealmDelegatingService = null;
    }

    protected void setUserRealmDefault(UserRealm userRealmDefault) {
        userRealmDefaultService = userRealmDefault;
    }

    protected void unsetUserRealmDefault(UserRealm userRealmDefault) {
        userRealmDefaultService = null;
    }

    public static UserRealm getUserRealm() throws Exception {
        UserRealm userRealm  = userRealmDelegatingService;

        if(userRealm == null){
            userRealm = userRealmDefaultService;
        }

        if(userRealm == null){
            String msg = "System has not been started properly. Some components have not started";
            log.error(msg);
            throw new Exception(msg);
        }
        return userRealm;
    }

    public static RegistryService getRegistryService() throws Exception {
        if (registryServiceInstance == null) {
            String msg = "Before activating Carbon Services bundle, an instance of "
                    + "RegistryService should be in existance";
            log.error(msg);
            throw new Exception(msg);
        }
        return registryServiceInstance;
    }
}
