/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.core.internal;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.http.HttpService;
import org.wso2.carbon.core.init.CarbonServerManager;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.utils.ServerConfiguration;

/**
 * @scr.component name="carbon.core.dscomponent"" immediate="true"
 * @scr.reference name="user.realm.delegating" interface="org.wso2.carbon.user.core.UserRealm"
 * cardinality="1..1" policy="dynamic" target="(RealmGenre=Delegating)"
 * bind="setUserRealmDelegating" unbind="unsetUserRealmDelegating"
 * @scr.reference name="registry.service" interface="org.wso2.carbon.registry.core.service.RegistryService"
 * cardinality="1..1" policy="dynamic"  bind="setRegistryService" unbind="unsetRegistryService"
 * @scr.reference name="http.service" interface="org.osgi.service.http.HttpService"
 * cardinality="1..1" policy="dynamic"  bind="setHttpService" unbind="unsetHttpService"
 */
public class CarbonCoreServiceComponent {

    private static Log log = LogFactory.getLog(CarbonCoreServiceComponent.class);
    private BundleContext bundleContext;

    private static UserRealm userRealmService;
    private static RegistryService registryServiceInstance;
    private static HttpService httpServiceInstance;
    private CarbonServerManager carbonServerManager;

    protected void activate(ComponentContext ctxt) {
        try {
            bundleContext = ctxt.getBundleContext();
            carbonServerManager = new CarbonServerManager(bundleContext);
        } catch (Throwable e) {
            log.error("Failed to activate Carbon Core bundle ", e);
        }
    }

    protected void deactivate(ComponentContext ctxt) {
        try {
            carbonServerManager.cleanupSystem();
        } catch (Throwable e) {
            log.error("Failed clean up Carbon core", e);
        }

        if ("false".equals(ServerConfiguration.getInstance().getFirstProperty("RequireCarbonServlet"))) {
            return;
        }
        try {
            getHttpService().unregister("/services");
        } catch (Exception e) {
            log.error("Failed to unregister servlets ", e);
        }
        log.debug("Carbon Core bundle is deactivated ");
    }

    protected void setUserRealmDelegating(UserRealm userRealmDelegating) {
        userRealmService = userRealmDelegating;
    }

    protected void unsetUserRealmDelegating(UserRealm userRealmDelegating) {
        userRealmService = null;
    }

    protected void setHttpService(HttpService httpService) {
        httpServiceInstance = httpService;
    }

    protected void unsetHttpService(HttpService httpService) {
        httpServiceInstance = null;
    }

    protected void setRegistryService(RegistryService registryService) {
        registryServiceInstance = registryService;
    }

    protected void unsetRegistryService(RegistryService registryService) {
        registryServiceInstance = null;
    }
    
    public static HttpService getHttpService() throws Exception {
        if (httpServiceInstance == null) {
            String msg = "Before activating Carbon Core bundle, an instance of "
                    + HttpService.class.getName() + " should be in existance";
            log.error(msg);
            throw new Exception(msg);
        }
        return httpServiceInstance;
    }

    public static UserRealm getUserRealm() throws Exception {
        if (userRealmService == null) {
            String msg = "Before activating Carbon Core bundle, an instance of "
                    + "UserRealm service should be in existance";
            log.error(msg);
            throw new Exception(msg);
        }
        return userRealmService;
    }

    public static RegistryService getRegistryService() throws Exception {
        if (registryServiceInstance == null) {
            String msg = "Before activating Carbon Core bundle, an instance of "
                    + "RegistryService should be in existance";
            log.error(msg);
            throw new Exception(msg);
        }
        return registryServiceInstance;
    }
}
