/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.core.persistence;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMException;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.impl.llom.util.AXIOMUtil;
import org.apache.axiom.om.util.UUIDGenerator;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.description.*;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.util.Utils;
import org.apache.axis2.util.JavaUtils;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.*;
import org.wso2.carbon.Axis2ModuleNotFound;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.core.RegistryResources;
import org.wso2.carbon.core.CarbonAxisConfigurator;
import org.wso2.carbon.core.transports.util.TransportSummary;
import org.wso2.carbon.core.util.ParameterUtil;
import org.wso2.carbon.registry.core.Association;
import org.wso2.carbon.registry.core.Collection;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.registry.core.jdbc.utils.Transaction;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.exceptions.ResourceNotFoundException;
import org.wso2.carbon.utils.CarbonUtils;
import org.wso2.carbon.utils.WSO2Constants;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;
import javax.xml.stream.XMLStreamException;
import java.io.ByteArrayOutputStream;
import java.util.*;

/**
 * Handles all the persistence related stuff for WSO2 Carbon
 */
public class PersistenceManager {
    private static Log log = LogFactory.getLog(PersistenceManager.class);
    private static final String GLOBALLY_ENGAGED_PARAM_NAME = "globallyEngaged";
    private static final String GLOBALLY_ENGAGED_CUSTOM = "globallyEngagedCustom";
    private static final String AVAILABLE_TRANSPORT_LISTENERS = "availableTransportListeners";
    private static final String AVAILABLE_TRANSPORT_SENDERS = "availableTransportSenders";

    private static final String TRANSPORT_LISTENER = "listener/";
    private static final String TRANSPORT_SENDER = "sender/";

    private Registry registry;
    private AxisConfiguration axisConfig;

    public PersistenceManager(AxisConfiguration axisConfig) throws AxisFault {
        this.axisConfig = axisConfig;
        registry = (Registry) axisConfig.getParameterValue(WSO2Constants.REGISTRY_INSTANCE);
        if (registry == null) {
            throw new AxisFault("WSO2 Registry is not available");
        }
    }

    public Resource getModule(String moduleName, String version) {

        try {
            String modulePath = RegistryResources.MODULES + moduleName + "/" + version;
            if (registry.resourceExists(modulePath)) {
                Resource resource = registry.get(modulePath);
                if(resource.getProperty(RegistryResources.SUCCESSFULLY_ADDED) != null){
                    return resource;
                }
            }
        } catch (ResourceNotFoundException e) {
            return null;
        } catch (RegistryException e) {
            String msg = "Could not get service group from registry";
            log.error(msg);
            throw new RuntimeException(msg, e);
        }
        return null;
    }

    public Resource getService(AxisService axisService) throws RegistryException {
        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + axisService.getAxisServiceGroup().getServiceGroupName()
                + RegistryResources.SERVICES + axisService.getName();
        if (registry.resourceExists(serviceResourcePath)) {
            Resource resource = registry.get(serviceResourcePath);
            if (resource.getProperty(RegistryResources.SUCCESSFULLY_ADDED) != null) {
                return resource;
            }
        }
        return null;
    }

    public Resource getServiceGroup(String serviceGroupId) {
        try {
            String resourcePath = RegistryResources.SERVICE_GROUPS + serviceGroupId;
            if (registry.resourceExists(resourcePath)) {
                Resource resource = registry.get(resourcePath);
                if(resource.getProperty(RegistryResources.SUCCESSFULLY_ADDED) != null){
                    return resource;
                }
            }
        } catch (ResourceNotFoundException e) {
            return null;
        } catch (RegistryException e) {
            String msg = "Could not get service group from registry";
            log.error(msg);
            throw new RuntimeException(msg, e);
        }
        return null;
    }


    public void addKeyStore(String ksName) {
        // TODO AZEEZ
    }

    public Resource getKeyStore(String ksName) {
        // TODO AZEEZ
        return null;
    }

    public String getConfigurationProperty(String propertyName) throws Exception {
        String resourcePath = RegistryResources.CONFIG + propertyName;
        String value = null;
        if (registry.resourceExists(resourcePath)) {
            Resource resource = registry.get(resourcePath);
            value = resource.getProperty(propertyName);
            resource.discard();
        }
        return value;
    }

    public void updateConfigurationProperty(String propertyName, String value)
            throws RegistryException {
        String resourcePath = RegistryResources.CONFIG + propertyName;
        Resource resource;
        if (!registry.resourceExists(resourcePath)) {
            resource = registry.newResource();
            resource.addProperty(propertyName, value);
            registry.put(resourcePath, resource);
            resource.discard();
        } else {
            resource = registry.get(resourcePath);
            resource.setProperty(propertyName, value);
            registry.put(resourcePath, resource);
            resource.discard();
        }
    }

    public void deleteService(AxisService axisService) throws RegistryException {
        Parameter param = axisService.getParameter(CarbonConstants.PRESERVE_SERVICE_HISTORY_PARAM);
        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + axisService.getAxisServiceGroup().getServiceGroupName()
                + RegistryResources.SERVICES + axisService.getName();
        if (registry.resourceExists(serviceResourcePath) && (param == null || !JavaUtils.isTrue(param.getValue().toString())) ) {
            registry.delete(serviceResourcePath);
        }
    }

    public void deleteServiceGroup(AxisServiceGroup serviceGroup) throws RegistryException {
        Iterator services = serviceGroup.getServices();
        Parameter param = serviceGroup.getParameter(CarbonConstants.PRESERVE_SERVICE_HISTORY_PARAM);
        if (services.hasNext() && ((AxisService) services.next()).isClientSide()) {
            return;
        }
        if(param == null || !JavaUtils.isTrue(param.getValue().toString())){
            if(log.isDebugEnabled()){
                log.debug("Delete Service group data from registry:" + serviceGroup.getServiceGroupName());
            }
            registry.delete(RegistryResources.SERVICE_GROUPS + serviceGroup.getServiceGroupName());
        }
    }

    public synchronized void handleNewServiceAddition(AxisService axisService) throws Exception {
        if (axisService.isClientSide()) {
            return;
        }
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            // Add Service
            Collection service = registry.newCollection();
            service.setProperty(RegistryResources.ServiceProperties.DOCUMENTATION, axisService
                    .getDocumentation());
            service.setProperty(RegistryResources.ServiceProperties.EXPOSED_ON_ALL_TANSPORTS,
                                String.valueOf(axisService.isEnableAllTransports()));
            String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                    + axisService.getAxisServiceGroup().getServiceGroupName()
                    + RegistryResources.SERVICES + axisService.getName();
            registry.put(serviceResourcePath, service);

            // Add Service Operations
            for (Iterator iter = axisService.getOperations(); iter.hasNext();) {
                AxisOperation axisOperation = (AxisOperation) iter.next();
                Collection operation = registry.newCollection();
                String doc = axisOperation.getDocumentation();
                if (doc != null) {
                    operation.setProperty(RegistryResources.ServiceProperties.DOCUMENTATION, doc);
                }
                operation.setProperty(RegistryResources.NAME, axisOperation.getName()
                        .getLocalPart());
                registry.put(serviceResourcePath + RegistryResources.ServiceProperties.OPERATIONS
                        + axisOperation.getName().getLocalPart(), operation);
            }

            // Add Service Bindings
            Map endPointMap = axisService.getEndpoints();
            for (Object o : endPointMap.entrySet()) {
                Map.Entry entry = (Map.Entry) o;
                AxisEndpoint point = (AxisEndpoint) entry.getValue();
                AxisBinding currentAxisBinding = point.getBinding();
                Collection binding = registry.newCollection();
                binding.setProperty(RegistryResources.NAME, currentAxisBinding.getName()
                        .getLocalPart());
                registry.put(serviceResourcePath + RegistryResources.ServiceProperties.BINDINGS
                        + currentAxisBinding.getName().getLocalPart(), binding);

                // Add binding operations
                Iterator operations = currentAxisBinding.getChildren();
                while (operations.hasNext()) {
                    AxisBindingOperation currentOperation = (AxisBindingOperation) operations
                            .next();
                    Collection bindingOperation = registry.newCollection();
                    bindingOperation.setProperty(RegistryResources.NAME, currentOperation.getName()
                            .getLocalPart());
                    registry.put(serviceResourcePath + RegistryResources.ServiceProperties.BINDINGS
                            + currentAxisBinding.getName().getLocalPart()
                            + RegistryResources.ServiceProperties.OPERATIONS
                            + currentOperation.getName().getLocalPart(), bindingOperation);
                }

            }

            // Need to commit these changes before proceeding further.
            if (transactionStarted) {
                registry.commitTransaction();
                registry.beginTransaction();
            }

            // Add the Service Policies
            List<Resource> servicePolicies = getServicePolicies(axisService);
            for (Resource servicePolicy : servicePolicies) {
                registry
                        .put(
                                serviceResourcePath
                                        + RegistryResources.POLICIES
                                        + servicePolicy
                                        .getProperty(
                                                RegistryResources.ModuleProperties.POLICY_UUID),
                                servicePolicy);
            }

            // If the service scope='soapsession' we need to engage addressing
            // if not already
            // engaged.
            if (axisService.getScope().equals(Constants.SCOPE_SOAP_SESSION)) {
                if (!axisService.isEngaged("addressing")) {
                    axisService.engageModule(axisService.getAxisConfiguration().getModule(
                            "addressing"));
                }
            }

            // Add the Modules Engaged to this service
            for (Object o1 : axisService.getEngagedModules()) {
                AxisModule axisModule = (AxisModule) o1;
                String moduleName = axisModule.getName();
                String moduleVersion = axisModule.getVersion();
                if (moduleVersion == null) {
                    moduleVersion = "SNAPSHOT";
                }

                if (!isGloballyEngaged(moduleName, moduleVersion)
                        && !axisService.getParent().isEngaged(axisModule.getName())) {
                    String moduleResourcePath = RegistryResources.MODULES + moduleName + "/"
                            + moduleVersion;
                    registry.addAssociation(serviceResourcePath, moduleResourcePath,
                                            RegistryResources.Associations.ENGAGED_MODULES);
                }
            }

            // Save the operation-module engagements
            for (Iterator iter = axisService.getOperations(); iter.hasNext();) {
                AxisOperation axisOperation = (AxisOperation) iter.next();
                for (Object o : axisOperation.getEngagedModules()) {
                    AxisModule axisModule = (AxisModule) o;
                    String moduleName = axisModule.getName();
                    String moduleVersion = axisModule.getVersion();
                    if (!isGloballyEngaged(moduleName, moduleVersion)
                            && !axisService.getParent().isEngaged(axisModule.getName())
                            && !axisService.isEngaged(axisModule.getName())) {

                        if (moduleVersion == null) {
                            moduleVersion = "SNAPSHOT";
                        }
                        String moduleResourcePath = RegistryResources.MODULES + moduleName + "/"
                                + moduleVersion;
                        String opResourcePath = serviceResourcePath
                                + RegistryResources.ServiceProperties.OPERATIONS
                                + axisOperation.getName().getLocalPart();
                        registry.addAssociation(opResourcePath, moduleResourcePath,
                                                RegistryResources.Associations.ENGAGED_MODULES);
                    }
                }
            }

            // add the service parameters
            for (Object o : axisService.getParameters()) {
                Parameter parameter = (Parameter) o;
                String paramName = parameter.getName();
                if (paramName != null && paramName.trim().length() != 0) {
                    if (parameter.getParameterElement() != null) {
                        Resource paramResource = registry.newResource();
                        paramResource.setContent(parameter.getParameterElement().toString());
                        paramResource.addProperty(RegistryResources.NAME, parameter.getName());
                        registry.put(serviceResourcePath + RegistryResources.PARAMETERS
                                + parameter.getName(), paramResource);
                        paramResource.discard();
                    } else if (parameter.getValue() != null
                            && parameter.getValue() instanceof String) {
                        parameter = ParameterUtil.createParameter(paramName.trim(),
                                                                  (String) parameter.getValue());
                        Resource paramResource = registry.newResource();
                        paramResource.setContent(parameter.getParameterElement().toString());
                        paramResource.addProperty(RegistryResources.NAME, parameter.getName());
                        registry.put(serviceResourcePath + RegistryResources.PARAMETERS
                                + parameter.getName(), paramResource);
                        paramResource.discard();
                    }
                }
            }

            // add transport associations
            if (!axisService.isEnableAllTransports()) {
                List exposedTransports = axisService.getExposedTransports();
                for (Object exposedTransport : exposedTransports) {
                    String transportResourcePath = RegistryResources.TRANSPORTS + exposedTransport;
                    registry.addAssociation(serviceResourcePath, transportResourcePath,
                                            RegistryResources.Associations.EXPOSED_TRANSPORTS);
                }
            }
            service = (Collection)registry.get(serviceResourcePath);
            service.addProperty(RegistryResources.SUCCESSFULLY_ADDED, "true");
            registry.put(serviceResourcePath, service);

            if (transactionStarted) {
                registry.commitTransaction();
            }

            if (log.isDebugEnabled()) {
                log.debug("Added new service - " + axisService.getName());
            }
        } catch (Throwable e) {
            log
                    .error("Unable to handle new service addition. Service: "
                            + axisService.getName(), e);
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            throw new Exception(e);
        }
    }

    /**
     * Extract all the service policies from the AxisService.
     *
     * @param axisService Service to get policies
     * @return policies list of policies
     * @throws Exception on error
     */
    private List<Resource> getServicePolicies(AxisService axisService) throws Exception {

        List<Resource> servicePolicyResources = new ArrayList<Resource>();

        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + axisService.getAxisServiceGroup().getServiceGroupName()
                + RegistryResources.SERVICES + axisService.getName();

        // Get Service Policy
        PolicySubject servicePolicySubject = axisService.getPolicySubject();
        List<PolicyComponent> servicePolicyList = new ArrayList<PolicyComponent>(servicePolicySubject.getAttachedPolicyComponents());
        Policy servicePolicy = org.apache.axis2.util.PolicyUtil.getMergedPolicy(servicePolicyList,
                                                                                axisService);
        if (servicePolicy != null) {
            if (servicePolicy.getId() == null) {
                // Generate an ID
                servicePolicy.setId(UUIDGenerator.getUUID());

                Resource servicePolicyResource = registry.newResource();
                servicePolicyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                                  servicePolicy.getId());
                // do a performance improvement
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(baos);
                servicePolicy.serialize(writer);
                writer.flush();
                servicePolicyResource.setContent(baos.toString());
                servicePolicyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE,
                                                  "" + PolicyInclude.AXIS_SERVICE_POLICY);

                servicePolicyResources.add(servicePolicyResource);
            } else if (getPolicyResourceFromList(servicePolicy.getId(),
                    servicePolicyResources) == null) {

                Resource servicePolicyResource = registry.newResource();
                servicePolicyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                                  servicePolicy.getId());
                // do a performance improvement
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(baos);
                servicePolicy.serialize(writer);
                writer.flush();
                servicePolicyResource.setContent(baos.toString());
                servicePolicyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE,
                                                  "" + PolicyInclude.AXIS_SERVICE_POLICY);

                servicePolicyResources.add(servicePolicyResource);
            }

            // Refer this policy from the service
            Resource serviceResource = registry.get(serviceResourcePath);
            serviceResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                        servicePolicy.getId());

            // Update the service resource in registry
            registry.put(serviceResourcePath, serviceResource);
            serviceResource.discard();
        }

        // Get Service Operation Policies
        Iterator serviceOperations = axisService.getOperations();
        while (serviceOperations.hasNext()) {
            AxisOperation currentAxisOperation = (AxisOperation) serviceOperations.next();

            Resource operationResource = registry.get(serviceResourcePath
                    + RegistryResources.ServiceProperties.OPERATIONS
                    + currentAxisOperation.getName().getLocalPart());

            PolicySubject serviceOperationPolicySubject = currentAxisOperation.getPolicySubject();
            List<PolicyComponent> serviceOperationPolicyList = new ArrayList<PolicyComponent>(serviceOperationPolicySubject
                    .getAttachedPolicyComponents());
            Policy operationPolicy = org.apache.axis2.util.PolicyUtil.getMergedPolicy(
                    serviceOperationPolicyList, axisService);
            if (operationPolicy != null) {
                if (operationPolicy.getId() == null) {
                    // Generate an ID
                    operationPolicy.setId(UUIDGenerator.getUUID());

                    Resource serviceOperationPolicyResource = registry.newResource();
                    serviceOperationPolicyResource.setProperty(
                            RegistryResources.ServiceProperties.POLICY_UUID, operationPolicy
                            .getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    operationPolicy.serialize(writer);
                    writer.flush();
                    serviceOperationPolicyResource.setContent(baos.toString());
                    serviceOperationPolicyResource.setProperty(
                            RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.AXIS_OPERATION_POLICY);
                    servicePolicyResources.add(serviceOperationPolicyResource);
                } else if (
                        getPolicyResourceFromList(operationPolicy.getId(), servicePolicyResources) ==
                                null) {
                    Resource serviceOperationPolicyResource = registry.newResource();
                    serviceOperationPolicyResource.setProperty(
                            RegistryResources.ServiceProperties.POLICY_UUID, operationPolicy
                            .getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    operationPolicy.serialize(writer);
                    writer.flush();
                    serviceOperationPolicyResource.setContent(baos.toString());
                    serviceOperationPolicyResource.setProperty(
                            RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.AXIS_OPERATION_POLICY);
                    servicePolicyResources.add(serviceOperationPolicyResource);
                }

                // Refer this policy from the operation resource
                operationResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                              operationPolicy.getId());
            }

            // Get Service Operation Message Policies
            Policy messageInPolicy = null;
            try {
                AxisMessage axisInMessage = axisService
                        .getOperation(currentAxisOperation.getName()).getMessage(
                        WSDLConstants.MESSAGE_LABEL_IN_VALUE);
                PolicySubject messageInPolicySubject = axisInMessage.getPolicySubject();
                List<PolicyComponent> messageInPolicyList = new ArrayList<PolicyComponent>(messageInPolicySubject
                        .getAttachedPolicyComponents());
                messageInPolicy = org.apache.axis2.util.PolicyUtil.getMergedPolicy(
                        messageInPolicyList, axisService);
            } catch (Exception e) {
                log.debug(e);
            }
            if (messageInPolicy != null) {
                if (messageInPolicy.getId() == null) {
                    // Generate an ID
                    messageInPolicy.setId(UUIDGenerator.getUUID());

                    Resource policyResource = registry.newResource();
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                               messageInPolicy.getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    messageInPolicy.serialize(writer);
                    writer.flush();
                    policyResource.setContent(baos.toString());
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.AXIS_MESSAGE_POLICY);
                    servicePolicyResources.add(policyResource);
                } else if (
                        getPolicyResourceFromList(messageInPolicy.getId(), servicePolicyResources) ==
                                null) {
                    Resource policyResource = registry.newResource();
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                               messageInPolicy.getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    messageInPolicy.serialize(writer);
                    writer.flush();
                    policyResource.setContent(baos.toString());
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.AXIS_MESSAGE_POLICY);
                    servicePolicyResources.add(policyResource);
                }

                // Refer this policy from the operation resource
                operationResource.setProperty(
                        RegistryResources.ServiceProperties.MESSAGE_IN_POLICY_UUID, messageInPolicy
                        .getId());

            }

            Policy messageOutPolicy = null;
            try {
                AxisMessage axisOutMessage = axisService.getOperation(
                        currentAxisOperation.getName()).getMessage(
                        WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
                PolicySubject messageOutPolicySubject = axisOutMessage.getPolicySubject();
                List<PolicyComponent> messageOutPolicyList = new ArrayList<PolicyComponent>(messageOutPolicySubject
                        .getAttachedPolicyComponents());
                messageOutPolicy = org.apache.axis2.util.PolicyUtil.getMergedPolicy(
                        messageOutPolicyList, axisService);
            } catch (Exception e) {
                log.debug(e);
            }
            if (messageOutPolicy != null) {
                if (messageOutPolicy.getId() == null) {
                    // Generate an ID
                    messageOutPolicy.setId(UUIDGenerator.getUUID());

                    Resource policyResource = registry.newResource();
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                               messageOutPolicy.getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    messageOutPolicy.serialize(writer);
                    writer.flush();
                    policyResource.setContent(baos.toString());
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.AXIS_MESSAGE_POLICY);
                    servicePolicyResources.add(policyResource);
                } else if (
                        getPolicyResourceFromList(messageOutPolicy.getId(), servicePolicyResources) ==
                                null) {
                    Resource policyResource = registry.newResource();
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                               messageOutPolicy.getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    messageOutPolicy.serialize(writer);
                    writer.flush();
                    policyResource.setContent(baos.toString());
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.AXIS_MESSAGE_POLICY);
                    servicePolicyResources.add(policyResource);
                }

                // Refer this policy from the operation resource
                operationResource.setProperty(
                        RegistryResources.ServiceProperties.MESSAGE_OUT_POLICY_UUID,
                        messageInPolicy.getId());
            }

            // Update the operation resource in registry
            registry.put(serviceResourcePath + RegistryResources.ServiceProperties.OPERATIONS
                    + currentAxisOperation.getName().getLocalPart(), operationResource);
            operationResource.discard();
        }

        // Get binding policies
        Map endPointMap = axisService.getEndpoints();
        for (Object o : endPointMap.entrySet()) {
            Map.Entry entry = (Map.Entry) o;
            AxisEndpoint point = (AxisEndpoint) entry.getValue();
            AxisBinding currentAxisBinding = point.getBinding();

            // Get Binding Policies
            PolicySubject bindingPolicySubject = currentAxisBinding.getPolicySubject();
            List<PolicyComponent> bindingPolicyList = new ArrayList<PolicyComponent>(bindingPolicySubject
                    .getAttachedPolicyComponents());
            Policy bindingPolicy = org.apache.axis2.util.PolicyUtil.getMergedPolicy(
                    bindingPolicyList, axisService);
            if (bindingPolicy != null) {
                if (bindingPolicy.getId() == null) {
                    // Generate an ID
                    bindingPolicy.setId(UUIDGenerator.getUUID());

                    Resource policyResource = registry.newResource();
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                               bindingPolicy.getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    bindingPolicy.serialize(writer);
                    writer.flush();
                    policyResource.setContent(baos.toString());
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.BINDING_POLICY);
                    servicePolicyResources.add(policyResource);
                } else if (
                        getPolicyResourceFromList(bindingPolicy.getId(), servicePolicyResources) ==
                                null) {
                    Resource policyResource = registry.newResource();
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                               bindingPolicy.getId());
                    // do a performance improvement
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(
                            baos);
                    bindingPolicy.serialize(writer);
                    writer.flush();
                    policyResource.setContent(baos.toString());
                    policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, ""
                            + PolicyInclude.BINDING_POLICY);
                    servicePolicyResources.add(policyResource);
                }

                // Refer this policy from the binding resource
                Resource bindingResource = registry.get(serviceResourcePath
                        + RegistryResources.ServiceProperties.BINDINGS
                        + currentAxisBinding.getName().getLocalPart());
                bindingResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                            bindingPolicy.getId());

                // Update the binding resource in registry
                registry.put(serviceResourcePath + RegistryResources.ServiceProperties.BINDINGS
                        + currentAxisBinding.getName().getLocalPart(), bindingResource);
                bindingResource.discard();
            }

            // Get Binding Operation Policies
            Iterator operations = currentAxisBinding.getChildren();
            while (operations.hasNext()) {
                AxisBindingOperation currentOperation = (AxisBindingOperation) operations.next();

                Resource bindingOperationResource = registry.get(serviceResourcePath
                        + RegistryResources.ServiceProperties.BINDINGS
                        + currentAxisBinding.getName().getLocalPart()
                        + RegistryResources.ServiceProperties.OPERATIONS
                        + currentOperation.getName().getLocalPart());

                PolicySubject bindingOperationPolicySubject = currentOperation.getPolicySubject();
                List<PolicyComponent> bindingOperationPolicyList = new ArrayList<PolicyComponent>(bindingOperationPolicySubject
                        .getAttachedPolicyComponents());
                Policy bindingOperationPolicy = org.apache.axis2.util.PolicyUtil.getMergedPolicy(
                        bindingOperationPolicyList, axisService);
                if (bindingOperationPolicy != null) {
                    if (bindingOperationPolicy.getId() == null) {
                        // Generate an ID
                        bindingOperationPolicy.setId(UUIDGenerator.getUUID());
                        Resource policyResource = registry.newResource();
                        policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                                   bindingOperationPolicy.getId());
                        // do a performance improvement
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        XMLStreamWriter writer = XMLOutputFactory.newInstance()
                                .createXMLStreamWriter(baos);
                        bindingOperationPolicy.serialize(writer);
                        writer.flush();
                        policyResource.setContent(baos.toString());
                        policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE,
                                                   "" + PolicyInclude.BINDING_OPERATION_POLICY);
                        servicePolicyResources.add(policyResource);
                    } else if (getPolicyResourceFromList(bindingOperationPolicy.getId(),
                                                         servicePolicyResources) == null) {
                        Resource policyResource = registry.newResource();
                        policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID,
                                                   bindingOperationPolicy.getId());
                        // do a performance improvement
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        XMLStreamWriter writer = XMLOutputFactory.newInstance()
                                .createXMLStreamWriter(baos);
                        bindingOperationPolicy.serialize(writer);
                        writer.flush();
                        policyResource.setContent(baos.toString());
                        policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE,
                                                   "" + PolicyInclude.BINDING_OPERATION_POLICY);
                        servicePolicyResources.add(policyResource);
                    }

                    // Refer this policy from the binding operation
                    bindingOperationResource.setProperty(
                            RegistryResources.ServiceProperties.POLICY_UUID, bindingOperationPolicy
                            .getId());

                }

                // Get Binding Operation Message Policies
                Policy bindingOperationMessageInPolicy = null;
                try {
                    PolicySubject bindingOperationMessageInPolicySubject = currentOperation
                            .getChild(WSDLConstants.MESSAGE_LABEL_IN_VALUE).getPolicySubject();
                    List<PolicyComponent> bindingOperationMessageInPolicyList = new ArrayList<PolicyComponent>(
                            bindingOperationMessageInPolicySubject.getAttachedPolicyComponents());
                    bindingOperationMessageInPolicy = org.apache.axis2.util.PolicyUtil
                            .getMergedPolicy(bindingOperationMessageInPolicyList, axisService);
                } catch (Exception e) {
                    log.debug(e);
                }
                if (bindingOperationMessageInPolicy != null) {
                    if (bindingOperationMessageInPolicy.getId() == null) {
                        // Generate an ID
                        bindingOperationMessageInPolicy.setId(UUIDGenerator.getUUID());

                        Resource bindingOperationMessageInPolicyResource = registry.newResource();
                        bindingOperationMessageInPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_UUID,
                                bindingOperationMessageInPolicy.getId());
                        // do a performance improvement
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        XMLStreamWriter writer = XMLOutputFactory.newInstance()
                                .createXMLStreamWriter(baos);
                        bindingOperationMessageInPolicy.serialize(writer);
                        writer.flush();
                        bindingOperationMessageInPolicyResource
                                .setContent(baos.toString());
                        bindingOperationMessageInPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_TYPE, ""
                                + PolicyInclude.BINDING_INPUT_POLICY);
                        servicePolicyResources.add(bindingOperationMessageInPolicyResource);
                    } else if (getPolicyResourceFromList(bindingOperationMessageInPolicy.getId(),
                                                         servicePolicyResources) == null) {
                        Resource bindingOperationMessageInPolicyResource = registry.newResource();
                        bindingOperationMessageInPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_UUID,
                                bindingOperationMessageInPolicy.getId());
                        // do a performance improvement
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        XMLStreamWriter writer = XMLOutputFactory.newInstance()
                                .createXMLStreamWriter(baos);
                        bindingOperationMessageInPolicy.serialize(writer);
                        writer.flush();
                        bindingOperationMessageInPolicyResource
                                .setContent(baos.toString());
                        bindingOperationMessageInPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_TYPE, ""
                                + PolicyInclude.BINDING_INPUT_POLICY);
                        servicePolicyResources.add(bindingOperationMessageInPolicyResource);
                    }

                    // Refer this policy from the binding operation
                    bindingOperationResource.setProperty(
                            RegistryResources.ServiceProperties.MESSAGE_IN_POLICY_UUID,
                            bindingOperationMessageInPolicy.getId());

                }

                Policy bindingOperationMessageOutPolicy = null;
                try {
                    PolicySubject bindingOperationMessageOutPolicySubject = currentOperation
                            .getChild(WSDLConstants.MESSAGE_LABEL_OUT_VALUE).getPolicySubject();
                    List<PolicyComponent> bindingOperationMessageOutPolicyList = new ArrayList<PolicyComponent>(
                            bindingOperationMessageOutPolicySubject.getAttachedPolicyComponents());
                    bindingOperationMessageOutPolicy = org.apache.axis2.util.PolicyUtil
                            .getMergedPolicy(bindingOperationMessageOutPolicyList, axisService);
                } catch (Exception e) {
                    log.debug(e);
                }
                if (bindingOperationMessageOutPolicy != null) {
                    if (bindingOperationMessageOutPolicy.getId() == null) {
                        // Generate an ID
                        bindingOperationMessageOutPolicy.setId(UUIDGenerator.getUUID());

                        Resource bindingOperationMessageOutPolicyResource = registry.newResource();
                        bindingOperationMessageOutPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_UUID,
                                bindingOperationMessageOutPolicy.getId());
                        // do a performance improvement
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        XMLStreamWriter writer = XMLOutputFactory.newInstance()
                                .createXMLStreamWriter(baos);
                        bindingOperationMessageOutPolicy.serialize(writer);
                        writer.flush();
                        bindingOperationMessageOutPolicyResource.setContent(baos.toString());
                        bindingOperationMessageOutPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_TYPE, ""
                                + PolicyInclude.BINDING_OUTPUT_POLICY);
                        servicePolicyResources.add(bindingOperationMessageOutPolicyResource);
                    } else if (getPolicyResourceFromList(bindingOperationMessageInPolicy.getId(),
                                                         servicePolicyResources) == null) {
                        Resource bindingOperationMessageOutPolicyResource = registry.newResource();
                        bindingOperationMessageOutPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_UUID,
                                bindingOperationMessageOutPolicy.getId());
                        // do a performance improvement
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        XMLStreamWriter writer = XMLOutputFactory.newInstance()
                                .createXMLStreamWriter(baos);
                        bindingOperationMessageOutPolicy.serialize(writer);
                        writer.flush();
                        bindingOperationMessageOutPolicyResource.setContent(baos.toString());
                        bindingOperationMessageOutPolicyResource.setProperty(
                                RegistryResources.ServiceProperties.POLICY_TYPE, ""
                                + PolicyInclude.BINDING_OUTPUT_POLICY);
                        servicePolicyResources.add(bindingOperationMessageOutPolicyResource);
                    }

                    // Refer this policy from the binding operation
                    bindingOperationResource.setProperty(
                            RegistryResources.ServiceProperties.MESSAGE_OUT_POLICY_UUID,
                            bindingOperationMessageOutPolicy.getId());

                }

                // Update binding operation resource in registry
                registry.put(serviceResourcePath + RegistryResources.ServiceProperties.BINDINGS
                        + currentAxisBinding.getName().getLocalPart()
                        + RegistryResources.ServiceProperties.OPERATIONS, bindingOperationResource);
                bindingOperationResource.discard();
            }
        }

        return servicePolicyResources;
    }

    private Resource getPolicyResourceFromList(String policyId, List policyResourceList) {

        Iterator iter = policyResourceList.iterator();
        while (iter.hasNext()) {
            Resource currentResource = (Resource) iter.next();
            if (currentResource.getProperty(RegistryResources.ServiceProperties.POLICY_UUID)
                    .equalsIgnoreCase(policyId)) {
                return currentResource;
            }
        }

        return null;
    }

    public String[] getServiceBindings(AxisService axisService) throws Exception {

        // Get all bindings
        ArrayList<String> bindingsList = new ArrayList<String>();
        Map endPointMap = axisService.getEndpoints();
        for (Object o : endPointMap.entrySet()) {
            Map.Entry entry = (Map.Entry) o;
            AxisEndpoint point = (AxisEndpoint) entry.getValue();

            String currentBinding = point.getBinding().getName().toString();
            if ((!currentBinding.contains("HttpBinding"))
                    && (!bindingsList.contains(currentBinding))) {
                bindingsList.add(currentBinding);
            }
        }
        String[] bindings = new String[bindingsList.size()];
        bindingsList.toArray(bindings);

        return bindings;
    }

    /**
     * Extract all the service policies from the AxisService.
     *
     * @param axisModule the Axis2 module
     * @return module policies
     * @throws Exception on error
     */
    private List<Resource> getModulePolicies(AxisModule axisModule) throws Exception {
        List<Resource> modulePolicies = new ArrayList<Resource>();

        PolicySubject modulePolicySubject = axisModule.getPolicySubject();
        List<PolicyComponent> policyList = new ArrayList<PolicyComponent>(modulePolicySubject.getAttachedPolicyComponents());

        // Get the merged module policy
        Policy policy = null;
        for (Iterator iterator = policyList.iterator(); iterator.hasNext();) {
            Object policyElement = iterator.next();
            if (policyElement instanceof Policy) {
                policy = (policy == null) ? (Policy) policyElement : policy.merge((Policy) policyElement);
            } else {
                PolicyReference policyReference = (PolicyReference) policyElement;

                String key = policyReference.getURI();
                int pos = key.indexOf("#");
                if (pos == 0) {
                    key = key.substring(1);
                } else if (pos > 0) {
                    key = key.substring(0, pos);
                }

                PolicyComponent attachedPolicyComponent = modulePolicySubject
                        .getAttachedPolicyComponent(key);

                if (attachedPolicyComponent != null && attachedPolicyComponent instanceof Policy) {
                    policy = (Policy) attachedPolicyComponent;
                }
            }
        }

        if (policy != null) {
            if (policy.getId() == null) {
                // Generate an ID
                policy.setId(UUIDGenerator.getUUID());
            }

            // Create a registry resource from the merged module policy
            Resource policyResource = registry.newResource();
            policyResource.setProperty(RegistryResources.ModuleProperties.POLICY_UUID, policy
                    .getId());
            policyResource.setProperty(RegistryResources.ModuleProperties.VERSION, axisModule
                    .getVersion());
            // do a performance improvement
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(baos);
            policy.serialize(writer);
            writer.flush();
            policyResource.setContent(baos.toString());
            policyResource.setProperty(RegistryResources.ModuleProperties.POLICY_TYPE, ""
                    + PolicyInclude.AXIS_MODULE_POLICY);

            modulePolicies.add(policyResource);
        }

        return modulePolicies;
    }

    public void setServiceProperty(AxisService service, String propertyName, String propertyValue)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                    + service.getAxisServiceGroup().getServiceGroupName()
                    + RegistryResources.SERVICES + service.getName();

            if (registry.resourceExists(serviceResourcePath)) {
                Resource serviceResource = registry.get(serviceResourcePath);
                serviceResource.setProperty(propertyName, propertyValue);
                registry.put(serviceResourcePath, serviceResource);
                serviceResource.discard();
            }

            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log
                    .error("Unable to set property " + propertyName + " to service "
                            + service.getName());
            throw new Exception(e);
        }
    }

    public void setServiceGroupProperty(AxisServiceGroup serviceGroup, String propertyName,
                                        String propertyValue) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                    + serviceGroup.getServiceGroupName();
            if (registry.resourceExists(serviceResourcePath)) {
                Resource serviceResource = registry.get(serviceResourcePath);
                serviceResource.setProperty(propertyName, propertyValue);
                registry.put(serviceResourcePath, serviceResource);
                serviceResource.discard();
            }

            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to set property " + propertyName + " to service group "
                    + serviceGroup.getServiceGroupName());
            throw new Exception(e);
        }
    }

    private void addServicePolicy(PolicyInclude include, List<Resource> servicePolicyDOs,
                                  int policyType) throws Exception {
        for (Object obj : include.getPolicyElements(policyType)) {

            Resource policyResource = registry.newResource();
            policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_TYPE, String
                    .valueOf(policyType));

            Policy policy;
            if (obj instanceof Policy) {
                policy = (Policy) obj;
            } else if (obj instanceof PolicyReference) {
                PolicyReference pRef = (PolicyReference) obj;
                PolicyRegistry policyRegistry = include.getPolicyRegistry();

                String key = pRef.getURI();
                if (key.startsWith("#")) {
                    key = key.substring(key.indexOf("#") + 1);
                }

                policy = policyRegistry.lookup(key);

            } else {
                throw new IllegalArgumentException("Invalid Policy Item found in collection. "
                        + "This is a programmatic error");
            }
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            XMLStreamWriter writer = XMLOutputFactory.newInstance().createXMLStreamWriter(baos);

            policy.serialize(writer);
            writer.flush();

            policyResource.setContent(baos.toString());
            policyResource.setProperty(RegistryResources.ServiceProperties.POLICY_UUID, policy
                    .getId());
            servicePolicyDOs.add(policyResource);
        }
    }

    public synchronized void handleExistingServiceInit(Resource service, AxisService axisService)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            AxisConfiguration axisConfig = axisService.getAxisConfiguration();

            List<AxisModule> serviceMods = new ArrayList<AxisModule>();

            // Disengage all the statically engaged modules (i.e. those module
            // engaged from the services.xml file)
            axisService.getEngagedModules().clear();

            // Engage modules to service
            Association[] engModules = registry.getAssociations(service.getPath(),
                                                                RegistryResources.Associations.ENGAGED_MODULES);
            for (Association association : engModules) {
                Resource mod = registry.get(association.getDestinationPath());
                String modName = mod.getProperty(RegistryResources.ModuleProperties.NAME);
                String modVersion = mod.getProperty(RegistryResources.ModuleProperties.VERSION);
                AxisModule axisModule = axisConfig.getModule(modName, modVersion);

                if (axisModule == null)
                    throw new Axis2ModuleNotFound("Module: " + modName + " not found", modName);

                if (!isGloballyEngaged(modName, modVersion)) {
                    axisService.disengageModule(axisModule);
                    // Differ service level module engagement until service level policies
                    // are processed
                    serviceMods.add(axisModule);
                }
                mod.discard();
            }

            String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                    + axisService.getAxisServiceGroup().getServiceGroupName()
                    + RegistryResources.SERVICES + axisService.getName();

            // Engage modules to operation
            Collection operations = (Collection) registry.get(service.getPath()
                    + RegistryResources.OPERATIONS);
            for (String opPath : operations.getChildren()) {
                Resource operation = registry.get(opPath);
                List<AxisModule> operationMods = new ArrayList<AxisModule>();
                AxisOperation axisOp = axisService.getOperation(new QName(operation
                        .getProperty(RegistryResources.NAME)));

                // Disengage all the statically engaged modules (i.e. those
                // module engaged from the services.xml file)
                axisOp.getEngagedModules().clear();

                Association[] associations = registry.getAssociations(opPath,
                                                                      RegistryResources.Associations.ENGAGED_MODULES);
                for (Association association : associations) {
                    Resource mod = registry.get(association.getDestinationPath());
                    String modName = mod.getProperty(RegistryResources.ModuleProperties.NAME);
                    String modVersion = mod.getProperty(RegistryResources.ModuleProperties.VERSION);
                    AxisModule axisModule = axisConfig.getModule(modName, modVersion);

                    if (axisModule == null)
                        throw new Axis2ModuleNotFound("Module: " + modName + " not found", modName);

                    if (!isGloballyEngaged(modName, modVersion)) {
                        if (!axisOp.isControlOperation()) {
                            if (!axisOp.isEngaged(axisModule)) {
                                // Differ operation level module engagement until operation level
                                // policies are processed
                                operationMods.add(axisModule);
                            }
                        }
                    }
                    mod.discard();
                }

                // Fetch and attach Service Operation and Message Policies
                for (Iterator iter = axisService.getOperations(); iter.hasNext();) {
                    AxisOperation axisOperation = (AxisOperation) iter.next();
                    if (!axisOperation.isControlOperation()) {
                        Resource operationResource = registry.get(serviceResourcePath
                                + RegistryResources.ServiceProperties.OPERATIONS
                                + axisOperation.getName().getLocalPart());
                        axisOperation.getPolicySubject().clear();
                        List operationPolicies = operationResource
                                .getPropertyValues(RegistryResources.ServiceProperties.POLICY_UUID);
                        if (operationPolicies != null) {
                            Iterator uuidIter = operationPolicies.iterator();
                            while (uuidIter.hasNext()) {
                                String currentPolicyUUID = (String) uuidIter.next();
                                String policyResourcePath = serviceResourcePath
                                        + RegistryResources.POLICIES + currentPolicyUUID;
                                if (registry.resourceExists(policyResourcePath)) {
                                    Resource policyResource = registry.get(policyResourcePath);
                                    Policy policy = PolicyEngine.getPolicy(policyResource
                                            .getContentStream());
                                    axisOperation.getPolicySubject().attachPolicy(policy);
                                    policyResource.discard();
                                } else {
                                    log.error("Failed to load Policy with ID " + currentPolicyUUID
                                            + ". The Policy does not exist.");
                                }

                            }
                        }

                        // Fetch and attach MessageIn policies for this operation
                        try {
                            axisOperation.getMessage(WSDLConstants.MESSAGE_LABEL_IN_VALUE)
                                    .getPolicySubject().clear();
                            List operationMessageInPolicies = operationResource
                                    .getPropertyValues(
                                            RegistryResources.ServiceProperties.MESSAGE_IN_POLICY_UUID);
                            if (operationMessageInPolicies != null) {
                                Iterator uuidIter = operationMessageInPolicies.iterator();
                                while (uuidIter.hasNext()) {
                                    String currentPolicyUUID = (String) uuidIter.next();
                                    String policyResourcePath = serviceResourcePath
                                            + RegistryResources.POLICIES + currentPolicyUUID;
                                    if (registry.resourceExists(policyResourcePath)) {
                                        Resource policyResource = registry.get(policyResourcePath);
                                        Policy policy = PolicyEngine.getPolicy(policyResource
                                                .getContentStream());
                                        axisOperation.getMessage(WSDLConstants.MESSAGE_LABEL_IN_VALUE)
                                                .getPolicySubject().attachPolicy(policy);
                                        policyResource.discard();
                                    } else {
                                        log.error("Failed to load Policy with ID " + currentPolicyUUID
                                                + ". The Policy does not exist.");
                                    }
                                }
                            }
                        } catch (Exception e) {
                            log.debug(e);
                        }

                        // Fetch and attach MessageOut policies for this operation
                        try {
                            axisOperation.getMessage(WSDLConstants.MESSAGE_LABEL_OUT_VALUE)
                                    .getPolicySubject().clear();
                            List operationMessageOutPolicies = operationResource
                                    .getPropertyValues(
                                            RegistryResources.ServiceProperties.MESSAGE_OUT_POLICY_UUID);
                            if (operationMessageOutPolicies != null) {
                                Iterator uuidIter = operationMessageOutPolicies.iterator();
                                while (uuidIter.hasNext()) {
                                    String currentPolicyUUID = (String) uuidIter.next();
                                    String policyResourcePath = serviceResourcePath
                                            + RegistryResources.POLICIES + currentPolicyUUID;
                                    if (registry.resourceExists(policyResourcePath)) {
                                        Resource policyResource = registry.get(policyResourcePath);
                                        Policy policy = PolicyEngine.getPolicy(policyResource
                                                .getContentStream());
                                        axisOperation.getMessage(WSDLConstants.MESSAGE_LABEL_OUT_VALUE)
                                                .getPolicySubject().attachPolicy(policy);
                                        policyResource.discard();
                                    } else {
                                        log.error("Failed to load Policy with ID " + currentPolicyUUID
                                                + ". The Policy does not exist.");
                                    }
                                }
                            }
                        } catch (Exception e) {
                            log.debug(e);
                        }

                        operationResource.discard();
                    }
                }

                // set operation description
                String documentation = operation
                        .getProperty(RegistryResources.ServiceProperties.DOCUMENTATION);
                if (documentation != null) {
                    try {
                        // tries to build an OMNode from the documentation
                        axisOp.setDocumentation(AXIOMUtil.stringToOM(documentation));
                    } catch (OMException e) {
                        axisService.setDocumentation(documentation);
                    } catch (XMLStreamException e) {
                        axisService.setDocumentation(documentation);
                    }
                }

                // Handle operation parameters
                String opParamPath = operation.getPath() + RegistryResources.PARAMETERS;
                if (registry.resourceExists(opParamPath)) {
                    Collection opParameters = (Collection) registry.get(opParamPath);
                    for (String paramPath : opParameters.getChildren()) {
                        Resource param = registry.get(paramPath);
                        StAXOMBuilder builder = new StAXOMBuilder(param.getContentStream());
                        Parameter parameter = ParameterUtil.createParameter(builder
                                .getDocumentElement());
                        Parameter p = axisOp
                                .getParameter(param.getProperty(RegistryResources.NAME));
                        if (p != null) {
                            if (!p.isLocked()) {
                                axisOp.addParameter(parameter);
                            }
                        } else {
                            axisOp.addParameter(parameter);
                        }
                        param.discard();
                    }
                    opParameters.discard();
                }
                operation.discard();

                // Engage modules for operation
                for (AxisModule mod : operationMods) {
                    axisOp.engageModule(mod);
                }
            }
            operations.discard();

            // Fetch and attach Service policies
            Resource serviceResource = registry.get(serviceResourcePath);
            axisService.getPolicySubject().clear();
            List servicePolicies = serviceResource
                    .getPropertyValues(RegistryResources.ServiceProperties.POLICY_UUID);
            serviceResource.discard();

            if (servicePolicies != null) {
                Iterator iter = servicePolicies.iterator();
                while (iter.hasNext()) {
                    String currentPolicyUUID = (String) iter.next();
                    String policyResourcePath = serviceResourcePath + RegistryResources.POLICIES
                            + currentPolicyUUID;
                    if (registry.resourceExists(policyResourcePath)) {
                        Resource policyResource = registry.get(policyResourcePath);
                        Policy policy = PolicyEngine.getPolicy(policyResource.getContentStream());
                        axisService.getPolicySubject().attachPolicy(policy);
                        policyResource.discard();
                    } else {
                        log.error("Failed to load Policy with ID " + currentPolicyUUID
                                + ". The Policy does not exist.");
                    }
                }
            }

            // Now engage service level modules
            for (AxisModule mod : serviceMods) {
                axisService.engageModule(mod);
            }

            // Fetch and attach Binding, Binding operation and their Message
            // policies
            Map endPointMap = axisService.getEndpoints();
            for (Object o : endPointMap.entrySet()) {
                Map.Entry entry = (Map.Entry) o;
                AxisEndpoint point = (AxisEndpoint) entry.getValue();
                AxisBinding currentAxisBinding = point.getBinding();

                Resource bindingResource = registry.get(serviceResourcePath
                        + RegistryResources.ServiceProperties.BINDINGS
                        + currentAxisBinding.getName().getLocalPart());
                currentAxisBinding.getPolicySubject().clear();
                List bindingPolicies = bindingResource
                        .getPropertyValues(RegistryResources.ServiceProperties.POLICY_UUID);
                bindingResource.discard();
                if (bindingPolicies != null) {
                    Iterator iter = bindingPolicies.iterator();
                    while (iter.hasNext()) {
                        String currentPolicyUUID = (String) iter.next();
                        String policyResourcePath = serviceResourcePath
                                + RegistryResources.POLICIES + currentPolicyUUID;
                        if (registry.resourceExists(policyResourcePath)) {
                            Resource policyResource = registry.get(policyResourcePath);
                            Policy policy = PolicyEngine.getPolicy(policyResource
                                    .getContentStream());
                            currentAxisBinding.getPolicySubject().attachPolicy(policy);
                            policyResource.discard();
                        } else {
                            log.error("Failed to load Policy with ID " + currentPolicyUUID
                                    + ". The Policy does not exist.");
                        }
                    }
                }

                // Fetch and attach binding operation policies
                Iterator operationsItr = currentAxisBinding.getChildren();
                while (operationsItr.hasNext()) {
                    AxisBindingOperation currentBindingOperation = (AxisBindingOperation) operationsItr
                            .next();
                    Resource bindingOperationResource = registry.get(serviceResourcePath
                            + RegistryResources.ServiceProperties.BINDINGS
                            + currentAxisBinding.getName().getLocalPart()
                            + RegistryResources.ServiceProperties.OPERATIONS
                            + currentBindingOperation.getName().getLocalPart());
                    currentBindingOperation.getPolicySubject().clear();
                    List bindingOperationPolicies = bindingOperationResource
                            .getPropertyValues(RegistryResources.ServiceProperties.POLICY_UUID);
                    if (bindingOperationPolicies != null) {
                        Iterator iterator = bindingOperationPolicies.iterator();
                        while (iterator.hasNext()) {
                            String currentPolicyUUID = (String) iterator.next();
                            String policyResourcePath = serviceResourcePath
                                    + RegistryResources.POLICIES + currentPolicyUUID;
                            if (registry.resourceExists(policyResourcePath)) {
                                Resource policyResource = registry.get(policyResourcePath);
                                Policy policy = PolicyEngine.getPolicy(policyResource
                                        .getContentStream());
                                currentBindingOperation.getPolicySubject().attachPolicy(policy);
                                policyResource.discard();
                            } else {
                                log.error("Failed to load Policy with ID " + currentPolicyUUID
                                        + ". The Policy does not exist.");
                            }
                        }
                    }

                    // Fetch and attach operation message In policies
                    try {
                        currentBindingOperation.getChild(WSDLConstants.MESSAGE_LABEL_IN_VALUE)
                                .getPolicySubject().clear();
                        List bindingOpInPolicies = bindingOperationResource
                                .getPropertyValues(
                                        RegistryResources.ServiceProperties.MESSAGE_IN_POLICY_UUID);
                        if (bindingOpInPolicies != null) {
                            Iterator iterator = bindingOpInPolicies.iterator();
                            while (iterator.hasNext()) {
                                String currentPolicyUUID = (String) iterator.next();
                                String policyResourcePath = serviceResourcePath
                                        + RegistryResources.POLICIES + currentPolicyUUID;
                                if (registry.resourceExists(policyResourcePath)) {
                                    Resource policyResource = registry.get(policyResourcePath);
                                    Policy policy = PolicyEngine.getPolicy(policyResource
                                            .getContentStream());
                                    currentBindingOperation.getChild(
                                            WSDLConstants.MESSAGE_LABEL_IN_VALUE)
                                            .getPolicySubject().attachPolicy(policy);
                                    policyResource.discard();
                                } else {
                                    log.error("Failed to load Policy with ID " + currentPolicyUUID
                                            + ". The Policy does not exist.");
                                }
                            }
                        }
                    } catch (Exception e) {
                        log.debug(e);
                    }

                    // Fetch and attach operation message Out policies
                    try {
                        currentBindingOperation.getChild(WSDLConstants.MESSAGE_LABEL_OUT_VALUE)
                                .getPolicySubject().clear();
                        List bindingOpOutPolicies = bindingOperationResource
                                .getPropertyValues(
                                        RegistryResources.ServiceProperties.MESSAGE_OUT_POLICY_UUID);
                        if (bindingOpOutPolicies != null) {
                            Iterator iterator = bindingOpOutPolicies.iterator();
                            while (iterator.hasNext()) {
                                String currentPolicyUUID = (String) iterator.next();
                                String policyResourcePath = serviceResourcePath
                                        + RegistryResources.POLICIES + currentPolicyUUID;
                                if (registry.resourceExists(policyResourcePath)) {
                                    Resource policyResource = registry.get(policyResourcePath);
                                    Policy policy = PolicyEngine.getPolicy(policyResource
                                            .getContentStream());
                                    currentBindingOperation.getChild(
                                            WSDLConstants.MESSAGE_LABEL_OUT_VALUE)
                                            .getPolicySubject().attachPolicy(policy);
                                    policyResource.discard();
                                } else {
                                    log.error("Failed to load Policy with ID " + currentPolicyUUID
                                            + ". The Policy does not exist.");
                                }
                            }
                        }
                    } catch (Exception e) {
                        log.debug(e);
                    }

                    bindingOperationResource.discard();
                }

            }

            ArrayList axisServiceParameters = axisService.getParameters();
            ListIterator iter = axisServiceParameters.listIterator();
            boolean isProxyService = false;

            while (iter.hasNext()) {
                Parameter elem = (Parameter) (iter.next());
                Object value = elem.getValue();
                if (value != null && "proxy".equals(value.toString())) {
                    isProxyService = true;
                }
            }

            // Add the service parameters
            if (isProxyService) {
                List availableParameters = axisService.getParameters();

                // Removing parameters from the registry
                ListIterator<Parameter> ite = availableParameters.listIterator();
                while (ite.hasNext()) {
                    Parameter serviceParameter = ite.next();
                    String resourcePath = RegistryResources.SERVICE_GROUPS + axisService.getName()
                            + RegistryResources.SERVICES + axisService.getName()
                            + RegistryResources.PARAMETERS + serviceParameter.getName();

                    if (registry.resourceExists(resourcePath)) {
                        registry.delete(resourcePath);
                    }
                }

                // Adding the parameters to the registry
                ListIterator<Parameter> ite2 = availableParameters.listIterator();
                while (ite2.hasNext()) {
                    Parameter serviceParameter = ite2.next();
                    updateServiceParameter(axisService, serviceParameter);
                }
            } else {
                // Add the Service Parameters
                String serviceParamsPath = service.getPath() + RegistryResources.PARAMETERS;
                if (registry.resourceExists(serviceParamsPath)) {
                    Collection parameters = (Collection) registry.get(serviceParamsPath);
                    for (String paramPath : parameters.getChildren()) {
                        Resource paramResource = registry.get(paramPath);
                        StAXOMBuilder builder = new StAXOMBuilder(paramResource.getContentStream());
                        Parameter parameter = ParameterUtil.createParameter(builder
                                .getDocumentElement());
                        Parameter p = axisService.getParameter(paramResource
                                .getProperty(RegistryResources.NAME));
                        if (p != null) {
                            if (!p.isLocked()) {
                                axisService.addParameter(parameter);
                            }
                        } else {
                            axisService.addParameter(parameter);
                        }
                        paramResource.discard();
                    }
                    parameters.discard();
                }
            }

            // add description
            String doc = service.getProperty(RegistryResources.ServiceProperties.DOCUMENTATION);
            if (doc != null) {
                try {
                    // tries to build an OMNode from the documentation
                    axisService.setDocumentation(AXIOMUtil.stringToOM(doc));
                } catch (OMException e) {
                    axisService.setDocumentation(doc);
                } catch (XMLStreamException e) {
                    axisService.setDocumentation(doc);
                }
            }

            // Handle existing transports
            if (isProxyService) {
                List availableTransports = axisService.getExposedTransports();
                ListIterator<String> i = availableTransports.listIterator();

                // Removing transports from the registry
                Association[] aa = registry.getAssociations(serviceResource.getPath(),
                        RegistryResources.Associations.EXPOSED_TRANSPORTS);
                for (Association a : aa) {
                    registry.removeAssociation(a.getSourcePath(), a.getDestinationPath(),
                            a.getAssociationType());
                }

                // Adding the transports to the registry
                while (i.hasNext()) {
                    String transport = i.next();
                    Resource transportResource = getTransport(transport);
                    serviceResource.setProperty(
                            RegistryResources.ServiceProperties.EXPOSED_ON_ALL_TANSPORTS, String
                                    .valueOf(false));
                    registry.addAssociation(serviceResource.getPath(), transportResource.getPath(),
                            RegistryResources.Associations.EXPOSED_TRANSPORTS);
                    registry.put(transportResource.getPath(), transportResource);
                    log.info("Added " + transport + " transport binding for " + axisService.getName()
                            + " service");
                }

            } else {
                if (!Boolean.valueOf(service
                        .getProperty(RegistryResources.ServiceProperties.EXPOSED_ON_ALL_TANSPORTS))){
                    axisService.setExposedTransports(new ArrayList());
                    Association[] associations = registry.getAssociations(service.getPath(),
                            RegistryResources.Associations.EXPOSED_TRANSPORTS);
                    for (Association association : associations) {
                        Resource resource = registry.get(association.getDestinationPath());
                        String transportProtocol = resource
                                .getProperty(RegistryResources.Transports.PROTOCOL_NAME);
                        axisService.addExposedTransport(transportProtocol);
                        resource.discard();
                    }
                }
            }

            // Activate/Deactivate service
            String serviceState = service.getProperty(RegistryResources.ServiceProperties.ACTIVE);
            if (serviceState == null || serviceState.trim().length() == 0) {
                serviceState = "true";
            }
            axisService.setActive(Boolean.parseBoolean(serviceState));
            if (transactionStarted) {
                registry.commitTransaction();
            }
            if (log.isDebugEnabled()) {
                log.debug("Initialized service - " + axisService.getName());
            }

        } catch (Axis2ModuleNotFound e){
            throw e;
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to handle service initialization. Service: " + axisService.getName(),
                      e);
            throw new Exception(e);
        }
    }

    private boolean isGloballyEngaged(String moduleName, String moduleVersion)
            throws RegistryException {

        String resourcePath = RegistryResources.MODULES + moduleName + "/" + moduleVersion;
        if (registry.resourceExists(resourcePath)) {
            Resource resource = registry.get(resourcePath);
            boolean globallyEngaged = Boolean.parseBoolean(resource
                    .getProperty(RegistryResources.ModuleProperties.GLOBALLY_ENGAGED));
            resource.discard();
            return globallyEngaged;
        }
        return false;
    }

    public void globallyEngageModule(AxisModule module) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {

            String resourcePath = PersistenceUtils.getResourcePath(module);

            if (transactionStarted) {
                registry.beginTransaction();
            }

            if (registry.resourceExists(resourcePath)) {
                Resource resource = registry.get(resourcePath);
                resource.setProperty(RegistryResources.ModuleProperties.GLOBALLY_ENGAGED,
                                     Boolean.TRUE.toString());
                registry.put(resourcePath, resource);
                resource.discard();
            } else {
                log.error("Trying to engage unavailable module " + module.getName());
                throw new Exception("Trying to engage unavailable module " + module.getName());
            }

            if (transactionStarted) {
                registry.commitTransaction();
            }

            if (log.isDebugEnabled()) {
                log.debug(module.getName() + " is globally engaged");
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + "globally.", e);
            throw new Exception(e);
        }

    }

    public void globallyDisengageModule(AxisModule module) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            String resourcePath = PersistenceUtils.getResourcePath(module);

            if (transactionStarted) {
                registry.beginTransaction();
            }

            if (registry.resourceExists(resourcePath)) {
                Resource resource = registry.get(resourcePath);
                resource.setProperty(RegistryResources.ModuleProperties.GLOBALLY_ENGAGED,
                                     Boolean.FALSE.toString());
                registry.put(resourcePath, resource);
                resource.discard();
            }

            if (transactionStarted) {
                registry.commitTransaction();
            }

            if (log.isDebugEnabled()) {
                log.debug(module.getName() + " is globally disengaged");
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to disengage " + module.getName() + "globally.", e);
            throw new Exception(e);
        }
    }

    public void engageModuleForServiceGroup(AxisModule module, AxisServiceGroup serviceGroup)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String serviceGroupResourcePath = PersistenceUtils.getResourcePath(serviceGroup);
            String moduleResourcePath = PersistenceUtils.getResourcePath(module);

            registry.addAssociation(serviceGroupResourcePath, moduleResourcePath,
                                    RegistryResources.Associations.ENGAGED_MODULES);

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + " module to "
                    + serviceGroup.getServiceGroupName() + "service group ", e);
            throw new Exception(e);
        }

    }

    public void disengageModuleForServiceGroup(AxisModule module, AxisServiceGroup serviceGroup)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String serviceGroupResourcePath = PersistenceUtils.getResourcePath(serviceGroup);
            String moduleResourcePath = PersistenceUtils.getResourcePath(module);

            registry.removeAssociation(serviceGroupResourcePath, moduleResourcePath,
                                       RegistryResources.Associations.ENGAGED_MODULES);

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + " module to "
                    + serviceGroup.getServiceGroupName() + "service group ", e);
            throw new Exception(e);
        }

    }

    public void engageModuleForService(AxisModule module, AxisService service) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String serviceResourcePath = PersistenceUtils.getResourcePath(service);
            String moduleResourcePath = PersistenceUtils.getResourcePath(module);

            registry.addAssociation(serviceResourcePath, moduleResourcePath,
                                    RegistryResources.Associations.ENGAGED_MODULES);

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + " module to " + service.getName()
                    + " service");
            throw new Exception(e);
        }

    }

    public void disengageModuleForService(AxisModule module, AxisService service) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String serviceResourcePath = PersistenceUtils.getResourcePath(service);
            String moduleResourcePath = PersistenceUtils.getResourcePath(module);

            registry.removeAssociation(serviceResourcePath, moduleResourcePath,
                                       RegistryResources.Associations.ENGAGED_MODULES);

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + " module to " + service.getName()
                    + " service");
            throw new Exception(e);
        }

    }

    public void engageModuleForOperation(AxisModule module, AxisOperation operation)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String operationPath = PersistenceUtils.getResourcePath(operation);
            String moduleResourcePath = PersistenceUtils.getResourcePath(module);

            registry.addAssociation(operationPath, moduleResourcePath,
                                    RegistryResources.Associations.ENGAGED_MODULES);

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + " module to "
                    + module.getOperations() + "operation ", e);
            throw new Exception(e);
        }

    }

    public void disengageModuleForOperation(AxisModule module, AxisOperation operation)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }

            String operationPath = PersistenceUtils.getResourcePath(operation);
            String moduleResourcePath = PersistenceUtils.getResourcePath(module);

            registry.removeAssociation(operationPath, moduleResourcePath,
                                       RegistryResources.Associations.ENGAGED_MODULES);

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to engage " + module.getName() + " module to "
                    + module.getOperations() + "operation ", e);
            throw new Exception(e);
        }

    }

    public synchronized void handleExistingServiceGroupInit(Resource sgResource, AxisServiceGroup serviceGroup)
            throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            String paramResource = sgResource.getPath() + RegistryResources.PARAMETERS;
            if (transactionStarted) {
                registry.beginTransaction();
            }

            // Add the Service Group Parameters
            if (registry.resourceExists(paramResource)) {
                Collection parameters = (Collection) registry.get(paramResource);
                for (String param : parameters.getChildren()) {
                    Resource resource = registry.get(param);
                    StAXOMBuilder builder = new StAXOMBuilder(resource.getContentStream());
                    Parameter parameter = ParameterUtil.createParameter(builder
                            .getDocumentElement());
                    Parameter p = serviceGroup.getParameter(resource
                            .getProperty(RegistryResources.NAME));
                    if (p != null) {
                        if (!p.isLocked()) {
                            serviceGroup.addParameter(parameter);
                        }
                    } else {
                        serviceGroup.addParameter(parameter);
                    }
                    resource.discard();
                }
                parameters.discard();
            }

            AxisConfiguration axisConfig = serviceGroup.getAxisConfiguration();

            // Disengage all the statically engaged modules (i.e. those module
            // engaged from the services.xml file)
            serviceGroup.getEngagedModules().clear();

            // Engage modules to service group
            Association[] associations = registry.getAssociations(sgResource.getPath(),
                                                                  RegistryResources.Associations.ENGAGED_MODULES);
            for (Association association : associations) {
                String moduleResourcePath = association.getDestinationPath();
                Resource moduleResource = registry.get(moduleResourcePath);
                String moduleName = moduleResource
                        .getProperty(RegistryResources.ModuleProperties.NAME);
                String moduleVersion = moduleResource
                        .getProperty(RegistryResources.ModuleProperties.VERSION);
                AxisModule axisModule = axisConfig.getModule(moduleName, moduleVersion);

                if (axisModule == null)
                    throw new Axis2ModuleNotFound("Module: " + moduleName + " not found", moduleName);

                if (!isGloballyEngaged(moduleName, moduleVersion)) {
                    serviceGroup.disengageModule(axisModule);
                    serviceGroup.engageModule(axisModule);
                }
                moduleResource.discard();
            }
            if (transactionStarted) {
                registry.commitTransaction();
            }
            if (log.isDebugEnabled()) {
                log.debug("Initialized Service Group - " + serviceGroup.getServiceGroupName());
            }

        } catch(Axis2ModuleNotFound e) {
            throw e;
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to handle service group init. Service group: "
                    + serviceGroup.getServiceGroupName(), e);
            throw new Exception(e);
        }
    }

    public synchronized void handleNewServiceGroupAddition(AxisServiceGroup serviceGroup) throws Exception {
        Iterator services = serviceGroup.getServices();
        if (services.hasNext() && ((AxisService) services.next()).isClientSide()) {
            return;
        }
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            Resource serviceGroupResource = registry.newCollection();
            Long lastUpdatedLong = CarbonUtils.lastUpdatedTime(serviceGroup);
            if (lastUpdatedLong == null) {
                lastUpdatedLong = (Long) serviceGroup
                        .getParameterValue(RegistryResources.ServiceGroupProperties.LAST_UPDATED);
            }
            // fallback
            if (lastUpdatedLong == null) {
                lastUpdatedLong = System.currentTimeMillis();
            }

            serviceGroupResource.addProperty(RegistryResources.ServiceGroupProperties.LAST_UPDATED,
                                             lastUpdatedLong.toString());
            String sgName = serviceGroup.getServiceGroupName();
            String sgResourcePath = RegistryResources.SERVICE_GROUPS + sgName;
            registry.put(sgResourcePath, serviceGroupResource);

            AxisConfiguration axisConfiguration = serviceGroup.getAxisConfiguration();

            // Handle ServiceGroup-Module engagement
            for (Object o : serviceGroup.getEngagedModules()) {
                AxisModule axisModule = (AxisModule) o;
                if (!axisConfiguration.isEngaged(axisModule.getName())) {
                    String moduleResourcePath = RegistryResources.MODULES + axisModule.getName()
                            + "/" + axisModule.getVersion();
                    registry.addAssociation(sgResourcePath, moduleResourcePath,
                                            RegistryResources.Associations.ENGAGED_MODULES);
                }
            }

            // Handle Service Group Parameters
            for (Object o : serviceGroup.getParameters()) {
                Parameter parameter = (Parameter) o;
                if (parameter.getName() != null && parameter.getName().trim().length() != 0) {
                    Resource param = registry.newResource();
                    if (parameter.getParameterElement() != null) {
                        param.setProperty(RegistryResources.NAME, parameter.getName());
                        param.setContent(parameter.getParameterElement().toString());
                        registry.put(sgResourcePath + RegistryResources.PARAMETERS
                                + parameter.getName(), param);
                        param.discard();
                    }
                }
            }

            serviceGroupResource.addProperty(RegistryResources.SUCCESSFULLY_ADDED, "true");
            registry.put(sgResourcePath, serviceGroupResource);

            if (transactionStarted) {
                registry.commitTransaction();
            }
            if (log.isDebugEnabled()) {
                log.debug("Added new service group - " + sgName);
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to handle new service group addition. Service group: "
                    + serviceGroup.getServiceGroupName(), e);
            throw new Exception(e);
        }
    }

    public synchronized void handleNewModuleAddition(AxisModule axisModule, String moduleName,
                                        String moduleVersion) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            Resource module = registry.newCollection();
            module.addProperty(RegistryResources.ModuleProperties.NAME, moduleName);
            if (!moduleVersion.equals(RegistryResources.ModuleProperties.UNDEFINED)) {
                module.addProperty(RegistryResources.ModuleProperties.VERSION, moduleVersion);
            }
            module.addProperty(RegistryResources.ModuleProperties.GLOBALLY_ENGAGED, String
                    .valueOf(CarbonAxisConfigurator.getInstance().isGlobalyEngaged(axisModule)));
            String registryResourcePath = RegistryResources.MODULES + moduleName + "/" + moduleVersion + "/";
            registry.put(registryResourcePath, module);

            // add the module parameters
            for (Object o : axisModule.getParameters()) {
                Parameter param = (Parameter) o;
                OMElement paramEle = param.getParameterElement();
                if (paramEle != null) {
                    Resource parameter = registry.newResource();
                    parameter.setContent(paramEle.toString());
                    parameter.addProperty(RegistryResources.NAME, param.getName());
                    registry.put(RegistryResources.MODULES + moduleName + "/" + moduleVersion
                            + RegistryResources.PARAMETERS + param.getName(), parameter);
                    parameter.discard();
                }
            }

            // Persist module policies
            List<Resource> modulePolicies = getModulePolicies(axisModule);
            for (Resource modulePolicy : modulePolicies) {
                registry.put(RegistryResources.MODULES + moduleName + "/" + moduleVersion
                        + RegistryResources.POLICIES
                        + modulePolicy.getProperty(RegistryResources.ModuleProperties.POLICY_UUID),
                             modulePolicy);
            }
            module.addProperty(RegistryResources.SUCCESSFULLY_ADDED, "true");
            registry.put(registryResourcePath, module);

            if (transactionStarted) {
                registry.commitTransaction();
            }
            if (log.isDebugEnabled()) {
                log.debug("Added new module - " + axisModule.getName() + "-"
                        + axisModule.getVersion());
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to handle new module addition. Module: "
                    + Utils.getModuleName(moduleName, moduleVersion), e);
            markFaultyModule(axisModule);
            throw new Exception(e);
        }
    }

    private void markFaultyModule(AxisModule axisModule) {
        axisModule.getParent().getFaultyModules().put(
                Utils.getModuleName(axisModule.getName(), axisModule.getVersion()), axisModule.getName());
    }

    public synchronized void handleExistingModuleInit(Resource module, AxisModule axisModule) throws Exception {
        boolean transactionStarted = Transaction.isStarted();
        try {
            String paramResource = module.getPath() + RegistryResources.PARAMETERS;
            if (transactionStarted) {
                registry.beginTransaction();
            }

            // Add the Module Parameters
            if (registry.resourceExists(paramResource)) {
                Collection parameters = (Collection) registry.get(paramResource);
                for (String param : parameters.getChildren()) {
                    Resource resource = registry.get(param);
                    StAXOMBuilder builder = new StAXOMBuilder(resource.getContentStream());
                    Parameter parameter = ParameterUtil.createParameter(builder
                            .getDocumentElement());
                    Parameter p = axisModule.getParameter(resource
                            .getProperty(RegistryResources.NAME));
                    if (p != null) {
                        if (!p.isLocked()) {
                            axisModule.addParameter(parameter);
                        }
                    } else {
                        axisModule.addParameter(parameter);
                    }
                    resource.discard();
                }
                parameters.discard();
            }

            axisModule.getPolicySubject().clear();
            // Load policies from DB into AxisModule. All other policies
            // contained within
            // axisModule will be rendered invalid by this time
            String modulePath = PersistenceUtils.getResourcePath(axisModule);
            String policiesResource = modulePath + RegistryResources.POLICIES;
            if (registry.resourceExists(policiesResource)) {
                // PolicyInclude policyInclude = axisModule.getPolicyInclude();
                // policyInclude.removeAllPolicyElements();
                Collection policies = (Collection) registry.get(policiesResource);
                for (String policyResource : policies.getChildren()) {
                    Resource resource = registry.get(policyResource);
                    Policy policy = PolicyEngine.getPolicy(resource.getContentStream());
                    axisModule.getPolicySubject().attachPolicy(policy);
                    // String policyType =
                    // resource.getProperty(RegistryResources.ModuleProperties.POLICY_TYPE);
                    // policyInclude.addPolicyElement(Integer.parseInt(policyType),
                    // policy);
                    resource.discard();
                }
                policies.discard();
            }

            if (Boolean.parseBoolean(module
                    .getProperty(RegistryResources.ModuleProperties.GLOBALLY_ENGAGED))) {
                axisModule.addParameter(new Parameter(GLOBALLY_ENGAGED_PARAM_NAME, "true"));
                axisConfig.engageModule(axisModule);
            }

            if (Boolean.parseBoolean(module
                    .getProperty(GLOBALLY_ENGAGED_CUSTOM))) {
                axisModule.addParameter(new Parameter(GLOBALLY_ENGAGED_PARAM_NAME, "true"));
            }

            if (transactionStarted) {
                registry.commitTransaction();
            }
            if (log.isDebugEnabled()) {
                log.debug("Initialized module - "
                        + Utils.getModuleName(axisModule.getName(), axisModule.getVersion()));
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            log.error("Unable to handle module init. Module: "
                    + Utils.getModuleName(axisModule.getName(), axisModule.getVersion()), e);
            markFaultyModule(axisModule);
            throw new Exception(e);
        }
    }

    /**
     * Get the registry resouce corresponding to a specified transport
     *
     * @param protocolName name of the transport
     * @param isListener   true for transport listener and false for transport sender
     * @return a registry resource or null if the resource doesn't exist
     * @throws Exception if an error occurs while retrieving the resource
     */
    public Resource getTransport(String protocolName, boolean isListener) throws Exception {
        String transportResourcePath;
        if (isListener) {
            transportResourcePath = RegistryResources.TRANSPORTS + protocolName +
                    "/" + TRANSPORT_LISTENER;
        } else {
            transportResourcePath = RegistryResources.TRANSPORTS + protocolName +
                    "/" + TRANSPORT_SENDER;
        }

        if (registry.resourceExists(transportResourcePath)) {
            return registry.get(transportResourcePath);
        }
        return null;
    }

    public Resource getTransport(String protocolName) throws RegistryException {
        String resourcePath = RegistryResources.TRANSPORTS + protocolName;
        if (registry.resourceExists(resourcePath)) {
            return registry.get(resourcePath);
        }
        return null;
    }

    public void setTransportProperty(String protocol, boolean isListener, String key,
                                     String value) throws Exception {
        Resource trpResource = getTransport(protocol, isListener);
        if (trpResource != null) {
            trpResource.addProperty(key, value);
            registry.put(trpResource.getPath(), trpResource);
            trpResource.discard();
        }
    }

    public void removeOperationParameter(AxisOperation operation, Parameter parameter)
            throws Exception {
        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + operation.getAxisService().getAxisServiceGroup().getServiceGroupName()
                + RegistryResources.SERVICES + operation.getAxisService().getName();
        String operationsResourcePath = serviceResourcePath + RegistryResources.OPERATIONS
                + operation.getName().getLocalPart();
        String opParamResourcePath = operationsResourcePath + RegistryResources.PARAMETERS
                + parameter.getName();
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            if (registry.resourceExists(opParamResourcePath)) {
                registry.delete(opParamResourcePath);
            }
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to update the operation parameter " + parameter.getName()
                    + " of operation " + operation.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void updateOperationParameter(AxisOperation operation, Parameter parameter)
            throws Exception {
        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + operation.getAxisService().getAxisServiceGroup().getServiceGroupName()
                + RegistryResources.SERVICES + operation.getAxisService().getName();
        String operationsResourcePath = serviceResourcePath + RegistryResources.OPERATIONS
                + operation.getName().getLocalPart();
        String opParamResourcePath = operationsResourcePath + RegistryResources.PARAMETERS
                + parameter.getName();
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            if (registry.resourceExists(opParamResourcePath)) {
                Resource opParam = registry.get(opParamResourcePath);
                opParam.setContent(parameter.getParameterElement().toString());
                registry.put(opParamResourcePath, opParam);
                opParam.discard();
            } else {
                Resource opParam = registry.newResource();
                opParam.setContent(parameter.getParameterElement().toString());
                registry.put(opParamResourcePath, opParam);
                opParam.discard();
            }
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to update the operation parameter " + parameter.getName()
                    + " of operation " + operation.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void removeServiceParameter(AxisService service, Parameter parameter) throws Exception {
        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + service.getAxisServiceGroup().getServiceGroupName() + RegistryResources.SERVICES
                + service.getName();
        String serviceParamResourcePath = serviceResourcePath + RegistryResources.PARAMETERS
                + parameter.getName();
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            if (registry.resourceExists(serviceParamResourcePath)) {
                registry.delete(serviceParamResourcePath);
            }
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to update the service parameter " + parameter.getName()
                    + " to the service " + service.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void updateServiceParameter(AxisService service, Parameter parameter) throws Exception {

        String serviceResourcePath = RegistryResources.SERVICE_GROUPS
                + service.getAxisServiceGroup().getServiceGroupName() + RegistryResources.SERVICES
                + service.getName();
        String serviceParamResourcePath = serviceResourcePath + RegistryResources.PARAMETERS
                + parameter.getName();
        boolean transactionStarted = Transaction.isStarted();

        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            Resource serviceParamResource;

            if (!registry.resourceExists(serviceParamResourcePath)) {
                serviceParamResource = registry.newResource();
            } else {
                serviceParamResource = registry.get(serviceParamResourcePath);
            }

            serviceParamResource.addProperty(RegistryResources.ParameterProperties.NAME, parameter
                    .getName());
            serviceParamResource.addProperty(RegistryResources.ParameterProperties.TYPE, Integer
                    .toString(parameter.getParameterType()));

            if (parameter.getParameterElement() != null) {
                serviceParamResource.setContent(parameter.getParameterElement().toString());
            } else if (parameter.getValue() != null && parameter.getValue() instanceof String) {
                serviceParamResource.setContent(parameter.getValue());
            }
            registry.put(serviceParamResourcePath, serviceParamResource);
            serviceParamResource.discard();
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to update the service parameter " + parameter.getName()
                    + " to the service " + service.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void updateServiceGroupParameter(AxisServiceGroup serviceGroup, Parameter parameter)
            throws Exception {

        String serviceGroupResourcePath = RegistryResources.SERVICE_GROUPS
                + serviceGroup.getServiceGroupName();
        String paramResourcePath = serviceGroupResourcePath + RegistryResources.PARAMETERS
                + parameter.getName();
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            Resource paramResource;

            if (!registry.resourceExists(paramResourcePath)) {
                paramResource = registry.newResource();
            } else {
                paramResource = registry.get(paramResourcePath);
            }

            paramResource.addProperty(RegistryResources.ParameterProperties.NAME, parameter
                    .getName());
            paramResource.addProperty(RegistryResources.ParameterProperties.TYPE, Integer
                    .toString(parameter.getParameterType()));

            if (parameter.getParameterElement() != null) {
                paramResource.setContent(parameter.getParameterElement().toString());
            } else if (parameter.getValue() != null && parameter.getValue() instanceof String) {
                paramResource.setContent(parameter.getValue());
            }

            registry.put(paramResourcePath, paramResource);
            paramResource.discard();

            if (transactionStarted) {
                registry.commitTransaction();
            }

        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to update the service group parameter " + parameter.getName()
                    + " to the service group " + serviceGroup.getServiceGroupName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void updateModuleParameter(AxisModule module, Parameter param) throws Exception {

        String moduleParamPath = PersistenceUtils.getResourcePath(module, param);

        Resource modParam;
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            if (registry.resourceExists(moduleParamPath)) {
                modParam = registry.get(moduleParamPath);
            } else {
                modParam = registry.newResource();

            }
            modParam.setContent(param.getParameterElement().toString());
            modParam.setProperty(RegistryResources.NAME, param.getName());
            registry.put(moduleParamPath, modParam);
            modParam.discard();
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to update module parameter " + param.getName() + " of module "
                    + module.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void removeModuleParameter(AxisModule module, Parameter param) throws Exception {

        String moduleParamPath = PersistenceUtils.getResourcePath(module, param);
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            if (registry.resourceExists(moduleParamPath)) {
                registry.delete(moduleParamPath);
            }
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to remove the module parameter " + param.getName()
                    + " to the module " + module.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void removeModule(AxisModule module) throws Exception {

        String modulePath = PersistenceUtils.getResourcePath(module);
        boolean transactionStarted = Transaction.isStarted();
        try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
            if (registry.resourceExists(modulePath)) {
                registry.delete(modulePath);
            }
            if (transactionStarted) {
                registry.commitTransaction();
            }
        } catch (Throwable e) {
            if (transactionStarted) {
                registry.rollbackTransaction();
            }
            String msg = "Unable to remove module " + module.getName();
            log.error(msg, e);
            throw new Exception(msg, e);
        }

    }

    /**
     * Add a transport listener to the registry
     *
     * @param protocolName name of the transport
     * @param isStarted true if the transport is started and false otherwise
     * @param params list of transport receiver parameters
     * @throws Exception on error
     */
    public void addInTransport(String protocolName, boolean isStarted, List<Parameter> params)
            throws Exception {

        Resource transportResource = getTransport(protocolName, true);
        Resource resourceAvailTransports;

        if (transportResource == null) {

            try {
                transportResource = registry.newCollection();
                transportResource.setProperty(RegistryResources.Transports.PROTOCOL_NAME,
                                              protocolName);
                transportResource.setProperty(RegistryResources.Transports.IS_STARTED, Boolean
                        .toString(isStarted));

                if (isStarted) {
                    resourceAvailTransports = getAvailableTransportListeners();
                    if (resourceAvailTransports == null) {
                        resourceAvailTransports = registry.newResource();
                        resourceAvailTransports.setContent(protocolName + ",");
                    } else {
                        String content = new String((byte[]) resourceAvailTransports.getContent());
                        content += protocolName + ",";
                        resourceAvailTransports.setContent(content);
                    }
                    registry.put(RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_LISTENERS,
						resourceAvailTransports);
				    resourceAvailTransports.discard();
                }

                // Create listener collection for the transport
                Resource listenerResource = registry.newCollection();
                registry.put(RegistryResources.TRANSPORTS + protocolName + "/" + TRANSPORT_LISTENER,
                        registry.newCollection());
                listenerResource.discard();

                // We also need to add transport specific properties as well.
                for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
                    Parameter param = iterator.next();
                    addTransportParameter(protocolName, param, true);
                }

                registry.put(RegistryResources.TRANSPORTS + protocolName, transportResource);

            } catch (Exception e) {
                String msg = "Unable to add the transport listener to the registry. Transport: " + protocolName;
                log.error(msg, e);
                throw new Exception(msg, e);
            }
        }

        if (transportResource != null) transportResource.discard();
    }

    /**
     * Add a transport sender to the registry
     *
     * @param protocolName name of the transport
     * @param isStarted true if the transport is started and false otherwise
     * @param params list of transport sender parameters
     * @throws Exception on error
     */
    public void addOutTransport(String protocolName, boolean isStarted, List<Parameter> params)
            throws Exception {

        Resource transportResource = getTransport(protocolName, false);
        Resource resourceAvailTransports;

        if (transportResource == null) {
            try {
                transportResource = registry.newCollection();
                transportResource.setProperty(RegistryResources.Transports.PROTOCOL_NAME,
                                              protocolName);
                transportResource.setProperty(RegistryResources.Transports.IS_STARTED, Boolean
                        .toString(isStarted));

                if (isStarted) {
                    resourceAvailTransports = getAvailableTransportSenders();

                    if (resourceAvailTransports == null) {
                        resourceAvailTransports = registry.newResource();
                        resourceAvailTransports.setContent(protocolName + ",");
                    } else {
                        String content = new String((byte[]) resourceAvailTransports.getContent());
                        content += protocolName + ",";
                        resourceAvailTransports.setContent(content);
                    }

                    registry.put(RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_SENDERS,
                            resourceAvailTransports);
                    resourceAvailTransports.discard();
                }

                // Create sender collection for the transport
                Resource senderResource = registry.newCollection();
                registry.put(RegistryResources.TRANSPORTS + protocolName + "/" + TRANSPORT_SENDER, registry.newCollection());
                senderResource.discard();

                // We also need to add transport specific properties as well.
                for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
                    Parameter param = iterator.next();
                    addTransportParameter(protocolName, param, false);
                }

                registry.put(RegistryResources.TRANSPORTS + protocolName, transportResource);

            } catch (Exception e) {
                String msg = "Unable to add the transport sender to the registry. Transport: " + protocolName;
                log.error(msg, e);
                throw new Exception(msg, e);
            }
        }

        if (transportResource != null) transportResource.discard();
    }

    /**
     * Add a transport parameter to the registry. If a parameter by the same name already
     * exists this method will overwrite the existing parameter.
     *
     * @param protocol Name of the transport
     * @param param Transport parameter to be added
     * @param listener true if listener and false if sender
     * @throws Exception on error
     */
    public void addTransportParameter(String protocol, Parameter param,
                                      boolean listener) throws Exception {

        String resourcePath = RegistryResources.TRANSPORTS + protocol + "/";
        if (listener) {
            resourcePath += TRANSPORT_LISTENER;
        } else {
            resourcePath += TRANSPORT_SENDER;
        }

        if (param.getValue() != null) {
            Resource resource = registry.newResource();
            resource.setContent(param.getValue().toString());
            registry.put(resourcePath + param.getName(), resource);
            resource.discard();
        }

        if (param.getParameterElement() != null) {
            Resource resourceElement = registry.newResource();
            resourceElement.setContent(param.getParameterElement().toString());
            registry.put(resourcePath + param.getName() + ".element", resourceElement);
            resourceElement.discard();
        }
    }

    /**
     * Remove a transport parameter from the registry.
     *
     * @param protocol Name of the transport
     * @param parameterName Name of the transport parameter to be deleted
     * @param listener true if listener and false if sender
     * @throws Exception on error
     */
    public void removeTransportParameter(String protocol, String parameterName,
                                         boolean listener) throws Exception {

        String resourcePath = RegistryResources.TRANSPORTS + protocol + "/";
        if (listener) {
            resourcePath += TRANSPORT_LISTENER;
        } else {
            resourcePath += TRANSPORT_SENDER;
        }

        if (registry.resourceExists(resourcePath + parameterName)) {
            registry.delete(resourcePath + parameterName);
        }

        if (registry.resourceExists(resourcePath + parameterName + ".element")) {
            registry.delete(resourcePath + parameterName + ".element");
        }
    }

    /**
     * Update the specified in transport at startup
     *
	 * @param protocolName name of the transport
	 * @param isStarted true if the transport is started or false otherwise
	 * @param params list of parameters to stored
	 * @throws Exception on error
	 */
	public void addInTransportOnStartup(String protocolName, boolean isStarted, List<Parameter> params)
			throws Exception {
		Resource transportResource = getTransport(protocolName, true);

		if (transportResource == null) {
			addInTransport(protocolName, isStarted, params);
			return;
		}
        boolean transactionStarted = Transaction.isStarted();
		try {
            if (transactionStarted) {
                registry.beginTransaction();
            }
			// We also need to add transport specific properties as well.
			for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
				Parameter param = iterator.next();

				if (!registry.resourceExists(RegistryResources.TRANSPORTS + protocolName + "/"
						+ TRANSPORT_LISTENER + param.getName())) {
					// We only add newly added parameters.
					addTransportParameter(protocolName, param, true);
                }
			}

			Collection resorceCollection = (Collection) registry.get(RegistryResources.TRANSPORTS
					+ protocolName + "/" + TRANSPORT_LISTENER);
			String[] children = resorceCollection.getChildren();
            resorceCollection.discard();

            for (int i = 0; i < children.length; i++) {
				String child = children[i];
				boolean hasRemoved = true;
				for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
					Parameter param = iterator.next();
					if (child.equals(RegistryResources.TRANSPORTS + protocolName + "/"
							+ TRANSPORT_LISTENER + param.getName() + ".element")
							|| child.equals(RegistryResources.TRANSPORTS + protocolName + "/"
									+ TRANSPORT_LISTENER + param.getName())) {
						hasRemoved = false;
						break;
					}
				}

				if (hasRemoved) {
					registry.delete(child);
				}
			}

			registry.put(RegistryResources.TRANSPORTS + protocolName + "/" + TRANSPORT_LISTENER, transportResource);
			if (transactionStarted) {
                registry.commitTransaction();
            }
		} catch (Exception e) {
			if (transactionStarted) {
                registry.rollbackTransaction();
            }
			String msg = "Unable to add the transport. Transport: " + protocolName;
			log.error(msg, e);
			throw new Exception(msg, e);
		}

        if (transportResource != null) transportResource.discard();
    }

    /**
     * Update the specified out transport at startup
     *
     * @param protocolName name of the transport
     * @param isStarted true if the transport is started or false otherwise
     * @param params the list of parameters to be stored
     * @throws Exception on error
     */
    public void addOutTransportOnStartup(String protocolName, boolean isStarted, List<Parameter> params)
			throws Exception {
		Resource transportResource = getTransport(protocolName, false);

		if (transportResource == null) {
			addOutTransport(protocolName, isStarted, params);
			return;
		}

		try {
			// We also need to add transport specific properties as well.
			for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
				Parameter param = iterator.next();

				if (!registry.resourceExists(RegistryResources.TRANSPORTS + protocolName + "/"
						+ TRANSPORT_SENDER + param.getName())) {
					// We only add newly added parameters.
					addTransportParameter(protocolName, param, false);
                }
			}

			Collection resorceCollection = (Collection) registry.get(RegistryResources.TRANSPORTS
					+ protocolName + "/" + TRANSPORT_SENDER);
			String[] children = resorceCollection.getChildren();
            resorceCollection.discard();

            for (int i = 0; i < children.length; i++) {
				String child = children[i];
				boolean hasRemoved = true;
				for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
					Parameter param = iterator.next();
					if (child.equals(RegistryResources.TRANSPORTS + protocolName + "/"
							+ TRANSPORT_SENDER + param.getName() + ".element")
							|| child.equals(RegistryResources.TRANSPORTS + protocolName + "/"
									+ TRANSPORT_SENDER + param.getName())) {
						hasRemoved = false;
						break;
					}
				}

				if (hasRemoved) {
					registry.delete(child);
				}
			}

			registry.put(RegistryResources.TRANSPORTS + protocolName + "/" + TRANSPORT_SENDER,
                    transportResource);
		} catch (Exception e) {
			String msg = "Unable to add the transport sender. Transport: " + protocolName;
			log.error(msg, e);
			throw new Exception(msg, e);
		}

        if (transportResource != null) transportResource.discard();
    }

    /**
     * Removes a transport listener/sender from the registry. If the transport is
     * enabled, cleans up the resources corresponding to available transports too.
     *
     * @param protocolName name of the transport
     * @param isListener true for listener and false for sender
     * @throws Exception on error
     */
    public void removeTransport(String protocolName, boolean isListener) throws Exception {
        Resource availableTrps;
        String availableTransportsPath, transportPath;

        if (isListener) {
            availableTrps = getAvailableTransportListeners();
            availableTransportsPath = RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_LISTENERS;
            transportPath = RegistryResources.TRANSPORTS + protocolName + "/listener";
        } else {
            availableTrps = getAvailableTransportSenders();
            availableTransportsPath = RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_SENDERS;
            transportPath = RegistryResources.TRANSPORTS + protocolName + "/sender";
        }

        if (registry.resourceExists(transportPath)) {
            // delete the transport listener/sender resource
            registry.delete(transportPath);
        }

        if (availableTrps != null) {
            // if available then remove from the lists
            String content = new String((byte[]) availableTrps.getContent());
            content = content.replace(protocolName + ",", "");
            availableTrps.setContent(content);
            registry.put(availableTransportsPath, availableTrps);
            availableTrps.discard();
        }

    }

    /**
     * Update the resources corresponding to an in transport in the registry
     *
     * @param protocolName name of the transport
     * @param params list of parameters to be saved
     * @throws Exception on error
     */
    public void updateInTransport(String protocolName, List<Parameter> params) throws Exception {
        Collection transportResource = (Collection) getTransport(protocolName, true);
        if (transportResource != null) {
            boolean transactionStarted = Transaction.isStarted();
            try {
                Resource resourceAvailTransports;
                List<String> paramNames = new ArrayList<String>();

                resourceAvailTransports = getAvailableTransportListeners();
                if (transactionStarted) {
                    registry.beginTransaction();
                }
				if (resourceAvailTransports == null) {
					resourceAvailTransports = registry.newResource();
					resourceAvailTransports.setContent(protocolName + ",");
					registry.put(RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_LISTENERS,
							resourceAvailTransports);
					resourceAvailTransports.discard();
				} else {
					String content;
					content = new String((byte[]) resourceAvailTransports.getContent());
					if (!content.contains(protocolName)) {
						content += protocolName + ",";
						resourceAvailTransports.setContent(content);
						registry.put(RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_LISTENERS,
								resourceAvailTransports);
						resourceAvailTransports.discard();
					}
				}

                // We also need to add transport specific properties as well.
                for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
                    Parameter param = iterator.next();
                    paramNames.add(param.getName());
                    addTransportParameter(protocolName, param, true);
                }

                String[] trpListenerParams = transportResource.getChildren();
                for (String listenerParam : trpListenerParams) {
                    if (!listenerParam.endsWith(".element")) {
                        String paramName = listenerParam.substring((listenerParam.lastIndexOf("/") + 1));
                        if (!paramNames.contains(paramName)) {
                            registry.delete(listenerParam);
                            if (registry.resourceExists(listenerParam + ".element")) {
                                registry.delete(listenerParam + ".element");
                            }
                        }
                    }
                }

                registry.put(RegistryResources.TRANSPORTS + protocolName + "/" + TRANSPORT_LISTENER,
                        transportResource);
                if (transactionStarted) {
                registry.commitTransaction();
            }
            } catch (Exception e) {
                if (transactionStarted) {
                registry.rollbackTransaction();
            }
                String msg = "Unable to update the transport listener in the registry. " +
                        "Transport: " + protocolName;
                log.error(msg, e);
                throw new Exception(msg, e);
            }

            transportResource.discard();
		} else {
			addInTransport(protocolName, true, params);
		}
    }


    /**
     * Update the resources corresponding to an out transport in the registry
     *
     * @param protocolName name of the transport
     * @param params list of parameters to be saved
     * @throws Exception on error
     */
   public void updateOutTransport(String protocolName, List<Parameter> params) throws Exception {
        Collection transportResource = (Collection) getTransport(protocolName, false);
        if (transportResource != null) {

            try {
                Resource resourceAvailTransports;
                List<String> paramNames = new ArrayList<String>();

                resourceAvailTransports = getAvailableTransportSenders();

                if (resourceAvailTransports == null) {
                    resourceAvailTransports = registry.newResource();
                    resourceAvailTransports.setContent(protocolName + ",");
                    registry.put(RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_SENDERS,
                            resourceAvailTransports);
                    resourceAvailTransports.discard();
                } else {
                    String content;
                    content = new String((byte[]) resourceAvailTransports.getContent());
                    if (!content.contains(protocolName)) {
                        content += protocolName + ",";
                        resourceAvailTransports.setContent(content);
                        registry.put(RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_SENDERS,
                                resourceAvailTransports);
                        resourceAvailTransports.discard();
                    }
                }

                // We also need to add transport specific properties as well.
                for (Iterator<Parameter> iterator = params.iterator(); iterator.hasNext();) {
                    Parameter param = iterator.next();
                    paramNames.add(param.getName());
                    addTransportParameter(protocolName, param, false);
                }

                String[] trpSenderParams = transportResource.getChildren();
                for (String senderParam : trpSenderParams) {
                    if (!senderParam.endsWith(".element")) {
                        String paramName = senderParam.substring((senderParam.lastIndexOf("/") + 1));
                        if (!paramNames.contains(paramName)) {
                            registry.delete(senderParam);
                            if (registry.resourceExists(senderParam + ".element")) {
                                registry.delete(senderParam + ".element");
                            }
                        }
                    }
                }

                registry.put(RegistryResources.TRANSPORTS + protocolName + "/" + TRANSPORT_SENDER,
                        transportResource);

            } catch (Exception e) {
                String msg = "Unable to update the transport sender in the registry. " +
                        "Transport: " + protocolName;
                log.error(msg, e);
                throw new Exception(msg, e);
            }

            transportResource.discard();
        } else {
            addOutTransport(protocolName, true, params);
        }
    }

    /**
     * Fetch a parameter corresponding to a specified in transport or an out transport
     *
     * @param protocolName name of the transport protocol
     * @param paramName name of the required transport parameter
     * @param isListener inidicates whether the transport is an in-transport or an out-transport
     * @return the Registry resource corresponding to the required parameter or null
     * @throws Exception on error
     */
    public Resource getTransportParam(String protocolName, String paramName,
                                      boolean isListener) throws Exception {

        String resourcePath;
        if (protocolName != null && paramName != null) {
            if (isListener) {
                resourcePath = RegistryResources.TRANSPORTS + protocolName + "/" +
                        TRANSPORT_LISTENER + paramName;
            } else {
                resourcePath = RegistryResources.TRANSPORTS + protocolName + "/" +
                        TRANSPORT_SENDER + paramName;
            }

            if (registry.resourceExists(resourcePath)) {
                return registry.get(resourcePath);
			}
		}
		return null;
	}

    /**
     * Get the list of parameters associated with a particular transport receiver or sender
     * implementation
     *
     * @param protocol name of the transport
     * @param isListener true for transport receiver and false for sender
     * @return list of transport parameters
     * @throws Exception on error
     */
    public List<Parameter> getTransportParameters(String protocol, boolean isListener) throws Exception {
        Collection trpResource = (Collection) getTransport(protocol, isListener);
        if (trpResource == null) {
            return null;
        }

        String[] trpListenerParams = trpResource.getChildren();
        List<Parameter> params = new ArrayList<Parameter>();
        Map<String, Parameter> parametersMap = new HashMap<String, Parameter>();
        Map<String, String> parameterElementsMap = new HashMap<String, String>();

        for (String key : trpListenerParams) {
            Resource resource = registry.get(key);
            String content = new String((byte[]) resource.getContent());

            if (key.endsWith(".element")) {
                // save the element content to the elements map
                parameterElementsMap.put(key.substring(0, key.lastIndexOf(".element")), content);

            } else {
                // create a new parameter using the key
                Parameter param = new Parameter();
                param.setName(key.substring(key.lastIndexOf("/") + 1));
                // set the parameter content
                // the parameter element will be in the elements map under the same key
                try {
                    param.setValue(AXIOMUtil.stringToOM(content));
                } catch (Exception e) {
                    param.setValue(content);
                }
                // save the parameter to the parmeters map
                parametersMap.put(key, param);
                resource.discard();
            }
        }

        // Iterate thru the two maps and set the element contents to the
        // respective parameters
        Iterator<String> keys = parametersMap.keySet().iterator();
        while (keys.hasNext()) {
            String key = keys.next();
            Parameter param = parametersMap.get(key);
            String elementContent = parameterElementsMap.get(key);
            if (elementContent != null) {
                param.setParameterElement(AXIOMUtil.stringToOM(elementContent));
            }
            params.add(param);
        }

        return params;
    }

    /**
     * Get the resource corresponding to list of available/enabled transports
     *
     * @return the availableTransports resource
     * @throws Exception on error
     */
	public Resource getAvailableTransportListeners() throws Exception {
		String resourcePath = RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_LISTENERS;
		if (registry.resourceExists(resourcePath)) {
			return registry.get(resourcePath);
		}
		return null;
	}

    /**
     * Get the resource corresponding to list of available/enabled transports
     *
     * @return the availableTransports resource
     * @throws Exception on error
     */
	public Resource getAvailableTransportSenders() throws Exception {
		String resourcePath = RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_SENDERS;
		if (registry.resourceExists(resourcePath)) {
			return registry.get(resourcePath);
		}
		return null;
	}

    /**
	 * Disable the specified transport and update the registry
     *
	 * @param transport name of the transport
	 * @throws Exception on error
	 */
	public void disableTransportListener(String transport) throws Exception {
		Resource resourceAvailTransports;
		String resourcePath;
		resourcePath = RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_LISTENERS;
		String content;
		resourceAvailTransports = getAvailableTransportListeners();
		if (resourceAvailTransports != null) {
			content = new String((byte[]) resourceAvailTransports.getContent());
			content = content.replace(transport + ",", "");
			resourceAvailTransports.setContent(content);
			registry.put(resourcePath, resourceAvailTransports);
			resourceAvailTransports.discard();
            log.info("Successfully removed transport " + transport + " from the list of" +
                    " available transport listeners");
        }
	}

    /**
	 * Disable the specified transport and update the registry
     *
	 * @param transport name of the transport
	 * @throws Exception on error
	 */
	public void disableTransportSender(String transport) throws Exception {
		Resource resourceAvailTransports;
		String resourcePath;
		resourcePath = RegistryResources.TRANSPORTS + AVAILABLE_TRANSPORT_SENDERS;
		String content;
		resourceAvailTransports = getAvailableTransportSenders();
		if (resourceAvailTransports != null) {
			content = new String((byte[]) resourceAvailTransports.getContent());
			content = content.replace(transport + ",", "");
			resourceAvailTransports.setContent(content);
			registry.put(resourcePath, resourceAvailTransports);
			resourceAvailTransports.discard();
            log.info("Successfully removed transport " + transport + " from the list of" +
                    " available transport senders");
        }
	}

    /**
	 * Removes an exposed transport from a given service.
	 *
	 * @param serviceName Name of the service where new transport to be removed.
	 * @param transportProtocol Name of the transport to be removed.
	 * @param transports
	 * @throws Exception
	 */
	public void removeExposedTransports(String serviceName, String transportProtocol,
			TransportSummary[] transports) throws Exception {

        AxisService axisService;
		Resource serviceResource;
		Resource transportResource;

		if (serviceName == null) {
			if (log.isDebugEnabled()) {
				log.debug("Invalid service name");
			}
			throw new Exception("Invalid service name");
		}

		if (transportProtocol == null) {
			if (log.isDebugEnabled()) {
				log.debug("Invalid transport name");
			}
			throw new Exception("Invalid transport name");
		}

		try {
			axisService = axisConfig.getServiceForActivation(serviceName);

			if (axisService == null) {
				if (log.isDebugEnabled()) {
					log.debug("No service found for the provided service name" + serviceName);
				}
				throw new Exception("No service found for the provided service name");
			}

			serviceResource = getService(axisService);
			transportResource = getTransport(transportProtocol);

			// Keep this commented for the time being.
			// registry.beginTransaction();

			registry.removeAssociation(serviceResource.getPath(), transportResource.getPath(),
					RegistryResources.Associations.EXPOSED_TRANSPORTS);
			axisService.removeExposedTransport(transportProtocol);
			registry.put(transportResource.getPath(), transportResource);
            transportResource.discard();

			for (TransportSummary transport : transports) {
				if (transport.isListenerActive()) {
					// Only the active transports are considered here.
					String protocol = transport.getProtocol();
					if (!protocol.equals(transportProtocol)) {
						axisService.addExposedTransport(protocol);
						transportResource = getTransport(protocol);
						registry.addAssociation(serviceResource.getPath(), transportResource
								.getPath(), RegistryResources.Associations.EXPOSED_TRANSPORTS);
						registry.put(transportResource.getPath(), transportResource);
                        transportResource.discard();
					}
				}
			}

			axisService.setEnableAllTransports(false);
			serviceResource.setProperty(
					RegistryResources.ServiceProperties.EXPOSED_ON_ALL_TANSPORTS, String
							.valueOf(false));
			registry.put(serviceResource.getPath(), serviceResource);
            serviceResource.discard();
			// Keep this commented for the time being.
			// registry.commitTransaction();
		} catch (Exception e) {
			if (registry != null) {
				// Keep this commented for the time being.
				// registry.rollbackTransaction();
			}
			log.error("Error while removing exposed transport " + transportProtocol, e);
			throw new Exception("Error while removing exposed transport", e);
		}
	}
}