/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.core.transports;

import org.wso2.carbon.core.transports.util.TransportParameter;

/**
 * All the transport specific bundles should have an implementation of this interface. Once you have
 * such implementation you can register it against the TransportService. Since Transport Management
 * bundle listen on TransportService - it can easily pick the registered transport specific bundles.
 * 
 */
public interface TransportService {

	/**
	 * Get the name of the transport protocol.
     *
     * @return Name of the transport as a strign
	 */
	public String getName();

	/**
	 * Get the transport in specific parameters.
     *
     * @return An array of transport parameters or null
	 */
	public TransportParameter[] getTransportInParameters();

    /**
     * Get the transport out specific parameters
     *
     * @return An array of transport parameters or null
     */
    public TransportParameter[] getTransportOutParameters();

    /**
	 * Check whether the transport is enabled at startup
     *
     * @return a boolean value
	 */
	public boolean isEnableAtStartup();

	/**
	 * Whether the transport listener is active. If it's already enabled then it's considered as active.
     *
     * @return true if the listener is active and false if not
	 */
	public boolean isListenerActive();

    /**
     * Whether the transport sender is active.
     *
     * @return true if the sender is active and false if not
     */
    public boolean isSenderActive();

    /**
     * Whether the dependencies required by the transport are available
     *
     * @param params An array of transport parameters
     * @return true if the dependencies are available and false if not
     */
    public boolean dependenciesAvailable(TransportParameter[] params);
}
