/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.registry.app;

import org.apache.abdera.protocol.server.CollectionAdapter;
import org.apache.abdera.protocol.server.RequestContext;
import org.apache.abdera.protocol.server.TargetType;
import org.apache.abdera.protocol.server.impl.AbstractWorkspaceProvider;
import org.apache.abdera.protocol.server.impl.SimpleWorkspaceInfo;
import org.apache.abdera.protocol.server.impl.TemplateTargetBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.config.RegistryContext;
import org.wso2.carbon.registry.core.jdbc.EmbeddedRegistry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;

public class RegistryProvider extends AbstractWorkspaceProvider {

    private static final Log log = LogFactory.getLog(RegistryProvider.class);

    RegistryAdapter adapter;
    public static final String baseURI = "/registry";

    public CollectionAdapter getCollectionAdapter(RequestContext request) {
        return adapter;
    }

    public RegistryProvider() throws Exception {
        RegistryContext context = RegistryContext.getSingleton();
        if (context == null) {

            String msg = "Registry is not properly initialized. Registry context is not found.";
            log.error(msg);
            throw new RegistryException(msg);
            
            //InputStream is = null;
            //try {
            //    is = new FileInputStream("registry.xml");
            //} catch (FileNotFoundException e) {
            //    // no problemo
            //}
            //context = new RegistryContext(is);
            //RegistryContext.setSingleton(context);
        }

        EmbeddedRegistry embeddedRegistry;
        try {
            embeddedRegistry = Utils.getEmbeddedRegistry();
        } catch (RegistryException e) {
            embeddedRegistry = context.getEmbeddedRegistry();
        }

        RegistryResolver resolver = new RegistryResolver(embeddedRegistry, baseURI);
        adapter = new RegistryAdapter(resolver, embeddedRegistry);
        
        this.setTargetResolver(resolver);

        this.setTargetBuilder(
                new TemplateTargetBuilder().setTemplate(TargetType.TYPE_COLLECTION,
                                                        "{target_base}/atom/{collection}").
                        setTemplate(RegistryResolver.TAGS_TYPE, "{target_base}/atom/{collection};tags"));
        SimpleWorkspaceInfo workspace = new SimpleWorkspaceInfo();
        workspace.setTitle("A Simple Workspace");
        workspace.addCollection(adapter);
        addWorkspace(workspace);
    }
}
