package org.wso2.carbon.registry.app;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.*;
import org.wso2.carbon.registry.core.utils.AuthorizationUtils;
import org.wso2.carbon.registry.core.utils.RegistryUtils;
import org.wso2.carbon.registry.core.internal.RegistryCoreServiceComponent;
import org.wso2.carbon.registry.core.config.RegistryConfiguration;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.internal.RegistryCoreServiceComponent;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.session.CurrentSession;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.authorizing.AuthorizingRealm;
import org.wso2.carbon.user.core.authorizing.AuthorizingRealmConfig;
import org.wso2.carbon.user.core.def.DefaultRealm;

import javax.sql.DataSource;
import java.net.MalformedURLException;

public class RemoteRegistryService implements RegistryService{

    private static final Log log = LogFactory.getLog(RemoteRegistryService.class);
    private Registry registry;
    private UserRealm userRealm;
    private UserRealm systemUserRealm;
    private String url;

    public RemoteRegistryService (String url,String username,String password, UserRealm userRealm, String chroot) throws RegistryException {
        try {
            RemoteRegistry remote = new RemoteRegistry(url, username, password);

            //Hack to authenticate the user with the remote registry as remote regitry
            //doesn't provide a way to log in
            remote.get("/");

            //If chroot option is provided, create a ChrootRegistry
            if (chroot != null ) {
                registry = new ChrootRegistry(remote,chroot);
            } else {
                registry = remote;
            }

            this.url = url;
            this.userRealm = userRealm;

            this.systemUserRealm = new AuthorizingRealm();
            AuthorizingRealmConfig config =
                    (AuthorizingRealmConfig) systemUserRealm.getRealmConfiguration();
            config.setRealm(userRealm);
            RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
            config.setAuthenticatedUserName(regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME));
            config.setCurrentUserReadable(true);
            config.setCurrentUserEditable(true);
            this.systemUserRealm.init(config);

            addRootCollection();
            RegistryUtils.addSystemCollection(registry, systemUserRealm);
            RegistryUtils.addMountCollection(registry, systemUserRealm);
            RegistryUtils.addUserProfileCollection(registry, systemUserRealm, RegistryConstants.PROFILES_PATH);
            RegistryUtils.addServiceStoreCollection(registry, systemUserRealm, RegistryConstants.GOVERNANCE_SERVICE_PATH);

        } catch (MalformedURLException e) {
            log.fatal("Registry URL is malformed, Registry configuration must be invalid",e);
            throw new RegistryException("URL is malformed");
        } catch (Exception e) {
            log.fatal("Error initializing the remote registry, Registry configuration must be invalid",e);
            throw new RegistryException("Error initializing the remote registry");
        }
    }

    public UserRegistry getUserRegistry() throws RegistryException {
        return new CarbonRegistry(RegistryConstants.ANONYMOUS_USER, registry, userRealm);
    }

    public UserRegistry getSystemRegistry() throws RegistryException {
        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);
        return new CarbonRegistry(username, registry, userRealm);
    }

    public UserRegistry getUserRegistry(String username, String password) throws RegistryException {
        try {

            RemoteRegistry userRemote = new RemoteRegistry(url, username, password);
            return new CarbonRegistry(username, userRemote, userRealm);

        } catch (MalformedURLException e) {
            log.fatal("Registry URL is malformed, Registry configuration must be invalid",e);
            throw new RegistryException("URL is malformed");
        } catch (Exception e) {
            log.fatal("Error initializing the remote registry, User credentials must be invalid",e);
            throw new RegistryException("Error initializing the remote registry");
        }
    }

    public UserRegistry getUserRegistry(String userName) throws RegistryException {
        return new UserRegistry(userName, registry, userRealm);
    }

    private void addRootCollection() throws RegistryException {
        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);

        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);

        try {

            AuthorizationUtils.setRootAuthorizations(RegistryConstants.ROOT_PATH, userRealm);

         } catch (Exception e) {

            String msg = "Failed to add the root collection to the registry.";
            log.fatal(msg, e);

            throw new RegistryException(msg, e);
        } finally {
            CurrentSession.removeUser();
            CurrentSession.removeRealm();
        }

    }
}
