/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.registry.app;

import org.wso2.carbon.registry.core.ResourceImpl;
import org.wso2.carbon.registry.core.exceptions.RegistryException;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.net.URLConnection;

public class RemoteResourceImpl extends ResourceImpl {
    URL contentURL;
    String authorizationString;
    InputStream contentStream;

    public void setContentURL(URL contentURL) {
        this.contentURL = contentURL;
    }

    public String getAuthorizationString() {
        return authorizationString;
    }

    public void setAuthorizationString(String authorizationString) {
        this.authorizationString = authorizationString;
    }

    public InputStream getContentStream() throws RegistryException {

        if (content != null) {
            InputStream is = super.getContentStream();
            if (is != null) {
                return is;
            }
        }

        if(contentURL != null) {
              return getContentStreamFromURL();
        }

        return contentStream;
    }

    public Object getContent() throws RegistryException {

        if (content != null) {
            if (content instanceof String) {
                // Remote registry put method handles String content in a different manner - this is
                // to reflect that change - to be consistent with embedded registry.
                return ((String) content).getBytes();
            }
            return content;
        }

        try {

            InputStream is = getContentStreamFromURL();

            if (is == null) {
                return null;    
            }

            ByteArrayOutputStream os = new ByteArrayOutputStream();
            int nextChar;
            while ( ( nextChar = is.read() ) != -1  ) {
                os.write(nextChar);
            }
            os.flush();
            content = os.toByteArray();
            return content;

        } catch (Exception e) {
           throw new RegistryException("Couldn't get content stream",e);
        }

    }

    private InputStream getContentStreamFromURL() throws RegistryException {

        if (contentURL != null) {
            try {
                URLConnection uc = new URL(contentURL.toString().replace(" ", "+")).openConnection();
                if (uc instanceof HttpsURLConnection) { //HTTPS URL?
                    ((HttpsURLConnection) uc).setHostnameVerifier(new HostnameVerifier() {
                        public boolean verify(String hostname, SSLSession session) {
                            return true;
                        }
                    });
                }
                if (authorizationString != null) {
                    uc.setRequestProperty("Authorization", authorizationString);
                }
                return uc.getInputStream();

            } catch (IOException e) {
                throw new RegistryException("Couldn't open stream to source URL " + contentURL);
            }
        }

        return null;
    }


    
}
