package org.wso2.carbon.registry.app;

import org.apache.axiom.om.util.Base64;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.registry.core.Collection;
import org.wso2.carbon.registry.core.utils.RegistryUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Date;
import java.net.URLDecoder;

public class ResourceServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(ResourceServlet.class);

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        try {
            UserRegistry registry = null;
            // Set up secure registry instance
            String authorizationString = request.getHeader("Authorization");
            if (authorizationString != null) {
                // spliting the Authorization string "Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=="
                String values[] = authorizationString.split("\\ ");
                if ("Basic".equals(values[0])) {
                    // Decode usename/password
                    authorizationString = new String(Base64.decode(values[1]));
                    values = authorizationString.split("\\:");
                    String userName = values[0];
                    String password = values[1];
                    try {
                        registry = Utils.getEmbeddedRegistry().getUserRegistry(userName, password);
                    } catch (RegistryException e) {
                        String msg = "Not authorized";
                        log.error(msg);
                        response.setStatus(401);
                        return;
                    }
                } 
            } else {
                registry = Utils.getEmbeddedRegistry().getUserRegistry(RegistryConstants.GUEST_USER);
            }

            if (registry == null) {
                String msg = "Registry is not initialized.";
                log.error(msg);
                response.setStatus(400);
                return;
            }

            //UserRegistry registry = Utils.getEmbeddedRegistry().getUserRegistry(
            //            RegistryConstants.GUEST_USER);
            String uri = request.getRequestURI();
            int idx = uri.indexOf("resource");
            String path = uri.substring(idx+8);
            if (path == null) {
                String msg = "Could not get the resource content. Path is not specified.";
                log.error(msg);
                response.setStatus(400);
                return;
            }

            Resource resource = null;
            try {                
                resource = registry.get(URLDecoder.decode(path, "utf-8"));
            } catch (RegistryException e) {
                String msg = "Error retrieving the resource " + path + ". " + e.getMessage();
                log.error(msg, e);
                throw e;
            }

            if (resource instanceof Collection) {
                String msg = "Could not get the resource content. Path " + path + " refers to a collection.";
                log.error(msg);
                response.setStatus(501);
                return;
            }

            // date based conditional get
            long ifModifiedSinceValue = request.getDateHeader("If-Modified-Since");
            long lastModifiedValue = resource.getLastModified().getTime();
            if (ifModifiedSinceValue > 0) {
                // convert the time values from mili to seconds
                ifModifiedSinceValue /= 1000;
                lastModifiedValue /= 1000;

                /* condition to check we have latested updates interms of dates */
                if (ifModifiedSinceValue >= lastModifiedValue) {
                    /* no need to response with data */
                    response.setStatus(304);
                    return;
                }
            }
            response.setDateHeader("Last-Modified", lastModifiedValue);

            // etag based conditional get
            String ifNonMatchValue = request.getHeader("if-none-match");
            String currentEtag = APPUtils.calculateEntityTag(resource);
            if (ifNonMatchValue != null) {
                if (ifNonMatchValue.equals(currentEtag)) {
                    /* the version is not modified */
                    response.setStatus(304);
                    return;
                }
            }
            response.setHeader("ETag", currentEtag);

            if (resource.getMediaType() != null && resource.getMediaType().length() > 0) {
                response.setContentType(resource.getMediaType());
            } else {
                response.setHeader(
                        "Content-Disposition", "attachment; filename=" + RegistryUtils.getResourceName(path));
                response.setContentType("application/download");
            }

            InputStream contentStream = null;
            if (resource.getContent() != null) {
                contentStream = resource.getContentStream();
            }
            if (contentStream != null) {

                try {
                    ServletOutputStream servletOutputStream = response.getOutputStream();
                    byte[] contentChunk = new byte[1024];
                    int byteCount;
                    while ((byteCount = contentStream.read(contentChunk)) != -1) {
                        servletOutputStream.write(contentChunk, 0, byteCount);
                    }

                    response.flushBuffer();
                    servletOutputStream.flush();

                } finally {
                    contentStream.close();
                }

            } else {
                Object content = resource.getContent();
                if (content != null) {

                    if (content instanceof byte[]) {
                        ServletOutputStream servletOutputStream = response.getOutputStream();
                        servletOutputStream.write((byte[])content);
                        response.flushBuffer();
                        servletOutputStream.flush();
                    } else {
                        PrintWriter writer = response.getWriter();
                        writer.write(content.toString());
                        writer.flush();
                    }
                }
            }

            resource.discard();

        } catch (RegistryException e) {
            String msg = "Failed to get resource content. " + e.getMessage();
            log.error(msg, e);
            response.setStatus(500);
            return;
        }
    }

}
