package org.wso2.carbon.registry.app;

import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.jdbc.EmbeddedRegistry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.axis2.context.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axiom.om.util.Base64;

import javax.servlet.http.HttpServletRequest;
import java.security.MessageDigest;

public class Utils {
    private static final Log log = LogFactory.getLog(Utils.class);

    private static EmbeddedRegistry embeddedRegistry;

    public static synchronized void setEmbeddedRegistry(EmbeddedRegistry embRegistry) {
        embeddedRegistry = embRegistry;
    }

    public static Registry getRegistry() throws RegistryException {

        MessageContext messageContext = MessageContext.getCurrentMessageContext();
        if (messageContext == null) {
            String msg = "Could not get the user's Registry session. Message context not found.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        HttpServletRequest request =
                (HttpServletRequest) messageContext.getProperty("transport.http.servletRequest");

        Registry registry =
                (Registry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        if (registry == null) {

            String msg = "User's Registry instance is not found. " +
                    "Creating a anonymous Registry instance for the user.";
            if (log.isDebugEnabled()) {
                log.debug(msg);
            }

            registry = getEmbeddedRegistry().getUserRegistry(RegistryConstants.ADMIN_USER,
                    RegistryConstants.ADMIN_PASSWORD);
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }

        return registry;
    }

    public static UserRegistry getRegistry(HttpServletRequest request) throws RegistryException {

        UserRegistry registry =
                (UserRegistry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);
        if (registry == null) {
            String auth = request.getHeader("Authorization");
            if (auth != null && auth.length() > 0) {
                String[] aParts = auth.trim().split(" ");
                if (aParts.length == 2) {
                    String credentials = aParts[1];
                    String decodedCredentials = new String(Base64.decode(credentials));
                    String[] cParts = decodedCredentials.trim().split(":");
                    if (cParts.length == 2) {
                        String userName = cParts[0];
                        String password = cParts[1];

                        registry = getEmbeddedRegistry().getUserRegistry(userName, password);
                        request.getSession()
                                .setAttribute(RegistryConstants.USER_REGISTRY, registry);
                    }
                }
            }
        }
        if (registry == null) {

            String msg = "User's Registry instance is not found. " +
                    "Creating a anonymous Registry instance for the user.";
            if (log.isDebugEnabled()) {
                log.debug(msg);
            }

            registry = getEmbeddedRegistry().getUserRegistry();
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }

        return registry;
    }

    public static EmbeddedRegistry getEmbeddedRegistry() throws RegistryException {
        if (embeddedRegistry == null) {
            String msg = "Embedded Registry service is not available. Make sure that the " +
                    "required version of the Registry core component is properly intalled.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        return embeddedRegistry;
    }

    private static char[] hexMap = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};

    public static String getUUID(String uri){
        char[] tempArray = new char[36];
        try{
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(uri.getBytes());
            byte[] password = md5.digest();
            int count = 0;
            for(int i=0;i<password.length;i++){

                if(count==8 || count==13 || count==18 || count==23){
                    tempArray[count++] = '-';
                }
                //if(count==8 || count==13 ||count==18||count==23){
                //    tempArray[i] = '-';
                //    continue;
                //}

                //System.out.println(Integer.toHexString(password[i]));

                //tempArray[i] = (char)(password[i] & 0x00000000F);
                //tempArray[count] = (char)((password[i] & 0x0000000F) >> 4);

                tempArray[count++] = hexMap[(password[i] & 0x000000F0) >> 4];
                tempArray[count++] = hexMap[(password[i] & 0x0000000F)];

            }
        }catch(Exception e){
            e.printStackTrace();
        }
        return (new String(tempArray));
    }

    private static final String[] hexamap = {
        "%00", "%01", "%02", "%03", "%04", "%05", "%06", "%07", "%08", "%09", "%0A", "%0B", "%0C", "%0D", "%0E", "%0F",
        "%10", "%11", "%12", "%13", "%14", "%15", "%16", "%17", "%18", "%19", "%1A", "%1B", "%1C", "%1D", "%1E", "%1F",
        "%20", "%21", "%22", "%23", "%24", "%25", "%26", "%27", "%28", "%29", "%2A", "%2B", "%2C", "%2D", "%2E", "%2F",
        "%30", "%31", "%32", "%33", "%34", "%35", "%36", "%37", "%38", "%39", "%3A", "%3B", "%3C", "%3D", "%3E", "%3F",
        "%40", "%41", "%42", "%43", "%44", "%45", "%46", "%47", "%48", "%49", "%4A", "%4B", "%4C", "%4D", "%4E", "%4F",
        "%50", "%51", "%52", "%53", "%54", "%55", "%56", "%57", "%58", "%59", "%5A", "%5B", "%5C", "%5D", "%5E", "%5F",
        "%60", "%61", "%62", "%63", "%64", "%65", "%66", "%67", "%68", "%69", "%6A", "%6B", "%6C", "%6D", "%6E", "%6F",
        "%70", "%71", "%72", "%73", "%74", "%75", "%76", "%77", "%78", "%79", "%7A", "%7B", "%7C", "%7D", "%7E", "%7F",
        "%80", "%81", "%82", "%83", "%84", "%85", "%86", "%87", "%88", "%89", "%8A", "%8B", "%8C", "%8D", "%8E", "%8F",
        "%90", "%91", "%92", "%93", "%94", "%95", "%96", "%97", "%98", "%99", "%9A", "%9B", "%9C", "%9D", "%9E", "%9F",
        "%A0", "%A1", "%A2", "%A3", "%A4", "%A5", "%A6", "%A7", "%A8", "%A9", "%AA", "%AB", "%AC", "%AD", "%AE", "%AF",
        "%B0", "%B1", "%B2", "%B3", "%B4", "%B5", "%B6", "%B7", "%B8", "%B9", "%BA", "%BB", "%BC", "%BD", "%BE", "%BF",
        "%C0", "%C1", "%C2", "%C3", "%C4", "%C5", "%C6", "%C7", "%C8", "%C9", "%CA", "%CB", "%CC", "%CD", "%CE", "%CF",
        "%D0", "%D1", "%D2", "%D3", "%D4", "%D5", "%D6", "%D7", "%D8", "%D9", "%DA", "%DB", "%DC", "%DD", "%DE", "%DF",
        "%E0", "%E1", "%E2", "%E3", "%E4", "%E5", "%E6", "%E7", "%E8", "%E9", "%EA", "%EB", "%EC", "%ED", "%EE", "%EF",
        "%F0", "%F1", "%F2", "%F3", "%F4", "%F5", "%F6", "%F7", "%F8", "%F9", "%FA", "%FB", "%FC", "%FD", "%FE", "%FF"};

    public static String encodeRegistryPath(String path) {
        StringBuffer sbuf = new StringBuffer();
        int len = path.length();
        for (int i = 0; i < len; i++) {
          int ch = path.charAt(i);
          if (('A' <= ch && ch <= 'Z')          // alpha numeric characters
           || ('a' <= ch && ch <= 'z')
           || ('0' <= ch && ch <= '9')) {
               sbuf.append((char)ch);
          } else if (ch == '-' || ch == '_'		// allowed characters
                  || ch == '.' || ch == '/'
                  || ch == ' ' || ch == ';' || ch == ':') {
            sbuf.append((char)ch);
          } else if (ch <= 0x007F) {		    // other ASCII
            sbuf.append(hexamap[ch]);
          } else if (ch <= 0x07FF) {		    // non-ASCII <= 0x7FF
            sbuf.append(hexamap[0xC0 | (ch >> 6)]);
            sbuf.append(hexamap[0x80 | (ch & 0x3F)]);
          } else {					            // 0x7FF < ch <= 0xFFFF
            sbuf.append(hexamap[0xE0 | (ch >> 12)]);
            sbuf.append(hexamap[0x80 | ((ch >> 6) & 0x3F)]);
            sbuf.append(hexamap[0x80 | (ch & 0x3F)]);
          }
        }
        return sbuf.toString();
    }    
}
