/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.jdbc.dao.ResourceVersionDAO;
import org.wso2.carbon.registry.core.jdbc.utils.Transaction;
import org.wso2.carbon.registry.core.jdbc.utils.VersionRetriever;
import org.wso2.carbon.registry.core.jdbc.dataobjects.ResourceDO;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class CollectionVersionImpl extends CollectionImpl {

    private static Log log = LogFactory.getLog(CollectionVersionImpl.class);

    private ResourceVersionDAO resourceVersionDAO;

    public VersionRetriever getVersionList() {
        return versionList;
    }

    public void setVersionList(VersionRetriever versionList) {
        this.versionList = versionList;
    }

    public int getVersionListIndex() {
        return versionListIndex;
    }

    public void setVersionListIndex(int versionListIndex) {
        this.versionListIndex = versionListIndex;
    }

    private VersionRetriever versionList;

    int versionListIndex = 0;

    public CollectionVersionImpl() {
        resourceVersionDAO = new ResourceVersionDAO();
    }

    public CollectionVersionImpl(String path, ResourceDO resourceDO) {
        super(path, resourceDO);                                              
        resourceVersionDAO = new ResourceVersionDAO();
    }

    public String[] getChildren() throws RegistryException {
        return (String[]) getContent();
    }

    public String[] getChildren(int start, int pageLen) throws RegistryException {

        try {
            String[] childPaths = new String[0];

            if (Transaction.isStarted()) {
                childPaths = resourceVersionDAO.getChildPaths(this.getResourceIDImpl(),
                                versionList, versionListIndex,
                            start, pageLen, snapshotID, Transaction.getConnection());

            } else {

                Connection conn = null;

                try {
                    conn = dataSource.getConnection();
                    childPaths = resourceVersionDAO.getChildPaths(this.getResourceIDImpl(),
                                versionList, versionListIndex,
                            start, pageLen, snapshotID, conn);

                } catch (SQLException e) {

                    String msg = "Failed to get the child paths " + pageLen + " child paths from " +
                            start + " of resource " + getPath() + ". " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {

                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException e) {
                            log.error("Failed to close the database connection used to " +
                                    "get child paths of collection " + getPath());
                        }
                    }
                }
            }

            return childPaths;
        } finally {
            clearSessionInformation();
        }
    }

    public int getChildCount() throws RegistryException {

        try {
            setSessionInformation();
            
            if (childCount == -1) {
                String[] childPaths = new String[0];

                if (Transaction.isStarted()) {
                    childPaths = resourceVersionDAO.getChildPaths(this.getResourceIDImpl(),
                                versionList, versionListIndex,
                                0, -1, snapshotID, Transaction.getConnection());

                } else {

                    Connection conn = null;

                    try {
                        conn = dataSource.getConnection();
                        childPaths = resourceVersionDAO.getChildPaths(this.getResourceIDImpl(),
                                versionList, versionListIndex,
                                0, -1, snapshotID, conn);

                    } catch (SQLException e) {

                        String msg = "Failed to get the child paths of resource " + getPath() + ". " + e.getMessage();
                        log.error(msg, e);
                        throw new RegistryException(msg, e);

                    } finally {

                        if (conn != null) {
                            try {
                                conn.close();
                            } catch (SQLException e) {
                                log.error("Failed to close the database connection used to " +
                                        "get child paths of collection " + getPath());
                            }
                        }
                    }
                }
                childCount = childPaths.length;
            }

            return childCount;

        } finally {
            clearSessionInformation();
        }
    }

    /**
     * Collection's content is a string array, which contains paths of its children. These paths
     * are loaded on demand to increase performance. It is recommended to use getChildren(...)
     * method to get child paths of a collection, which provides pagination. Calling this method
     * will load all child paths.
     *
     * @return String array of child paths.
     * @throws org.wso2.carbon.registry.core.exceptions.RegistryException On any error.
     */
    public Object getContent() throws RegistryException {
        if (content != null) return content;
        try {
            String[] childPaths = new String[0];
            
            if (Transaction.isStarted()) {
                childPaths = resourceVersionDAO.getChildPaths(this.getResourceIDImpl(),
                                versionList, versionListIndex,
                            0, -1, snapshotID, Transaction.getConnection());

            } else {

                Connection conn = null;

                try {
                    conn = dataSource.getConnection();
                    childPaths = resourceVersionDAO.getChildPaths(this.getResourceIDImpl(),
                                versionList, versionListIndex,
                            0, -1, snapshotID, conn);

                } catch (SQLException e) {

                    String msg = "Failed to get the child paths of resource " + getPath() + ". " + e.getMessage();
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {

                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException e) {
                            log.error("Failed to close the database connection used to " +
                                    "get child paths of collection " + getPath());
                        }
                    }
                }
            }

            Object content = childPaths;
            setContentWithNoUpdate(content);

            return content;
        } finally {
            clearSessionInformation();
        }
    }
}
