/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core;

import java.util.Date;

/**
 * Representation of a log entry. Log entry is a record of a single action performed on the
 * registry.
 */
public class LogEntry {

    // Filters for log queries - these represent the possible actions that get logged.
    public static final int ALL = -1;
    public static final int ADD = 0;
    public static final int UPDATE = 1;
    public static final int COMMENT = 2;
    public static final int TAG = 3;
    public static final int RATING = 4;
    public static final int DELETE_RESOURCE = 5;
    public static final int RESTORE = 6;
    public static final int RENAME = 7;
    public static final int MOVE = 8;
    public static final int COPY = 9;
    public static final int CREATE_REMOTE_LINK = 10;
    public static final int CREATE_SYMBOLIC_LINK = 11;
    public static final int REMOVE_LINK = 12;

    /** Path of the resource on which the action is performed. */
    private String resourcePath;

    /** User who has performed the action. */
    private String userName;

    /** Date and time at which the action is performed. */
    private Date date;

    /** Name of the actions. e.g. put, tag, comment */
    private int action;

    /**
     * Additional data to describe the actions. This depends on the action. e.g. comment text of the
     * comment action, tag name of the tag action.
     */
    private String actionData;

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public int getAction() {
        return action;
    }

    public void setAction(int action) {
        this.action = action;
    }

    public String getActionData() {
        return actionData;
    }

    public void setActionData(String actionData) {
        this.actionData = actionData;
    }

    public String getTitle() {
        StringBuffer entryBuf = new StringBuffer();
        switch (getAction()) {
            case UPDATE:
                entryBuf.append("Update of ");
                break;
            case COMMENT:
                entryBuf.append("Comment on ");
                break;
            case TAG:
                entryBuf.append("Tag of ");
                break;
            case RATING:
                entryBuf.append("Rating of ");
                break;
            default:
        }
        entryBuf.append(getResourcePath());
        return entryBuf.toString();
    }

    public String getText() {
        StringBuffer entryBuf = new StringBuffer();
        //entryBuf.append("<a href='/wso2registry/system/people/");
        //entryBuf.append(getUserName());
        //entryBuf.append("'>");
        entryBuf.append(getUserName());
        //entryBuf.append("</a>");
        switch (getAction()) {
            case ADD:
                entryBuf.append(" added the resource ");
                break;
            case UPDATE:
                entryBuf.append(" updated the resource '");
                break;
            case COMMENT:
                entryBuf.append(" commented on the resource ");
                break;
            case TAG:
                entryBuf.append(" tagged the resource ");
                break;
            case RATING:
                entryBuf.append(" rated the resource ");
                break;
            case RENAME:
                entryBuf.append(" renamed the resource ");
                break;
            case RESTORE:
                entryBuf.append(" restored the resource ");
                break;
            case COPY:
                entryBuf.append(" copied the resource ");
                break;
            case MOVE:
                entryBuf.append(" moved the resource ");
                break;
            case CREATE_REMOTE_LINK:
                entryBuf.append(" created a remote link ");
                break;
            case CREATE_SYMBOLIC_LINK:
                entryBuf.append(" created a symbolic link ");
                break;
            case REMOVE_LINK:
                entryBuf.append(" removed link ");
                break;
            default:
        }
        //entryBuf.append("<a href='/wso2registry/web");
        //entryBuf.append(getResourcePath());
        //entryBuf.append("'>");
        entryBuf.append(getResourcePath());
        entryBuf.append(" on ");
        entryBuf.append(getDate().toString());
        entryBuf.append(".");
        return entryBuf.toString();
    }
}
