/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.registry.core;

import org.wso2.carbon.registry.core.exceptions.RegistryException;

import java.io.InputStream;
import java.util.Date;
import java.util.List;
import java.util.Properties;

public interface Resource {

    String getId();

    String getAuthorUserName();

    Date getCreatedTime();

    Date getLastModified();

    String getDescription();

    void setDescription(String description);

    String getPath();

    /**
     * If resource is versioned, the associated version of the resource does not get modified by
     * any means. Therefore, the path of that version is the permanent path (permalink) of the
     * current state of the resource.
     *
     * @return Permanent path (permalink) of the resource.
     */
    String getPermanentPath();

    String getMediaType();

    int getState();

    void setMediaType(String mediaType);

    String getParentPath();

    String getProperty(String key);

    /**
     * Returns the list of values for the given property name. Note that these values are
     * read-only. Changes made to these values will not be persisted on putting the resource.
     *
     * @param key Key of the property.
     * @return List of values of the given property key.
     */
    List getPropertyValues(String key);

    /**
     * Returns all properties of the resource. Properties are stored as
     * key (String) -> values (List) pairs. It is not recommended to use this method to access
     * properties. Instead, use other property related Resource API methods provided.
     *
     * Note that these values are read-only. Changes made to these values will not be
     * persisted on putting the resource.
     *
     * @return All properties of the resource.
     */
    Properties getProperties();

    void setProperty(String key, String value);

    void setProperty(String key, List<String> value);

    void addProperty(String key, String value);

    void setProperties(Properties properties);

    void editPropertyValue(String key, String oldValue, String newValue);

    void removeProperty(String key);

    void removePropertyValue(String key, String value);

    Object getContent() throws RegistryException;

    void setContent(Object content) throws RegistryException;

    String getLastUpdaterUserName();

    InputStream getContentStream() throws RegistryException;

    void setContentStream(InputStream contentStream) throws RegistryException;

    List<String> getAspects();

    void addAspect(String name);

    void removeAspect(String name);

    void discard();
}
