package org.wso2.carbon.registry.core.config;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.CarbonConstants;

import javax.xml.namespace.QName;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;

public class RegistryConfiguration {

    private static final Log log = LogFactory.getLog(RegistryConfiguration.class);

    public static final String REGISTRY_CONFIG = "Registry";
    public static final String TYPE = "Type";
    public static final String USERNAME = "Username";
    public static final String PASSWORD = "Password";
    public static final String URL = "Url";
    public static final String REGISTRY_ROOT = "registryRoot";
    public static final String READ_ONLY = "ReadOnly";
    public static final String SYSTEM_USER_NAME = "RegistrySystemUserName";

    public static final String REMOTE_REGISTRY = "remote";
    public static final String EMBEDDED_REGISTRY = "embedded";

    private Map<String, String> registryConfiguration = new HashMap<String, String>();

    public RegistryConfiguration(String carbonXMLPath) throws RegistryException {

        try {
            File carbonXML = new File(carbonXMLPath);
            InputStream inSXml = new FileInputStream(carbonXML);
            OMElement config = new StAXOMBuilder(inSXml).getDocumentElement();

            OMElement registryConfig = config.getFirstChildWithName(new QName(
                    "http://wso2.org/projects/carbon/carbon.xml", REGISTRY_CONFIG));

            if (registryConfig != null) {

                for (Iterator childElements = registryConfig.getChildElements();
                     childElements.hasNext();) {
                    OMElement element = (OMElement) childElements.next();
                    registryConfiguration.put(element.getLocalName(), element.getText());

                }

                //Default to embedded configuration
            } else {
                registryConfiguration.put(TYPE, EMBEDDED_REGISTRY);
            }
            validate();

        } catch (Exception e) {
            throw new RegistryException("Error occured while initialization", e);
        }
    }

    private void validate() throws RegistryException {

        registryConfiguration.put(SYSTEM_USER_NAME, CarbonConstants.REGISTRY_SYSTEM_USERNAME);

        String type = registryConfiguration.get(TYPE);
        if(type == null || type.trim().length() == 0){
            type = EMBEDDED_REGISTRY;
            registryConfiguration.put(TYPE, EMBEDDED_REGISTRY);
        }
        if (REMOTE_REGISTRY.equals(type)) {
            if (registryConfiguration.get(URL) == null) {
                throw new RegistryException("URL not given");
            }
            if (registryConfiguration.get(REGISTRY_ROOT) != null && !registryConfiguration.get(REGISTRY_ROOT).startsWith("/")) {
                log.error("Invalid Registry Configuration : CHROOT must start with a /");
                throw new RegistryException("Invalid Registry Configuration : CHROOT must start with a /");
            }
        } else if (EMBEDDED_REGISTRY.equals(type)) {
            // validate
        } else {
            throw new RegistryException("Unkown type");
        }
    }

    public String getValue(String key) {
        return registryConfiguration.get(key);
    }

    public String getRegistryType() {
        return registryConfiguration.get(RegistryConfiguration.TYPE);
    }

}
