/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.config;

import org.wso2.carbon.registry.core.jdbc.dao.*;

public class StaticConfiguration {
    private static boolean versioningProperties = false;
    private static boolean versioningTags = false;
    private static boolean versioningRatings = false;
    private static boolean versioningAssociations = false;

    public static boolean isVersioningProperties() {
        return versioningProperties;
    }

    public static void setVersioningProperties(boolean versioningProperties) {
        StaticConfiguration.versioningProperties = versioningProperties;
    }

    public static boolean isVersioningTags() {
        return versioningTags;
    }

    public static void setVersioningTags(boolean versioningTags) {
        StaticConfiguration.versioningTags = versioningTags;
    }

    public static boolean isVersioningRatings() {
        return versioningRatings;
    }

    public static void setVersioningRatings(boolean versioningRatings) {
        StaticConfiguration.versioningRatings = versioningRatings;
    }

    public static boolean isVersioningAssociations() {
        return versioningAssociations;
    }

    public static void setVersioningAssociations(boolean versioningAssociations) {
        StaticConfiguration.versioningAssociations = versioningAssociations;
    }

    public static boolean isVersioningComments() {
        return versioningComments;
    }

    public static void setVersioningComments(boolean versioningComments) {
        StaticConfiguration.versioningComments = versioningComments;
    }

    private static boolean versioningComments;

    // defining getters for DAO returns dao object for the configurations

    /**
     * returns comments DAO depending on the configurations
     * @return
     */
    public static CommentsDAO getCommentsDAO() {
        if (versioningComments) {
            return new CommentsVersionDAO();
        }
        else {
            return new CommentsDAO();
        }
    }

    /**
     * returns ratings DAO depending on the configurations
     * @return
     */
    public static RatingsDAO getRatingsDAO() {
        if (versioningRatings) {
            return new RatingsVersionDAO();
        }
        else {
            return new RatingsDAO();
        }
    }

    /**
     * returns tag DAO depending on the configurations
     * @return
     */
    public static TagsDAO getTagsDAO() {
        if (versioningTags) {
            return new TagsVersionDAO();
        }
        else {
            return new TagsDAO();
        }
    }
}