/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc;

import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.jdbc.realm.RegistryRealm;
import org.wso2.carbon.registry.core.jdbc.utils.RegistryDataSource;
import org.wso2.carbon.registry.core.config.DataBaseConfiguration;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

import javax.sql.DataSource;

public class InMemoryJDBCRegistry extends JDBCRegistry {

    public static final String INMEMORY_DB_URL = "jdbc:hsqldb:mem:aname";
    public static final String INMEMORY_DB_DRIVER_NAME = "org.hsqldb.jdbcDriver";
    public static final String INMEMORY_DB_USER_NAME = "sa";
    public static final String INMEMORY_DB_PASSWORD = "";

    /**
     * Creates an in-memory HSQL database based registry without the support for authentication and
     * authorization.
     *
     * @throws org.wso2.carbon.registry.core.exceptions.RegistryException
     */
    public InMemoryJDBCRegistry() throws RegistryException {

        DataBaseConfiguration dbConfiguration = new DataBaseConfiguration();
        dbConfiguration.setDbUrl(INMEMORY_DB_URL);
        dbConfiguration.setDriverName(INMEMORY_DB_DRIVER_NAME);
        dbConfiguration.setUserName(INMEMORY_DB_USER_NAME);
        dbConfiguration.setPassWord(INMEMORY_DB_PASSWORD);
        RegistryDataSource dataSource = new RegistryDataSource(dbConfiguration);

        startInProcessDatabase(dataSource);

        super.configure(dataSource, null);
    }

    /**
     * Creates the in-memory HSQL database and creates all tables necessary for the registry. Then
     * constructs the registry with a datasource pointing to the HSQL database.
     *
     * @param registryRealm User manager registryRealm to use with the registry. It is strongly
     *                      recommended to use a registryRealm configured with a in-memory
     *                      (transient) database with the in-memory registry.
     */
    public InMemoryJDBCRegistry(RegistryRealm registryRealm) throws RegistryException {
        // create a datasource using the HSQL database. jdbc registry should use this datasource.

        DataBaseConfiguration dbConfiguration = new DataBaseConfiguration();
        dbConfiguration.setDbUrl(INMEMORY_DB_URL);
        dbConfiguration.setDriverName(INMEMORY_DB_DRIVER_NAME);
        dbConfiguration.setUserName(INMEMORY_DB_USER_NAME);
        dbConfiguration.setPassWord(INMEMORY_DB_PASSWORD);
        RegistryDataSource dataSource = new RegistryDataSource(dbConfiguration);

        startInProcessDatabase(dataSource);
        if (registryContext != null) {
            registryContext.setDataSource(dataSource);
        }
        super.configure(dataSource, registryRealm);
    }


    private void startInProcessDatabase(DataSource dataSource) throws RegistryException {
        DatabaseCreator databaseCreator = new DatabaseCreator(dataSource);

        try {
            databaseCreator.createRegistryDatabase();
        } catch (Exception e) {
            String msg = "Error in creatig the database";
            throw new RegistryException(msg, e);
        }
    }
}
