/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.handlers.builtin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.*;
import org.wso2.carbon.registry.core.config.StaticConfiguration;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.jdbc.dao.TagsDAO;
import org.wso2.carbon.registry.core.jdbc.dao.ResourceDAO;
import org.wso2.carbon.registry.core.jdbc.handlers.Handler;
import org.wso2.carbon.registry.core.jdbc.handlers.RequestContext;
import org.wso2.carbon.registry.core.jdbc.Repository;

/**
 * Handles paths of the form <pure resource path>;tags
 * e.g. /projects/wsf-php/config.xml;tags
 */
public class TagCollectionURLHandler extends Handler {

    private static final Log log = LogFactory.getLog(TagCollectionURLHandler.class);
    private ResourceDAO resourceDAO = new ResourceDAO();
    private TagsDAO tagsDAO;

    public TagCollectionURLHandler() {
        tagsDAO = StaticConfiguration.getTagsDAO();
    }

    public Resource get(RequestContext requestContext) throws RegistryException {

        ResourcePath resourcePath = requestContext.getResourcePath();

        if (resourcePath.parameterExists("tags") &&
                resourcePath.getParameterValue("tags") == null) {

            String path = resourcePath.getPath();
            String tagStr = "";

            ResourceIDImpl resourceIDImpl = resourceDAO.getResourceID(path);
            if (resourceIDImpl != null) {
                ResourceImpl resourceImpl = resourceDAO.getResourceMetaData(resourceIDImpl);
                if (resourceImpl != null) {
                    String [] tags = tagsDAO.getTags(resourceImpl);
                    for (String tag : tags) {
                        tagStr += tag;
                    }
                }
            }

            ResourceImpl resource = new ResourceImpl();
            resource.setMediaType(RegistryConstants.TAG_MEDIA_TYPE);
            resource.setContent(tagStr);
            resource.setPath(resourcePath.getCompletePath());
            resource.addProperty("resourcePath", path);
            
            requestContext.setProcessingComplete(true);
            return resource;
        }

        return null;
    }

    public void put(RequestContext requestContext) throws RegistryException {
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }

    public void invokeAspect(RequestContext requestContext) throws RegistryException {
    }
}
