/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.realm;

import org.wso2.carbon.user.core.*;
import org.wso2.carbon.user.core.def.DefaultRealm;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.SQLException;
import java.io.File;

public class InMemoryRegistryRealm implements UserRealm {

    private static final Log log = LogFactory.getLog(InMemoryRegistryRealm.class);

    private UserRealm userRealm;

    private static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        return dir.delete();
    }


    public InMemoryRegistryRealm() throws RegistryException {

        String derbyDBName =  "target/databasetest/CARBON_TEST";
        String dbFolder="target/databasetest";
        if ((new File(dbFolder)).exists()) {
            deleteDir(new File(dbFolder));    
        }
        // create an inmemory realm
        try {
            Class clazz = Class.forName("org.h2.Driver");
        } catch (ClassNotFoundException e) {
            String msg = "Derby database embedded driver is not available in the class path. " +
                    "Could not create the database for the user manager.";
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        BasicDataSource ds = new BasicDataSource();
        String connnectionUrl = "jdbc:h2:" + derbyDBName;
        ds.setUrl(connnectionUrl);
        ds.setDriverClassName("org.h2.Driver");

        DatabaseCreator creator = new DatabaseCreator(ds);
        try {
            creator.createRegistryDatabase();
        } catch (Exception e) {
            String msg = "Failed to create database tables of the User Manager. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        UserRealm realm = null;
        try {
            realm = new DefaultRealm();
            DefaultRealmConfig config = (DefaultRealmConfig) realm.getRealmConfiguration();
            config.setConnectionURL(connnectionUrl);
            config.setSysteUserName("TestSystemUserName");
            config.setDigestFunction("MD5");
            realm.init(config);
        } catch (UserStoreException e) {
            String msg = "Failed to initialize the user manager. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        this.userRealm = new RegistryRealm(realm);
    }

    public Object getRealmConfiguration() throws UserStoreException {
        return userRealm.getRealmConfiguration();
    }

    public void init(Object o) throws UserStoreException {
        userRealm.init(o);
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return userRealm.getAuthenticator();
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return userRealm.getAuthorizer();
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return userRealm.getUserStoreReader();
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return userRealm.getUserStoreAdmin();
    }

    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return userRealm.getAccessControlAdmin();
    }
    
    public void cleanUp() throws UserStoreException{
	
    }


    public UserClaimsAdmin getClaimsAdmin(String storeName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public UserProfileAdmin getUserProfileAdmin(String storeName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public void setClaimsAdmin(UserClaimsAdmin claimsAdmin) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setUserProfileAdmin(UserProfileAdmin profileAdmin) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }
    
    
}
