/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.realm;

import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.utils.RegistryUtils;
import org.wso2.carbon.registry.core.config.RegistryConfiguration;
import org.wso2.carbon.registry.core.internal.RegistryCoreServiceComponent;
import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class RegistryAccessControlAdmin implements AccessControlAdmin {

    private static final Log log = LogFactory.getLog(RegistryAccessControlAdmin.class);

    private UserRealm coreRealm;

    public RegistryAccessControlAdmin(UserRealm coreRealm) {
        this.coreRealm = coreRealm;
    }

    //public RegistryAccessControlAdmin(DataSource dataSource, String algo) {
    //    super(dataSource, algo);
    //}

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {
        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String systemuser = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);   
        if (userName.equals(systemuser) ||
            userName.equals(RegistryConstants.ADMIN_USER)) {

            String msg = "Could not change authorizations of the system defined user: " + userName;
            throw new UserStoreException(msg, false);
        }

        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().clearUserAuthorization(userName, unchrootedResourceId, action);
    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String systemuser = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);   
        
        if (userName.equals(systemuser) ||
            userName.equals(RegistryConstants.ADMIN_USER)) {

            String msg = "Could not change authorizations of the system defined user: " + userName;
            throw new UserStoreException(msg, false);
        }

        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().denyUser(userName, unchrootedResourceId, action);
    }


    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {

            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }

        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().clearRoleAuthorization(roleName, unchrootedResourceId, action);
    }

    public void clearUserAuthorization(String userName) throws UserStoreException {
        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Could not change authorizations of the system defined role: " + userName;
            throw new UserStoreException(msg, false);
        }
        getAccessControlAdmin().clearUserAuthorization(userName);
    }
    
    public void clearRoleAuthorization(String roleName) throws UserStoreException {
        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {
            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }
        getAccessControlAdmin().clearRoleAuthorization(roleName);
    }
    
    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {

            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }

        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().denyRole(roleName, unchrootedResourceId, action);
    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().authorizeUser(userName, unchrootedResourceId, action);
    }

    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().authorizeRole(roleName, unchrootedResourceId, action);
    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        getAccessControlAdmin().clearResourceAuthorizations(unchrootedResourceId);
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {
        String unchrootedFromResourceId = RegistryUtils.getUnChrootedPath(fromResourceId);
        String unchrootedToResourceId = RegistryUtils.getUnChrootedPath(toResourceId);
        getAccessControlAdmin().copyAuthorizations(unchrootedFromResourceId, unchrootedToResourceId);
    }
    public boolean isUserAuthorized(String userName, String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        return getAccessControlAdmin().isUserAuthorized(userName, unchrootedResourceId, action);
    }

    public boolean isRoleAuthorized(String roleName, String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        return getAccessControlAdmin().isRoleAuthorized(roleName, unchrootedResourceId, action);
    }

    public String[] getAllowedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        return getAccessControlAdmin().getAllowedUsersForResource(unchrootedResourceId, action);
    }

    public String[] getDeniedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        return getAccessControlAdmin().getDeniedUsersForResource(unchrootedResourceId, action);
    }

    public String[] getAllowedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        return getAccessControlAdmin().getAllowedRolesForResource(unchrootedResourceId, action);
    }

    public String[] getDeniedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        String unchrootedResourceId = RegistryUtils.getUnChrootedPath(resourceId);
        return getAccessControlAdmin().getDeniedRolesForResource(unchrootedResourceId, action);
    }
    

    private AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return coreRealm.getAccessControlAdmin();
    }
    
   
}
