/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.realm.pathbased;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.jdbc.dao.ResourceDAO;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserStoreException;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;

public class PathBasedAuthorizer implements Authorizer {

    private static final Log log = LogFactory.getLog(PathBasedAuthorizer.class);

    private Authorizer coreAuthorizer;
    private DataSource dataSource;
    private ResourceDAO resourceDAO = new ResourceDAO();

    public PathBasedAuthorizer(Authorizer authorizer, DataSource dataSource) {
        this.coreAuthorizer = authorizer;
        this.dataSource = dataSource;
    }

    public boolean isUserAuthorized(String userName, String resourcePath, String action)
            throws UserStoreException {
        return coreAuthorizer.isUserAuthorized(userName, getResourceID(resourcePath), action);
    }

    public boolean isRoleAuthorized(String roleName, String resourcePath, String action)
            throws UserStoreException {
        return coreAuthorizer.isRoleAuthorized(roleName, getResourceID(resourcePath), action);
    }

    public String[] getAllowedUsersForResource(String resourcePath, String action)
            throws UserStoreException {
        return coreAuthorizer.getAllowedUsersForResource(getResourceID(resourcePath), action);
    }

    public String[] getDeniedUsersForResource(String resourcePath, String action)
            throws UserStoreException {
        return coreAuthorizer.getDeniedUsersForResource(getResourceID(resourcePath), action);
    }

    public String[] getAllowedRolesForResource(String resourcePath, String action)
            throws UserStoreException {
        return coreAuthorizer.getAllowedRolesForResource(getResourceID(resourcePath), action);
    }

    public String[] getDeniedRolesForResource(String resourcePath, String action)
            throws UserStoreException {
        return coreAuthorizer.getDeniedRolesForResource(getResourceID(resourcePath), action);
    }

    private String getResourceID(String resourcePath) throws UserStoreException {
        return resourcePath;

    }
}
