/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.realm.pathbased;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.user.core.*;

import javax.sql.DataSource;

/**
 * Registry stores authorizations for resources against the UUID of the resource. This makes it
 * flexible to rename, move, copy, etc resources without rewritting all the authorizations. But for
 * the users of the Registry and User Manager APIs, UUIDs don't make much sense. API users always
 * refer to the resources using the path. Therefore, this class and its authorizer and access
 * control admin classes, convert the given path to the UUID and checks the authorizations. Thus,
 * users of this class can only deal with the resource paths.
 */
public class PathBasedRegistryRealm implements UserRealm {

    private static final Log log = LogFactory.getLog(PathBasedRegistryRealm.class);

    private UserRealm coreRealm;
    private PathBasedAccessControlAdmin pathBasedAccessControlAdmin;
    private PathBasedAuthorizer pathBasedAuthorizer;

    public PathBasedRegistryRealm(UserRealm userRealm, DataSource dataSource) throws RegistryException {
        coreRealm = userRealm;

        try {
            pathBasedAuthorizer = new PathBasedAuthorizer(coreRealm.getAuthorizer(), dataSource);
            pathBasedAccessControlAdmin =
                    new PathBasedAccessControlAdmin(coreRealm.getAccessControlAdmin(), dataSource);
        } catch (UserStoreException e) {
            String msg = "Failed to initialize the path based realm. " + e.getMessage();
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return pathBasedAuthorizer;
    }

    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return pathBasedAccessControlAdmin;
    }

    public Object getRealmConfiguration() throws UserStoreException {
        return coreRealm.getRealmConfiguration();
    }

    public void init(Object configBean) throws UserStoreException {
        coreRealm.init(configBean);
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return coreRealm.getAuthenticator();
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return coreRealm.getUserStoreReader();
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return coreRealm.getUserStoreAdmin();
    }
    
    public void cleanUp() throws UserStoreException{
	
    }
    
    public UserProfileAdmin getUserProfileAdmin() throws UserStoreException{
        return null;
    }
    
    public UserClaimsAdmin getClaimsAdmin() throws UserStoreException {
        return null;
    }

    public UserClaimsAdmin getClaimsAdmin(String storeName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public UserProfileAdmin getUserProfileAdmin(String storeName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public void setClaimsAdmin(UserClaimsAdmin claimsAdmin) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void setUserProfileAdmin(UserProfileAdmin profileAdmin) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }
    
}
