/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

public class VersionInputStream extends InputStream {
    int available = 0;
    int read = 0;
    List<Long> versionList;
    private final static int SIZE_OF_BYTE = 8;
    private final static int SIZE_OF_LONG =  64;
    private final static int SIZE_OF_MULTIPLIER = SIZE_OF_LONG / SIZE_OF_BYTE; 

    public VersionInputStream(List<Long> versionList) {
        this.versionList = versionList;
        available = versionList.size() * SIZE_OF_MULTIPLIER;
        read = 0;
    }

    public int read() throws IOException {
        int nextIndex = read / SIZE_OF_MULTIPLIER;
        int shiftOffset = SIZE_OF_MULTIPLIER - 1 - read % SIZE_OF_MULTIPLIER;

        if(nextIndex >= versionList.size()) {
            return -1;
        }
        long nextVersion = versionList.get(nextIndex);
        int valueToRead = (int) ((nextVersion >> (shiftOffset * SIZE_OF_BYTE)) & 0xff);
        read ++;
        available --;
        return valueToRead; 
    }

    public int available() throws IOException {
	    return available;
    }
}
