/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.utils;

import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.InputStream;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;


public class VersionRetriever {
    InputStream inputStream;
    private final static int SIZE_OF_BYTE = 8;
    private final static int SIZE_OF_LONG =  64;
    List<Long> versionList;
    int maxRetrievedVersion;

    private static Log log = LogFactory.getLog(VersionRetriever.class);

    public VersionRetriever(InputStream inputStream) {
        this.inputStream = inputStream;
        versionList = new ArrayList<Long>();
        maxRetrievedVersion = -1;
    }

    public long getVersion(int versionIndex) throws RegistryException {
        while (versionIndex > maxRetrievedVersion) {
            long nextVersion = getNextVersion();
            if(nextVersion == 0) {
                // the stream is over
                return -1;
            }
            versionList.add(nextVersion);
            maxRetrievedVersion ++;
        }
        return versionList.get(versionIndex);
    }

    private long getNextVersion() throws RegistryException {
        // Logi is read the input stream 8 time and build the
        // 64 bit long value
        int shiftLimit = SIZE_OF_LONG/SIZE_OF_BYTE;
        long versionValue = 0;
        try {
            for(int i = 0; i < shiftLimit; i ++) {
                int nextByte = (int)inputStream.read();
                if(i != 0) {
                    // left shift one byte
                    versionValue <<= SIZE_OF_BYTE;
                }
                // do 'or' with the current byte.
                versionValue |= nextByte;
            }
        }
        catch (IOException e) {
            String msg = "Failed to read the stream to get the versio nvalue. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
        return versionValue;
    }
}
