/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.service;

import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;

public interface RegistryService {

    /**
     * Creates a UserRegistry instance for anonymous user. Permissions set for anonymous user will
     * be applied for all operations performed using this instance.
     *
     * @return UserRegistry for the anonymous user.
     * @throws RegistryException
     */
    UserRegistry getUserRegistry() throws RegistryException;

    /**
     * Returns a registry to be used for system operations. Human users should not be allowed
     * log in using this registry.
     *
     * @return User registry for system user.
     * @throws RegistryException
     */
    UserRegistry getSystemRegistry() throws RegistryException;

    /**
     * Creates UserRegistry instances for normal users. Applications should use this method to
     * create UserRegistry instances, unless there is a specific need documented in other methods.
     * User name and the password will be authenticated by the EmbeddedRegistry before creating
     * the requested UserRegistry instance.
     *
     * @param userName User name of the user.
     * @param password Password of the user.
     * @return UserRegistry instance for the given user.
     * @throws RegistryException
     */
    UserRegistry getUserRegistry(String userName, String password) throws RegistryException;

    /**
     * Creates a UserRegistry instance for the given user. This method will NOT authenticate the
     * user before creating the UserRegistry instance. It assumes that the user is authenticated
     * outside the EmbeddedRegistry.
     *
     * @param userName User name of the user.
     * @return UserRegistry instance for the given user.
     * @throws RegistryException
     */
    UserRegistry getUserRegistry(String userName) throws RegistryException;
}
