/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.registry.ws;

import org.wso2.carbon.registry.core.*;
import org.wso2.carbon.registry.core.config.RegistryContext;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.ws.utils.CommonUtil;
import org.apache.axiom.om.OMElement;


import java.util.Date;
import java.util.Map;
import java.io.OutputStream;
import java.io.Writer;
import java.io.Reader;


public class WSRegistry implements Registry {

     public Resource newResource() throws RegistryException{
         UserRegistry registry = CommonUtil.getRegistry();
         return registry.newResource();
     }
    public Collection newCollection() throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.newCollection();
    }
    /**
     * Returns the resource at the given path.
     *
     * @param path Path of the resource. e.g. /project1/server/deployment.xml
     * @return Resource instance
     * @throws org.wso2.carbon.registry.core.exceptions.RegistryException
     *          is thrown if the resource is not in the registry
     */
    public Resource get(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.get(path);
    }
     public Resource WSget(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.get(path);
    }
    /**
     * Returns the meta data of the resource at the given path.
     *
     * @param path Path of the resource. e.g. /project1/server/deployment.xml
     * @return Resource instance
     * @throws org.wso2.carbon.registry.core.exceptions.RegistryException
     *          is thrown if the resource is not in the registry
     */
    public Resource getMetaData(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getMetaData(path);
    }

    /**
     * Returns the Collection at the given path, with the content paginated according to
     * the arguments.
     *
     * @param path the path of the collection.  MUST point to a collection!
     * @param start the initial index of the child to return.  If there are fewer children than
     *              the specified value, a RegistryException will be thrown.
     * @param pageSize the maximum number of results to return
     * @return a Collection containing the specified results in the content
     * @throws RegistryException if the resource is not found, or if the path does not
     *                           reference a Collection, or if the start index is greater than
     *                           the number of children.
     */
    public Collection get(String path, int start, int pageSize) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.get(path,start,pageSize);
    }

    public Collection getChildCollection(String path, int start, int pageSize) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.get(path,start,pageSize);
    }

    /**
     * Check whether a resource exists at the given path
     *
     * @param path Path of the resource to be checked
     * @return true if a resource exists at the given path, false otherwise.
     * @throws org.wso2.carbon.registry.core.exceptions.RegistryException
     *          if an error occurs
     */
    public boolean resourceExists(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.resourceExists(path);

    }

    /**
     * Adds or updates resources in the registry. If there is no resource at the given path,
     * resource is added. If a resource already exist at the given path, it will be replaced with
     * the new resource.
     *
     * @param suggestedPath the path which we'd like to use for the new resource.
     * @param resource      Resource instance for the new resource
     * @return the actual path that the server chose to use for our Resource
     * @throws org.wso2.carbon.registry.core.exceptions.RegistryException
     *
     */
    public String put(String suggestedPath, Resource resource) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.put(suggestedPath,resource);
    }

    /**
     * Deletes the resource at the given path. If the path refers to a directory, all child
     * resources of the directory will also be deleted.
     *
     * @param path Path of the resource to be deleted.
     * @throws RegistryException is thrown depending on the implementation.
     */
    public void delete(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.delete(path);
    }
    /**
     * Creates a resource by fetching the resource content from the given URL.
     *
     * @param suggestedPath path where we'd like to add the new resource. Although this path is
     *                      specified by the caller of the method, resource may not be actually
     *                      added at this path.
     * @param sourceURL     where to fetch the resource content
     * @param metadata      a template Resource
     * @return actual path to the new resource
     * @throws RegistryException if we couldn't get or store the new resource
     */
    public String importResource(String suggestedPath,
                                 String sourceURL,
                                 Resource metadata) throws RegistryException {
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.importResource(suggestedPath,sourceURL,metadata);
    }
    /**
     * Move or rename a resource in the registry.  This is equivalent to 1) delete the resource,
     * then 2) add the resource to the new location.  The operation is atomic, so if it fails the
     * old resource will still be there.
     *
     * @param currentPath current path of the resource
     * @param newName     where we'd like to move the reosurce
     * @return the actual path for the new resource
     * @throws RegistryException if something went wrong
     */
    public String rename(String currentPath, String newName) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.rename(currentPath, newName);

    }

    public String move(String currentPath, String newPath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.move(currentPath,newPath);
    }

    public String copy(String sourcePath, String targetPath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.copy(sourcePath,targetPath);
    }

    public void createVersion(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.createVersion(path);
    }
    /**
     * Get a list of all versions of the resource located at the given path. Version paths are
     * returned in the form /projects/myresource?v=12
     *
     * @param path path of a current version of a resource
     * @return a String array containing the individual paths of versions
     * @throws RegistryException if there is an error
     */
    public String[] getVersions(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getVersions(path);
    }

    /**
     * Reverts a resource to a given version.
     *
     * @param versionPath path of the version to be reverted. It is not necessary to provide the
     *                    path of the resource as it can be derived from the version path.
     * @throws RegistryException if there is an error
     */
    public void restoreVersion(String versionPath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.restoreVersion(versionPath);
    }


    ////////////////////////////////////////////////////////
    // Associations
    ////////////////////////////////////////////////////////
    /**
     * Adds an association stating that the resource at "associationPath" associate on the resource at
     * "associationPath". Paths may be the resource paths of the current versions or paths of the old
     * versions. If a path refers to the current version, it should contain the path in the form
     * /c1/c2/r1. If it refers to an old version, it should be in the form /c1/c2/r1?v=2.
     *
     * @param sourcePath       Path of the source resource
     * @param targetPath       Path of the target resource
     * @param associationType  Type of the association
     * @throws RegistryException Depends on the implementation
     */
   public void addAssociation(String sourcePath,
                        String targetPath,
                        String associationType) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.addAssociation(sourcePath,targetPath,associationType);
    }
    /**
     * To remove an association for a given resource
     *
     * @param sourcePath       Path of the source resource
     * @param targetPath       Path of the target resource
     * @param associationType  Type of the association
     * @throws RegistryException Depends on the implementation
     */
    public void removeAssociation(String sourcePath,
                                  String targetPath,
                                  String associationType) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.removeAssociation(sourcePath,targetPath,associationType);
    }
    /**
     * Get all associations of the given resource. This is a chain of association starting from the
     * given resource both upwards (source to destination) and downwards (destination to source). T
     * his is useful to analyse how changes to other resources would affect the
     * given resource.
     *
     * @param resourcePath Path of the resource to analyse associations.
     * @return List of Association
     * @throws RegistryException : If something went wrong
     */
    public Association[] getAllAssociations(String resourcePath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getAllAssociations(resourcePath);
    }

    /**
     * Get all associations of the given resource for a give association type.
     * This is a chain of association starting from the
     * given resource both upwards (source to destination) and downwards (destination to source). T
     * his is useful to analyse how changes to other resources would affect the
     * given resource.
     *
     * @param resourcePath    Path of the resource to analyse associations.
     * @param associationType : Type of the association , that could be dependecy , or some other type
     * @return List of Association
     * @throws RegistryException : If something went wrong
     */
    public Association[] getAssociations(String resourcePath, String associationType)
            throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getAssociations(resourcePath,associationType);
    }

    ////////////////////////////////////////////////////////
    // Tagging
    ////////////////////////////////////////////////////////

    /**
     * Applies the given tag to the resource in the given path. If the given tag is not defined in
     * the registry, it will be defined.
     *
     * @param resourcePath Path of the resource to be tagged.
     * @param tag          Tag. Any string can be used for the tag.
     * @throws RegistryException is thrown if a resource does not exist in the given path.
     */
    public void applyTag(String resourcePath, String tag) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.applyTag(resourcePath,tag);
    }

    /**
     * Returns the paths of all Resources that are tagged with the given tag.
     *
     * @param tag the tag to search for
     * @return an array of TaggedResourcePaths
     * @throws RegistryException if an error occurs
     */
    public TaggedResourcePath[] getResourcePathsWithTag(String tag) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getResourcePathsWithTag(tag);
    }

    /**
     * Returns all tags used for tagging the given resource.
     *
     * @param resourcePath Path of the resource
     * @return Tags tag names
     * @throws RegistryException is thrown if a resource does not exist in the given path.
     */
    public Tag[] getTags(String resourcePath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getTags(resourcePath);
    }

    /**
     * Removes a tag on a resource. If the resource at the path is owned by the current user, all
     * taggings done using the given tag will be removed. If the resource is not owned by the
     * current user, only the tagging done by the current user will be removed.
     *
     * @param path Resource path tagged with the given tag.
     * @param tag  Name of the tag to be removed.
     * @throws RegistryException if there's a problem
     */
    public void removeTag(String path, String tag) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.removeTag(path,tag);
    }

    ////////////////////////////////////////////////////////
    // Comments
    ////////////////////////////////////////////////////////

    /**
     * Adds a comment to a resource.
     *
     * @param resourcePath Path of the resource to add the comment.
     * @param comment      Comment instance for the new comment.
     * @return the path of the new comment.
     * @throws RegistryException is thrown if a resource does not exist in the given path.
     */
    public String addComment(String resourcePath, Comment comment) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.addComment(resourcePath,comment);
    }

    /**
     * Change the text of an existing comment.
     *
     * @param commentPath path to comment resource ("..foo/r1;comment:1")
     * @param text        new text for the comment.
     * @throws RegistryException Registry implementations may handle exceptions and throw
     *                           RegistryException if the exception has to be propagated to the
     *                           client.
     */
    public void editComment(String commentPath, String text) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.editComment(commentPath,text);
    }

    /**
     * Get all comments for the given resource.
     *
     * @param resourcePath path of the resource.
     * @return an array of Comment objects.
     * @throws RegistryException Registry implementations may handle exceptions and throw
     *                           RegistryException if the exception has to be propagated to the
     *                           client.
     */
    public Comment[] getComments(String resourcePath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getComments(resourcePath);
    }

    ////////////////////////////////////////////////////////
    // Ratings
    ////////////////////////////////////////////////////////

    /**
     * Rate the given resource.
     *
     * @param resourcePath Path of the resource.
     * @param rating       Rating value between 1 and 5.
     * @throws RegistryException Registry implementations may handle exceptions and throw
     *                           RegistryException if the exception has to be propagated to the
     *                           client.
     */
    public void rateResource(String resourcePath, int rating) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.rateResource(resourcePath,rating);
    }

    /**
     * Returns the average rating for the given resource. This is the average of all ratings done by
     * all users for the given resource.
     *
     * @param resourcePath Path of the resource.
     * @return Average rating between 1 and 5.
     * @throws RegistryException if an error occurs
     */
    public float getAverageRating(String resourcePath) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getAverageRating(resourcePath);
    }

    /**
     * Returns the rating given to the specified resource by the given user
     *
     * @param path     Path of the resource
     * @param userName username of the user
     * @return rating given by the given user
     * @throws RegistryException if there is a problem
     */
    public int getRating(String path, String userName) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return getRating(path,userName);
    }

    /**
     * Executes a custom query which lives at the given path in the Registry.
     *
     * @param path       Path of the query to execute.
     * @param parameters a Map of query parameters (name -> value)
     * @return a Collection containing any resource paths which match the query
     * @throws RegistryException depends on the implementation.
     */
    public Collection executeQuery(String path, Map parameters) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.executeQuery(path,parameters);
    }
    public Collection WSexecuteQuery(String path, String[] key,String[] value) throws RegistryException{
           UserRegistry registry = CommonUtil.getRegistry();
           return registry.executeQuery(path,CommonUtil.createMap(key,value));
       }

    /**
     * Returns the logs of the activities occured in the registy.
     *
     * @see LogEntry Accepted values for action parameter
     *
     * @param resourcePath If given, only the logs related to the resource path will be returned. If
     *                     null, logs for all resources will be returned.
     * @param action       Only the logs pertaining to this action will be returned.  For
     *                     acceptable values, see LogEntry.
     * @param userName     If given, only the logs for activities done by the given user will be
     *                     returned. If null, logs for all users will be returned.
     * @param from         If given, logs for activities occured after the given date will be
     *                     returned. If null, there will not be a bound for the starting date.
     * @param to           If given, logs for activities occured before the given date will be
     *                     returned. If null, there will not be a bound for the ending date.
     * @param recentFirst  If true, returned activities will be most-recent first. If false,
     *                     returned activities will be oldest first.
     * @return Array of LogEntry objects representing the logs
     * @throws RegistryException if there is a problem
     */
    public LogEntry[] getLogs(String resourcePath,
                              int action,
                              String userName,
                              Date from,
                              Date to,
                              boolean recentFirst) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getLogs(resourcePath,action,userName,from,to,recentFirst);
    }

    /**
     * Returns the logs of the activities occured in the registy.
     *
     * @see LogEntry Accepted values for action parameter
     *
     * @param resourcePath If given, only the logs related to the resource path will be returned. If
     *                     null, logs for all resources will be returned.
     * @param action       Only the logs pertaining to this action will be returned.  For
     *                     acceptable values, see LogEntry.
     * @param userName     If given, only the logs for activities done by the given user will be
     *                     returned. If null, logs for all users will be returned.
     * @param from         If given, logs for activities occured after the given date will be
     *                     returned. If null, there will not be a bound for the starting date.
     * @param to           If given, logs for activities occured before the given date will be
     *                     returned. If null, there will not be a bound for the ending date.
     * @param recentFirst  If true, returned activities will be most-recent first. If false,
     *                     returned activities will be oldest first.
     * @return LogEntryCollection representing collection of log entries
     * @throws RegistryException if there is a problem
     */
    public LogEntryCollection getLogCollection(String resourcePath,
                                               int action,
                                               String userName,
                                               Date from,
                                               Date to,
                                               boolean recentFirst) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getLogCollection(resourcePath,action,userName,from,to,recentFirst);
    }

    /**
     * Get a list of the available Aspects for this Registry
     * @return a String array containing available Aspect names
     */
    public String [] getAvailableAspects(){
        try{
            UserRegistry registry = CommonUtil.getRegistry();
            return registry.getAvailableAspects();
        }catch(RegistryException e)
        {
            String msg = "Failed to get Available Aspects " +  e.getMessage();
            return null;
        }
    }

    /**
     * Associate an Aspect with a resource.
     *
     * @param resourcePath Path of the resource
     * @param aspect       Name of the aspect
     * @throws RegistryException : If some thing went wrong while doing associating the phase
     */
    public void associateAspect(String resourcePath, String aspect) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.associateAspect(resourcePath,aspect);
    }

    /**
     * This invokes an action on a specified Aspect, which must be associated with the Resource
     * at the given path.
     *
     * @param resourcePath Path of the resource
     * @param aspectName   Name of the aspect
     * @param action       Which action was selected - actions are aspect-specific
     * @throws RegistryException if the Aspect isn't associated with the Resource, or the action
     *                           isn't valid, or an Aspect-specific problem occurs.
     */
    public void invokeAspect(String resourcePath, String aspectName, String action)
            throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.invokeAspect(resourcePath,aspectName,action);
    }

    /**
     * Obtain a list of the available actions on a given resource for a given Aspect.  The
     * Aspect must be associated with the Resource (@see associateAspect).  The actions are
     * determined by asking the Aspect itself, so they may change depending on the state of the
     * Resource, the user who's asking, etc)
     *
     * @param resourcePath path of the Resource
     * @param aspectName   name of the Aspect to query for available actions
     * @return a String[] of action names
     * @throws RegistryException if the Aspect isn't associated or an Aspect-specific problem occurs
     */
    public String[] getAspectActions(String resourcePath, String aspectName)
            throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getAspectActions(resourcePath,aspectName);
    }

    /**
     * Start a new transaction
     *
     * @throws RegistryException If an error occurs while starting a transaction
     */
    public void beginTransaction() throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.beginTransaction();
    }

    /**
     * Commit the currently active transaction
     *
     * @throws RegistryException If an error occurs while committing a transaction
     */
    public void commitTransaction() throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.commitTransaction();
    }

    /**
     * Rollback the currently active transaction
     *
     * @throws RegistryException If an error occurs while rolling back a transaction
     */
    public void rollbackTransaction() throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.rollbackTransaction();
    }

    /**
     * Get the configuration for this Registry
     *
     * @return the currently active RegistryContext, or null
     */
    public RegistryContext getRegistryContext(){
        try{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getRegistryContext();
        }catch(RegistryException e)
        {
            return null;
        }
    }

    /**
     * Search the content of resources
     *
     * @param keywords keywords to look for
     * @return the result set as a collection
     */
    public Collection searchContent(String keywords) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.searchContent(keywords);
    }

    /**
     * Create a symbolic link or mount a registy
      *
     * @param path the mount path
     * @param target the point to be mounted
     */
    public void createLink(String path, String target) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.createLink(path, target);
    }
    public void WScreateLink(String path, String target) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.createLink(path, target);
    }
    /**
     * Create a symbolic link or mount a registy
      *
     * @param path the mount path
     * @param target the point to be mounted
     * @param subTargetPath sub path in the remote instance to be mounted
     */
    public void createLink(String path, String target, String subTargetPath)
            throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.createLink(path, target, subTargetPath);
    }
    public void createLinkWithMounting(String path, String target, String subTargetPath)
            throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.createLink(path, target, subTargetPath);
    }
     /**
     * Remove a symbolic link or mount point created
      *
     * @param path the mount path
     */
    public void removeLink(String path) throws RegistryException{
        UserRegistry registry = CommonUtil.getRegistry();
        registry.removeLink(path);
    }

    /**
     * check in the input axiom element into database
     * @param path path to check in
     * @param reader axiom element containing resource
     * @throws RegistryException
     */
    public void restore(String path, Reader reader) throws RegistryException {
        UserRegistry registry = CommonUtil.getRegistry();
        registry.restore(path, reader);
    }

    /**
     * check out the given path as an xml
     * @param path path to check out
     * @param writer output stream to write the response
     * @throws RegistryException
     */
    public void dump(String path, Writer writer) throws RegistryException {
        UserRegistry registry = CommonUtil.getRegistry();
        registry.dump(path, writer);
    }

    public String getEventingServiceURL(String path) throws RegistryException {
        UserRegistry registry = CommonUtil.getRegistry();
        return registry.getEventingServiceURL(path);
    }

    public void setEventingServiceURL(String path, String eventingServiceURL) throws RegistryException {
        UserRegistry registry = CommonUtil.getRegistry();
        registry.setEventingServiceURL(path, eventingServiceURL);
    }
}


