CREATE TABLE REG_CLUSTER_LOCK (
             REG_LOCK_NAME VARCHAR (20),
             REG_LOCK_STATUS VARCHAR (20),
             REG_LOCKED_TIME TIMESTAMP,
             PRIMARY KEY (REG_LOCK_NAME)
);

CREATE TABLE REG_LOG (
             REG_LOG_ID INTEGER GENERATED ALWAYS AS IDENTITY,
             REG_PATH VARCHAR (2000),
             REG_USER_ID VARCHAR (20) NOT NULL,
             REG_LOGGED_TIME TIMESTAMP NOT NULL,
             REG_ACTION INTEGER NOT NULL,
             REG_ACTION_DATA VARCHAR (500),
             PRIMARY KEY (REG_LOG_ID)
);

CREATE TABLE REG_PATH(
             REG_PATH_ID INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
             REG_PATH_VALUE VARCHAR(2000) NOT NULL,
             REG_PATH_PARENT_ID INT,
             CONSTRAINT PK_REG_PATH PRIMARY KEY(REG_PATH_ID)
);

CREATE TABLE REG_CONTENT (
             REG_CONTENT_ID INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
             REG_CONTENT_DATA BLOB,
             CONSTRAINT PK_REG_CONTENT PRIMARY KEY(REG_CONTENT_ID)
);

CREATE TABLE REG_CONTENT_HISTORY (
             REG_CONTENT_ID INTEGER NOT NULL,
             REG_CONTENT_DATA BLOB,
             REG_DELETED   SMALLINT,
             CONSTRAINT PK_REG_CONTENT_HISTORY PRIMARY KEY(REG_CONTENT_ID)
);

CREATE TABLE REG_RESOURCE (
            REG_PATH_ID         INTEGER NOT NULL,
            REG_NAME            VARCHAR(256),
            REG_VERSION         INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
            REG_MEDIA_TYPE      VARCHAR(500),
            REG_CREATOR         VARCHAR(20) NOT NULL,
            REG_CREATED_TIME    TIMESTAMP NOT NULL,
            REG_LAST_UPDATOR    VARCHAR(20),
            REG_LAST_UPDATED_TIME    TIMESTAMP NOT NULL,
            REG_DESCRIPTION     VARCHAR(1000),
            REG_CONTENT_ID      INTEGER
            --CONSTRAINT PK_REG_RESOURCE PRIMARY KEY(REG_VERSION)
            --CONSTRAINT PK_REG_RESOURCE PRIMARY KEY(PATH_ID,NAME,VERSION)
);

ALTER TABLE REG_RESOURCE ADD CONSTRAINT REG_RESOURCE_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
ALTER TABLE REG_RESOURCE ADD CONSTRAINT REG_RESOURCE_FK_BY_CONTENT_ID FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT (REG_CONTENT_ID);
CREATE INDEX REG_RESOURCE_IND_BY_NAME ON REG_RESOURCE(REG_NAME);
CREATE INDEX REG_RESOURCE_IND_BY_PATH_ID_NAME ON REG_RESOURCE(REG_PATH_ID, REG_NAME);

CREATE TABLE REG_RESOURCE_HISTORY (
            REG_PATH_ID         INTEGER NOT NULL,
            REG_NAME            VARCHAR(256),
            REG_VERSION         INTEGER DEFAULT 0,
            REG_MEDIA_TYPE      VARCHAR(500),
            REG_CREATOR         VARCHAR(20) NOT NULL,
            REG_CREATED_TIME    TIMESTAMP NOT NULL,
            REG_LAST_UPDATOR    VARCHAR(20),
            REG_LAST_UPDATED_TIME    TIMESTAMP NOT NULL,
            REG_DESCRIPTION     VARCHAR(1000),
            REG_CONTENT_ID      INTEGER,
            REG_DELETED         SMALLINT,
            CONSTRAINT PK_REG_RESOURCE_HISTORY PRIMARY KEY(REG_VERSION)
);

ALTER TABLE REG_RESOURCE_HISTORY ADD CONSTRAINT REG_RESOURCE_HIST_FK_BY_PATHID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
ALTER TABLE REG_RESOURCE_HISTORY ADD CONSTRAINT REG_RESOURCE_HIST_FK_BY_CONTENT_ID FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT_HISTORY (REG_CONTENT_ID);
CREATE INDEX REG_RESOURCE_HISTORY_IND_BY_NAME ON REG_RESOURCE_HISTORY(REG_NAME);
CREATE INDEX REG_RESOURCE_HISTORY_IND_BY_PATH_ID_NAME ON REG_RESOURCE(REG_PATH_ID, REG_NAME);

CREATE TABLE REG_COMMENTS (
            REG_ID        INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
            REG_COMMENT_TEXT      VARCHAR(500) NOT NULL,
            REG_USER_ID           VARCHAR(20) NOT NULL,
            REG_COMMENTED_TIME    TIMESTAMP NOT NULL,
            CONSTRAINT PK_REG_COMMENTS PRIMARY KEY(REG_ID)
);

CREATE TABLE REG_RESOURCE_COMMENTS (
            REG_COMMENT_ID          INTEGER NOT NULL,
            REG_VERSION             INTEGER DEFAULT 0,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_COMMENTS PRIMARY KEY(REG_VERSION, REG_COMMENT_ID)
);


ALTER TABLE REG_RESOURCE_COMMENTS ADD CONSTRAINT REG_RESOURCE_COMMENTS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
ALTER TABLE REG_RESOURCE_COMMENTS ADD CONSTRAINT REG_RESOURCE_COMMENTS_FK_BY_COMMENT_ID FOREIGN KEY (REG_COMMENT_ID) REFERENCES REG_COMMENTS (REG_ID);
CREATE INDEX REG_RESOURCE_COMMENTS_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_COMMENTS(REG_PATH_ID, REG_RESOURCE_NAME);

CREATE TABLE REG_RATINGS (
            REG_ID     INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
            REG_RATING        INTEGER NOT NULL,
            REG_USER_ID       VARCHAR(20) NOT NULL,
            REG_RATED_TIME    TIMESTAMP NOT NULL,
            CONSTRAINT PK_REG_RATINGS PRIMARY KEY(REG_ID)
);

CREATE TABLE REG_RESOURCE_RATINGS (
            REG_RATING_ID           INTEGER NOT NULL,
            REG_VERSION             INTEGER DEFAULT 0,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_RATINGS PRIMARY KEY(REG_VERSION, REG_RATING_ID)
);

ALTER TABLE REG_RESOURCE_RATINGS ADD CONSTRAINT REG_RESOURCE_RATINGS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
ALTER TABLE REG_RESOURCE_RATINGS ADD CONSTRAINT REG_RESOURCE_RATINGS_FK_BY_RATING_ID FOREIGN KEY (REG_RATING_ID) REFERENCES REG_RATINGS (REG_ID);
CREATE INDEX REG_RESOURCE_RATINGS_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_RATINGS(REG_PATH_ID, REG_RESOURCE_NAME);

CREATE TABLE REG_TAGS (
            REG_ID         INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
            REG_TAG_NAME       VARCHAR(500) NOT NULL,
            REG_USER_ID        VARCHAR(20) NOT NULL,
            REG_TAGGED_TIME    TIMESTAMP NOT NULL,
            CONSTRAINT PK_REG_TAGS PRIMARY KEY(REG_ID)
);

CREATE TABLE REG_RESOURCE_TAGS (
            REG_TAG_ID              INTEGER NOT NULL,
            REG_VERSION             INTEGER DEFAULT 0,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_TAGS PRIMARY KEY(REG_VERSION, REG_TAG_ID)
);

ALTER TABLE REG_RESOURCE_TAGS ADD CONSTRAINT REG_RESOURCE_TAGS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
ALTER TABLE REG_RESOURCE_TAGS ADD CONSTRAINT REG_RESOURCE_TAGS_FK_BY_TAG_ID FOREIGN KEY (REG_TAG_ID) REFERENCES REG_TAGS (REG_ID);
CREATE INDEX REG_RESOURCE_TAGS_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_TAGS(REG_PATH_ID, REG_RESOURCE_NAME);

CREATE TABLE REG_PROPERTIES (
            REG_ID         INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
            REG_NAME       VARCHAR(100) NOT NULL,
            REG_VALUE        VARCHAR(500),
            CONSTRAINT PK_REG_PROPERTIES PRIMARY KEY(REG_ID)
);

CREATE TABLE REG_RESOURCE_PROPERTIES (
            REG_PROPERTY_ID         INTEGER NOT NULL,
            REG_VERSION             INTEGER DEFAULT 0,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_PROPERTIES PRIMARY KEY(REG_VERSION, REG_PROPERTY_ID)
);

ALTER TABLE REG_RESOURCE_PROPERTIES ADD CONSTRAINT REG_RESOURCE_PROPERTIES_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
ALTER TABLE REG_RESOURCE_PROPERTIES ADD CONSTRAINT REG_RESOURCE_PROPERTIES_FK_BY_TAG_ID FOREIGN KEY (REG_PROPERTY_ID) REFERENCES REG_PROPERTIES (REG_ID);
CREATE INDEX REG_RESOURCE_PROPERTIES_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_PROPERTIES(REG_PATH_ID, REG_RESOURCE_NAME);

-- CREATE TABLE REG_ASSOCIATIONS (
-- SRC_PATH_ID     INTEGER,
-- SRC_RESOURCE_NAME    VARCHAR(256),
-- SRC_VERSION     INTEGER,
-- TGT_PATH_ID     INTEGER,
-- TGT_RESOURCE_NAME    VARCHAR(256),
-- TGT_VERSION     INTEGER
-- );
--
-- ALTER TABLE REG_ASSOCIATIONS ADD CONSTRAINT REG_ASSOCIATIONS_FK_BY_SRC_PATH_ID FOREIGN KEY (SRC_PATH_ID) REFERENCES REG_PATH (PATH_ID);
-- ALTER TABLE REG_ASSOCIATIONS ADD CONSTRAINT REG_ASSOCIATIONS_FK_BY_TGT_PATH_ID FOREIGN KEY (TGT_PATH_ID) REFERENCES REG_PATH (PATH_ID);
-- CREATE INDEX REG_ASSOCIATIONS_IND_BY_SRC_VERSION ON REG_ASSOCIATIONS(SRC_VERSION);
-- CREATE INDEX REG_ASSOCIATIONS_IND_BY_TGT_VERSION ON REG_ASSOCIATIONS(TGT_VERSION);
-- CREATE INDEX REG_ASSOCIATIONS_IND_BY_SRC_RESOURCE_NAME ON REG_ASSOCIATIONS(SRC_RESOURCE_NAME);
-- CREATE INDEX REG_ASSOCIATIONS_IND_BY_TGT_RESOURCE_NAME ON REG_ASSOCIATIONS(TGT_RESOURCE_NAME);



CREATE TABLE REG_ASSOCIATION (
            REG_ASSOCIATION_ID INTEGER GENERATED ALWAYS AS IDENTITY,
            REG_SOURCEPATH VARCHAR (2000) NOT NULL,
            REG_TARGETPATH VARCHAR (2000) NOT NULL,
            REG_ASSOCIATION_TYPE VARCHAR (2000) NOT NULL,
            PRIMARY KEY (REG_ASSOCIATION_ID)
);

CREATE TABLE REG_SNAPSHOTS (
            REG_SNAPSHOT_ID     INTEGER NOT NULL GENERATED BY DEFAULT AS IDENTITY,
            REG_PATH_ID            INTEGER NOT NULL,
            REG_RESOURCE_NAME       VARCHAR(256),
            REG_RESOURCE_VIDS     BLOB NOT NULL,
            CONSTRAINT PK_REG_SNAPSHOTS PRIMARY KEY(REG_SNAPSHOT_ID)
);

ALTER TABLE REG_SNAPSHOTS ADD CONSTRAINT REG_SNAPSHOTS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);
