--create table REG_CLUSTER_LOCK
IF NOT  EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_CLUSTER_LOCK]') AND TYPE IN (N'U'))
CREATE TABLE  REG_CLUSTER_LOCK (
             REG_LOCK_NAME VARCHAR (20),
             REG_LOCK_STATUS VARCHAR (20),
             REG_LOCKED_TIME DATETIME,
             PRIMARY KEY (REG_LOCK_NAME)
)

--create table REG_LOG

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_LOG]') AND TYPE IN (N'U'))

CREATE TABLE REG_LOG (
             REG_LOG_ID INTEGER IDENTITY(1,1) NOT NULL,
             REG_PATH VARCHAR (2000),
             REG_USER_ID VARCHAR (20) NOT NULL,
             REG_LOGGED_TIME DATETIME NOT NULL,
             REG_ACTION INTEGER NOT NULL,
             REG_ACTION_DATA VARCHAR (500),
             PRIMARY KEY (REG_LOG_ID)
)

--create table regpath

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_PATH]') AND TYPE IN (N'U'))
CREATE TABLE  REG_PATH(
             REG_PATH_ID INTEGER IDENTITY(1,1) NOT NULL,
             REG_PATH_VALUE VARCHAR(2000) NOT NULL,
             REG_PATH_PARENT_ID INT,
             CONSTRAINT PK_REG_PATH PRIMARY KEY(REG_PATH_ID)
)

--create table regcontent

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_CONTENT]') AND TYPE IN (N'U'))
CREATE TABLE  REG_CONTENT (
             REG_CONTENT_ID INTEGER IDENTITY(1,1) NOT NULL,
             REG_CONTENT_DATA VARBINARY(MAX),
             CONSTRAINT PK_REG_CONTENT PRIMARY KEY(REG_CONTENT_ID)
)

--create table REG_CONTENT_HISTORY
IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_CONTENT_HISTORY]') AND TYPE IN (N'U'))
CREATE TABLE  REG_CONTENT_HISTORY (
             REG_CONTENT_ID INTEGER NOT NULL,
             REG_CONTENT_DATA VARBINARY(MAX),
             REG_DELETED   SMALLINT,
             CONSTRAINT PK_REG_CONTENT_HISTORY PRIMARY KEY(REG_CONTENT_ID)
)

GO

--create table REG_RESOURCE
IF NOT  EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE]') AND TYPE IN (N'U'))
CREATE TABLE  REG_RESOURCE (
            REG_PATH_ID         INTEGER NOT NULL,
            REG_NAME            VARCHAR(256),
            REG_VERSION          INTEGER IDENTITY(1,1) NOT NULL,
            REG_MEDIA_TYPE      VARCHAR(500),
            REG_CREATOR         VARCHAR(20) NOT NULL,
            REG_CREATED_TIME    DATETIME NOT NULL,
            REG_LAST_UPDATOR    VARCHAR(20),
            REG_LAST_UPDATED_TIME   DATETIME NOT NULL,
            REG_DESCRIPTION     VARCHAR(1000),
            REG_CONTENT_ID      INTEGER,
            --CONSTRAINT PK_REG_RESOURCE PRIMARY KEY(REG_VERSION)
)

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_FK_BY_PATH_ID]') AND PARENT_OBJECT_ID = OBJECT_ID(N'DBO.[REG_RESOURCE]'))
ALTER TABLE REG_RESOURCE ADD CONSTRAINT REG_RESOURCE_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_FK_BY_CONTENT_ID]') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE]'))
ALTER TABLE REG_RESOURCE ADD CONSTRAINT REG_RESOURCE_FK_BY_CONTENT_ID FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT (REG_CONTENT_ID);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_IND_BY_NAME')
DROP INDEX REG_RESOURCE.REG_RESOURCE_IND_BY_NAME
CREATE INDEX REG_RESOURCE_IND_BY_NAME ON REG_RESOURCE(REG_NAME);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_IND_BY_PATH_ID_NAME')
DROP INDEX REG_RESOURCE.REG_RESOURCE_IND_BY_PATH_ID_NAME
CREATE INDEX REG_RESOURCE_IND_BY_PATH_ID_NAME ON REG_RESOURCE(REG_PATH_ID, REG_NAME);


--create table REG_RESOURCE_HISTORY
IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_HISTORY]') AND TYPE IN (N'U'))
CREATE TABLE  REG_RESOURCE_HISTORY (
            REG_PATH_ID         INTEGER NOT NULL,
            REG_NAME            VARCHAR(256),
            REG_VERSION         INTEGER NOT NULL,
            REG_MEDIA_TYPE      VARCHAR(500),
            REG_CREATOR         VARCHAR(20) NOT NULL,
            REG_CREATED_TIME    DATETIME NOT NULL,
            REG_LAST_UPDATOR    VARCHAR(20),
            REG_LAST_UPDATED_TIME  DATETIME NOT NULL,
            REG_DESCRIPTION     VARCHAR(1000),
            REG_CONTENT_ID      INTEGER,
            REG_DELETED         SMALLINT,
            CONSTRAINT PK_REG_RESOURCE_HISTORY PRIMARY KEY(REG_VERSION)
)

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE object_id = OBJECT_ID(N'[dbo].[REG_RESOURCE_HIST_FK_BY_PATHID]') AND parent_object_id = OBJECT_ID(N'[dbo].[REG_RESOURCE_HISTORY]'))
ALTER TABLE REG_RESOURCE_HISTORY ADD CONSTRAINT REG_RESOURCE_HIST_FK_BY_PATHID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE object_id = OBJECT_ID(N'[dbo].[REG_RESOURCE_HIST_FK_BY_CONTENT_ID]') AND parent_object_id = OBJECT_ID(N'[dbo].[REG_RESOURCE_HISTORY]'))
ALTER TABLE REG_RESOURCE_HISTORY ADD CONSTRAINT REG_RESOURCE_HIST_FK_BY_CONTENT_ID FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT_HISTORY (REG_CONTENT_ID);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_HISTORY_IND_BY_NAME')
DROP INDEX REG_RESOURCE_HISTORY.REG_RESOURCE_HISTORY_IND_BY_NAME
CREATE INDEX REG_RESOURCE_HISTORY_IND_BY_NAME ON REG_RESOURCE_HISTORY(REG_NAME);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_HISTORY_IND_BY_PATH_ID_NAME')
DROP INDEX REG_RESOURCE_HISTORY.REG_RESOURCE_HISTORY_IND_BY_PATH_ID_NAME
CREATE INDEX REG_RESOURCE_HISTORY_IND_BY_PATH_ID_NAME ON REG_RESOURCE_HISTORY(REG_PATH_ID, REG_NAME);

--create table REG_COMMENTS

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_COMMENTS]') AND TYPE IN (N'U'))
CREATE TABLE  REG_COMMENTS (
            REG_ID      INTEGER IDENTITY(1,1) NOT NULL,
            REG_COMMENT_TEXT      VARCHAR(500) NOT NULL,
            REG_USER_ID           VARCHAR(20) NOT NULL,
            REG_COMMENTED_TIME    DATETIME NOT NULL,
            CONSTRAINT PK_REG_COMMENTS PRIMARY KEY(REG_ID)
)

--create table REG_RESOURCE_COMMENTS
IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_COMMENTS]') AND TYPE IN (N'U'))
CREATE TABLE  REG_RESOURCE_COMMENTS (
            REG_COMMENT_ID          INTEGER NOT NULL,
            REG_VERSION             INTEGER DEFAULT 0,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_COMMENTS PRIMARY KEY(REG_VERSION, REG_COMMENT_ID)
)

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].REG_RESOURCE_COMMENTS_FK_BY_PATH_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_COMMENTS'))
ALTER TABLE REG_RESOURCE_COMMENTS ADD CONSTRAINT REG_RESOURCE_COMMENTS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].REG_RESOURCE_COMMENTS_FK_BY_COMMENT_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_COMMENTS'))
ALTER TABLE REG_RESOURCE_COMMENTS ADD CONSTRAINT REG_RESOURCE_COMMENTS_FK_BY_COMMENT_ID FOREIGN KEY (REG_COMMENT_ID) REFERENCES REG_COMMENTS (REG_ID);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_COMMENTS_IND_BY_PATH_ID_AND_RESOURCE_NAME')
DROP INDEX REG_RESOURCE_COMMENTS.REG_RESOURCE_COMMENTS_IND_BY_PATH_ID_AND_RESOURCE_NAME
CREATE INDEX REG_RESOURCE_COMMENTS_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_COMMENTS(REG_PATH_ID, REG_RESOURCE_NAME);

--create table  REG_RATINGS
IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RATINGS]') AND TYPE IN (N'U'))
CREATE TABLE REG_RATINGS (
            REG_ID      INTEGER IDENTITY(1,1) NOT NULL,
            REG_RATING        INTEGER NOT NULL,
            REG_USER_ID       VARCHAR(20) NOT NULL,
            REG_RATED_TIME    DATETIME NOT NULL,
            CONSTRAINT PK_REG_RATINGS PRIMARY KEY(REG_ID)
)

--create table REG_RESOURCE_RATINGS

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_RATINGS]') AND TYPE IN (N'U'))
CREATE TABLE  REG_RESOURCE_RATINGS (
            REG_RATING_ID           INTEGER NOT NULL,
            REG_VERSION             INTEGER,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_RATINGS PRIMARY KEY(REG_VERSION, REG_RATING_ID)
)

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].REG_RESOURCE_RATINGS_FK_BY_PATH_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[dbo].REG_RESOURCE_RATINGS'))
ALTER TABLE REG_RESOURCE_RATINGS ADD CONSTRAINT REG_RESOURCE_RATINGS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].REG_RESOURCE_RATINGS_FK_BY_RATING_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[dbo].REG_RESOURCE_RATINGS'))
ALTER TABLE REG_RESOURCE_RATINGS ADD CONSTRAINT REG_RESOURCE_RATINGS_FK_BY_RATING_ID FOREIGN KEY (REG_RATING_ID) REFERENCES REG_RATINGS (REG_ID);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_RATINGS_IND_BY_PATH_ID_AND_RESOURCE_NAME')
DROP INDEX REG_RESOURCE_RATINGS.REG_RESOURCE_RATINGS_IND_BY_PATH_ID_AND_RESOURCE_NAME
CREATE INDEX REG_RESOURCE_RATINGS_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_RATINGS(REG_PATH_ID, REG_RESOURCE_NAME);

--create table  REG_TAGS

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_TAGS]') AND TYPE IN (N'U'))
CREATE TABLE  REG_TAGS (
            REG_ID         INTEGER IDENTITY(1,1) NOT NULL,
            REG_TAG_NAME       VARCHAR(500) NOT NULL,
            REG_USER_ID        VARCHAR(20) NOT NULL,
            REG_TAGGED_TIME    DATETIME NOT NULL,
            CONSTRAINT PK_REG_TAGS PRIMARY KEY(REG_ID)
)



--create table  REG_RESOURCE_TAGS

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_TAGS]') AND TYPE IN (N'U'))
CREATE TABLE   REG_RESOURCE_TAGS (
            REG_TAG_ID              INTEGER NOT NULL,
            REG_VERSION             INTEGER DEFAULT 0,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_TAGS PRIMARY KEY(REG_VERSION, REG_TAG_ID)
)

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_TAGS_FK_BY_PATH_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_TAGS'))
ALTER TABLE REG_RESOURCE_TAGS ADD CONSTRAINT REG_RESOURCE_TAGS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_TAGS_FK_BY_TAG_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_TAGS'))
ALTER TABLE REG_RESOURCE_TAGS ADD CONSTRAINT REG_RESOURCE_TAGS_FK_BY_TAG_ID FOREIGN KEY (REG_TAG_ID) REFERENCES REG_TAGS (REG_ID);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_TAGS_IND_BY_PATH_ID_AND_RESOURCE_NAME')
DROP INDEX REG_RESOURCE_TAGS.REG_RESOURCE_TAGS_IND_BY_PATH_ID_AND_RESOURCE_NAME
CREATE INDEX REG_RESOURCE_TAGS_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_TAGS(REG_PATH_ID, REG_RESOURCE_NAME);

--CREATE TABLE REG_PROPERTIES

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_PROPERTIES]') AND TYPE IN (N'U'))
CREATE TABLE REG_PROPERTIES (
            REG_ID        INTEGER IDENTITY(1,1) NOT NULL,
            REG_NAME       VARCHAR(100) NOT NULL,
            REG_VALUE        VARCHAR(500),
            CONSTRAINT PK_REG_PROPERTIES PRIMARY KEY(REG_ID)
)

--CREATE TABLE REG_RESOURCE_PROPERTIES

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_RESOURCE_PROPERTIES]') AND TYPE IN (N'U'))
CREATE TABLE  REG_RESOURCE_PROPERTIES (
            REG_PROPERTY_ID         INTEGER NOT NULL,
            REG_VERSION             INTEGER,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR(256),
            CONSTRAINT PK_REG_RESOURCE_PROPERTIES PRIMARY KEY(REG_VERSION, REG_PROPERTY_ID)
)

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_PROPERTIES_FK_BY_PATH_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_PROPERTIES'))
ALTER TABLE REG_RESOURCE_PROPERTIES ADD CONSTRAINT REG_RESOURCE_PROPERTIES_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_PROPERTIES_FK_BY_TAG_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_RESOURCE_PROPERTIES'))
ALTER TABLE REG_RESOURCE_PROPERTIES ADD CONSTRAINT REG_RESOURCE_PROPERTIES_FK_BY_TAG_ID FOREIGN KEY (REG_PROPERTY_ID) REFERENCES REG_PROPERTIES (REG_ID);

IF EXISTS (SELECT NAME FROM SYSINDEXES WHERE NAME = 'REG_RESOURCE_PROPERTIES_IND_BY_PATH_ID_AND_RESOURCE_NAME')
DROP INDEX REG_RESOURCE_PROPERTIES.REG_RESOURCE_PROPERTIES_IND_BY_PATH_ID_AND_RESOURCE_NAME
CREATE INDEX REG_RESOURCE_PROPERTIES_IND_BY_PATH_ID_AND_RESOURCE_NAME ON REG_RESOURCE_PROPERTIES(REG_PATH_ID, REG_RESOURCE_NAME);

--CREATE TABLE  REG_ASSOCIATION

IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_ASSOCIATION]') AND TYPE IN (N'U'))
CREATE TABLE  REG_ASSOCIATION (
            REG_ASSOCIATION_ID  INTEGER IDENTITY(1,1) NOT NULL,
            REG_SOURCEPATH VARCHAR (2000) NOT NULL,
            REG_TARGETPATH VARCHAR (2000) NOT NULL,
            REG_ASSOCIATION_TYPE VARCHAR (2000) NOT NULL,
            PRIMARY KEY (REG_ASSOCIATION_ID)
)

--CREATE TABLE  REG_SNAPSHOTS
IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].[REG_SNAPSHOTS]') AND TYPE IN (N'U'))
CREATE TABLE REG_SNAPSHOTS (
            REG_SNAPSHOT_ID     INTEGER IDENTITY(1,1) NOT NULL,
            REG_PATH_ID            INTEGER NOT NULL,
            REG_RESOURCE_NAME            VARCHAR (256),
            REG_RESOURCE_VIDS     VARBINARY(MAX) NOT NULL,
            CONSTRAINT PK_REG_SNAPSHOTS PRIMARY KEY(REG_SNAPSHOT_ID)
)
IF NOT EXISTS (SELECT * FROM SYS.FOREIGN_KEYS WHERE OBJECT_ID = OBJECT_ID(N'[DBO].REG_SNAPSHOTS_FK_BY_PATH_ID') AND PARENT_OBJECT_ID = OBJECT_ID(N'[DBO].REG_SNAPSHOTS'))
ALTER TABLE REG_SNAPSHOTS ADD CONSTRAINT REG_SNAPSHOTS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID);

-- ################################
-- USER MANAGER TABLES
-- ################################

--CREATE TABLE   UM_USERS

--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_USERS]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_USERS (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             USER_NAME VARCHAR(255) NOT NULL,
--             USER_PASSWORD VARCHAR(255) NOT NULL,
--             PRIMARY KEY (ID),
--             UNIQUE(USER_NAME)
--)

--CREATE TABLE   UM_USER_ATTRIBUTES

--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_USER_ATTRIBUTES]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_USER_ATTRIBUTES (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             ATTR_NAME VARCHAR(255) NOT NULL,
--             ATTR_VALUE VARCHAR(255),
--             USER_ID INTEGER,
--             FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,
--             PRIMARY KEY (ID)
--)
--
--CREATE TABLE   UM_ROLES

--IF NOT  EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_ROLES]') AND TYPE IN (N'U'))
--CREATE TABLE UM_ROLES (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             ROLE_NAME VARCHAR(255) NOT NULL,
--             PRIMARY KEY (ID),
--             UNIQUE(ROLE_NAME)
--)


--CREATE TABLE   UM_ROLES

--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_ROLE_ATTRIBUTES]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_ROLE_ATTRIBUTES (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             ATTR_NAME VARCHAR(255) NOT NULL,
--             ATTR_VALUE VARCHAR(255),
--             ROLE_ID INTEGER,
--             FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,
--             PRIMARY KEY (ID)
--)

--CREATE TABLE UM_PERMISSIONS

--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_PERMISSIONS]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_PERMISSIONS (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             RESOURCE_ID VARCHAR(255) NOT NULL,
--             ACTION VARCHAR(255) NOT NULL,
--             PRIMARY KEY (ID)
--)

--IF EXISTS (SELECT name FROM sysindexes WHERE name = 'INDEX_UM_PERMISSIONS_RESOURCE_ID_ACTION')
--DROP INDEX UM_PERMISSIONS.INDEX_UM_PERMISSIONS_RESOURCE_ID_ACTION
--CREATE INDEX INDEX_UM_PERMISSIONS_RESOURCE_ID_ACTION ON UM_PERMISSIONS (RESOURCE_ID, ACTION);

--CREATE TABLE UM_ROLE_PERMISSIONS

--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_ROLE_PERMISSIONS]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_ROLE_PERMISSIONS (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             PERMISSION_ID INTEGER NOT NULL,
--             ROLE_ID INTEGER NOT NULL,
--             IS_ALLOWED SMALLINT NOT NULL,
--             UNIQUE (PERMISSION_ID, ROLE_ID),
--             FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE,
--             FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,
--             PRIMARY KEY (ID)
--)

--CREATE TABLE UM_USER_PERMISSIONS

--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_USER_PERMISSIONS]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_USER_PERMISSIONS (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             PERMISSION_ID INTEGER NOT NULL,
--             USER_ID INTEGER NOT NULL,
--             IS_ALLOWED SMALLINT NOT NULL,
--             UNIQUE (PERMISSION_ID, USER_ID),
--             FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE,
--             FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,
--             PRIMARY KEY (ID)
--)
--
--create table UM_USER_ROLES
--IF NOT EXISTS (SELECT * FROM SYS.OBJECTS WHERE OBJECT_ID = OBJECT_ID(N'[dbo].[UM_USER_ROLES]') AND TYPE IN (N'U'))
--CREATE TABLE  UM_USER_ROLES (
--             ID INTEGER IDENTITY(1,1) NOT NULL,
--             ROLE_ID INTEGER NOT NULL,
--             USER_ID INTEGER NOT NULL,
--             UNIQUE (USER_ID, ROLE_ID),
--             FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,
--             FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,
--             PRIMARY KEY (ID)
--)
