CREATE TABLE REG_CLUSTER_LOCK (
             REG_LOCK_NAME VARCHAR2(20),
             REG_LOCK_STATUS VARCHAR2(20),
             REG_LOCKED_TIME TIMESTAMP,
             PRIMARY KEY (REG_LOCK_NAME))
/
CREATE TABLE REG_LOG (
             REG_LOG_ID INTEGER,
             REG_PATH VARCHAR2(2000),
             REG_USER_ID VARCHAR2(20) NOT NULL,
             REG_LOGGED_TIME TIMESTAMP  NOT NULL,
             REG_ACTION INTEGER NOT NULL,
             REG_ACTION_DATA VARCHAR2(500),
             PRIMARY KEY (REG_LOG_ID))
/
CREATE SEQUENCE REG_LOG_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_LOG_TRIGGER
                    BEFORE INSERT
                    ON REG_LOG
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_LOG_ID IS NOT NULL THEN
                       SELECT REG_LOG_SEQUENCE.nextval INTO :NEW.REG_LOG_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_PATH(
             REG_PATH_ID INTEGER NOT NULL,
             REG_PATH_VALUE VARCHAR2(2000) NOT NULL,
             REG_PATH_PARENT_ID INT,
             CONSTRAINT PK_REG_PATH PRIMARY KEY(REG_PATH_ID))
/
CREATE SEQUENCE REG_PATH_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_PATH_TRIGGER
                    BEFORE INSERT
                    ON REG_PATH
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_PATH_ID IS NOT NULL THEN
                       SELECT REG_PATH_SEQUENCE.nextval INTO :NEW.REG_PATH_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_CONTENT (
             REG_CONTENT_ID INTEGER NOT NULL,
             REG_CONTENT_DATA BLOB,
             CONSTRAINT PK_REG_CONTENT PRIMARY KEY(REG_CONTENT_ID))
/
CREATE SEQUENCE REG_CONTENT_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_CONTENT_TRIGGER
                    BEFORE INSERT
                    ON REG_CONTENT
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_CONTENT_ID IS NOT NULL THEN
                       SELECT REG_CONTENT_SEQUENCE.nextval INTO :NEW.REG_CONTENT_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_CONTENT_HISTORY (
             REG_CONTENT_ID INTEGER NOT NULL,
             REG_CONTENT_DATA BLOB,
             REG_DELETED   SMALLINT,
             CONSTRAINT PK_REG_CONTENT_HISTORY PRIMARY KEY(REG_CONTENT_ID))
/
CREATE TABLE REG_RESOURCE (
            REG_PATH_ID         INTEGER NOT NULL,
            REG_NAME            VARCHAR2(256),
            REG_VERSION         INTEGER NOT NULL,
            REG_MEDIA_TYPE      VARCHAR2(500),
            REG_CREATOR         VARCHAR2(20) NOT NULL,
            REG_CREATED_TIME    TIMESTAMP NOT NULL,
            REG_LAST_UPDATOR    VARCHAR2(20),
            REG_LAST_UPDATED_TIME    TIMESTAMP NOT NULL,
            REG_DESCRIPTION     VARCHAR2(1000),
            REG_CONTENT_ID      INTEGER )
            --CONSTRAINT PK_REG_RESOURCE PRIMARY KEY(REG_VERSION)
/
CREATE SEQUENCE REG_RESOURCE_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_RESOURCE_TRIGGER
                    BEFORE INSERT
                    ON REG_RESOURCE
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_VERSION IS NOT NULL THEN
                       SELECT REG_RESOURCE_SEQUENCE.nextval INTO :NEW.REG_VERSION FROM dual;
		                  END IF;
                    END;
/
ALTER TABLE REG_RESOURCE ADD CONSTRAINT REG_RESOURCE_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
ALTER TABLE REG_RESOURCE ADD CONSTRAINT REG_RESOURCE_FK_BY_CONTENT_ID FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT (REG_CONTENT_ID)
/
--CREATE INDEX REG_RESOURCE_IND_BY_NAME ON REG_RESOURCE(REG_NAME)

CREATE INDEX REG_RESOURCE_IND_BY_PATH_ID ON REG_RESOURCE(REG_PATH_ID, REG_NAME)
/
CREATE TABLE REG_RESOURCE_HISTORY (
            REG_PATH_ID         INTEGER NOT NULL,
            REG_NAME            VARCHAR2(256),
            REG_VERSION         INTEGER NOT NULL,
            REG_MEDIA_TYPE      VARCHAR2(500),
            REG_CREATOR         VARCHAR2(20) NOT NULL,
            REG_CREATED_TIME    TIMESTAMP NOT NULL,
            REG_LAST_UPDATOR    VARCHAR2(20),
            REG_LAST_UPDATED_TIME    TIMESTAMP NOT NULL,
            REG_DESCRIPTION     VARCHAR2(1000),
            REG_CONTENT_ID      INTEGER,
            REG_DELETED         SMALLINT,
            CONSTRAINT PK_REG_RESOURCE_HISTORY PRIMARY KEY(REG_VERSION))
/
ALTER TABLE REG_RESOURCE_HISTORY ADD CONSTRAINT REG_RES_HIST_FK_BY_PATHID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
ALTER TABLE REG_RESOURCE_HISTORY ADD CONSTRAINT REG_RES_HIST_FK_BY_CONTENT FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT_HISTORY (REG_CONTENT_ID)
/
CREATE INDEX REG_RES_HIST_IND_BY_NAME ON REG_RESOURCE_HISTORY(REG_NAME)
/
CREATE TABLE REG_COMMENTS (
            REG_ID        INTEGER NOT NULL,
            REG_COMMENT_TEXT      VARCHAR2(500) NOT NULL,
            REG_USER_ID           VARCHAR2(20) NOT NULL,
            REG_COMMENTED_TIME    TIMESTAMP NOT NULL,
            CONSTRAINT PK_REG_COMMENTS PRIMARY KEY(REG_ID))
/
CREATE SEQUENCE REG_COMMENTS_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_COMMENTS_TRIGGER
                    BEFORE INSERT
                    ON REG_COMMENTS
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_ID IS NOT NULL THEN
                       SELECT REG_COMMENTS_SEQUENCE.nextval INTO :NEW.REG_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_RESOURCE_COMMENTS (
            REG_COMMENT_ID          INTEGER NOT NULL,
            REG_VERSION             INTEGER,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR2(256),
            CONSTRAINT PK_REG_RESOURCE_COMMENTS PRIMARY KEY(REG_VERSION, REG_COMMENT_ID))
/
ALTER TABLE REG_RESOURCE_COMMENTS ADD CONSTRAINT REG_RES_COMM_FK_BY_PATH FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
ALTER TABLE REG_RESOURCE_COMMENTS ADD CONSTRAINT REG_RES_COMM_FK_BY_COMMENT FOREIGN KEY (REG_COMMENT_ID) REFERENCES REG_COMMENTS (REG_ID)
/
CREATE INDEX REG_RES_COMM_BY_PATH_ID ON REG_RESOURCE_COMMENTS(REG_PATH_ID, REG_RESOURCE_NAME)
/
CREATE TABLE REG_RATINGS (
            REG_ID     INTEGER NOT NULL ,
            REG_RATING        INTEGER NOT NULL,
            REG_USER_ID       VARCHAR2(20) NOT NULL,
            REG_RATED_TIME    TIMESTAMP NOT NULL,
            CONSTRAINT PK_REG_RATINGS PRIMARY KEY(REG_ID))
/
CREATE SEQUENCE REG_RATINGS_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_RATINGS_TRIGGER
                    BEFORE INSERT
                    ON REG_RATINGS
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_ID IS NOT NULL THEN
                       SELECT REG_RATINGS_SEQUENCE.nextval INTO :NEW.REG_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_RESOURCE_RATINGS (
            REG_RATING_ID           INTEGER NOT NULL,
            REG_VERSION             INTEGER,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR2(256),
            CONSTRAINT PK_REG_RESOURCE_RATINGS PRIMARY KEY(REG_VERSION, REG_RATING_ID))
 /
ALTER TABLE REG_RESOURCE_RATINGS ADD CONSTRAINT REG_RES_RATINGS_FK_BY_PATH FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
ALTER TABLE REG_RESOURCE_RATINGS ADD CONSTRAINT REG_RES_RATINGS_FK_BY_RATING FOREIGN KEY (REG_RATING_ID) REFERENCES REG_RATINGS (REG_ID)
/
CREATE INDEX REG_RATINGS_IND_BY_PATH_ID ON REG_RESOURCE_RATINGS(REG_PATH_ID, REG_RESOURCE_NAME)
/
CREATE TABLE REG_TAGS (
            REG_ID         INTEGER NOT NULL,
            REG_TAG_NAME       VARCHAR2(500) NOT NULL,
            REG_USER_ID        VARCHAR2(20) NOT NULL,
            REG_TAGGED_TIME    TIMESTAMP NOT NULL,
            CONSTRAINT PK_REG_TAGS PRIMARY KEY(REG_ID))
/
CREATE SEQUENCE REG_TAGS_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_TAGS_TRIGGER
                    BEFORE INSERT
                    ON REG_TAGS
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_ID IS NOT NULL THEN
                       SELECT REG_TAGS_SEQUENCE.nextval INTO :NEW.REG_ID FROM dual;
		                  END IF;
                    END;
/

CREATE TABLE REG_RESOURCE_TAGS (
            REG_TAG_ID              INTEGER NOT NULL,
            REG_VERSION             INTEGER,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR2(256),
            CONSTRAINT PK_REG_RESOURCE_TAGS PRIMARY KEY(REG_VERSION, REG_TAG_ID))
/
ALTER TABLE REG_RESOURCE_TAGS ADD CONSTRAINT REG_RESOURCE_TAGS_FK_BY_PATH FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
ALTER TABLE REG_RESOURCE_TAGS ADD CONSTRAINT REG_RESOURCE_TAGS_FK_BY_TAG FOREIGN KEY (REG_TAG_ID) REFERENCES REG_TAGS (REG_ID)
/
CREATE INDEX REG_TAGS_IND_BY_PATH_ID ON REG_RESOURCE_TAGS(REG_PATH_ID, REG_RESOURCE_NAME)
/
CREATE TABLE REG_PROPERTIES (
            REG_ID         INTEGER NOT NULL,
            REG_NAME       VARCHAR2(100) NOT NULL,
            REG_VALUE        VARCHAR2(500),
            CONSTRAINT PK_REG_PROPERTIES PRIMARY KEY(REG_ID))
/
CREATE SEQUENCE REG_PROPERTIES_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_PROPERTIES_TRIGGER
                    BEFORE INSERT
                    ON REG_PROPERTIES
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_ID IS NOT NULL THEN
                       SELECT REG_PROPERTIES_SEQUENCE.nextval INTO :NEW.REG_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_RESOURCE_PROPERTIES (
            REG_PROPERTY_ID         INTEGER NOT NULL,
            REG_VERSION             INTEGER,
            REG_PATH_ID             INTEGER,
            REG_RESOURCE_NAME       VARCHAR2(256),
            CONSTRAINT PK_REG_RESOURCE_PROPERTIES PRIMARY KEY(REG_VERSION, REG_PROPERTY_ID))
/
ALTER TABLE REG_RESOURCE_PROPERTIES ADD CONSTRAINT REG_RES_PRO_FK_BY_PATH FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
ALTER TABLE REG_RESOURCE_PROPERTIES ADD CONSTRAINT REG_RES_PROP_FK_BY_TAG FOREIGN KEY (REG_PROPERTY_ID) REFERENCES REG_PROPERTIES (REG_ID)
/
CREATE INDEX REG_RESC_PROP_IND_BY_PATH_ID ON REG_RESOURCE_PROPERTIES(REG_PATH_ID, REG_RESOURCE_NAME)
/
CREATE TABLE REG_ASSOCIATION (
            REG_ASSOCIATION_ID INTEGER,
            REG_SOURCEPATH VARCHAR2 (2000) NOT NULL,
            REG_TARGETPATH VARCHAR2 (2000) NOT NULL,
            REG_ASSOCIATION_TYPE VARCHAR2 (2000) NOT NULL,
            PRIMARY KEY (REG_ASSOCIATION_ID))
/
CREATE SEQUENCE REG_ASSOCIATION_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_ASSOCIATION_TRIGGER
                    BEFORE INSERT
                    ON REG_ASSOCIATION
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_ASSOCIATION_ID IS NOT NULL THEN
                       SELECT REG_ASSOCIATION_SEQUENCE.nextval INTO :NEW.REG_ASSOCIATION_ID FROM dual;
		                  END IF;
                    END;
/
CREATE TABLE REG_SNAPSHOTS (
            REG_SNAPSHOT_ID     INTEGER NOT NULL,
            REG_PATH_ID            INTEGER NOT NULL,
            REG_RESOURCE_NAME      VARCHAR2(256),
            REG_RESOURCE_VIDS     BLOB NOT NULL,
            CONSTRAINT PK_REG_SNAPSHOTS PRIMARY KEY(REG_SNAPSHOT_ID))
/
CREATE SEQUENCE REG_SNAPSHOTS_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_SNAPSHOTS_TRIGGER
                    BEFORE INSERT
                    ON REG_SNAPSHOTS
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                     BEGIN
                      IF :NEW.REG_SNAPSHOT_ID IS NOT NULL THEN
                       SELECT REG_SNAPSHOTS_SEQUENCE.nextval INTO :NEW.REG_SNAPSHOT_ID FROM dual;
		                  END IF;
                    END;
/
ALTER TABLE REG_SNAPSHOTS ADD CONSTRAINT REG_SNAPSHOTS_FK_BY_PATH_ID FOREIGN KEY (REG_PATH_ID) REFERENCES REG_PATH (REG_PATH_ID)
/
COMMIT
/
