CREATE TABLE UM_USERS (
				ID INTEGER IDENTITY(1, 1),
				USER_NAME VARCHAR(255) NOT NULL,
				USER_PASSWORD VARCHAR(255) NOT NULL,
				PRIMARY KEY (ID),
				UNIQUE(USER_NAME));

CREATE TABLE UM_USER_ATTRIBUTES (           
				ID INTEGER IDENTITY(1, 1),
				ATTR_NAME VARCHAR(255) NOT NULL,
				ATTR_VALUE VARCHAR(255),
				PROFILE_ID VARCHAR(255), 
				USER_ID INTEGER,
				FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,
				PRIMARY KEY (ID));

CREATE TABLE UM_ROLES (
				ID INTEGER IDENTITY(1, 1),
				ROLE_NAME VARCHAR(255) NOT NULL,
				PRIMARY KEY (ID),
				UNIQUE(ROLE_NAME));

CREATE TABLE UM_ROLE_ATTRIBUTES (
				ID INTEGER IDENTITY(1, 1),
				ATTR_NAME VARCHAR(255) NOT NULL,
				ATTR_VALUE VARCHAR(255),
				ROLE_ID INTEGER,
				FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,
				PRIMARY KEY (ID));

CREATE TABLE UM_PERMISSIONS (
				ID INTEGER IDENTITY(1, 1),
				RESOURCE_ID VARCHAR(255),
				ACTION VARCHAR(255) NOT NULL,
				PRIMARY KEY (ID));

CREATE INDEX INDEX_UM_PERMISSIONS_RESOURCE_ID_ACTION
				ON UM_PERMISSIONS (RESOURCE_ID, ACTION);

CREATE TABLE UM_ROLE_PERMISSIONS (
				ID INTEGER IDENTITY(1, 1),
				PERMISSION_ID INTEGER NOT NULL,
				ROLE_ID INTEGER NOT NULL,
				IS_ALLOWED SMALLINT NOT NULL,
				--UNIQUE (PERMISSION_ID, ROLE_ID),
				FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE,
				FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,
				PRIMARY KEY (ID));

CREATE TABLE UM_USER_PERMISSIONS (
				ID INTEGER IDENTITY(1, 1),
				PERMISSION_ID INTEGER NOT NULL,
				USER_ID INTEGER NOT NULL,
				IS_ALLOWED SMALLINT NOT NULL,
				--UNIQUE (PERMISSION_ID, USER_ID),
				FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE,
				FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,
				PRIMARY KEY (ID));

CREATE TABLE UM_USER_ROLES (
				ID INTEGER IDENTITY(1, 1),
				ROLE_ID INTEGER NOT NULL,
				USER_ID INTEGER NOT NULL,
				--UNIQUE (USER_ID, ROLE_ID),
				FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,
				FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,
				PRIMARY KEY (ID));

CREATE TABLE UM_USER_DATA (
				ID INTEGER IDENTITY(1, 1), 
				USER_ID INTEGER NOT NULL,
				EMAIL VARCHAR(255),
				FIRST_NAME VARCHAR(255),
				LAST_NAME VARCHAR(255),
				BIRTH_DATE VARCHAR(255),
				FULL_NAME VARCHAR(255),
				NAME_PREFIX VARCHAR(255),
				GENDER VARCHAR(10),
				TIME_ZONE VARCHAR(255),	
				COMPANY_NAME VARCHAR(255),
				JOB_TITLE VARCHAR(255),
				PRIMARY_PHONE VARCHAR(255),
				HOME_PHONE VARCHAR(255),
				WORK_PHONE VARCHAR(255),
				MOBILE_PHONE VARCHAR(255),
				STREET_ADDRESS VARCHAR(255),
				CITY VARCHAR(255),
				STATE VARCHAR(255),
				COUNTRY VARCHAR(255),
				POSTAL_CODE VARCHAR(255),
				WEB_PAGE VARCHAR(255),
				LANGUAGE VARCHAR(255),
				BLOG VARCHAR(255),			
				PROFILE_ID VARCHAR(255),
				FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID),
				PRIMARY KEY (ID));

CREATE TABLE UM_BIN_DATA (
				ID INTEGER IDENTITY(1, 1),
				USER_ID INTEGER NOT NULL,
				CONTENT_NAME VARCHAR(255) NOT NULL,
				UM_CONTENT VARBINARY(MAX) NOT NULL,
				PROFILE_ID VARCHAR(255) NOT NULL,
				FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID),
				PRIMARY KEY (ID));


CREATE TABLE HYBRID_ROLES (
				ID INTEGER IDENTITY(1, 1),
				ROLE_ID VARCHAR(255) NOT NULL,
				PRIMARY KEY (ID),
				UNIQUE(ROLE_ID));

CREATE TABLE HYBRID_USER_ROLES (
				ID INTEGER IDENTITY(1, 1),
				USER_ID VARCHAR(255),
				ROLE_ID VARCHAR(255) NOT NULL,
				PRIMARY KEY (ID));

CREATE TABLE HYBRID_PERMISSIONS (
				ID INTEGER IDENTITY(1, 1),
				RESOURCE_ID VARCHAR(255),
				ACTION VARCHAR(255) NOT NULL,
				PRIMARY KEY (ID));

CREATE TABLE HYBRID_ROLE_PERMISSIONS (
				ID INTEGER IDENTITY(1, 1),
				PERMISSION_ID INTEGER NOT NULL,
				ROLE_ID VARCHAR(255) NOT NULL,
				IS_ALLOWED SMALLINT NOT NULL,
				UNIQUE (PERMISSION_ID, ROLE_ID),
				FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE  CASCADE,
				PRIMARY KEY (ID));

CREATE TABLE HYBRID_USER_PERMISSIONS (
				ID INTEGER IDENTITY(1, 1),
				PERMISSION_ID INTEGER NOT NULL,
				USER_ID VARCHAR(255) NOT NULL,
				IS_ALLOWED SMALLINT NOT NULL,
				UNIQUE (PERMISSION_ID, USER_ID),
				FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE CASCADE,
				PRIMARY KEY (ID));