package org.wso2.carbon.server.admin.ui;

import org.apache.axis2.AxisFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.server.admin.ui.types.carbon.ServerData;

import javax.servlet.http.HttpSession;
import java.lang.Exception;
import java.rmi.RemoteException;

public class ServerAdminClient {

    private static final Log log = LogFactory.getLog(ServerAdminClient.class);

    private ServerAdminStub serverAdminStub;
    private HttpSession session;

    public ServerAdminClient(ConfigurationContext ctx, String serverURL,
                             String cookie, HttpSession session) throws AxisFault {
        this.session = session;
        String serviceEPR = serverURL + "ServerAdmin";
        serverAdminStub = new ServerAdminStub(ctx, serviceEPR);
        ServiceClient client = serverAdminStub._getServiceClient();
        Options options = client.getOptions();
        options.setManageSession(true);
        options.setProperty(org.apache.axis2.transport.http.HTTPConstants.COOKIE_STRING, cookie);
        options.setTimeOutInMilliSeconds(10000);
    }

    public String restart() throws Exception {
        try {
            serverAdminStub.restart();
            invalidateSession();
        } catch (Exception e) {
            if (e.getMessage().indexOf("cannot.restart.derby.embedded.on.windows") != -1){
                return "cannot.restart.derby.embedded.on.windows";
            }
            String msg = "Cannot restart the server. Backend service may be unavailable";
            log.error(msg, e);
            throw new Exception(msg, e);
        }
        return null;
    }

    public String restartGracefully() throws Exception {
        try {
            serverAdminStub.restartGracefully();
            invalidateSession();
        } catch (Exception e) {
            if (e.getMessage().indexOf("cannot.restart.derby.embedded.on.windows") != -1){
                return "cannot.restart.derby.embedded.on.windows";
            }
            String msg = "Cannot restart the server. Backend service may be unavailable";
            log.error(msg, e);
            throw new Exception(msg, e);
        }
        return null;
    }

    public void shutdown() throws Exception {
        try {
            serverAdminStub.shutdown();
            invalidateSession();
        } catch (RemoteException e) {
            String msg = "Cannot shutdown the server. Backend service may be unavailable";
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public void shutdownGracefully() throws Exception {
        try {
            serverAdminStub.shutdownGracefully();
            invalidateSession();
        } catch (RemoteException e) {
            String msg = "Cannot shutdown the server. Backend service may be unavailable";
            log.error(msg, e);
            throw new Exception(msg, e);
        }
    }

    public ServerData getServerData() throws Exception {
        ServerData data;
        try {
            data = serverAdminStub.getServerData();
        } catch (RemoteException e) {
            String msg = "Cannot get server data. Backend service may be unavailable";
            log.error(msg, e);
            throw new Exception(msg, e);
        }
        return data;
    }

    private void invalidateSession() {
        try {
            session.invalidate();
        } catch (Exception ignored) { // Ignore invalidation of invalidated sessions
        }
    }

}
