package org.wso2.carbon.server.admin.internal;

import org.apache.axis2.AxisFault;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.service.component.ComponentContext;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.server.admin.service.ServerAdmin;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.utils.ConfigurationContextService;
import org.wso2.carbon.utils.MBeanRegistrar;
import org.wso2.carbon.utils.ServerConfiguration;
import org.eclipse.osgi.framework.console.CommandProvider;

/**
 * @scr.component name="serveradmin.service.component"" immediate="true"
 * @scr.reference name="userrealm_default"
 * interface="org.wso2.carbon.user.core.UserRealm"
 * cardinality="1..1" policy="dynamic" target="(RealmGenre=Default)"
 * bind="setUserRealmDefault" unbind="unsetUserRealmDefault"
 * @scr.reference name="registry.service"
 * interface="org.wso2.carbon.registry.core.service.RegistryService"
 * cardinality="1..1" policy="dynamic"  bind="setRegistryService" unbind="unsetRegistryService"
 * @scr.reference name="config.context.service"
 * interface="org.wso2.carbon.utils.ConfigurationContextService"
 * cardinality="1..1" policy="dynamic"  bind="setConfigurationContextService" unbind="unsetConfigurationContextService"
 * @scr.reference name="server.configuration"
 * interface="org.wso2.carbon.utils.ServerConfiguration"
 * cardinality="1..1" policy="dynamic" bind="setServerConfiguration" unbind="unsetServerConfiguration"
 * @scr.reference name="userrealm_delegating"
 * interface="org.wso2.carbon.user.core.UserRealm"
 * cardinality="0..1" policy="dynamic" target="(RealmGenre=Delegating)"
 * bind="setUserRealmDelegating" unbind="unsetUserRealmDelegating"
 */
public class ServerAdminServiceComponent {

    private static final Log log = LogFactory.getLog(ServerAdminServiceComponent.class);

    private static String registryDriver;
    private static String usermanagerDriver;
    private boolean registeredMBeans;
    public static final String SERVER_ADMIN_MODULE_NAME = "ServerAdminModule";
    private static ConfigurationContext configContext;
    private static ServerConfiguration serverConfig;
    private RegistryService registryService;
    private UserRealm userRealmDefault;
    private static UserRealm userRealmDelegating;

    protected void activate(ComponentContext ctxt) {
        try {
            //Register ServerAdmin MBean
            registerMBeans(serverConfig);
            // Engaging ServerAdmin as an global module
            configContext.getAxisConfiguration().engageModule(SERVER_ADMIN_MODULE_NAME);
            setUserManagerDriver(userRealmDefault);
            setRegistryDriver(registryService);
            
            ServerAdmin.classloader = Thread.currentThread().getContextClassLoader();

            ctxt.getBundleContext().registerService(CommandProvider.class.getName(),
                                                    new ServerAdminCommandProvider(),
                                                    null);
            log.debug("ServerAdmin bundle is activated");
        } catch (Throwable e) {
            log.error("Failed to activate ServerAdmin bundle", e);
        }
    }

    protected void deactivate(ComponentContext ctxt) {
        log.debug("ServerAdmin bundle is deactivated");
    }

    private void registerMBeans(ServerConfiguration serverConfig) {
        if (serverConfig.getFirstProperty("Ports.JMX") != null) {
            if (registeredMBeans) return;
            MBeanRegistrar.registerMBean(new ServerAdmin());
            registeredMBeans = true;
        }
    }

    private void setUserManagerDriver(UserRealm realm) {
        try {
            DefaultRealmConfig config = (DefaultRealmConfig) realm.getRealmConfiguration();
            usermanagerDriver = config.getDriverName();
        } catch (UserStoreException e) {
            String msg = "Cannot get user store driver";
            log.error(msg, e);
        }
    }

    private void setRegistryDriver(RegistryService registry) {
        try {
            if (registry.getSystemRegistry().getRegistryContext() != null) {
                registryDriver = registry.getSystemRegistry().getRegistryContext().getDataSource().
                        getConnection().getMetaData().getDriverName();
            }

        } catch (Exception e) {
            String msg = "Cannot get registry driver";
            log.error(msg, e);
        }
    }

    public static String getRegistryDatabaseDriver() {
        return registryDriver;
    }

    public static String getUserManagerDriver() {
        return usermanagerDriver;
    }

    public static ConfigurationContext getConfigContext() {
        return configContext;
    }

    public static AxisConfiguration getAxisConfig(){
        if(configContext != null){
            return configContext.getAxisConfiguration();
        }
        return null;
    }

    protected void setUserRealmDefault(UserRealm userRealmDefault) {
        this.userRealmDefault = userRealmDefault;
    }

    protected void unsetUserRealmDefault(UserRealm userRealmDefault) {
        this.userRealmDefault = null;
        usermanagerDriver = null;
    }

    public static UserRealm getUserRealmDelegating() {
        return userRealmDelegating;
    }

    protected void setUserRealmDelegating(UserRealm userRealmDele) {
        userRealmDelegating = userRealmDele;
    }

    protected void unsetUserRealmDelegating(UserRealm userRealmDele) {
        userRealmDelegating = null;
    }

    protected void setConfigurationContextService(ConfigurationContextService contextService) {
        configContext = contextService.getServerConfigContext();
    }

    protected void unsetConfigurationContextService(ConfigurationContextService contextService) {
        AxisConfiguration axisConf = configContext.getAxisConfiguration();
        AxisModule statModule = axisConf.getModule(ServerAdminServiceComponent.SERVER_ADMIN_MODULE_NAME);
        if (statModule != null) {
            try {
                axisConf.disengageModule(statModule);
            } catch (AxisFault axisFault) {
                log.error("Failed disengage module: " + ServerAdminServiceComponent.SERVER_ADMIN_MODULE_NAME);
            }
        }
        configContext = null;
    }

    protected void setServerConfiguration(ServerConfiguration serverConfiguration) {
        serverConfig = serverConfiguration;
    }

    protected void unsetServerConfiguration(ServerConfiguration serverConfiguration) {
        serverConfig = null;
    }

    protected void setRegistryService(RegistryService registryService) {
        this.registryService = registryService;
    }

    protected void unsetRegistryService(RegistryService registryService) {
        this.registryService = null;
        registryDriver = null;
    }

    public static ServerConfiguration getServerConfig() {
        return serverConfig;
    }
}
