/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.server.admin.module.handler;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axis2.AxisFault;
import org.apache.axis2.clustering.ClusteringConstants;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.handlers.AbstractHandler;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.utils.ServerConstants;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.xml.namespace.QName;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 *
 */
public class AuthenticationHandler extends AbstractHandler {
    private static final Log log = LogFactory.getLog(AuthenticationHandler.class);
    private MessageContext msgContext;

    public InvocationResponse invoke(MessageContext messageContext) throws AxisFault {
        this.msgContext = messageContext;
        authenticate((String) messageContext.getProperty(MessageContext.REMOTE_ADDR));
        return InvocationResponse.CONTINUE;
    }

    protected void authenticate(String remoteIP) throws AxisFault {
        if (callToGeneralService()) {
            return;
        }
        try {
            if (!isAuthenticated(remoteIP)) {
                throw new AxisFault("Access Denied. Please login first.",
                        ServerConstants.AUTHORIZATION_FAULT_CODE);
            }
        } catch (AuthenticationException e) {
            throw new AxisFault("Access Denied. Session timed out.",
                    ServerConstants.AUTHORIZATION_FAULT_CODE);
        } catch (Throwable e) {
            log.error(e.getMessage(), e);
            throw new AxisFault("Authentication failure");
        }
    }

    private boolean isAuthenticated(String remoteIP) throws AuthenticationException {
        AxisService service = msgContext.getAxisService();
        AxisOperation operation = msgContext.getAxisOperation();

        String opName = operation.getName().getLocalPart();
        String serviceName = service.getName();

        Parameter authenticationParameter = operation.getParameter("DoAuthentication");

        if (authenticationParameter != null && "false".equals(authenticationParameter.getValue())) {
            return true;
        }

        HttpServletRequest request = (HttpServletRequest) msgContext
                .getProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST);

        HttpSession httpSession = request.getSession();

        if ((serviceName.equals(ServerConstants.AUTHENTICATION_ADMIN_SERVICE) || serviceName
                .equals(ClusteringConstants.NODE_MANAGER_SERVICE))) {
            if (opName.equals("login") || opName.equals("loginWithDelegation")) {
                SOAPBody soapBody = msgContext.getEnvelope().getBody();

                OMElement usernameElem = soapBody.getFirstElement().getFirstChildWithName(
                        new QName(ServerConstants.AUTHENTICATION_SERVICE_NS,
                                ServerConstants.AUTHENTICATION_SERVICE_USERNAME));
                if (usernameElem != null) {
                    String userName = usernameElem.getText();
                    httpSession.setAttribute(ServerConstants.ADMIN_USERNAME, userName);
                    return true;
                }
            } else if (opName.equals("logout")) {
                return true;
            }
        }

        String loginStatus = (String) httpSession.getAttribute(ServerConstants.ADMIN_LOGGED_IN);

        if (loginStatus == null) {
            throw new AuthenticationException();
        }

        boolean isAuthenticated = "true".equals(loginStatus);
        if (!isAuthenticated) {
            SimpleDateFormat date = new SimpleDateFormat("'['yyyy-MM-dd HH:mm:ss,SSSS']'");
            log.warn("Illegal access attempt at " + date.format(new Date()) + " from IP address "
                    + remoteIP + " : Service is " + serviceName);
        }
        // return true;
        return isAuthenticated;
    }

    private boolean callToGeneralService() {
        boolean isGeneral = true;
        AxisService service = msgContext.getAxisService();
        Parameter param = service.getParameter("adminService");
        if (param != null && "true".equals(param.getValue())) {
            isGeneral = false;
        }
        return isGeneral;
    }

}
