package org.wso2.carbon.server.admin.module.handler;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.handlers.AbstractHandler;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.utils.ServerConstants;
import org.wso2.carbon.server.admin.internal.ServerAdminServiceComponent;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

public class AuthorizationHandler extends AbstractHandler {

    private static Log log = LogFactory.getLog(AuthorizationHandler.class.getClass());

    public static final String AUTHZ_FAULT_CODE = "WSO2CarbonAuthorizationFailure";

    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {

        if (this.callToGeneralService(msgContext)) {
            return InvocationResponse.CONTINUE;
        }

        AxisService service = msgContext.getAxisService();
        AxisOperation operation = msgContext.getAxisOperation();

        String opName = operation.getName().getLocalPart();

        if (service.getName().equals("AuthenticationAdminService")
                && (opName.equals("login") || opName.equals("loginWithDelegation") || opName
                        .equals("logout"))) {
            return InvocationResponse.CONTINUE;
        }

        Parameter actionParam = operation.getParameter("AuthorizationAction");
        Parameter resourceParam = operation.getParameter("AuthorizationResource");
        Parameter authorizationParameter = operation.getParameter("DoAuthorization");

        String serviceName = service.getName();
        String userName = null;
        if (authorizationParameter != null && "false".equals(authorizationParameter.getValue())) {
            return InvocationResponse.CONTINUE;
        }

        try {
            if (actionParam != null) {
                String action = (String) actionParam.getValue();
                String resourceId = "System";
                if (resourceParam != null) {
                    resourceId = (String) resourceParam.getValue();
                }

                HttpServletRequest request = (HttpServletRequest) msgContext
                        .getProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST);

                HttpSession httpSession = request.getSession(false);
                if (httpSession != null) {
                    userName = (String) httpSession.getAttribute(ServerConstants.ADMIN_USERNAME);
                    UserRealm userRealm = ServerAdminServiceComponent.getUserRealmDelegating();
                    if (userRealm != null
                            && !userRealm.getAuthorizer().isUserAuthorized(userName, resourceId,
                                    action)) {
                        AxisFault afault = new AxisFault("Access Denied. You are not authorized.");
                        afault.setFaultCode(AUTHZ_FAULT_CODE);
                        throw afault;
                    }
                }
            }

        } catch (AxisFault e) {
            log.error("Access Denied. Failed authorization attempt to access service '"
                    + serviceName + "' operation '" + opName + "' by '" + userName + "'", e);
            throw e;
        } catch (Exception e) {
            String msg = "Exception occurred while trying to authorize. " + e.getMessage();
            log.error("System failure while authorizaing the user '" + userName
                    + "' to the service '" + serviceName + "' operation '" + opName + "'", e);
            throw new AxisFault(msg, ServerConstants.AUTHORIZATION_FAULT_CODE);
        } catch (Throwable e) {
            String msg = "Exception occurred while trying to authorize.";
            log.error(msg, e);
            throw new AxisFault(msg, ServerConstants.AUTHORIZATION_FAULT_CODE);
        }

        return InvocationResponse.CONTINUE;
    }

    private boolean callToGeneralService(MessageContext msgContext) {
        boolean isGeneral = true;
        AxisService service = msgContext.getAxisService();
        Parameter param = service.getParameter("adminService");
        if (param != null && "true".equals(param.getValue())) {
            isGeneral = false;
        }
        return isGeneral;
    }
}
