/*                                                                             
 * Copyright 2004,2005 The Apache Software Foundation.                         
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.carbon.server;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.utils.ServerConstants;
import org.wso2.carbon.CarbonConstants;

import java.io.File;

/**
 *
 */
public class RepositoryMetadataWriter {
    private static final Log log = LogFactory.getLog(RepositoryMetadataWriter.class);

    public static Thread shutdownHookThread;
    private TomcatServer tomcatServer;

    private static final String CARBON_HOME = "carbon.home";
    private static final String AXIS2_HOME = "axis2.home";

    public RepositoryMetadataWriter() {
        if (System.getProperty(CARBON_HOME) == null) {
            System.setProperty(CARBON_HOME, ".");
        }
        System.setProperty(AXIS2_HOME, System.getProperty(CARBON_HOME));
    }


    public void startServer() {
        try {
            tomcatServer = new TomcatServer();
            tomcatServer.start();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void shutdown() throws Exception {
        tomcatServer.stop();
    }

    public static void main(String[] args) {
        System.setProperty(CarbonConstants.START_TIME, System.currentTimeMillis() + "");
        String cmd = null;

        // Set the System properties
        for (String arg : args) {
            if (arg.startsWith("-D")) {
                int indexOfEq = arg.indexOf("=");
                String property;
                String value;
                if (indexOfEq != -1) {
                    property = arg.substring(2, indexOfEq);
                    value = arg.substring(indexOfEq + 1);
                } else {
                    property = arg.substring(2);
                    value = "true";
                }
                System.setProperty(property, value);
            } else {
                if (cmd == null) {
                    cmd = arg;
                }
            }
        }

        if(System.getProperty("carbon.axis2.repo") == null){
            System.err.println("Please specify the Axis2 repository using the -Dcarbon.axis2.repo argument");
            printUsages();
            System.exit(1);
        }
        System.setProperty(ServerConstants.REPO_WRITE_MODE, "true");
        final RepositoryMetadataWriter main = new RepositoryMetadataWriter();

        log.info("Starting Repository Metadata Writer");
        log.info("Using Java Home        : " + System.getProperty("java.home"));
        log.info("Using Java Version     : " + System.getProperty("java.version"));
        String carbonHome;
        if ((carbonHome = System.getProperty("carbon.home")).equals(".")) {
            carbonHome = new File(".").getAbsolutePath();
        }
        log.info("Using Carbon Home      : " + carbonHome);

        main.startServer();
    }

    private static void printUsages() {
        String osName = System.getProperty("os.name");
        System.out.println("Deploy services & modules in an Axis2 repository and write the metadata to the registry");
        System.out.println("Usage: repowriter." + ((osName.toLowerCase().indexOf("win") == -1) ? "sh" : "bat ") +
                           " -Dcarbon.axis2.repo=<repo>");
    }
}
