package org.wso2.carbon.ui;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.Bundle;
import org.osgi.framework.ServiceReference;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.ui.deployment.beans.CarbonUIDefinitions;
import org.wso2.carbon.CarbonConstants;

public class CarbonSecuredHttpContext extends SecuredComponentEntryHttpContext {

    public static String LOGGED_USER = "logged-user";
    private static Log log = LogFactory.getLog(CarbonSecuredHttpContext.class);
    private Bundle bundle = null;
    protected BundleResourcePathRegistry resourcePathRegistry;

    public CarbonSecuredHttpContext(Bundle bundle, String s,
                                    BundleResourcePathRegistry resourcePathRegistry,
                                    Registry registry) {
        super(bundle, s, resourcePathRegistry);
        this.registry = registry;
        this.bundle = bundle;
    }

    public boolean handleSecurity(HttpServletRequest request,
                                  HttpServletResponse response) throws IOException {
        String requestedURI = request.getRequestURI();
        HttpSession session;
        String sessionId;
        boolean authenticated = false;
        try {
            session = request.getSession();
            sessionId = session.getId();
            Boolean authenticatedObj = (Boolean) session.getAttribute("authenticated");
            if (authenticatedObj != null) {
                authenticated = authenticatedObj.booleanValue();
            }
        } catch (Exception e) {
            log.error("Error occurred while checking security", e);
            return false;
        }

        String context = request.getContextPath();
        if ("".equals(context)) {
            context = "/";
        }

        if (requestedURI.equals(context) || requestedURI.equals(context + "/")) {
            PrintWriter pw = response.getWriter();
            pw.println("<html>");
            pw.println("<body>");
            pw.println("<b>Available sub-contexts</b>");
            if ("/".equals(context)) {
                pw.println("<li><a href=\"./carbon\">carbon</a></li>");
            } else {
                pw.println("<li><a href=\"" + context + "/carbon\">carbon</a></li>");
            }
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
            return false;
        }
        //when war is deployed on top of an existing app server we cannot use root context
        //for deployment. Hence a new context is added.
        //Now url changes from eg: carbon/admin/index.jsp to wso2/carbon/admin/index.jsp
        //In this case before doing anything, we need to remove web app context (eg: wso2) .       
        String tmp = requestedURI;
        String customWarContext = "";
        if (requestedURI.startsWith("/carbon")
                && !(requestedURI.startsWith("/carbon/carbon/"))) {
            //one can name the folder as 'carbon'
            requestedURI = tmp;
        } else if (requestedURI.indexOf("filedownload") == -1
                && requestedURI.indexOf("fileupload") == -1) {
            //replace first context
            String tmp1 = tmp.replaceFirst("/", "");
            int end = tmp1.indexOf("/");
            if (end > -1) {
                customWarContext = tmp1.substring(0, end);
                //one can rename the war file as 'registry'.
                //This will conflict with our internal 'registry' context
                if (!(requestedURI.startsWith("/registry/registry/"))
                        && !(requestedURI.startsWith("/registry/carbon/"))
                        && (customWarContext.equals("registry")
                        || customWarContext.equals("gadgets"))) {
                    requestedURI = tmp;
                } else {
                    requestedURI = tmp.substring(end + 1);
                }
            }
        }

        if (Utils.isDeveloperMode())
        	return true;
        //Disabling http access to admin console if it is not in developer mode
        //user guide documents should be allowed to access via http protocol
        if (!request.isSecure()
                && !(requestedURI.endsWith(".html"))) {

            //by passing items required for try-it & IDE plugins
            if (requestedURI.endsWith(".css") ||
                    requestedURI.endsWith(".gif") ||
                    requestedURI.endsWith(".GIF") ||
                    requestedURI.endsWith(".jpg") ||
                    requestedURI.endsWith(".JPG") ||
                    requestedURI.endsWith(".png") ||
                    requestedURI.endsWith(".PNG") ||
                    requestedURI.endsWith(".xsl") ||
                    requestedURI.endsWith(".xslt") ||
                    requestedURI.endsWith(".js") ||
                    requestedURI.endsWith(".ico") ||
                    requestedURI.endsWith("/filedownload") ||
                    requestedURI.endsWith("/fileupload") ||
                    requestedURI.indexOf("/fileupload/") > -1 ||
                    requestedURI.indexOf("admin/jsp/WSRequestXSSproxy_ajaxprocessor.jsp") > -1 ||
                    requestedURI.indexOf("registry/atom") > -1 ||
                    requestedURI.indexOf("gadgets/") > -1 ||
                    requestedURI.indexOf("registry/resource") > -1) {
                return true;
            }

            ServletContext servletContext = session.getServletContext();
            String adminConsoleURL = CarbonUIUtil.getAdminConsoleURL(request);
            if (adminConsoleURL != null) {
                if (log.isTraceEnabled()) {
                    log.trace("Request came to admin console via http.Forwarding to : " +
                            adminConsoleURL);
                }
                response.sendRedirect(adminConsoleURL);
                return false;
            }
        }

        //remove web application context & 'carbon' sub context from uri
        String resourceURI = requestedURI.replaceFirst("/carbon/", "../");

        if (log.isDebugEnabled()) {
            log.debug("CarbonSecuredHttpContext -> handleSecurity() requestURI:" + requestedURI +
                    " id:" + sessionId + " resourceURI:" + resourceURI);
        }

        //retrieve urls that should be by-passed from security check
        HashMap<String, String> urlsToBeByPassed = new HashMap<String, String>();
        if (bundle != null) {
            ServiceReference reference = bundle.getBundleContext()
                    .getServiceReference(CarbonUIDefinitions.class.getName());
            CarbonUIDefinitions carbonUIDefinitions = null;
            if (reference != null) {
                carbonUIDefinitions =
                        (CarbonUIDefinitions) bundle.getBundleContext().getService(reference);
                if (carbonUIDefinitions != null) {
                    urlsToBeByPassed = carbonUIDefinitions.getUnauthenticatedUrls();
                    /*
                         if(log.isDebugEnabled()){
                             Iterator<String> itr = urlsToBeByPassed.keySet().iterator();
                             while(itr.hasNext()){
                                 log.debug("Should bypass url : "+itr.next());
                             }
                         }
                         */
                }
            }
        }

        //if the current uri is marked to be by-passed, let it pass through
        if (!urlsToBeByPassed.isEmpty()) {
            if (urlsToBeByPassed.containsKey(resourceURI)) {
                if (log.isDebugEnabled()) {
                    log.debug("By passing authentication check for URI : " + resourceURI);
                }

                // Before bypassing, set the backendURL properly so that it doesn't fail
                String backendServerURL = request.getParameter("backendURL");
                if (backendServerURL == null) {
                    backendServerURL = CarbonUIUtil
                            .getServerURL(session.getServletContext(), request.getSession());
                }
                session.setAttribute(CarbonConstants.SERVER_URL, backendServerURL);
                return true;
            }
        }

        if (requestedURI.indexOf("login.jsp") > -1
                || requestedURI.indexOf("admin/layout/template.jsp") > -1
                || requestedURI.endsWith("/filedownload")
                || requestedURI.endsWith("/fileupload")
                || requestedURI.indexOf("/fileupload/") > -1
                || requestedURI.indexOf("admin/jsp/WSRequestXSSproxy_ajaxprocessor.jsp") > -1) {

            if (requestedURI.indexOf("login.jsp") > -1 && authenticated) {
                //User has typed the login page url, while being logged in
                response.sendRedirect("/carbon/admin/index.jsp");
                return true;
            } else {
                return true;
            }
        }
        boolean sessionAuth = false;

        if (requestedURI.indexOf("login_action.jsp") > -1) {
            try {
                sessionAuth = getAuthenticator(request).authenticate();
                if (sessionAuth) {
                    session.setAttribute("authenticated", sessionAuth);

                    // TODO: I'm commenting out code which redirects to the previously requested
                    // path
                    // this bit seems to be buggy, often returning error pages.
                    /*
                          * String previousRequestUri = (String) session.getAttribute("requestedUri"); if
                          * (previousRequestUri != null && previousRequestUri.trim().length() > 0) { if
                          * (log.isTraceEnabled()) { log.trace("Forwarding to previously request page : "
                          * + previousRequestUri); } response.sendRedirect(previousRequestUri); } else {
                          * response.sendRedirect("../../carbon/admin/index.jsp?loginStatus=true"); }
                          */

                    response.sendRedirect("../../carbon/admin/index.jsp?loginStatus=true");
                    session.setAttribute(LOGGED_USER,
                                         getAuthenticator(request).getAuthenticatedUser());
                    session.getServletContext().setAttribute(LOGGED_USER, getAuthenticator(
                            request).getAuthenticatedUser());
                    // remove menu items which are no longer required after user being signed in.
                    CarbonUIUtil.removeMenuDefinitionOnSignIn(request);
                } else {
                    response.sendRedirect("../../carbon/admin/login.jsp?loginStatus=false");
                }

            } catch (Exception e) {
                log.error("error occurred while login", e);
                response.sendRedirect("../../carbon/admin/login.jsp?loginStatus=failed");
            }
            return false;
        } else if (requestedURI.indexOf("logout_action.jsp") > 1) {
            // Logout the user from the back-end
            getAuthenticator(request).unauthenticate();
            HttpSession currentSession = request.getSession(false);
            if (currentSession != null) {
                //check if current session has expired
                session.removeAttribute(LOGGED_USER);
                session.getServletContext().removeAttribute(LOGGED_USER);
                try {
                    session.invalidate();
                } catch (Exception ignored) { // Ignore exception when invalidating and invalidated session
                }
            }
            response.sendRedirect("../../carbon/admin/index.jsp");
            return false;
        }
        if (requestedURI.endsWith("/carbon/")) {
            response.sendRedirect("./admin/index.jsp");
            return false;
        } else
        if (requestedURI.indexOf("/registry/atom") == -1 && requestedURI.endsWith("/carbon")) {
            String redirectUrl = "/carbon/admin/index.jsp";
            if (customWarContext != "" && customWarContext.trim().length() > 0) {
                redirectUrl = "/" + customWarContext + redirectUrl;
            }
            response.sendRedirect(redirectUrl);
            return false;
        } else if (requestedURI.endsWith(".css") ||
                requestedURI.endsWith(".gif") ||
                requestedURI.endsWith(".GIF") ||
                requestedURI.endsWith(".jpg") ||
                requestedURI.endsWith(".JPG") ||
                requestedURI.endsWith(".png") ||
                requestedURI.endsWith(".PNG") ||
                requestedURI.endsWith(".xsl") ||
                requestedURI.endsWith(".xslt") ||
                requestedURI.endsWith(".js") ||
                requestedURI.startsWith("/registry") ||
                requestedURI.endsWith(".html") ||
                requestedURI.endsWith(".ico") ||
                tmp.startsWith("/openid/") || //This is ugly - till we find a solution
                requestedURI.indexOf("/openid/") > -1 ||
                requestedURI.indexOf("/openidserver") > -1 ||
                requestedURI.indexOf("/passivests") > -1 ||
                requestedURI.indexOf("/gadgets") > -1) {
            return true;
        } else if (requestedURI.endsWith(".jsp") && authenticated) {
            return true;
        }

        String contextPath = (request.getContextPath().equals("")) ? "" : request.getContextPath();
        if (!authenticated) {
            if (requestedURI.endsWith(
                    "ajaxprocessor.jsp")) {  //To prevent login page appearing when ajax calls happen
                return true;
            } else {
                //saving originally requested url
                //should not forward to error page after login
                if (!requestedURI.endsWith("admin/error.jsp")) {
                    String queryString = request.getQueryString();
                    String tmpURI = "";
                    if (queryString != null) {
                        tmpURI = requestedURI + "?" + queryString;
                    } else {
                        tmpURI = requestedURI;
                    }
                    tmpURI = "../.." + tmpURI;
                    request.getSession(false).setAttribute("requestedUri", tmpURI);
                }
                response.sendRedirect(contextPath + "/carbon/admin/login.jsp");
            }
            return false;
        }

        if (request.getSession().isNew()) {
            response.sendRedirect(contextPath + "/carbon/admin/login.jsp");
            return false;
        }

        return true;
    }
}
