package org.wso2.carbon.ui;

import java.rmi.RemoteException;
import java.util.ArrayList;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.axis2.AxisFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.ui.authentication.AuthenticationAdminServiceStub;
import org.wso2.carbon.ui.authentication.GetSystemPermissionOfUser;
import org.wso2.carbon.ui.authentication.GetSystemPermissionOfUserResponse;
import org.wso2.carbon.ui.authentication.Login;
import org.wso2.carbon.ui.authentication.LoginResponse;
import org.wso2.carbon.utils.ServerConstants;

public class DefaultCarbonAuthenticator extends CarbonAuthenticator {

	private HttpServletRequest request = null;
	protected static final Log log = LogFactory.getLog(DefaultCarbonAuthenticator.class);
	protected static final String AUTHENTICATION_ADMIN_SERVICE = "AuthenticationAdminService";

	public DefaultCarbonAuthenticator(HttpServletRequest request) {
		super(request);
		this.request = request;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean authenticate() throws RemoteException {
		String userName = null;
		String password = null;

		userName = request.getParameter("username");
		password = request.getParameter("password");
		return authenticate(userName, password);
	}

	/**
	 * 
	 * @param request
	 */
	public void unauthenticate() {
		ServletContext servletContext = request.getSession().getServletContext();
		ConfigurationContext configContext = (ConfigurationContext) servletContext
				.getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

		String backendServerURL = CarbonUIUtil.getServerURL(servletContext, request.getSession());
		if (backendServerURL == null) {
			return;
		}
		String serviceURL = backendServerURL + AUTHENTICATION_ADMIN_SERVICE;
		try {
			AuthenticationAdminServiceStub stub = new AuthenticationAdminServiceStub(configContext,
					serviceURL);
			Options option = stub._getServiceClient().getOptions();
			String cookie = (String) request.getSession().getAttribute(
					ServerConstants.ADMIN_SERVICE_COOKIE);
			option.setProperty(HTTPConstants.COOKIE_STRING, cookie);
			option.setManageSession(true);
			stub.logout();
		} catch (RemoteException ignored) {
		} catch (Exception ignored) {
		}
	}

	/**
	 * 
	 * @param userName
	 * @param password
	 * @return
	 * @throws RemoteException
	 */
	protected boolean authenticate(String userName, String password) throws RemoteException {
		ServletContext servletContext = null;
		ConfigurationContext configContext = null;

		try {

			servletContext = request.getSession().getServletContext();
			configContext = (ConfigurationContext) servletContext
					.getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

			if (configContext == null) {
				log.error("Configuration context is null.");
			}
			// Obtain the back-end server URL from the request. If not obtain it from the http
			// session and then from the ServletContext.
            HttpSession session = request.getSession();
            String backendServerURL = request.getParameter("backendURL");
            if (backendServerURL == null) {
                backendServerURL = CarbonUIUtil.getServerURL(servletContext, request.getSession());
            }

			// Back-end server URL is stored in the session, even if it is an incorrect one. This
			// value will be displayed in the server URL text box. Usability improvement.
			session.setAttribute(CarbonConstants.SERVER_URL, backendServerURL);

			String serviceURL = backendServerURL + AUTHENTICATION_ADMIN_SERVICE;
			AuthenticationAdminServiceStub stub = new AuthenticationAdminServiceStub(configContext,
					serviceURL);
			Login loginRequest = new Login();
			loginRequest.setUsername(userName);
			loginRequest.setPassword(password);
			loginRequest.setRemoteAddress(request.getRemoteAddr());
			Options option = stub._getServiceClient().getOptions();
			option.setManageSession(true);
			LoginResponse loginResponse = stub.login(loginRequest);

			boolean isLogged = loginResponse.get_return();
			if (isLogged) {
				setUserPermissions(userName, request, stub);
			}
			return isLogged;

		} catch (AxisFault axisFault) {
			throw axisFault;
		} catch (RemoteException e) {
			throw e;
		} catch (Exception e) {
			throw new AxisFault("Exception occured", e);
		}
	}

	/**
	 * 
	 * @param userName
	 * @param request
	 * @param stub
	 * @throws RemoteException
	 */
	protected void setUserPermissions(String userName, HttpServletRequest request,
			AuthenticationAdminServiceStub stub) throws RemoteException {
		HttpSession session = null;
		GetSystemPermissionOfUser getPermissionReq = null;
		try {
			session = request.getSession();
			getPermissionReq = new GetSystemPermissionOfUser();
			getPermissionReq.setUsername(userName);
			GetSystemPermissionOfUserResponse permissions = stub
					.getSystemPermissionOfUser(getPermissionReq);
			String[] permissionArray = permissions.get_return();

			ArrayList<String> list = new ArrayList<String>();
			for (String permission : permissionArray) {
				list.add(permission);
			}
			session.setAttribute(ServerConstants.USER_PERMISSIONS, list);
			String cookie = (String) stub._getServiceClient().getServiceContext().getProperty(
					HTTPConstants.COOKIE_STRING);
			session.setAttribute(ServerConstants.ADMIN_SERVICE_COOKIE, cookie);
		} catch (AxisFault e) {
			throw e;
		} catch (RemoteException e) {
			throw e;
		} catch (Exception e) {
			throw new AxisFault("Exception occured", e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public String getAuthenticatedUser() {
		return request.getParameter("username");
	}

}
