/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.ui;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.equinox.http.helper.BundleEntryHttpContext;
import org.osgi.framework.Bundle;
import org.wso2.carbon.registry.core.Registry;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

public class SecuredComponentEntryHttpContext extends BundleEntryHttpContext {

	private String bundlePath;

	protected Registry registry;

    protected BundleResourcePathRegistry resourcePathRegistry;

    protected Log log = LogFactory.getLog(this.getClass());

	public SecuredComponentEntryHttpContext(Bundle bundle) {
		super(bundle);
	}

	public SecuredComponentEntryHttpContext(Bundle bundle, String bundlePath, BundleResourcePathRegistry resourcePathRegistry) {
		super(bundle, bundlePath);
		this.bundlePath = bundlePath;
        this.resourcePathRegistry = resourcePathRegistry;
    }

	public SecuredComponentEntryHttpContext(Bundle bundle, String s, Registry registry) {
		super(bundle, s);
		this.registry = registry;
    }

    public URL getResource(String resourceName) {
        URL url = super.getResource(resourceName);
        if (url != null)
            return url;

        //Searching the resourceBundle for the given bundle resource paths.        
        String resourcePath = CarbonUIUtil.getBundleResourcePath(resourceName);
        Bundle resourceBundle = resourcePathRegistry.getBundle(resourcePath);
        url = (resourceBundle != null) ? getResource(resourceName, resourceBundle) : null;

        if (url != null) {
            return url;
        } else {
            if (bundlePath != null) {
                resourceName = bundlePath + resourceName;
            }
            int lastSlash = resourceName.lastIndexOf('/');
            if (lastSlash == -1) {
                return null;
            }
            String file = resourceName.substring(lastSlash + 1);
            if (file.endsWith(".js") && isListed(file)) {
                try {
                    return new URL("carbon://generate/" + file);
                } catch (MalformedURLException e) {
                    return null;
                }
            }
            return null;
        }
    }

    public URL getResource(String resourceName, Bundle resourceBundle) {
		if (bundlePath != null)
			resourceName = bundlePath + resourceName;

		int lastSlash = resourceName.lastIndexOf('/');
		if (lastSlash == -1)
			return null;

		String path = resourceName.substring(0, lastSlash);
		if (path.length() == 0)
			path = "/"; //$NON-NLS-1$
		String file = resourceName.substring(lastSlash + 1);
		Enumeration entryPaths = resourceBundle.findEntries(path, file, false);

		if (entryPaths != null && entryPaths.hasMoreElements())
			return (URL) entryPaths.nextElement();

		return null;
	}

    /**
	 * Returns the corresponding CarbonAuthenticator based on the request.
	 *
	 * @param request
	 * @return
	 */
	protected CarbonAuthenticator getAuthenticator(HttpServletRequest request) {
		String password = request.getParameter("password");
		String ssoSessionId = request.getParameter(CarbonSSOAuthenticator.SSO_SESSION_ID);
		String ivuser = request.getHeader(WebSealAuthenticator.WEBSEAL_USER);

		if (ssoSessionId != null && password == null) {
			// This is SSO related login - we don't have the password.
			// Now the back-end service need to validate ssoSession id with back-end
			// SSO-Service.
			return new CarbonSSOAuthenticator(request);
		} else if (ivuser != null) {
			// If we have a value for this - then it's an authentication request redirected by
			// WebSeal.
			return new WebSealAuthenticator(request);
		} else {
			return new DefaultCarbonAuthenticator(request);
		}
	}

	/**
	 * TODO: fix this from a configuration file
	 *
	 * @param file
	 *            File
	 * @return bool
	 */
	private boolean isListed(String file) {
		return "global-params.js".equals(file);
	}
}
