package org.wso2.carbon.ui;

import java.rmi.RemoteException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.axiom.om.util.Base64;
import org.apache.axis2.AxisFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.ui.authentication.AuthenticationAdminServiceStub;
import org.wso2.carbon.ui.authentication.LoginWithDelegation;
import org.wso2.carbon.ui.authentication.LoginWithDelegationResponse;

public class WebSealAuthenticator extends DefaultCarbonAuthenticator {

	public static final String WEBSEAL_USER = "iv-user";
	protected static final Log log = LogFactory.getLog(WebSealAuthenticator.class);
	private HttpServletRequest request;

	/**
	 * 
	 * @param request
	 */
	public WebSealAuthenticator(HttpServletRequest request) {
		super(request);
		this.request = request;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean authenticate() throws RemoteException {

		String credentials = request.getHeader("Authorization");
		String websealuser = null;
		String password = null;
		String username = null;

		username = request.getHeader(WEBSEAL_USER);

		if (credentials != null) {
			credentials = credentials.trim();
		} else {
			if (log.isDebugEnabled()) {
				log.debug("Empty Authorization header");
			}
			return false;
		}

		if (credentials != null && credentials.startsWith("Basic ")) {
			credentials = new String(Base64.decode(credentials.substring(6)));
			int i = credentials.indexOf(':');
			if (i == -1) {
				websealuser = credentials;
			} else {
				websealuser = credentials.substring(0, i);
			}

			if (i != -1) {
				password = credentials.substring(i + 1);
				if (password != null && password.equals("")) {
					password = null;
				}
			}
		}

		return authenticate(websealuser, password, username);
	}

	private boolean authenticate(String websealUser, String password, String userName)
			throws RemoteException {
		ServletContext servletContext = null;
		ConfigurationContext configContext = null;

		try {

			if (websealUser == null || password == null || password == null) {
				if (log.isDebugEnabled()) {
					if (websealUser == null) {
						log.debug("No valid webseal user name provided");
					}
					if (password == null) {
						log.debug("No valid webseal user password provided");
					}
					if (password == null) {
						log.debug("No valid webseal authneticated user name provided");
					}
				}
				return false;
			}

			servletContext = request.getSession().getServletContext();
			configContext = (ConfigurationContext) servletContext
					.getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

			if (configContext == null) {
				log.error("Configuration context is null.");
			}
			// Obtain the back-end server URL from the request. If not obtain it from the http
			// session and then from the ServletContext.
            HttpSession session = request.getSession();
            String backendServerURL = request.getParameter("backendURL");
            if (backendServerURL == null) {
                backendServerURL = CarbonUIUtil.getServerURL(servletContext, request.getSession());
            }

			// Back-end server URL is stored in the session, even if it is an incorrect one. This
			// value will be displayed in the server URL text box. Usability improvement.
			session.setAttribute(CarbonConstants.SERVER_URL, backendServerURL);

			String serviceURL = backendServerURL + AUTHENTICATION_ADMIN_SERVICE;
			AuthenticationAdminServiceStub stub = new AuthenticationAdminServiceStub(configContext,
					serviceURL);
			LoginWithDelegation loginRequest = new LoginWithDelegation();
			loginRequest.setUsername(websealUser);
			loginRequest.setPassword(password);
			loginRequest.setAuthenticatedUser(userName);
			loginRequest.setRemoteAddress(request.getRemoteAddr());
			Options option = stub._getServiceClient().getOptions();
			option.setManageSession(true);
			LoginWithDelegationResponse loginResponse = stub.loginWithDelegation(loginRequest);

			boolean isLogged = loginResponse.get_return();
			if (isLogged) {
				setUserPermissions(userName, request, stub);
			}
			return isLogged;

		} catch (AxisFault axisFault) {
			throw axisFault;
		} catch (RemoteException e) {
			throw e;
		} catch (Exception e) {
			throw new AxisFault("Exception occured", e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public String getAuthenticatedUser() {
		return request.getHeader(WEBSEAL_USER);
	}
}
