package org.wso2.carbon.ui.deployment.beans;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.ui.CarbonUIUtil;
import org.wso2.carbon.ui.MenuAdminClient;

public class CarbonUIDefinitions {
    private static Log log = LogFactory.getLog(CarbonUIDefinitions.class);
    private Menu[] menuDefinitions;
    private Servlet[] servletDefinitions;
    private HashMap<String, String> unauthenticatedUrls = new HashMap<String, String>();
    private HashMap<String, String> skipTilesUrls = new HashMap<String, String>();


    public void addUnauthenticatedUrls(List<String> urls) {
        Iterator<String> iterator = urls.iterator();
        while (iterator.hasNext()) {
            String url = iterator.next();
            addUnauthenticatedUrl(url);
        }
    }

    public void addUnauthenticatedUrl(String url) {
        unauthenticatedUrls.put(url, url);
    }

    public HashMap<String, String> getUnauthenticatedUrls() {
        return unauthenticatedUrls;
    }

    public void addSkipTilesUrls(List<String> urls) {
        Iterator<String> iterator = urls.iterator();
        while (iterator.hasNext()) {
            String url = iterator.next();
            addSkipTilesUrl(url);
        }
    }

    public void addSkipTilesUrl(String url) {
        skipTilesUrls.put(url, url);
    }

    public HashMap<String, String> getSkipTilesUrls() {
        return skipTilesUrls;
    }


    /**
     * Removes a given menu item from menu definitions.
     * Menu items available to all users will be effected.
     *
     * @param menuId
     * @see CarbonUIUtil#removeMenuDefinition(String, javax.servlet.http.HttpServletRequest)
     */
    public void removeMenuDefinition(String menuId) {
        //TODO : consider removing child menu items as well
        ArrayList<Menu> modifiedMenuDefs = new ArrayList<Menu>();
        boolean modified = false;
        if (menuId != null && menuId.trim().length() > 0) {
            for (int a = 0; a < menuDefinitions.length; a++) {
                Menu menu = menuDefinitions[a];
                if (menu != null) {
                    if (!menuId.equals(menu.getId())) {
                        modifiedMenuDefs.add(menu);
                        modified = true;
                    } else {
                        if (log.isDebugEnabled()) {
                            log.debug("Removing menu item : " + menuId);
                        }
                    }
                }
            }
            if (modified) {
                Menu[] newMenuDefs = new Menu[modifiedMenuDefs.size()];
                menuDefinitions = modifiedMenuDefs.toArray(newMenuDefs);
            }
        }
    }

    /**
     * @param loggedInUserName
     * @param userPermissions
     * @param request
     * @return
     */
    public Menu[] getMenuDefinitions(String loggedInUserName,
                                     ArrayList<String> userPermissions,
                                     HttpServletRequest request) {
        return getMenuDefinitions(loggedInUserName, userPermissions, request, menuDefinitions);
    }

    /**
     * Get menu definitions based on Logged in user & user permission
     *
     * @param loggedInUserName
     * @param userPermissions
     * @return
     */
    public Menu[] getMenuDefinitions(String loggedInUserName,
                                     ArrayList<String> userPermissions,
                                     HttpServletRequest request,
                                     Menu[] currentMenuItems) {
        if (loggedInUserName != null) {
            //If a user is logged in, send filtered set of menus
            ArrayList<Menu> filteredMenuDefs = new ArrayList<Menu>();

            //Get permissions granted for logged in user
            HashMap<String, String> grantedPermissions =
                    getGrantedPermissions(loggedInUserName, userPermissions);

            for (int a = 0; a < currentMenuItems.length; a++) {
                Menu menu = currentMenuItems[a];
                //* is the default value, which denotes no special
                //permission is required
                if (!menu.getRequirePermission().equals("*")
                        && !("admin".equals(loggedInUserName))) {
                    String requiredPermission = menu.getRequirePermission();
                    if (grantedPermissions.get(requiredPermission) != null) {
                        filteredMenuDefs.add(menu);
                        if (log.isDebugEnabled()) {
                            log.debug("User : " + loggedInUserName +
                                    " is allowed to access menu -> " + menu);
                        }
                    }
                } else {
                    filteredMenuDefs.add(menu);
                }
            }
            Menu[] filteredMenus = new Menu[filteredMenuDefs.size()];
            request.getSession().setAttribute(MenuAdminClient.USER_MENU_ITEMS_FILTERED, "true");
            return filteredMenuDefs.toArray(filteredMenus);
        } else {
            request.getSession().setAttribute(MenuAdminClient.USER_MENU_ITEMS_FILTERED, "false");
            return currentMenuItems;
        }
    }


    private HashMap<String, String> getGrantedPermissions(String userName,
                                                          ArrayList<String> userPermissions) {
        HashMap<String, String> userRoles = new HashMap<String, String>();
        Iterator<String> iterator = userPermissions.iterator();
        while (iterator.hasNext()) {
            String permission = iterator.next();
            if (log.isDebugEnabled()) {
                log.debug("User : " + userName + " has permission : " + permission);
            }
            userRoles.put(permission, permission);
        }
        return userRoles;
    }


    public void setMenuDefinitions(Menu[] menuDefinitions, String action) {
        if (CarbonConstants.ADD_UI_COMPONENT.equals(action)) {
            addMenuItems(menuDefinitions);
        } else if (CarbonConstants.REMOVE_UI_COMPONENT.equals(action)) {
            //TODO
        }
    }

    private void addMenuItems(Menu[] newMenuDefinitions) {
        int noOfExistingMenuItems = 0;
        int noOfNewMenuItems = 0;
        if (menuDefinitions != null) {
            noOfExistingMenuItems = menuDefinitions.length;
        }
        if (newMenuDefinitions != null) {
            noOfNewMenuItems = newMenuDefinitions.length;
        }
        if (log.isDebugEnabled()) {
            log.debug("No of existing menu items : " + noOfExistingMenuItems);
            log.debug("No of new menu items : " + noOfNewMenuItems);
        }

        //Creating a new Menu Array to hold up all menu definitions
        Menu[] allMenus = new Menu[noOfExistingMenuItems + noOfNewMenuItems];
        for (int a = 0; a < noOfExistingMenuItems; a++) {
            allMenus[a] = menuDefinitions[a];
        }
        //start from where previous for loop left
        for (int b = noOfExistingMenuItems; b < allMenus.length; b++) {
            allMenus[b] = newMenuDefinitions[b - noOfExistingMenuItems];
        }

        if (log.isDebugEnabled()) {
            log.debug("Listing all menu items as of now...");
            for (int c = 0; c < allMenus.length; c++) {
                log.debug("--->" + allMenus[c]);
            }
        }
        this.menuDefinitions = null;
        this.menuDefinitions = allMenus;
    }

    /**
     * @param menuDefinitionsToBeRemoved
     */
    private void removeMenuItems(Menu[] menuDefinitionsToBeRemoved) {
        //TODO : complete
        int noOfExistingMenuItems = 0;
        int noOfNewMenuItems = 0;
        if (menuDefinitions != null) {
            noOfExistingMenuItems = menuDefinitions.length;
        }
        if (menuDefinitionsToBeRemoved != null) {
            noOfNewMenuItems = menuDefinitionsToBeRemoved.length;
        }
        if (log.isDebugEnabled()) {
            log.debug("No of existing menu items : " + noOfExistingMenuItems);
            log.debug("No of new menu items : " + noOfNewMenuItems);
        }

        //Creating a new Menu Array to hold up all menu definitions
        Menu[] allMenus = new Menu[noOfExistingMenuItems + noOfNewMenuItems];
        for (int a = 0; a < noOfExistingMenuItems; a++) {
            allMenus[a] = menuDefinitions[a];
        }
        //start from where previous for loop left
        for (int b = noOfExistingMenuItems; b < allMenus.length; b++) {
            allMenus[b] = menuDefinitionsToBeRemoved[b - noOfExistingMenuItems];
        }

        if (log.isDebugEnabled()) {
            log.debug("Listing all menu items as of now...");
            for (int c = 0; c < allMenus.length; c++) {
                log.debug("--->" + allMenus[c]);
            }
        }
        this.menuDefinitions = null;
        this.menuDefinitions = allMenus;
    }

    /**
     * @return
     */
    public Servlet[] getServletDefinitions() {
        return servletDefinitions;
    }


    /**
     * @param servletDefinitions
     * @param action
     */
    public void setServletDefinitions(Servlet[] servletDefinitions, String action) {
        if (CarbonConstants.ADD_UI_COMPONENT.equals(action)) {
            addServletItems(servletDefinitions);
        } else if (CarbonConstants.REMOVE_UI_COMPONENT.equals(action)) {
            //TODO
        }
    }

    /**
     * @param newServletDefinitions
     */
    private void addServletItems(Servlet[] newServletDefinitions) {
        int noOfExistingItems = 0;
        int noOfNewItems = 0;
        if (servletDefinitions != null) {
            noOfExistingItems = servletDefinitions.length;
        }
        if (newServletDefinitions != null) {
            noOfNewItems = newServletDefinitions.length;
        }
        if (log.isDebugEnabled()) {
            log.debug("No of existing servlet items : " + noOfExistingItems);
            log.debug("No of new servlet items : " + noOfNewItems);
        }

        //Creating a new Servlet Array to hold up all servlet definitions
        Servlet[] allServlets = new Servlet[noOfExistingItems + noOfNewItems];
        for (int a = 0; a < noOfExistingItems; a++) {
            allServlets[a] = servletDefinitions[a];
        }
        //start from where previous for loop left
        for (int b = noOfExistingItems; b < allServlets.length; b++) {
            allServlets[b] = newServletDefinitions[b - noOfExistingItems];
        }

        if (log.isDebugEnabled()) {
            log.debug("Listing all menu items as of now...");
            for (int c = 0; c < allServlets.length; c++) {
                log.debug("--->" + allServlets[c]);
            }
        }
        this.servletDefinitions = null;
        this.servletDefinitions = allServlets;
    }
}
