/*                                                                             
 * Copyright 2004,2005 The Apache Software Foundation.                         
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.carbon.ui.taglibs;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import java.io.IOException;
import java.util.ResourceBundle;

/**
 * A tag for selecting & deselecting a group of items & applying some common operation
 * on these items
 */
public class ItemGroupSelector extends BodyTagSupport {

    private static final Log log = LogFactory.getLog(Paginator.class);
    private String selectAllInPageFunction;
    private String selectAllFunction;
    private String selectNoneFunction;
    private String deleteFunction;
    private String deleteButtonId;
    private String resourceBundle;
    private String selectAllInPageKey;
    private String selectAllKey;
    private String selectNoneKey;
    private String deleteKey;
    private int numberOfPages = 1;

    public String getSelectAllInPageFunction() {
        return selectAllInPageFunction;
    }

    public void setSelectAllInPageFunction(String selectAllInPageFunction) {
        this.selectAllInPageFunction = selectAllInPageFunction;
    }

    public String getSelectAllFunction() {
        return selectAllFunction;
    }

    public void setSelectAllFunction(String selectAllFunction) {
        this.selectAllFunction = selectAllFunction;
    }

    public String getSelectNoneFunction() {
        return selectNoneFunction;
    }

    public void setSelectNoneFunction(String selectNoneFunction) {
        this.selectNoneFunction = selectNoneFunction;
    }

    public String getDeleteFunction() {
        return deleteFunction;
    }

    public void setDeleteFunction(String deleteFunction) {
        this.deleteFunction = deleteFunction;
    }

    public String getDeleteButtonId() {
        return deleteButtonId;
    }

    public void setDeleteButtonId(String deleteButtonId) {
        this.deleteButtonId = deleteButtonId;
    }

    public String getResourceBundle() {
        return resourceBundle;
    }

    public void setResourceBundle(String resourceBundle) {
        this.resourceBundle = resourceBundle;
    }

    public String getSelectAllInPageKey() {
        return selectAllInPageKey;
    }

    public void setSelectAllInPageKey(String selectAllInPageKey) {
        this.selectAllInPageKey = selectAllInPageKey;
    }

    public String getSelectAllKey() {
        return selectAllKey;
    }

    public void setSelectAllKey(String selectAllKey) {
        this.selectAllKey = selectAllKey;
    }

    public String getSelectNoneKey() {
        return selectNoneKey;
    }

    public void setSelectNoneKey(String selectNoneKey) {
        this.selectNoneKey = selectNoneKey;
    }

    public String getDeleteKey() {
        return deleteKey;
    }

    public void setDeleteKey(String deleteKey) {
        this.deleteKey = deleteKey;
    }

    public int getNumberOfPages() {
        return numberOfPages;
    }

    public void setNumberOfPages(int numberOfPages) {
        this.numberOfPages = numberOfPages;
    }

    public int doEndTag() throws JspException {
        String selectAllInPage = "Select all in this page";
        String selectAll = "Select all in all pages";
        String selectNone = "Select none";
        String delete = "Delete";
        if (resourceBundle != null) {
            try {
                ResourceBundle bundle = ResourceBundle.getBundle(resourceBundle,
                                                                 pageContext.getRequest().getLocale());
                selectAllInPage = bundle.getString(selectAllInPageKey);
                selectAll = bundle.getString(selectAllKey);
                selectNone = bundle.getString(selectNoneKey);
                delete = bundle.getString(deleteKey);
            } catch (Exception e) {
                log.warn("Error while i18ning paginator", e);
            }
        }

        JspWriter writer = pageContext.getOut();

        String content = "<table>" +
                         "<tr>" +
                         "<td><a href=\"#\" onclick=\"" + selectAllInPageFunction + ";return false;\"  " +
                         "style=\"cursor:pointer\">" + selectAllInPage + "</a>&nbsp<b>|</b>&nbsp;" +
                         "</td>";
        if (numberOfPages > 1) {
            content += "<td><a href=\"#\" onclick=\"" + selectAllFunction + ";return false;\"  " +
                       "style=\"cursor:pointer\">" + selectAll + "</a>&nbsp<b>|</b>&nbsp;</td>";
        }

        content += "<td><a href=\"#\" onclick=\"" + selectNoneFunction + ";return false;\"  " +
                   "style=\"cursor:pointer\">" + selectNone + "</a></td>" +
                   "<td width=\"20%\">&nbsp;</td>" +
                   "<td><a href='#' id=\"" + deleteButtonId + "\" onclick=\"" + deleteFunction + ";return false;\"/>" + delete + "</a></td>" +
                   "</tr>" +
                   "</table>";
        try {
            writer.write(content);
        } catch (IOException e) {
            String msg = "Cannot write ItemSelector tag content";
            log.error(msg, e);
            throw new JspException(msg, e);
        }
        return 0;
    }
}
