package org.wso2.carbon.ui.taglibs;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.http.HttpServletRequest;
import java.util.ResourceBundle;
import java.util.Locale;
import java.util.Enumeration;
import java.io.IOException;

public class JSi18n extends BodyTagSupport {
    private static final Log log = LogFactory.getLog(JSi18n.class);

    private static final String ORG_WSO2_CARBON_I18N_JSRESOURCES =
            "org.wso2.carbon.i18n.JSResources";

    private String resourceBundle;
    private HttpServletRequest request;
    private String i18nObjectName = "jsi18n";

    public String getI18nObjectName() {
        return i18nObjectName;
    }

    public void setI18nObjectName(String i18nObjectName) {
        this.i18nObjectName = i18nObjectName;
    }

    public HttpServletRequest getRequest() {
        return request;
    }

    public void setRequest(HttpServletRequest request) {
        this.request = request;
    }

    public String getResourceBundle() {
        return resourceBundle;
    }

    public void setResourceBundle(String resourceBundle) {
        this.resourceBundle = resourceBundle;
    }

    public int doEndTag() throws JspException {
        if (request != null) {
            // Retrieve locale from request headers
            Locale locale = request.getLocale();

            String jsString =
                    "<script type=\"text/javascript\" src='../yui/build/utilities/utilities.js'>" +
                            "<script type=\"text/javascript\" src='../yui/build/yahoo/yahoo-min.js'></script>" +
                            "<script type=\"text/javascript\" src='../yui/build/json/json-min.js'></script>" +
                            "<script type=\"text/javascript\"> var tmpPairs = '{";

            boolean firstPair = true;

            // Retrieve the default carbon JS resource bundle
            ResourceBundle defaultBunndle = ResourceBundle.getBundle(
                    ORG_WSO2_CARBON_I18N_JSRESOURCES, locale);

            // Retrieve keys from the default bundle
            for (Enumeration e = defaultBunndle.getKeys(); e.hasMoreElements();) {
                String key = (String) e.nextElement();
                String value = defaultBunndle.getString(key);
                if (firstPair) {
                    jsString = jsString + "\"" + key + "\":\"" + value + "\"";
                    firstPair = false;
                } else {
                    jsString = jsString + ",\"" + key + "\":\"" + value + "\"";
                }
            }

            if (resourceBundle != null) {
                // Retrieve the resource bundle
                ResourceBundle bundle = ResourceBundle.getBundle(resourceBundle, locale);

                // Retrieve keys from the user defined bundle
                for (Enumeration e = bundle.getKeys(); e.hasMoreElements();) {
                    String key = (String) e.nextElement();
                    String value = bundle.getString(key);
                    if (firstPair) {
                        jsString = jsString + "\"" + key + "\":\"" + value + "\"";
                        firstPair = false;
                    } else {
                        jsString = jsString + ",\"" + key + "\":\"" + value + "\"";
                    }
                }
            }

            jsString = jsString + "}'; var " + i18nObjectName +
                    " = YAHOO.lang.JSON.parse(tmpPairs);</script>";

            StringBuffer content = new StringBuffer();
            content.append(jsString);

            // Write to output
            JspWriter writer = pageContext.getOut();
            try {
                writer.write(content.toString());
            } catch (IOException e) {
                String msg = "Cannot write i18n tag content";
                log.error(msg, e);

                try {
                    //exit gracefully
                    writer.write("");
                } catch (IOException e1) {/*do nothing*/}
            }

        }

        return 0;
    }
}
