/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.ui.transports.fileupload;

import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.client.Options;
import org.apache.axis2.Constants;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.utils.*;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.CarbonException;
import org.wso2.carbon.ui.fileupload.AnyFileUploaderStub;
import org.wso2.carbon.ui.fileupload.utils.UploadedFileItem;

import javax.activation.DataHandler;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
/*
 * 
 */

public abstract class AbstractFileUploadExecutor{

    protected static final Log log = LogFactory.getLog(AbstractFileUploadExecutor.class);

    protected ConfigurationContext configurationContext;

    protected Map<String, ArrayList<FileItemData>> fileItemsMap;

    protected Map<String, ArrayList<String>> formFieldsMap;

    public abstract boolean execute(HttpServletRequest request,
                           HttpServletResponse response) throws CarbonException, IOException;

    boolean executeGeneric(HttpServletRequest request, HttpServletResponse response,
                                  ConfigurationContext configurationContext) throws CarbonException, IOException{
        this.configurationContext = configurationContext;
        parseRequest(request);
        return execute(request, response);
    }

    void parseRequest(HttpServletRequest request) throws CarbonException{
        fileItemsMap = new HashMap<String, ArrayList<FileItemData>>();
        formFieldsMap = new HashMap<String, ArrayList<String>>();

        ServletRequestContext servletRequestContext = new ServletRequestContext(request);
        boolean isMultipart =
                ServletFileUpload.isMultipartContent(servletRequestContext);
        try {
            if (isMultipart) {

                List items = parseRequest(servletRequestContext);
                boolean multiItems = false;
                if (items.size() > 1) {
                    multiItems = true;
                }
                // Add the uploaded items to the corresponding maps.
                for (Iterator iter = items.iterator(); iter.hasNext();) {
                    FileItem item = (FileItem) iter.next();
                    String fieldName = item.getFieldName().trim();
                    if (item.isFormField()) {
                        if (formFieldsMap.get(fieldName) == null) {
                            formFieldsMap.put(fieldName, new ArrayList<String>());
                        }
                        formFieldsMap.get(fieldName).add(new String(item.get()));
                    }else {
                        String fileName = item.getName();
                        if ((fileName == null || fileName.length() == 0) && multiItems) {
                            continue;
                        }
                        if (fileItemsMap.get(fieldName) == null) {
                            fileItemsMap.put(fieldName, new ArrayList<FileItemData>());
                        }
                        fileItemsMap.get(fieldName).add(new FileItemData(item));
                    }
                }
            }
        } catch (Exception e) {
            String msg = "File upload FAILED";
            log.error(msg, e);
            throw new CarbonException(msg, e);
        }
    }

    protected String getWorkingDir(){
        return (String) configurationContext.getProperty(WSO2Constants.WORK_DIR);
    }

    protected String generateUUID(){
        return String.valueOf(System.currentTimeMillis() + Math.random());
    }

    protected String getFileName(String fileName) {
        String fileNameOnly;
        if (fileName.indexOf("\\") < 0) {
            fileNameOnly = fileName.substring(fileName.lastIndexOf("/") + 1,
                    fileName.length());
        } else {
            fileNameOnly = fileName.substring(fileName.lastIndexOf("\\") + 1,
                    fileName.length());
        }
        return fileNameOnly;
    }

    protected List parseRequest(ServletRequestContext requestContext) throws FileUploadException {
        // Create a factory for disk-based file items
        FileItemFactory factory = new DiskFileItemFactory();
        // Create a new file upload handler
        ServletFileUpload upload = new ServletFileUpload(factory);
        // Parse the request
        return upload.parseRequest(requestContext);
    }

    //Old methods, these should be refactored.
    protected void checkServiceFileExtensionValidity(String fileExtension,
                                                     String[] allowedExtensions) throws Exception {
        boolean isExtensionValid = false;
        StringBuffer allowedExtensionsStr = new StringBuffer();
        for (int i = 0; i < allowedExtensions.length; i++) {
            allowedExtensionsStr.append(allowedExtensions[i]).append(",");
            if (fileExtension.endsWith(allowedExtensions[i])) {
                isExtensionValid = true;
                break;
            }
        }
        if (!isExtensionValid) {
            throw new Exception(" Illegal file type." +
                    " Allowed file extensions are " + allowedExtensionsStr);
        }
    }

    protected File uploadFile(HttpServletRequest request,
                              String repoDir,
                              HttpServletResponse response,
                              String extension) throws IOException {

        response.setContentType("text/html; charset=utf-8");
        ServletRequestContext servletRequestContext = new ServletRequestContext(request);
        boolean isMultipart =
                ServletFileUpload.isMultipartContent(servletRequestContext);
        File uploadedFile = null;
        if (isMultipart) {
            try {
                // Create a new file upload handler
                List items = parseRequest(servletRequestContext);

                // Process the uploaded items
                for (Iterator iter = items.iterator(); iter.hasNext();) {
                    FileItem item = (FileItem) iter.next();
                    if (!item.isFormField()) {
                        String fileName = item.getName();
                        String fileExtension = fileName;
                        fileExtension = fileExtension.toLowerCase();
                        if (extension != null && !fileExtension.endsWith(extension)) {
                            throw new Exception(" Illegal file type. Only " +
                                    extension + " files can be uploaded");

                        }
                        String fileNameOnly = getFileName(fileName);
                        uploadedFile = new File(repoDir, fileNameOnly);
                        item.write(uploadedFile);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.error("File upload failed", e);
                response.getWriter().write("<script type=\"text/javascript\">" +
                        "top.wso2.wsf.Util.alertWarning('File upload FAILED. " +
                        e.getMessage() + "');" +
                        "</script>");
            }
        }
        return uploadedFile;
    }

    /**
     * This is the common method that can be used for Fileupload.
     * extraStoreDirUUID is the name of the javascript that's going to
     * execute on the client side at the secound run.
     *
     * @param request
     * @param response
     * @param extraStoreDirUUID
     * @return Status true/fase.
     * @throws org.wso2.carbon.CarbonException
     *
     * @throws IOException
     */
    protected boolean executeCommon(HttpServletRequest request, HttpServletResponse response,
                                    String extraStoreDirUUID)
            throws CarbonException, IOException {

//        ConfigurationContext configurationContext = (ConfigurationContext)
//                request.getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

        String serverURL = (String) request.getAttribute(CarbonConstants.SERVER_URL);
        HttpSession session = request.getSession();
        String cookie = (String) session.getAttribute(ServerConstants.ADMIN_SERVICE_COOKIE);

        String serviceEPR = serverURL + "AnyFileUploader";
        AnyFileUploaderStub anyFileUploaderStub =
                new AnyFileUploaderStub(configurationContext, serviceEPR);
        ServiceClient client = anyFileUploaderStub._getServiceClient();
        Options options = client.getOptions();
        options.setManageSession(true);
        options.setProperty(org.apache.axis2.transport.http.HTTPConstants.COOKIE_STRING, cookie);
        options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
        options.setTimeOutInMilliSeconds(10000);

        PrintWriter out = response.getWriter();
        response.setContentType("text/plain; charset=utf-8");
        try {
            Set<String> keys = fileItemsMap.keySet();
            boolean multiItems = false;
            if (fileItemsMap.size() > 1) {
                multiItems = true;
            }
            // Process the uploaded items
            UploadedFileItem[] uploadedFileItems = new UploadedFileItem[fileItemsMap.size()];
            Iterator<String> iterator = keys.iterator();
            int i = 0;
            while (iterator.hasNext()) {
                String fieldName = iterator.next();
                String fileName = fileItemsMap.get(fieldName).get(0).getFileItem().getName();
                if ((fileName == null || fileName.length() == 0) && multiItems) {
                    continue;
                }
                FileItemData fileItemData = fileItemsMap.get(fieldName).get(0);
                UploadedFileItem uploadedFileItem = new UploadedFileItem();
                uploadedFileItem.setDataHandler(fileItemData.getDataHandler());
                uploadedFileItem.setFileName(fileName);
                uploadedFileItem.setFileType("");
                uploadedFileItems[i] = uploadedFileItem;
                i++;
            }
            String[] uuidArray = anyFileUploaderStub.uploadFiles(uploadedFileItems);
            String uuids = "";
            for (String uuid : uuidArray) {
                uuids += uuid + ",";
            }
            uuids = uuids.substring(0, uuids.length() - 1);
            out.write(uuids);
            out.flush();
        } catch (Exception e) {
            log.error("File upload FAILED", e);
            out.write("<script type=\"text/javascript\">" +
                    "top.wso2.wsf.Util.alertWarning('File upload FAILED. File may be non-existent or invalid.');" +
                    "</script>");
        } finally {
            out.close();
        }
        return true;
    }


    /**
     * This is a helper method that will be used upload main entity (ex: wsdd, jar, class etc) and
     * its resources to a given deployer.
     *
     * @param request
     * @param response
     * @param uploadDirName
     * @param extensions
     * @param utilityString
     * @return boolean
     * @throws IOException
     */
    protected boolean uploadArtifacts(HttpServletRequest request,
                                      HttpServletResponse response,
                                      String uploadDirName,
                                      String[] extensions,
                                      String utilityString)
            throws IOException {
        String axis2Repo = ServerConfiguration.getInstance().
                getFirstProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION);
        PrintWriter out = response.getWriter();
        String webContext = (String) request.getAttribute(CarbonConstants.WEB_CONTEXT);
        if (CarbonUtils.isURL(axis2Repo)) {
//            out.write("<script type=\"text/javascript\">" +
//                      "wso2.wsf.Util.alertWarning('You are not permitted to upload jars to URL repository " +
//                      axis2Repo + "');" +
//                      "</script>");
//            out.flush();


            response.sendRedirect(getContextRoot(request)+ "/carbon/service-mgt/index.jsp?message=You are not permitted to upload jars to URL repository "
                    + axis2Repo);
            return false;
        }

        String tmpDir = (String) configurationContext.getProperty(WSO2Constants.WORK_DIR);
        String uuid = String.valueOf(System.currentTimeMillis() + Math.random());
        tmpDir = tmpDir + File.separator + "artifacts" + File.separator + uuid + File.separator;
        new File(tmpDir).mkdirs();

        response.setContentType("text/html; charset=utf-8");

        ServletRequestContext servletRequestContext = new ServletRequestContext(request);
        boolean isMultipart =
                ServletFileUpload.isMultipartContent(servletRequestContext);
        if (isMultipart) {
            try {
                // Create a new file upload handler
                List items = parseRequest(servletRequestContext);
                // Process the uploaded items
                for (Iterator iter = items.iterator(); iter.hasNext();) {
                    FileItem item = (FileItem) iter.next();
                    if (!item.isFormField()) {
                        String fileName = item.getName();
                        String fileExtension = fileName;
                        fileExtension = fileExtension.toLowerCase();

                        String fileNameOnly = getFileName(fileName);
                        File uploadedFile;

                        String fieldName = item.getFieldName();

                        if (fieldName != null && fieldName.equals("jarResource")) {
                            if (fileExtension.endsWith(".jar")) {
                                File servicesDir =
                                        new File(tmpDir + File.separator + uploadDirName, "lib");
                                if (!servicesDir.exists()) {
                                    servicesDir.mkdirs();
                                }
                                uploadedFile = new File(servicesDir, fileNameOnly);
                                item.write(uploadedFile);
                            }
                        } else {
                            File servicesDir = new File(tmpDir, uploadDirName);
                            if (!servicesDir.exists()) {
                                servicesDir.mkdirs();
                            }
                            uploadedFile = new File(servicesDir, fileNameOnly);
                            item.write(uploadedFile);
                        }
                    }
                }

                //First lets filter for jar resources
                String repo = configurationContext.getAxisConfiguration().getRepository().getPath();

                //Writting the artifacts to the proper location
                FileManipulator fm = new FileManipulator();
                String parent = repo + File.separator + uploadDirName;
                File mainDir = new File(tmpDir + File.separator + uploadDirName);
                File libDir = new File(mainDir, "lib");
                File[] resourceLibFile = fm.getMatchingFiles(libDir.getAbsolutePath(), null, "jar");


                for (int j = 0; j < resourceLibFile.length; j++) {
                    File dst = new File(parent, "lib");
                    File src = resourceLibFile[j];
                    String[] files = libDir.list();
                    for (int i = 0; i < files.length; i++) {
                        copyFile(src, new File(dst, files[i]));
                    }
                }

                for (int i = 0; i < extensions.length; i++) {
                    File[] mainFiles =
                            fm.getMatchingFiles(mainDir.getAbsolutePath(), null, extensions[i]);
                    for (int j = 0; j < mainFiles.length; j++) {
                        File dst = new File(parent);
                        String[] files = mainDir.list();
                        for (int k = 0; k < files.length; k++) {
                            File f = new File(dst, files[k]);
                            if (!f.isDirectory()) {
                                copyFile(mainFiles[j], f);
                            }
                        }

                    }
                }
//                out.write("<script type=\"text/javascript\">" +
//                          "completeServiceFileUpload('Files have been uploaded " +
//                          "successfully. This page will be auto refreshed shortly with " +
//                          "the status of the created " + utilityString + " service ');" +
//                          "</script>");
//                String context = request.getContextPath();
//                out.write("<script type=\"text/javascript\">" +
//                          "location.href=../" + webContext + "'/service-mgt/index.jsp?message=Files have been uploaded" +
//                          "successfully. This page will be auto refreshed shortly with " +
//                          "the status of the created " + utilityString + " service ';" +
//                          "</script>");
                response.sendRedirect(getContextRoot(request)+ "/carbon/service-mgt/index.jsp?message=Files have been uploaded "
                        + "successfully. This page will be auto refreshed shortly with "
                        + "the status of the created " + utilityString + " service");
                return true;
            } catch (Exception e) {
                log.error("File upload failed", e);
//                out.write("<script type=\"text/javascript\">" +
//                          "wso2.wsf.Util.alertWarning('File upload FAILED. " +
//                          e.getMessage() + "');" +
//                          "</script>");
                response.sendRedirect(getContextRoot(request)+ "/carbon/service-mgt/index.jsp?message=File upload FAILED. "
                        + e.getMessage());
            }
        }
        return false;
    }

    private void copyFile(File src, File dst) throws IOException, FileNotFoundException {
        String dstAbsPath = dst.getAbsolutePath();
        String dstDir = dstAbsPath.substring(0, dstAbsPath.lastIndexOf(File.separator));
        File dir = new File(dstDir);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        DataHandler dh = new DataHandler(src.toURL());
        FileOutputStream out = new FileOutputStream(dst);
        dh.writeTo(out);
        out.flush();
    }

    protected List<FileItemData> getAllFileItems(){
        Collection<ArrayList<FileItemData>> listCollection = fileItemsMap.values();
        List<FileItemData> fileItems = new ArrayList<FileItemData>();
        for (ArrayList<FileItemData> fileItemData : listCollection) {
            fileItems.addAll(fileItemData);
        }
        return fileItems;
    }

    protected String getContextRoot(HttpServletRequest request) {
        String contextPath = (request.getContextPath().equals("")) ? "" : request.getContextPath();
        int index;
        if (contextPath.equals("/fileupload")){
            contextPath = "";
        } else if ((index = contextPath.indexOf("/fileupload")) > -1) {
            contextPath = contextPath.substring(0, index);
        }
        return contextPath;
    }
}
