/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core;

import java.util.List;
import java.util.Map;

/**
 * Manipulates data in the User Store
 */
public interface UserStoreAdmin extends UserStoreReader {

    /**
     * Add user to the user store
     */
    public void addUser(String userName, Object credential) throws UserStoreException;

    /**
     * Update user password
     */
    public void updateUser(String userName, Object newCredential, Object oldCredential)
            throws UserStoreException;

    /**
     * Update user name
     */
    public void updateUserName(String userName, String newUserName) throws UserStoreException;

    /**
     * Update method for admin
     */
    public void updateUser(String userName, Object newCredential) throws UserStoreException;

    /**
     * Delete user from user store
     */
    public void deleteUser(String userName) throws UserStoreException;

    /**
     * Add role to user store
     */
    public void addRole(String roleName) throws UserStoreException;

    /**
     * Update role name
     */
    public void updateRoleName(String roleName, String newRoleName) throws UserStoreException;

    /**
     * Delete role from user store
     */
    public void deleteRole(String roleName) throws UserStoreException;

    /**
     * Set role properties in the user store
     */
    public void setRoleProperties(String roleName, Map<String, String> properties)
            throws UserStoreException;

    /**
     * Add user to role
     */
    public void addUserToRole(String userName, String roleName) throws UserStoreException;

    /**
     * Delete user from role
     */
    public void removeUserFromRole(String userName, String roleName) throws UserStoreException;

    /**
     * Updates or add the user properties
     */
    public void setUserProperty(String username, String attributeId, String attributeValue, String profileName)
            throws UserStoreException;

    /**
     * Set user properties
     */
    public void setUserProperties(String userName, Map<String, String> properties, String profileName)
            throws UserStoreException;

    public void deleteUserProperties(String userName, String[] properties, String profileName)
            throws UserStoreException;

    public void deleteUserProperty(String userName, String property, String profileName) throws UserStoreException;

    public void setUserBinaryContent(String userName, String contentName, byte[] data, String profileName)
            throws UserStoreException;

    public void deleteBinaryContent(String userName, String contentName, String profileName) throws UserStoreException;

    public void setUserClaimValue(String userName, String claimURI, String claimValue,
            String profileName) throws UserStoreException;

    public void setUserClaimValues(String userName, Map<String, String> claims, String profileName)
            throws UserStoreException;

    public void deleteUserClaimValue(String userName, String claimName, String claimValue,
            String profileName) throws UserStoreException;

    public void deleteUserClaimValues(String userName, String[] claims, String profileName)
            throws UserStoreException;

    public void setProfilePicture(String userName, byte[] picture, String profileName)
            throws UserStoreException;

}
