/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core.authorizing;

import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.i18n.Messages;


/**
 * Manipulates access control data in the user store Cannot change admin
 * privileges through this interface
 * 
 * @see org.wso2.registry.usersAccessControlAdmin
 * 
 */
public class ACLAccessControlAdmin
        extends ACLAuthorizer implements AccessControlAdmin {

    private Authorizer authorizer = null;

    private AccessControlAdmin admin = null;

    /** Constructor */
    public ACLAccessControlAdmin(Authorizer athzr, AccessControlAdmin admin,
            AuthorizingRealmConfig authConfig) {
        super(athzr, authConfig);
        this.authorizer = athzr;
        this.admin = admin;
    }

    /**
     * Authorize user
     */
    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.authorizeUser(userName, resourceId, action);

    }

    /**
     * Deny access to user
     */
    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.denyUser(userName, resourceId, action);
    }

    /**
     * Grant access to role
     */
    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.authorizeRole(roleName, resourceId, action);
    }

    /**
     * Deny access to role
     */
    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.denyRole(roleName, resourceId, action);
    }

    /**
     * Removes granted access from user
     */
    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.clearUserAuthorization(userName, resourceId, action);
    }

    /**
     * Removes granted access from Role
     */
    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.clearRoleAuthorization(roleName, resourceId, action);
    }

    /**
     * Clears all authorizations on a Role
     */
    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.clearResourceAuthorizations(resourceId);
    }

    /**
     * Duplicates granted/denied access
     */
    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.copyAuthorizations(fromResourceId, toResourceId);
    }
    
    

    public void clearRoleAuthorization(String roleName) throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.clearRoleAuthorization(roleName);
    }
    
    

    public void clearUserAuthorization(String userName) throws UserStoreException {
        doAuthorizationToAuthorize();
        admin.clearUserAuthorization(userName);
    }

    protected void doAuthorizationToAuthorize() throws UserStoreException {

        if (config.isEnableAdminBehavior() && isAdmin) {
            // do nothing user is authenticated
        } else if (!authorizer.isUserAuthorized(config.getAuthenticatedUserName(),
                AccessControlConstants.USER_RESOURCE, AccessControlConstants.READ)) {
            throw new UnauthorizedException(Messages.getMessage("unAuthorized", new String[] {
                    AccessControlConstants.USER_RESOURCE, AccessControlConstants.READ }));
        }
    }
}
