package org.wso2.carbon.user.core.claim;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.ClaimMapping;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.utils.ServerConstants;

public class DefaultClaimBuilder {

    private static Log log = LogFactory.getLog(DefaultClaimBuilder.class);

    private static final String CLAIM_CONFIG = "claim-config.xml";

    public static void setBundleContext(BundleContext bundleContext) {
        DefaultClaimBuilder.bundleContext = bundleContext;
    }

    private static BundleContext bundleContext;

    public Map<String, ClaimMapping> buildClaimMappingsFromDatabase(DataSource ds, String realmName)
            throws ClaimBuilderException {
        Map<String, ClaimMapping> claims = new HashMap<String, ClaimMapping>();
        try {
            ClaimDAO claimDAO = new ClaimDAO(ds, realmName);
            List<ClaimMapping> lst = claimDAO.loadClaimMappings();
            for (Iterator<ClaimMapping> ite = lst.iterator(); ite.hasNext();) {
                ClaimMapping cm = ite.next();
                String uri = cm.getClaim().getClaimUri();
                claims.put(uri, cm);
            }
        } catch (UserStoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return claims;
    }

    /**
     * 
     * @return
     * @throws ClaimBuilderExceptio
     */
    public Map<String, ClaimMapping> buildClaimMappingsFromConfigFile()
            throws ClaimBuilderException {
        OMElement element = null;
        OMElement dialectRoot = null;
        String message = null;
        Iterator<OMElement> dialectIterator = null;
        Map<String, ClaimMapping> claims = null;

        try {
            element = getRootElement();
        } catch (Exception e) {
            message = "Error while reading claim configuration";
            log.error(message, e);
            throw new ClaimBuilderException(message, e);
        }

        dialectRoot = element.getFirstChildWithName(new QName(ClaimConstants.LOCAL_NAME_DIALECTS));

        if (dialectRoot == null) {
            message = "In valid schema <Dialects> element not present";
            if (log.isDebugEnabled()) {
                log.debug(message);
            }
            throw new ClaimBuilderException(message);
        }

        dialectIterator = dialectRoot.getChildrenWithLocalName(ClaimConstants.LOCAL_NAME_DIALECT);

        claims = new HashMap<String, ClaimMapping>();

        while (dialectIterator.hasNext()) {
            OMElement dialect = null;
            String dialectUri = null;
            Iterator<OMElement> claimIterator = null;

            dialect = dialectIterator.next();
            dialectUri = dialect.getAttribute(new QName(ClaimConstants.ATTR_DIALECT_URI))
                    .getAttributeValue();
            claimIterator = dialect.getChildrenWithLocalName(ClaimConstants.LOCAL_NAME_CLAIM);

            while (claimIterator.hasNext()) {
                OMElement claimElement = null;
                Claim claim = null;
                ClaimMapping claimMapping = null;
                String displayName = null;
                String claimUri = null;
                String attributeId = null;
                String description = null;
                String regEx = null;
                int displayOrder = 0;

                claimElement = claimIterator.next();
                validateSchema(claimElement);

                claim = new Claim();
                claim.setDialectURI(dialectUri);

                claimUri = claimElement.getFirstChildWithName(
                        new QName(ClaimConstants.LOCAL_NAME_CLAIM_URI)).getText();
                claim.setClaimUri(claimUri);

                if (claimElement.getFirstChildWithName(new QName(
                        ClaimConstants.LOCAL_NAME_DISPLAY_NAME)) != null) {
                    displayName = claimElement.getFirstChildWithName(
                            new QName(ClaimConstants.LOCAL_NAME_DISPLAY_NAME)).getText();
                    claim.setDisplayTag(displayName);
                }

                description = claimElement.getFirstChildWithName(
                        new QName(ClaimConstants.LOCAL_NAME_DESCRIPTION)).getText();
                claim.setDescription(description);

                if (claimElement.getFirstChildWithName(new QName(ClaimConstants.LOCAL_NAME_REG_EX)) != null) {
                    regEx = claimElement.getFirstChildWithName(
                            new QName(ClaimConstants.LOCAL_NAME_REG_EX)).getText();
                    claim.setRegEx(regEx);
                }

                if (claimElement
                        .getFirstChildWithName(new QName(ClaimConstants.LOCAL_NAME_REQUIRED)) != null) {
                    claim.setRequired(true);
                }

                if (claimElement.getFirstChildWithName(new QName(
                        ClaimConstants.LOCAL_NAME_SUPPORTED_BY_DEFAULT)) != null) {
                    claim.setSupportedByDefault(true);
                }

                if (claimElement.getFirstChildWithName(new QName(
                        ClaimConstants.LOCAL_NAME_DISPLAY_OREDR)) != null) {
                    displayOrder = Integer.parseInt(claimElement.getFirstChildWithName(
                            new QName(ClaimConstants.LOCAL_NAME_DISPLAY_OREDR)).getText());
                    claim.setDisplayOrder(displayOrder);
                }

                attributeId = claimElement.getFirstChildWithName(
                        new QName(ClaimConstants.LOCAL_NAME_ATTR_ID)).getText();

                claimMapping = new ClaimMapping(claim, attributeId);
                claims.put(claimUri, claimMapping);
            }
        }
        return claims;
    }

    /**
     * 
     * @param claimElement
     * @throws ClaimBuilderException
     */
    private void validateSchema(OMElement claimElement) throws ClaimBuilderException {
        String message = null;

        if (claimElement.getFirstChildWithName(new QName(ClaimConstants.LOCAL_NAME_CLAIM_URI)) == null) {
            message = "In valid schema <ClaimUri> element not present";
            if (log.isDebugEnabled()) {
                log.debug(message);
            }
            throw new ClaimBuilderException(message);
        }

        if (claimElement.getFirstChildWithName(new QName(ClaimConstants.LOCAL_NAME_DESCRIPTION)) == null) {
            message = "In valid schema <Description> element not present";
            if (log.isDebugEnabled()) {
                log.debug(message);
            }
            throw new ClaimBuilderException(message);
        }

        if (claimElement.getFirstChildWithName(new QName(ClaimConstants.LOCAL_NAME_ATTR_ID)) == null) {
            message = "In valid schema <AttributeId> element not present";
            if (log.isDebugEnabled()) {
                log.debug(message);
            }
            throw new ClaimBuilderException(message);
        }
    }

    /**
     * 
     * @return
     * @throws XMLStreamException
     * @throws IOException
     * @throws ClaimBuilderException
     */
    private OMElement getRootElement() throws XMLStreamException, IOException,
            ClaimBuilderException {
        String carbonHome = null;
        InputStream inStream = null;
        StAXOMBuilder builder = null;
        boolean isClaimsLoaded = false;

        carbonHome = System.getProperty(ServerConstants.CARBON_HOME);

        if (carbonHome != null) {
            File claimConfigXml = new File(carbonHome + File.separator + "conf", CLAIM_CONFIG);
            if (claimConfigXml.exists()) {
                inStream = new FileInputStream(claimConfigXml);
                isClaimsLoaded = true;
            }
        }

        if (!isClaimsLoaded) {
            if (bundleContext != null) {
                inStream = bundleContext.getBundle().getResource(CLAIM_CONFIG).openStream();
                if (inStream == null) {
                    String message = "Claim configuration not found.";
                    if (log.isDebugEnabled()) {
                        log.debug(message);
                    }
                    throw new FileNotFoundException(message);
                }
            } else {
                String message = "Claim configuration cannot be loaded from the bundle";
                if (log.isDebugEnabled()) {
                    log.debug(message);
                }
                throw new ClaimBuilderException(message);
            }
        }

        builder = new StAXOMBuilder(inStream);
        return builder.getDocumentElement();
    }
}