/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core.def;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.BitSet;
import java.util.Iterator;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.util.DefaultCommons;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.util.DatabaseUtil;

@SuppressWarnings( { "ThrowFromFinallyBlock" })
public class DefaultAccessControlAdmin extends DefaultAuthorizer implements AccessControlAdmin {

    private static Log log = LogFactory.getLog(DefaultAccessControlAdmin.class);
    
    public DefaultAccessControlAdmin(DataSource dataSource, String algo, String systemUserName) {
        super(dataSource, algo, systemUserName);
    }

    public DefaultAccessControlAdmin(DataSource dataSource, String algo, SQLHelper sqlHelper, String systemUserName) {
        super(dataSource, algo, sqlHelper, systemUserName);
    }

    public DefaultAccessControlAdmin(DataSource dataSource, String systemUserName) {
        super(dataSource, DefaultRealmConfig.PERMISSION_BLOCK_FIRST, systemUserName);
    }

    // --------- AccessControlAdmin ------------
    public void authorizeUser(String userName, String resourceId, String action)
        throws UserStoreException {
        //Persist the permission
        addAllowDenyUserPermissionInDatabase(userName, resourceId, action, UserCoreConstants.ALLOW);
        authorizeUserInTree(userName, resourceId, action);
    }

    public void denyUser(String userName, String resourceId, String action)
        throws UserStoreException {
        //Persist the permission
        addAllowDenyUserPermissionInDatabase(userName, resourceId, action, UserCoreConstants.DENY);
        denyUserInTree(userName, resourceId, action);
    }

    public void authorizeRole(String roleName, String resourceId, String action)
        throws UserStoreException {
        //Persist the permission
        addAllowDenyRolePermissionInDatabase(roleName, resourceId, action, UserCoreConstants.ALLOW);
        authorizeRoleInTree(roleName, resourceId, action);
    }

    public void denyRole(String roleName, String resourceId, String action)
        throws UserStoreException {
        //Persist the permission
        addAllowDenyRolePermissionInDatabase(roleName, resourceId, action, UserCoreConstants.DENY);
        denyRoleInTree(roleName, resourceId, action);
    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
        throws UserStoreException {
        clearUserAuthorizationInDatabase(userName, resourceId, action);
        SearchResult sr = getNode(root, Util.toComponenets(resourceId));
        if (sr. getUnprocessedPaths() == null || sr.getUnprocessedPaths().isEmpty()) {
            TreeNode.Permission p = Util.actionToPermission(action);

            Map<String, BitSet> allowUsers = sr.getLastNode().getUserAllowPermissions();
            BitSet bs = allowUsers.get(userName);
            if (bs != null) {
                bs.clear(p.ordinal());
            }

            Map<String, BitSet> denyUsers = sr.getLastNode().getUserDenyPermissions();
            bs = denyUsers.get(userName);
            if (bs != null) {
                bs.clear(p.ordinal());
            }
        }
    }

    public void clearRoleAuthorization(String roleName, String resourceId, String action)
        throws UserStoreException {
        clearRoleAuthorizationInDatabase(roleName, resourceId, action);
        SearchResult sr = getNode(root, Util.toComponenets(resourceId));
        if (sr.getUnprocessedPaths() == null) {
            TreeNode.Permission p = Util.actionToPermission(action);

            Map<String, BitSet> allowRoles = sr.getLastNode().getRoleAllowPermissions();
            BitSet bs = allowRoles.get(roleName);
            if (bs != null) {
                bs.clear(p.ordinal());
            }

            Map<String, BitSet> denyRoles = sr.getLastNode().getRoleDenyPermissions();
            bs = denyRoles.get(roleName);
            if (bs != null) {
                bs.clear(p.ordinal());
            }
        }
    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        clearResourceAuthorizationsInDatabase(resourceId);
        SearchResult sr = getNode(root, Util.toComponenets(resourceId));
        if (sr.getUnprocessedPaths() == null) {
            sr.getLastNode().getUserAllowPermissions().clear();
            sr.getLastNode().getUserDenyPermissions().clear();
            sr.getLastNode().getRoleAllowPermissions().clear();
            sr.getLastNode().getRoleDenyPermissions().clear();
        }
        
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
        throws UserStoreException {
        // TODO should not be required anymore - correct?
    }

    
    
    public void clearRoleAuthorization(String roleName) throws UserStoreException {
        this.clearRoleAuthorizationInDatabase(roleName);
        clearRoleAuthorization(roleName, root);
    }
    
    public void clearUserAuthorization(String userName) throws UserStoreException {
        this.clearUserAuthorizationInDatabase(userName);
        clearUserAuthorization(userName, root);
    }

    public void clearRoleAuthorization(String roleName, TreeNode node) {
        Map<String, BitSet> allowRoles = node.getRoleAllowPermissions();
        Map<String, BitSet> denyRoles = node.getRoleDenyPermissions();

        BitSet bs = allowRoles.get(roleName);
        if (bs != null) {
            allowRoles.remove(roleName);
        }

        bs = denyRoles.get(roleName);
        if (bs != null) {
            denyRoles.remove(roleName);
        }

        Map<String, TreeNode> childMap = node.getChildren();
        if (childMap != null && childMap.size() > 0) {
            Iterator<TreeNode> ite = childMap.values().iterator();
            while (ite.hasNext()) {
                TreeNode childNode = ite.next();
                clearRoleAuthorization(roleName, childNode);
            }
        }
    }

    public void clearUserAuthorization(String userName, TreeNode node) {
        Map<String, BitSet> allowUser = node.getUserAllowPermissions();
        Map<String, BitSet> denyUser = node.getUserDenyPermissions();

        BitSet bs = allowUser.get(userName);
        if (bs != null) {
            allowUser.remove(userName);
        }

        bs = denyUser.get(userName);
        if (bs != null) {
            denyUser.remove(userName);
        }

        Map<String, TreeNode> childMap = node.getChildren();
        if (childMap != null && childMap.size() > 0) {
            Iterator<TreeNode> ite = childMap.values().iterator();
            while (ite.hasNext()) {
                TreeNode childNode = ite.next();
                clearUserAuthorization(userName, childNode);
            }
        }
    }

        /**
     * Gets the permission if it exists, else create the permission and returns
     * the id.
     *
     * @return the id of the permission
     */
    public int getOrAddPermissionIdInDatabase(Connection dbConnection, String resourceId, String action)
            throws UserStoreException {

        int permissionId = -1;
        PreparedStatement prepStmt1 = null;
        PreparedStatement prepStmt2 = null;
        ResultSet rs1 = null;
        ResultSet rs2 = null;
        try {
            prepStmt1 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.GET_PERMISSION));
            prepStmt1.setString(1, resourceId);
            prepStmt1.setString(2, action);

            rs1 = prepStmt1.executeQuery();

            if (rs1.next()) {
                permissionId = rs1.getInt(DefaultRealmConstants.COLUMN_NAME_ID);
            }
            if (prepStmt1 != null) {
                prepStmt1.close();
            }

            prepStmt2 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_PERMISSION));

            if (permissionId == -1) {
                // no permission so add it
                prepStmt2.setString(1, resourceId);
                prepStmt2.setString(2, action);
                prepStmt2.executeUpdate();
                dbConnection.commit();

                // now read the id again
                prepStmt2 = dbConnection.prepareStatement(sqlHelper
                        .getSQL(DefaultRealmConstants.GET_PERMISSION));
                prepStmt2.setString(1, resourceId);
                prepStmt2.setString(2, action);

                rs2 = prepStmt2.executeQuery();

                if (rs2.next()) {
                    permissionId = rs2.getInt(DefaultRealmConstants.COLUMN_NAME_ID);
                }

                dbConnection.commit();
            }

        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(null, rs1, rs2, prepStmt1, prepStmt2);
        }

        return permissionId;

    }

    protected void addAllowDenyUserPermissionInDatabase(String userName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt1 = null;
        PreparedStatement prepStmt2 = null;
        try {
            dbConnection = getConnection(dbConnection);
            DefaultCommons commons = new DefaultCommons();
            int userId = commons.getUserId(sqlHelper, dbConnection, userName);

            if (userId == -1) {
                throw new UserStoreException(Messages.getMessage("nullUser"));
            }

            int permissionId = this.getOrAddPermissionIdInDatabase(dbConnection, resourceId, action);

            prepStmt1 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_USER_PERMISSION));
            prepStmt1.setString(1, userName);
            prepStmt1.setString(2, resourceId);
            prepStmt1.setString(3, action);
            prepStmt1.executeUpdate();

            prepStmt2 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_USER_PERMISSION));
            prepStmt2.setInt(1, permissionId);
            prepStmt2.setShort(2, allow);
            prepStmt2.setInt(3, userId);
            prepStmt2.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt1, prepStmt2);
        }

    }

    protected void addAllowDenyRolePermissionInDatabase(String roleName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt1 = null;
        PreparedStatement prepStmt2 = null;
        try {
            dbConnection = getConnection(dbConnection);
            DefaultCommons commons = new DefaultCommons();
            int roleId = commons.getRoleId(sqlHelper, dbConnection, roleName);

            if (roleId == -1) {
                throw new UserStoreException(Messages.getMessage("nullRole"));
            }

            int permissionId = this.getOrAddPermissionIdInDatabase(dbConnection, resourceId, action);

            prepStmt1 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_ROLE_PERMISSION));
            prepStmt1.setString(1, roleName);
            prepStmt1.setString(2, resourceId);
            prepStmt1.setString(3, action);
            prepStmt1.executeUpdate();
            
            prepStmt2 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_ROLE_PERMISSION));
            prepStmt2.setInt(1, permissionId);
            prepStmt2.setShort(2, allow);
            prepStmt2.setInt(3, roleId);
            prepStmt2.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt1, prepStmt2);
        }
    }

    
    public void clearUserAuthorizationInDatabase(String userName, String resourceId, String action)
            throws UserStoreException {
        Connection dbConnection = null;
        ResultSet rs = null;
        PreparedStatement getPermission = null;
        PreparedStatement prepStmt = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
        
            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_USER_PERMISSION));
            prepStmt.setString(1, userName);
            prepStmt.setString(2, resourceId);
            prepStmt.setString(3, action);
            prepStmt.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt);
        }
    }

    public void clearRoleAuthorizationInDatabase(String roleName, String resourceId, String action)
            throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt = null;
        ResultSet rs = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_ROLE_PERMISSION));
            prepStmt.setString(1, roleName);
            prepStmt.setString(2, resourceId);
            prepStmt.setString(3, action);
            prepStmt.executeUpdate();

            dbConnection.commit();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt);
        }
    }

    public void clearResourceAuthorizationsInDatabase(String resourceId) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt = null;
        ResultSet rs = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ON_DELETE_PERMISSION_UM_USER_PERMISSIONS));
            prepStmt.setString(1, resourceId);
            prepStmt.executeUpdate();
            
            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ON_DELETE_PERMISSION_UM_ROLE_PERMISSIONS));
            prepStmt.setString(1, resourceId);
            prepStmt.executeUpdate();
            
            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_PERMISSION_ON_RESOURCE));
            prepStmt.setString(1, resourceId);
            prepStmt.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt);
        }
    }
    
    
    
    public void clearRoleAuthorizationInDatabase(String roleName) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt = null;
        try {
            dbConnection = getConnection(null);
            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ON_DELETE_ROLE_UM_ROLE_PERMISSIONS));
            prepStmt.setString(1, roleName);
            prepStmt.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error("Error! " + e.getMessage(), e);
            throw new UserStoreException("Error! " + e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt);
        }
        
    }
    
    public void clearUserAuthorizationInDatabase(String userName) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt = null;
        try {
            dbConnection = getConnection(null);
            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ON_DELETE_USER_UM_USER_PERMISSIONS));
            prepStmt.setString(1, userName);
            prepStmt.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error("Error! " + e.getMessage(), e);
            throw new UserStoreException("Error! " + e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt);
        }
        
    }
    

    private Connection getConnection(Connection dbConnection)
            throws SQLException, UserStoreException {
        dbConnection = dataSource.getConnection();
        if (dbConnection == null) {
            throw new UserStoreException(Messages.getMessage("null_connection"));
        }
        dbConnection.setAutoCommit(false);
        return dbConnection;
    }

}