/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core.def;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.axiom.om.util.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.AuthenticatorException;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.util.DefaultSQLHelperImpl;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.util.DatabaseUtil;

/**
 * @see org.wso2.usermanager.Authenticator
 */
public class DefaultAuthenticator implements Authenticator {

    private static Log log = LogFactory.getLog(DefaultAuthenticator.class);

    protected DataSource dataSource = null;

    protected DefaultRealm realm = null;

    protected SQLHelper sqlHelper = new DefaultSQLHelperImpl();

    protected String systemUserName;

    protected DefaultRealmConfig config = null;

    public DefaultAuthenticator(DataSource dataSource, DefaultRealmConfig config) {
        this.dataSource = dataSource;
        this.systemUserName = config.getSysteUserName();
        this.config = config;
    }

    /**
     * @see org.wso2.usermanager.Authenticator#authenticate(String, Object)
     */
    public boolean authenticate(String userName, Object credentials) throws AuthenticatorException {
        boolean isAuth = false;
        Connection dbConnection = null;
        ResultSet rs = null;
        PreparedStatement prepStmt = null;
        try {
            if (userName == null || credentials == null) {
                return false;
            }

            if (userName.equals(systemUserName)) {
                return false;
            }

            String password = ((String) credentials).trim();
            userName = userName.trim();
            if (userName.length() == 0 || password.length() == 0) {
                return false;
            }

            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new AuthenticatorException(Messages.getMessage("null_connection"));
            }

            dbConnection.setAutoCommit(false);

            prepStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.GET_USER));
            prepStmt.setString(1, userName);
            rs = prepStmt.executeQuery();
            if (rs.next()) {
                String digestInput = password;
                if (config.isSaltedPassword()) {
                    String salt = rs.getString(DefaultRealmConstants.COLUMN_NAME_SALT_VALUE);
                    digestInput = password + salt;
                }
                MessageDigest dgst = MessageDigest.getInstance(config.getDigestFunction());
                byte[] digestedPasword = dgst.digest(digestInput.getBytes());
                String finalValue = Base64.encode(digestedPasword);
                String dbCred = rs.getString(DefaultRealmConstants.COLUMN_NAME_CREDENTIAL);
                isAuth = finalValue.equals(dbCred);
            }
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new AuthenticatorException(Messages.getMessage("errorReadingFromUserStore"), e);
        } catch (NoSuchAlgorithmException e) {
            log.error(e.getMessage(), e);
            throw new AuthenticatorException(Messages.getMessage("errorCreatingPasswordDigest"), e);
        } finally {
            try {
                DatabaseUtil.closeAllConnections(dbConnection, rs, prepStmt);
            } catch (Exception e) {
                throw new AuthenticatorException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return isAuth;
    }
}
