package org.wso2.carbon.user.core.def.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;

import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.def.SQLHelper;


public class DefaultCommons {
    
    public int getUserId(SQLHelper sqlHelper, Connection dbConnection,
			String userName) throws UserStoreException {
		int id = -1;
		CacheManager cacheManager = CacheManager.create();
		Cache cache = cacheManager.getCache(UserCoreConstants.USER_ID_CACHE);
		//Cache does not exist, create new
		if (cache == null) {
			cache = new Cache(UserCoreConstants.USER_ID_CACHE,
					UserCoreConstants.MAX_OBJECTS_IN_CACHE, false, false,
					UserCoreConstants.TTL2_CACHE,
					UserCoreConstants.IDLE_TIME_IN_CACHE);
			cacheManager.addCache(cache);
		}

		//look for username in cache
		Object obj = cache.get(userName);
		if (obj != null) {
			//if username found, return the id
			Element element = (Element) obj;
			id = ((Integer)element.getValue()).intValue();
		} else {
			//username not in cache, do a database call & find id
			try {
				PreparedStatement getUserIdStmt = dbConnection
						.prepareStatement(sqlHelper
								.getSQL(DefaultRealmConstants.GET_USER_ID));
				getUserIdStmt.setString(1, userName);
				ResultSet rs = getUserIdStmt.executeQuery();
				if (rs.next()) {
					id = rs.getInt(sqlHelper
							.getColumnName(DefaultRealmConstants.COLUMN_ID_ID));
					//add username,id to cache for future calls
					Element element = new Element(userName, id);
					cache.put(element);					
				}
				getUserIdStmt.close();
			} catch (SQLException e) {
				throw new UserStoreException("errorReadingFromUserStore", e);
			}
		}
		return id;
	}

    public int getRoleId(SQLHelper sqlHelper, Connection dbConnection, String roleName) throws UserStoreException {
        int id = -1;
        
		CacheManager cacheManager = CacheManager.create();
		Cache cache = cacheManager.getCache(UserCoreConstants.ROLE_ID_CACHE);
		//Cache does not exist, create new
		if (cache == null) {
			cache = new Cache(UserCoreConstants.ROLE_ID_CACHE,
					UserCoreConstants.MAX_OBJECTS_IN_CACHE, false, false,
					UserCoreConstants.TTL2_CACHE,
					UserCoreConstants.IDLE_TIME_IN_CACHE);
			cacheManager.addCache(cache);
		}

		//look for rolename in cache
		Object obj = cache.get(roleName);
		if (obj != null) {
			//if rolename found, return the id
			Element element = (Element) obj;
			id = ((Integer)element.getValue()).intValue();
		} else {
	        try {
	            PreparedStatement prepStmt = dbConnection.prepareStatement(sqlHelper.getSQL(DefaultRealmConstants.GET_ID_FOR_ALL_ROLES));
	            ResultSet rs = prepStmt.executeQuery();
	            while (rs.next()) {
	                String role = rs.getString(sqlHelper.getColumnName(DefaultRealmConstants.COLUMN_ID_ROLE_NAME));
	                if(role.compareToIgnoreCase(roleName)==0){
	                    id = rs.getInt(sqlHelper.getColumnName(DefaultRealmConstants.COLUMN_ID_ID));
	        			//add rolename,id to cache for future calls
	        			Element element = new Element(roleName, id);
	        			cache.put(element);        
	                    
	                    return id;
	                }
	            }
	            prepStmt.close();
	        } catch (SQLException e) {
	            throw new UserStoreException("errorReadingFromUserStore", e);
	        }	
		}        
        return id;
    }

    public String[] getUserRoles(SQLHelper sqlHelper, Connection dbConnection, String userName) throws UserStoreException {
        String[] names = new String[0];
        try {
            PreparedStatement getUserRolesStmt = dbConnection
                    .prepareStatement(sqlHelper.getSQL(DefaultRealmConstants.GET_USER_ROLES));
            getUserRolesStmt.setString(1, userName);
            ResultSet rs = getUserRolesStmt.executeQuery();
            List lst = new LinkedList();
            String colName = sqlHelper.getColumnName(DefaultRealmConstants.COLUMN_ID_ROLE_NAME);
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getUserRolesStmt.close();
        } catch (SQLException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return names;
    }

}
