/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core.def.util;

import java.sql.PreparedStatement;

import javax.sql.DataSource;

import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.def.SQLHelper;


/**
 * Represents the user store. If you change this object it will change the way
 * the DefaultRealmBehaves.
 */
public class DefaultSQLHelperImpl implements SQLHelper {

    protected DataSource dataSource = null;

    public DefaultSQLHelperImpl(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public DefaultSQLHelperImpl() {

    }

    public String getColumnName(int columnID) {
        String sqlString = null;
        switch (columnID) {
        case DefaultRealmConstants.COLUMN_ID_ATTR_NAME:
            sqlString = DefaultRealmConstants.COLUMN_NAME_ATTR_NAME;
            break;
        case DefaultRealmConstants.COLUMN_ID_ATTR_VALUE:
            sqlString = DefaultRealmConstants.COLUMN_NAME_ATTR_VALUE;
            break;
        case DefaultRealmConstants.COLUMN_ID_ID:
            sqlString = DefaultRealmConstants.COLUMN_NAME_ID;
            break;
        case DefaultRealmConstants.COLUMN_ID_USER_NAME:
            sqlString = DefaultRealmConstants.COLUMN_NAME_USER_NAME;
            break;
        case DefaultRealmConstants.COLUMN_ID_ROLE_NAME:
            sqlString = DefaultRealmConstants.COLUMN_NAME_ROLE_NAME;
            break;
        case DefaultRealmConstants.COLUMN_ID_ROLE_ID:
            sqlString = DefaultRealmConstants.COLUMN_NAME_ROLE_ID;
            break;
        case DefaultRealmConstants.COLUMN_ID_USER_ID:
            sqlString = DefaultRealmConstants.COLUMN_NAME_USER_ID;
            break;
        case DefaultRealmConstants.COLUMN_ID_IS_ALLOWED:
            sqlString = DefaultRealmConstants.COLUMN_NAME_IS_ALLOWED;
            break;
        case DefaultRealmConstants.COLUMN_ID_CREDENTIAL:
            sqlString = DefaultRealmConstants.COLUMN_NAME_CREDENTIAL;
            break;
        case DefaultRealmConstants.COLUMN_ID_ACTION:
            sqlString = DefaultRealmConstants.COLUMN_NAME_ACTION;
            break;
        case DefaultRealmConstants.COLUMN_ID_RESOURCE_ID:
            sqlString = DefaultRealmConstants.COLUMN_NAME_RESOURCE_ID;
            break;
        default:
            break;
        }
        return sqlString;

    }

    public String getSQL(int sqlID) {
        String sqlString = null;
        switch (sqlID) {
        case DefaultRealmConstants.GET_USER:
            sqlString = DefaultRealmConstants.GET_USER_SQL;
            break;
        case DefaultRealmConstants.ADD_USER:
            sqlString = DefaultRealmConstants.ADD_USER_SQL;
            break;
        case DefaultRealmConstants.ADD_ROLE:
            sqlString = DefaultRealmConstants.ADD_ROLE_SQL;
            break;
        case DefaultRealmConstants.ADD_USER_ROLE:
            sqlString = DefaultRealmConstants.ADD_USER_ROLE_SQL;
            break;
        case DefaultRealmConstants.ADD_ROLE_ATTRIBUTE:
            sqlString = DefaultRealmConstants.ADD_ROLE_ATTRIBUTE_SQL;
            break;
        case DefaultRealmConstants.UPDATE_USER:
            sqlString = DefaultRealmConstants.UPDATE_USER_SQL;
            break;
        case DefaultRealmConstants.DELETE_USER:
            sqlString = DefaultRealmConstants.DELETE_USER_SQL;
            break;
        case DefaultRealmConstants.DELETE_ROLE:
            sqlString = DefaultRealmConstants.DELETE_ROLE_SQL;
            break;
        case DefaultRealmConstants.DELETE_USER_ROLE:
            sqlString = DefaultRealmConstants.DELETE_USER_ROLE_SQL;
            break;
        case DefaultRealmConstants.DELETE_ROLE_ATTRIBUTE:
            sqlString = DefaultRealmConstants.DELETE_ROLE_ATTRIBUTE_SQL;
            break;
        case DefaultRealmConstants.GET_ROLE:
            sqlString = DefaultRealmConstants.GET_ROLE_SQL;
            break;
        case DefaultRealmConstants.ADD_PERMISSION:
            sqlString = DefaultRealmConstants.ADD_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.ADD_ROLE_PERMISSION:
            sqlString = DefaultRealmConstants.ADD_ROLE_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.ADD_USER_PERMISSION:
            sqlString = DefaultRealmConstants.ADD_USER_PERMISSION_SQL;
            break;

        case DefaultRealmConstants.DELETE_PERMISSION_ON_RESOURCE:
            sqlString = DefaultRealmConstants.DELETE_PERMISSION_ON_RESOURCE_SQL;
            break;
        case DefaultRealmConstants.DELETE_USER_PERMISSION:
            sqlString = DefaultRealmConstants.DELETE_USER_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.DELETE_ROLE_PERMISSION:
            sqlString = DefaultRealmConstants.DELETE_ROLE_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.GET_PERMISSION:
            sqlString = DefaultRealmConstants.GET_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.GET_ROLE_AUTHORIZED:
            sqlString = DefaultRealmConstants.GET_ROLE_AUTHORIZED_SQL;
            break;
        case DefaultRealmConstants.GET_USER_AUTHORIZED:
            sqlString = DefaultRealmConstants.GET_USER_AUTHORIZED_SQL;
            break;
        case DefaultRealmConstants.GET_ROLES_FOR_RESOURCE:
            sqlString = DefaultRealmConstants.GET_ROLES_FOR_RESOURCE_SQL;
            break;
        case DefaultRealmConstants.GET_USERS_ON_RESOURCE:
            sqlString = DefaultRealmConstants.GET_USERS_ON_RESOURCE_SQL;
            break;
        case DefaultRealmConstants.GET_ROLE_PERMISSION:
            sqlString = DefaultRealmConstants.GET_ROLE_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.GET_USER_PERMISSION:
            sqlString = DefaultRealmConstants.GET_USER_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.GET_RESOURCE_PERMISSION:
            sqlString = DefaultRealmConstants.GET_RESOURCE_PERMISSION_SQL;
            break;
        case DefaultRealmConstants.GET_RESOURCE_PERMISSION_COUNT:
            sqlString = DefaultRealmConstants.GET_RESOURCE_PERMISSION_COUNT_SQL;
            break;
        case DefaultRealmConstants.GET_USER_ROLES:
            sqlString = DefaultRealmConstants.GET_USER_ROLES_SQL;
            break;
        case DefaultRealmConstants.GET_ROLE_ATTRIBUTES:
            sqlString = DefaultRealmConstants.GET_ROLE_ATTRIBUTES_SQL;
            break;
        case DefaultRealmConstants.GET_USER_ATTRIBUTES:
            sqlString = DefaultRealmConstants.GET_USER_ATTRIBUTES_SQL;
            break;
        case DefaultRealmConstants.GET_ATTRIBUTE_NAMES:
            sqlString = DefaultRealmConstants.GET_ATTRIBUTE_NAMES_SQL;
            break;
        case DefaultRealmConstants.GET_USERS_IN_ROLE:
            sqlString = DefaultRealmConstants.GET_USERS_IN_ROLE_SQL;
            break;
        case DefaultRealmConstants.GET_USERS_WITH_PROPERTY:
            sqlString = DefaultRealmConstants.GET_USERS_WITH_PROPERTY_SQL;
            break;
        case DefaultRealmConstants.GET_USERS_WITH_PROPERTY_VALUE:
            sqlString = DefaultRealmConstants.GET_USERS_WITH_PROPERTY_VALUE_SQL;
            break;
        case DefaultRealmConstants.GET_USERS_ALL:
            sqlString = DefaultRealmConstants.GET_USERS_ALL_SQL;
            break;
        case DefaultRealmConstants.GET_ROLES_ALL:
            sqlString = DefaultRealmConstants.GET_ROLES_ALL_SQL;
            break;
        case DefaultRealmConstants.GET_USER_ID:
            sqlString = DefaultRealmConstants.GET_USER_ID_SQL;
            break;
        case DefaultRealmConstants.GET_ROLES_PERMISSION_FOR_RESOURCE:
            sqlString = DefaultRealmConstants.GET_ROLES_PERMISSION_FOR_RESOURCE_SQL;
            break;
        case DefaultRealmConstants.GET_USER_PERMISSIONS:
            sqlString = DefaultRealmConstants.GET_USER_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.GET_ROLE_PERMISSIONS:
            sqlString = DefaultRealmConstants.GET_ROLE_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.GET_EXISTING_ROLE_PERMISSIONS:
            sqlString = DefaultRealmConstants.GET_EXISTING_ROLE_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.GET_EXISTING_USER_PERMISSIONS:
            sqlString = DefaultRealmConstants.GET_EXISTING_USER_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.GET_USER_COUNT:
            sqlString = DefaultRealmConstants.GET_USER_COUNT_SQL;
            break;
        case DefaultRealmConstants.UPDATE_USER_NAME:
            sqlString = DefaultRealmConstants.UPDATE_USER_NAME_SQL;
            break;
        case DefaultRealmConstants.UPDATE_ROLE_NAME:
            sqlString = DefaultRealmConstants.UPDATE_ROLE_NAME_SQL;
            break;
        case DefaultRealmConstants.GET_ID_FOR_ALL_ROLES:
            sqlString = DefaultRealmConstants.GET_ID_FOR_ALL_ROLES_SQL;
            break;
        case DefaultRealmConstants.ON_DELETE_USER_UM_USER_ROLES:
            sqlString = DefaultRealmConstants.ON_DELETE_USER_UM_USER_ROLES_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_USER_UM_USER_DATA:
            sqlString = DefaultRealmConstants.ON_DELETE_USER_UM_USER_DATA_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_USER_UM_BIN_DATA:
            sqlString = DefaultRealmConstants.ON_DELETE_USER_UM_BIN_DATA_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_USER_UM_USER_ATTRIBUTES:
            sqlString = DefaultRealmConstants.ON_DELETE_USER_UM_USER_ATTRIBUTES_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_USER_UM_USER_PERMISSIONS:
            sqlString = DefaultRealmConstants.ON_DELETE_USER_UM_USER_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.ON_DELETE_ROLE_UM_ROLE_ATTRIBUTES:
            sqlString = DefaultRealmConstants.ON_DELETE_ROLE_UM_ROLE_ATTRIBUTES_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_ROLE_UM_ROLE_PERMISSIONS:
            sqlString = DefaultRealmConstants.ON_DELETE_ROLE_UM_ROLE_PERMISSIONS_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_ROLE_UM_USER_ROLES:
            sqlString = DefaultRealmConstants.ON_DELETE_ROLE_UM_USER_ROLES_SQL;
            break;            
        case DefaultRealmConstants.ON_DELETE_PERMISSION_UM_USER_PERMISSIONS:
            sqlString = DefaultRealmConstants.ON_DELETE_PERMISSION_UM_USER_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.ON_DELETE_PERMISSION_UM_ROLE_PERMISSIONS:
            sqlString = DefaultRealmConstants.ON_DELETE_PERMISSION_UM_ROLE_PERMISSIONS_SQL;
            break;
        case DefaultRealmConstants.ADD_USER_OLD:
            sqlString = DefaultRealmConstants.ADD_USER_OLD_SQL;
            break;
        case DefaultRealmConstants.UPDATE_USER_OLD:
            sqlString = DefaultRealmConstants.UPDATE_USER_OLD_SQL;
            break;
        default:
            break;
        }
        return sqlString;
    }

    public String getUserProfileRelatedSQL(int sqlID) {
        String sqlString = null;
        switch (sqlID) {
        case DefaultRealmConstants.IS_USER_DATA:
            sqlString = DefaultRealmConstants.IS_USER_DATA_SQL;
            break;
        case DefaultRealmConstants.UPDATE_USER_DATA:
            sqlString = DefaultRealmConstants.UPDATE_USER_DATA_SQL;
            break;
        case DefaultRealmConstants.ADD_SINGLE_USER_DATA:
            sqlString = DefaultRealmConstants.ADD_SINGLE_USER_DATA_SQL;
            break;
        case DefaultRealmConstants.UPDATE_SINGLE_USER_DATA:
            sqlString = DefaultRealmConstants.UPDATE_SINGLE_USER_DATA_SQL;
            break;
        case DefaultRealmConstants.GET_USER_PROPERTY:
            sqlString = DefaultRealmConstants.GET_USER_PROPERTY_SQL;
            break;
        case DefaultRealmConstants.ADD_USER_PROPERTY:
            sqlString = DefaultRealmConstants.ADD_USER_PROPERTY_SQL;
            break;
        case DefaultRealmConstants.UPDATE_USER_PROPERTY:
            sqlString = DefaultRealmConstants.UPDATE_USER_PROPERTY_SQL;
            break;
        case DefaultRealmConstants.ADD_USER_DATA:
            sqlString = DefaultRealmConstants.ADD_USER_DATA_SQL;
            break;
        case DefaultRealmConstants.GET_USER_DATA_VALUES:
            sqlString = DefaultRealmConstants.GET_USER_DATA_VALUES_SQL;
            break;
        case DefaultRealmConstants.GET_USER_PROPERTY_VALUE:
            sqlString = DefaultRealmConstants.GET_USER_PROPERTY_VALUE_SQL;
            break;
        case DefaultRealmConstants.GET_USER_PROPERTY_NAMES:
            sqlString = DefaultRealmConstants.GET_USER_PROPERTY_NAMES_SQL;
            break;
        case DefaultRealmConstants.DELETE_CUSTOM_PROPERTY:
            sqlString = DefaultRealmConstants.DELETE_CUSTOM_PROPERTY_SQL;
            break;
        case DefaultRealmConstants.ADD_BINARY_CONTENT:
            sqlString = DefaultRealmConstants.ADD_BINARY_CONTENT_SQL;
            break;
        case DefaultRealmConstants.UPDATE_BINARY_CONTENT:
            sqlString = DefaultRealmConstants.UPDATE_BINARY_CONTENT_SQL;
            break;
        case DefaultRealmConstants.DELETE_BINARY_CONTENT:
            sqlString = DefaultRealmConstants.DELETE_BINARY_CONTENT_SQL;
            break;
        case DefaultRealmConstants.GET_BINARY_CONTENT:
            sqlString = DefaultRealmConstants.GET_BINARY_CONTENT_SQL;
            break;
        case DefaultRealmConstants.COUNT_BINARY_CONTENT:
            sqlString = DefaultRealmConstants.COUNT_BINARY_CONTENT_SQL;
            break;
        case DefaultRealmConstants.GET_PROFILES_IN_USER_DATA:
            sqlString = DefaultRealmConstants.GET_PROFILES_IN_USER_DATA_SQL;
            break;
        case DefaultRealmConstants.GET_PROFILES_IN_USER_DATA_FOR_USER:
            sqlString = DefaultRealmConstants.GET_PROFILES_IN_USER_DATA_FOR_USER_SQL;
            break;
        case DefaultRealmConstants.GET_PROFILES_IN_USER_ATTRIBUTES:
            sqlString = DefaultRealmConstants.GET_PROFILES_IN_USER_ATTRIBUTES_SQL;
            break;
        case DefaultRealmConstants.GET_PROFILES_IN_USER_ATTRIBUTES_FOR_USER:
            sqlString = DefaultRealmConstants.GET_PROFILES_IN_USER_ATTRIBUTES_FOR_USER_SQL;
            break;
        default:
            break;
        }
        return sqlString;
    }

    public void setUserIDInPrepStatement(String userName, int index, PreparedStatement prepStmt){
        
    }
}
