package org.wso2.carbon.user.core.delegating;

import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.internal.UserCoreServiceComponent;

public class DelegatingAccessControlAdmin extends DelegatingAuthorizer implements
        AccessControlAdmin {

    
    public DelegatingAccessControlAdmin(UserRealm defaultRealm) {
        super(defaultRealm);
        this.defaultRealm = defaultRealm;
    }

    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (defaultRealm.getUserStoreReader().isExistingRole(roleName)) {
            defaultRealm.getAccessControlAdmin().authorizeRole(roleName, resourceId, action);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingRole(roleName)) {
                    hybrealm.getAccessControlAdmin().authorizeRole(roleName, resourceId, action);
                }
            }
        }

    }

    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (defaultRealm.getUserStoreReader().isExistingRole(roleName)) {
            defaultRealm.getAccessControlAdmin().clearRoleAuthorization(roleName, resourceId,
                    action);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingRole(roleName)) {
                    hybrealm.getAccessControlAdmin().clearRoleAuthorization(roleName, resourceId,
                            action);
                }
            }
        }

    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {

        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            defaultRealm.getAccessControlAdmin().authorizeUser(userName, resourceId, action);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                    hybrealm.getAccessControlAdmin().authorizeUser(userName, resourceId, action);
                }
            }
        }

    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {

        defaultRealm.getAccessControlAdmin().clearResourceAuthorizations(resourceId);
        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            hybrealm.getAccessControlAdmin().clearResourceAuthorizations(resourceId);
        }

    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {

        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            defaultRealm.getAccessControlAdmin().clearUserAuthorization(userName, resourceId,
                    action);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                    hybrealm.getAccessControlAdmin().clearUserAuthorization(userName, resourceId,
                            action);
                }
            }
        }

    }

    
    public void clearRoleAuthorization(String roleName) throws UserStoreException {
        if (defaultRealm.getUserStoreReader().isExistingRole(roleName)) {
            defaultRealm.getAccessControlAdmin().clearRoleAuthorization(roleName);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingRole(roleName)) {
                    hybrealm.getAccessControlAdmin().clearRoleAuthorization(roleName);
                }
            }
        }
        
    }
    
    public void clearUserAuthorization(String userName) throws UserStoreException {
        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            defaultRealm.getAccessControlAdmin().clearUserAuthorization(userName);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                    hybrealm.getAccessControlAdmin().clearUserAuthorization(userName);
                }
            }
        }
        
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {

        defaultRealm.getAccessControlAdmin().copyAuthorizations(fromResourceId, toResourceId);
        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            hybrealm.getAccessControlAdmin().copyAuthorizations(fromResourceId, toResourceId);
        }

    }

    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (defaultRealm.getUserStoreReader().isExistingRole(roleName)) {
            defaultRealm.getAccessControlAdmin().denyRole(roleName, resourceId, action);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingRole(roleName)) {
                    hybrealm.getAccessControlAdmin().denyRole(roleName, resourceId, action);
                }
            }
        }

    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            defaultRealm.getAccessControlAdmin().denyUser(userName, resourceId, action);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                    hybrealm.getAccessControlAdmin().denyUser(userName, resourceId, action);
                }
            }
        }

    }


    

}
