package org.wso2.carbon.user.core.delegating;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.AuthenticatorException;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.internal.UserCoreServiceComponent;

public class DelegatingAuthenticator implements Authenticator {

    /** logger */
    private static Log log = LogFactory.getLog(DelegatingAuthenticator.class);

    private UserRealm defaultRealm = null;
    
    public DelegatingAuthenticator(UserRealm defaultRealm) {
        this.defaultRealm = defaultRealm;
    }

    public boolean authenticate(String userName, Object credentials) throws AuthenticatorException {
        try {
            boolean isAuthed = false;
            if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
                isAuthed = defaultRealm.getAuthenticator().authenticate(userName, credentials);
            } else {
                UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
                if (hybrealm != null) {
                    if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                        isAuthed = hybrealm.getAuthenticator().authenticate(userName, credentials);
                    }
                }
            }
            return isAuthed;
        } catch (UserStoreException e) {
            //not logging already logged
            throw new AuthenticatorException();
        }
    }

}
