package org.wso2.carbon.user.core.delegating;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserClaimsAdmin;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserProfileAdmin;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.internal.UserCoreServiceComponent;

public class  DelegatingRealm implements UserRealm{
    
    private DelegatingAccessControlAdmin acAdmin = null;
    private DelegatingAuthorizer authorizer = null;
    private DelegatingUserStoreReader usReader = null;
    private DelegatingUserStoreAdmin usAdmin = null;
    private DelegatingAuthenticator authenticator = null;
    private UserRealm defaultRealm = null;
    private static Log log = LogFactory.getLog(DelegatingRealm.class);

    public DelegatingRealm(UserRealm defaultRealm){
        this.defaultRealm = defaultRealm;
        acAdmin = new DelegatingAccessControlAdmin(defaultRealm);
        authorizer = new DelegatingAuthorizer(defaultRealm);
        usReader = new DelegatingUserStoreReader(defaultRealm);
        usAdmin = new DelegatingUserStoreAdmin(defaultRealm);
        authenticator = new DelegatingAuthenticator(defaultRealm);
    }
    
    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException{
        return acAdmin;
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return authenticator;
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return authorizer;
    }

    public Object getRealmConfiguration() throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return usAdmin;
    }    
    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return usReader;
    }

    public void init(Object configBean) throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }
    
    
    public void cleanUp() throws UserStoreException{
	
    }

	public UserClaimsAdmin getClaimsAdmin(String storeType) throws UserStoreException {
        UserRealm realm = null;

        try {
            if (UserCoreConstants.EXTERNAL_USERSTORE.equals(storeType)) {
                realm = UserCoreServiceComponent.getHybridRealm();
            } else {
                realm = defaultRealm;
            }
            if (realm != null) {
                // Underlying realm does not has any meaning to the store type -
                // so we can simply ignore it.
                return realm.getClaimsAdmin(null);
            } else {
                if (log.isDebugEnabled()) {
                    String message = "No matching realm found for the provided user store type ";
                    log.debug(message + storeType);
                }
            }
        } catch (Exception e) {
            log.error("Error occured while retrieving claim admin", e);
            throw new UserStoreException("Error occured while retrieving claim admin", e);
        }
        return null;
    }

    public UserProfileAdmin getUserProfileAdmin(String storeType) throws UserStoreException {
        UserRealm realm = null;

        try {
            if (UserCoreConstants.EXTERNAL_USERSTORE.equals(storeType)) {
                realm = UserCoreServiceComponent.getHybridRealm();
            } else {
                realm = defaultRealm;
            }
            if (realm != null) {
                // Underlying realm does not has any meaning to the store type -
                // so we can simply ignore it.
                return realm.getUserProfileAdmin(null);
            } else {
                if (log.isDebugEnabled()) {
                    String message = "No matching realm found for the provided user store type ";
                    log.debug(message + storeType);
                }
            }
        } catch (Exception e) {
            log.error("Error occured while retrieving profile admin", e);
            throw new UserStoreException("Error occured while retrieving profile admin", e);
        }
        return null;
    }

    public void setClaimsAdmin(UserClaimsAdmin claimsAdmin) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void setUserProfileAdmin(UserProfileAdmin profileAdmin) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }
    

}
