package org.wso2.carbon.user.core.delegating;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.internal.UserCoreServiceComponent;

public class DelegatingUserStoreReader implements UserStoreReader {

    protected UserRealm defaultRealm = null;

    public DelegatingUserStoreReader(UserRealm defaultRealm) {
        this.defaultRealm = defaultRealm;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String[] users = new String[0];
        if (defaultRealm.getUserStoreReader().isExistingRole(roleName)) {
            users = defaultRealm.getUserStoreReader().getUsersInRole(roleName);
        }

        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            if (hybrealm.getUserStoreReader().isExistingRole(roleName)) {
                users = hybrealm.getUserStoreReader().getUsersInRole(roleName);
            }
        }
        return users;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] names = new String[0];

        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            names = defaultRealm.getUserStoreReader().getUserRoles(userName);
        } else {
            UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
            if (hybrealm != null) {
                if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                    names = hybrealm.getUserStoreReader().getUserRoles(userName);
                }
            }
        }
        return names;
    }

    public String[] getAllRoleNames() throws UserStoreException {

        String[] roles = new String[0];

        roles = defaultRealm.getUserStoreReader().getAllRoleNames();

        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            String[] hybroles = hybrealm.getUserStoreReader().getAllRoleNames();
            List<String> lst = new ArrayList<String>();

            for (String role : roles) {
                lst.add(role);
            }

            for (String role : hybroles) {
                lst.add(role);
            }

            roles = lst.toArray(new String[lst.size()]);
        }
        return roles;
    }

    public String[] getAllUserNames() throws UserStoreException {

        String[] users = new String[0];

        users = defaultRealm.getUserStoreReader().getAllUserNames();

        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            String[] hybusers = hybrealm.getUserStoreReader().getAllUserNames();
            List<String> lst = new ArrayList<String>();

            for (String user : users) {
                lst.add(user);
            }

            for (String user : hybusers) {
                lst.add(user);
            }

            users = lst.toArray(new String[lst.size()]);
        }
        return users;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {

        String[] users = new String[0];

        if (defaultRealm.getUserStoreReader().isExistingRole(roleName)) {
            return true;
        }

        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            if (hybrealm.getUserStoreReader().isExistingRole(roleName)) {
                return true;
            }
        }
        return false;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {

        String[] users = new String[0];

        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return true;
        }

        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                return true;
            }
        }
        return false;
    }

    public Claim[] getUserClaimValues(String userName, String profileName)
            throws UserStoreException {

        if (defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader().getUserClaimValues(userName, profileName);
        }

        UserRealm hybrealm = UserCoreServiceComponent.getHybridRealm();
        if (hybrealm != null) {
            if (hybrealm.getUserStoreReader().isExistingUser(userName)) {
                return hybrealm.getUserStoreReader().getUserClaimValues(userName, profileName);
            }
        }
        return new Claim[0];
    }

    public byte[] getProfilePicture(String userName, String profileName) throws UserStoreException {

        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader().getProfilePicture(userName, profileName);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(userName)) {
            return realm.getUserStoreReader().getProfilePicture(userName, profileName);
        }
        return new byte[0];
    }

    public byte[] getUserBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {

        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader().getUserBinaryContent(userName, contentName,
                    profileName);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(userName)) {
            return realm.getUserStoreReader().getUserBinaryContent(userName, contentName,
                    profileName);
        }
        return new byte[0];
    }

    public String getUserClaimValue(String userName, String claim, String profileName)
            throws UserStoreException {

        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader()
                    .getUserClaimValue(userName, claim, profileName);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(userName)) {
            return realm.getUserStoreReader().getUserClaimValue(userName, claim, profileName);
        }
        return null;
    }

    public Map<String, String> getUserProperties(String userName, String[] propertyNames,
            String profileName) throws UserStoreException {

        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader().getUserProperties(userName, propertyNames,
                    profileName);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(userName)) {
            return realm.getUserStoreReader().getUserProperties(userName, propertyNames,
                    profileName);
        }
        return new HashMap<String, String>();
    }

    public String getUserPropertyValue(String userName, String propertyName, String profileName)
            throws UserStoreException {

        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader().getUserPropertyValue(userName, propertyName,
                    profileName);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(userName)) {
            return realm.getUserStoreReader().getUserPropertyValue(userName, propertyName,
                    profileName);
        }
        return null;
    }

    public Map<String, String> getUserClaimValues(String userName, String[] claims,
            String profileName) throws UserStoreException {

        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(userName)) {
            return defaultRealm.getUserStoreReader().getUserClaimValues(userName, claims,
                    profileName);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(userName)) {
            return realm.getUserStoreReader().getUserClaimValues(userName, claims, profileName);
        }
        return new HashMap<String, String>();
    }

    public String[] getAllProfileNames() throws UserStoreException {
        return defaultRealm.getUserStoreReader().getAllProfileNames();
    }

    public String[] getProfileNames(String username) throws UserStoreException {
        if (defaultRealm != null && defaultRealm.getUserStoreReader().isExistingUser(username)) {
            return defaultRealm.getUserStoreReader().getProfileNames(username);
        }

        UserRealm realm = UserCoreServiceComponent.getHybridRealm();
        if (realm != null && realm.getUserStoreReader().isExistingUser(username)) {
            return realm.getUserStoreReader().getProfileNames(username);
        }
        return new String[0];
    }

    // ********************* not going to implement
    // ****************************************************

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public String getDefaultUserProfileName(String userName) throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public Map<String, String> getRoleProperties(String roleName) throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public Map<String, String> getUserProperties(String userName, String profileName)
            throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public Map<String, String> getUserProperties(String userName) throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

    public boolean isExistingUserProfile(String userName, String profileName)
            throws UserStoreException {
        throw new UserStoreException("NotImplemented");
    }

}
