package org.wso2.carbon.user.core.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.DefaultAccessControlAdmin;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.util.DatabaseUtil;

public class HybridAccessControlAdmin extends HybridAuthorizer implements AccessControlAdmin {
    private static Log log = LogFactory.getLog(DefaultAccessControlAdmin.class);

    public HybridAccessControlAdmin(DataSource dataSource, String algo, UserRealm realm, HybridRealmConfig config) {
        super(dataSource, algo, realm, config);
    }

    public HybridAccessControlAdmin(DataSource dataSource, UserRealm realm, HybridRealmConfig config) {
        super(dataSource, DefaultRealmConfig.PERMISSION_BLOCK_FIRST, realm, config);
    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        defAdmin.authorizeUserInTree(userName, resourceId, action);
        // Persist the permission
        addAllowDenyUserPermission(userName, resourceId, action, UserCoreConstants.ALLOW);
    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        defAdmin.denyUserInTree(userName, resourceId, action);
        // Persist the permission
        addAllowDenyUserPermission(userName, resourceId, action, UserCoreConstants.DENY);
    }

    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        defAdmin.authorizeRoleInTree(roleName, resourceId, action);
        // Persist the permission
        addAllowDenyRolePermission(roleName, resourceId, action, UserCoreConstants.ALLOW);
    }

    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        defAdmin.denyRoleInTree(roleName, resourceId, action);
        // Persist the permission
        addAllowDenyRolePermission(roleName, resourceId, action, UserCoreConstants.DENY);
    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        defAdmin.clearResourceAuthorizations(resourceId);
    }
    
    public void copyAuthorizations(String fromResourceId, String toResourceId)
        throws UserStoreException {
        // TODO should not be required anymore - correct?
    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {
       defAdmin.clearUserAuthorization(userName, resourceId, action);
    }

    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {
        defAdmin.clearRoleAuthorization(roleName, resourceId, action);
    }

    
    public void clearRoleAuthorization(String roleName) throws UserStoreException {
        defAdmin.clearRoleAuthorization(roleName);
    }

    public void clearUserAuthorization(String userName) throws UserStoreException {
        log.error("This operation is not allowed"); //users are not deleted
    }
    
    protected void addAllowDenyRolePermission(String roleName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt1 = null;
        PreparedStatement prepStmt2 = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            int permissionId = defAdmin.getOrAddPermissionIdInDatabase(dbConnection, resourceId, action);

            prepStmt1 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_ROLE_PERMISSION));
            prepStmt1.setString(1, roleName);
            prepStmt1.setString(2, resourceId);
            prepStmt1.setString(3, action);
            prepStmt1.executeUpdate();
            
            prepStmt2 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_ROLE_PERMISSION));
            prepStmt2.setInt(1, permissionId);
            prepStmt2.setShort(2, allow);
            prepStmt2.setString(3, roleName);
            prepStmt2.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt1, prepStmt2);
        }
    }

    protected void addAllowDenyUserPermission(String userName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt1 = null;
        PreparedStatement prepStmt2 = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            int permissionId = defAdmin.getOrAddPermissionIdInDatabase(dbConnection, resourceId, action);

            prepStmt1 = dbConnection.prepareStatement(sqlHelper.getSQL(DefaultRealmConstants.DELETE_USER_PERMISSION));
            prepStmt1.setString(1, userName);
            prepStmt1.setString(2, resourceId);
            prepStmt1.setString(3, action);
            prepStmt1.executeUpdate();

            prepStmt2 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_USER_PERMISSION));
            prepStmt2.setInt(1, permissionId);
            prepStmt2.setShort(2, allow);
            prepStmt2.setString(3, userName);
            prepStmt2.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, prepStmt1, prepStmt2);
        }

    }

}
