package org.wso2.carbon.user.core.hybrid;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.sql.DataSource;

import net.sf.ehcache.CacheManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultAccessControlAdmin;
import org.wso2.carbon.user.core.def.SQLHelper;

public class HybridAuthorizer implements Authorizer {

    /** logger */
    private static Log log = LogFactory.getLog(HybridAuthorizer.class);

    /** Algorithm to authorize users */
    private String algorithm = null;

    private HybridRealmConfig config = null;
    protected SQLHelper sqlHelper = null;
    protected DefaultAccessControlAdmin defAdmin = null;
    protected DataSource dataSource = null;
    protected HybridInternalRoleAdmin internalRoleAdmin= null; 
    
    protected CacheManager cacheManager = null;

    public HybridAuthorizer(DataSource dataSource, String algo, UserRealm realm, HybridRealmConfig config) {
        this.dataSource = dataSource;
        sqlHelper = new HybridSQLHelper();
        defAdmin = new DefaultAccessControlAdmin(dataSource, algo, sqlHelper,
                HybridRealm.DUMMY_SYSTEM_USER);
        this.algorithm = algo;
        this.config = config;
        this.internalRoleAdmin = new HybridInternalRoleAdmin(dataSource, realm, config);
    }

    public String[] getAllowedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        return defAdmin.getAllowedRolesForResource(resourceId, action);
    }

    public String[] getAllowedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        return defAdmin.getAllowedUsersForResource(resourceId, action);
    }

    public String[] getDeniedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        return defAdmin.getDeniedRolesForResource(resourceId, action);
    }

    public String[] getDeniedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        return defAdmin.getDeniedUsersForResource(resourceId, action);
    }

    public boolean isRoleAuthorized(String roleName, String resourceId, String action)
            throws UserStoreException {
        return defAdmin.isRoleAuthorized(roleName, resourceId, action);
    }

    public boolean isUserAuthorized(String userName, String resourceId, String action)
            throws UserStoreException {
        
        String roles[] = new String[0];
        UserStoreReader reader = config.getUserStoreReader();
        try {
            roles = reader.getUserRoles(userName);
        } catch (UserStoreException e) {
           log.error("Unable to retrieve roles for authorization" + e.getMessage(), e);
           //do not throw exception. we need to continute
        }

        // get internal roles defined by external roles
        String[] intnlRoles = internalRoleAdmin.getInternalRolesOfUser(userName);
        int length = intnlRoles.length;
        
        if (length > 0) {
            List<String> roleList = new ArrayList<String>();

            for (String role : roles) {
                roleList.add(role);
            }

            for (String role : intnlRoles) {
                roleList.add(role);
            }
            roles = roleList.toArray(new String[roleList.size()]);
        }       
        
        if(log.isDebugEnabled()){
            log.debug("Roles of the users are :: " + Arrays.toString(roles));
        }
        
        boolean isAuthed = defAdmin.isUserAuthorizedFromGivenRoles(userName, roles, resourceId, action);
        
        if(log.isDebugEnabled()){
            log.debug("Authorized from given roles :: " + isAuthed);
        }
        
        return isAuthed;
    }
    
    public void populatePermissionTreeFromDB() throws UserStoreException {
        defAdmin.populatePermissionTreeFromDB();
    }

}
