package org.wso2.carbon.user.core.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.def.DefaultUserStoreAdmin;
import org.wso2.carbon.user.core.def.SQLHelper;
import org.wso2.carbon.user.core.i18n.Messages;

public class HybridInternalRoleAdmin{
    
    private static Log log = LogFactory.getLog(HybridInternalRoleAdmin.class);
     
    private DefaultUserStoreAdmin defUSAdmin = null;
    protected SQLHelper sqlHelper = null;
    protected DataSource dataSource = null;
    private UserRealm realm = null;
    private HybridRealmConfig config = null;
    
    public HybridInternalRoleAdmin(DataSource dataSource, UserRealm realm, HybridRealmConfig config){
        this.dataSource = dataSource;
        sqlHelper = new HybridSQLHelper();
        this.realm = realm;
        defUSAdmin = new DefaultUserStoreAdmin(dataSource, sqlHelper, HybridRealm.DUMMY_SYSTEM_USER);
        this.config = config;
    }
    
    
    public String[] getInternalRolesOfUser(String userName) throws UserStoreException{
       String[] roles = defUSAdmin.getUserRoles(userName);
       String[] allRoles = new String[roles.length+1];
       int i = 0;
       for(String role : roles){
           allRoles[i] = role;
           i++;
       }
       allRoles[i] = UserCoreConstants.EVERYONE_ROLE;
       return allRoles;
    }
    
    public String[] getUsersInInternalRole(String roleName) throws UserStoreException{
        if(UserCoreConstants.EVERYONE_ROLE.equals(roleName)){
            return config.getUserStoreReader().getAllUserNames();
        }
        return defUSAdmin.getUsersInRole(roleName);
    }
    
    public void addInternalRole(String roleName) throws UserStoreException {
        if(UserCoreConstants.EVERYONE_ROLE.equals(roleName)){
            throw new  UserStoreException("User already exists.");
        }
        defUSAdmin.addRole(roleName);
    }
    
    public String[] getInternalRoles()  throws UserStoreException {
        String[] roles = defUSAdmin.getAllRoleNames();
        String[] allRoles = new String[roles.length+1];
        int i = 0;
        for(String role : roles){
            allRoles[i] = role;
            i++;
        }
        allRoles[i] = UserCoreConstants.EVERYONE_ROLE;
        return allRoles;
    }
    
    public void addUserToInternalRole(String userName, String roleName) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement addUserToRoleStmt = null;
        try {
            
            if(UserCoreConstants.EVERYONE_ROLE.equals(roleName)){
                throw new UserStoreException("The user is already in everyone role");
            }
            
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
        
            addUserToRoleStmt = dbConnection
                    .prepareStatement(sqlHelper.getSQL(DefaultRealmConstants.ADD_USER_ROLE));
            addUserToRoleStmt.setString(1, userName);
            addUserToRoleStmt.setString(2, roleName);
            addUserToRoleStmt.executeUpdate();
            dbConnection.commit();
            addUserToRoleStmt.close();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(Messages.getMessage("errorAddingUserToRole"), e);
        } finally {
            try {
                if (addUserToRoleStmt!=null) {
                    addUserToRoleStmt.close();
                }
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        } 
    }

    public void removeUserFromInternalRole(String userName, String roleName) throws UserStoreException {
        if(UserCoreConstants.EVERYONE_ROLE.equals(roleName)){
            throw new  UserStoreException("Removing users from everyone role is not allowed.");
        }
        defUSAdmin.removeUserFromRole(userName, roleName);
    }
    
    public void deleteInternalRole(String roleName) throws UserStoreException {
        Connection dbConnection = null;
        PreparedStatement prepStmt2 = null;
        PreparedStatement prepStmt3 = null;
        PreparedStatement prepStmt4 = null;
        try {
            if(UserCoreConstants.EVERYONE_ROLE.equals(roleName)){
                throw new  UserStoreException("Deleting everyone role is not allowed.");
            }
            
            this.realm.getAccessControlAdmin().clearRoleAuthorization(roleName);
            dbConnection = dataSource.getConnection();
            dbConnection.setAutoCommit(false);
            
            prepStmt3 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ON_DELETE_ROLE_UM_USER_ROLES));
            prepStmt3.setString(1, roleName);
            prepStmt3.executeUpdate();
            prepStmt3.close();
            prepStmt4 = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_ROLE));
            prepStmt4.setString(1, roleName);
            prepStmt4.executeUpdate();
            dbConnection.commit();
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            try {
                if (prepStmt2 != null) {
                    prepStmt2.close();
                }
                if (prepStmt3 != null) {
                    prepStmt3.close();
                }
                if (prepStmt4 != null) {
                    prepStmt4.close();
                }
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        
    }   

}
