package org.wso2.carbon.user.core.hybrid;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserClaimsAdmin;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserProfileAdmin;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.profile.BaseProfileAdmin;


public class HybridRealm implements UserRealm {

    private static Log log = LogFactory.getLog(HybridRealm.class);
    
    private Authenticator authenticator = null;

    private Authorizer authorizer = null;
    private AccessControlAdmin aclAdmin = null;

    private UserStoreAdmin usAdmin = null;
    private UserStoreReader usReader = null;

    private DataSource dataSource = null;
    private HybridRealmConfig config = null;
    private HybridInternalRoleAdmin internalRoleAdmin = null;
    private UserClaimsAdmin claimsAdmin = null;
    private UserProfileAdmin profileAdmin = null;
    
    public static final String DUMMY_SYSTEM_USER = "org.wso2.carbon.user.dummy";
    
    public void init(Object configBean) throws UserStoreException {

        if (!(configBean instanceof HybridRealmConfig)) {
            return;
        }
        HybridRealmConfig config = (HybridRealmConfig) configBean;

        BasicDataSource basicDataSource = (BasicDataSource)config.getDataSource();
        if (basicDataSource == null) {
            basicDataSource = new BasicDataSource();
            basicDataSource.setUrl(config.getConnectionURL());
            basicDataSource.setDriverClassName(config.getDriverName());
            basicDataSource.setUsername(config.getConnectionUserName());
            basicDataSource.setPassword(config.getConnectionPassword());
        }
        this.config = config;
        dataSource = (DataSource) basicDataSource;

        //claimsAdmin = new HybridClaimsAdmin(config.getsfsddfs);
        //profileAdmin = new HybridProfileAdmin();
        if(config.getAuthenticator() == null)
            throw new UserStoreException("authenticatorMustExist");
        
        authenticator = config.getAuthenticator();
        
        usAdmin = config.getUserStoreAdmin();
        UserStoreReader reader = config.getUserStoreReader();
        claimsAdmin = new HybridClaimsAdmin(config.getClaimMapping(), dataSource);
        profileAdmin = new BaseProfileAdmin(config.getProfileConfigs(), dataSource,
                UserCoreConstants.EXTERNAL_USERSTORE);
        usReader = new HybridUserStoreReader(dataSource, reader, claimsAdmin, profileAdmin);

        authorizer = new HybridAuthorizer(dataSource, DefaultRealmConfig.PERMISSION_BLOCK_FIRST, this,
                config);
        ((HybridAuthorizer) authorizer).populatePermissionTreeFromDB();
        aclAdmin = new HybridAccessControlAdmin(dataSource, this, config);

        internalRoleAdmin = new HybridInternalRoleAdmin(dataSource, this, config);
        
    }

    public void cleanUp() throws UserStoreException{
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            Statement stmt = dbConnection.createStatement();
            int count = 0;
            count = stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_USER_ROLE_TABLE);
            if(log.isDebugEnabled()) log.debug("Deleted " + count);
            count = stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_ROLE_PERMISSION_TABLE);
            if(log.isDebugEnabled()) log.debug("Deleted " + count);
            count = stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_USER_PERMISSION_TABLE);
            if(log.isDebugEnabled()) log.debug("Deleted " + count);
            count = stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_PERMISSION_TABLE);
            if(log.isDebugEnabled()) log.debug("Deleted " + count);
            count = stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_ROLE_TABLE);
            if(log.isDebugEnabled()) log.debug("Deleted " + count);
            dbConnection.commit();
            
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }
    
    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return aclAdmin;
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return authenticator;
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return authorizer;
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return this.usAdmin;
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return this.usReader;
    }

    public HybridInternalRoleAdmin getInternalRoleAdmin() throws UserStoreException {
        return internalRoleAdmin;
    }
    
    public Object getRealmConfiguration() throws UserStoreException {
        HybridRealmConfig retConfig = null;
        if (config == null) {
            retConfig = new HybridRealmConfig();
        }
        return retConfig;
    }

    
    public DataSource getDataSource() {
        return dataSource;
    }

    @Override
    protected void finalize() throws Throwable {
        dataSource = null;
        super.finalize();
    }
    
    public UserProfileAdmin getUserProfileAdmin(String storeName) throws UserStoreException{
        return profileAdmin;
    }
    
    public void setUserProfileAdmin(UserProfileAdmin profileAdmin) throws UserStoreException{
        this.profileAdmin = (BaseProfileAdmin)profileAdmin;
    }
    
    public UserClaimsAdmin getClaimsAdmin(String storeName) throws UserStoreException{
        return claimsAdmin;
    }
    
    public void setClaimsAdmin(UserClaimsAdmin claimsAdmin) throws UserStoreException{
        this.claimsAdmin = claimsAdmin;
    }
    
}
