package org.wso2.carbon.user.core.hybrid;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserClaimsAdmin;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserProfileAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultUserStoreAdmin;
import org.wso2.carbon.user.core.def.DefaultUserStoreReader;
import org.wso2.carbon.user.core.def.SQLHelper;

public class HybridUserStoreReader extends DefaultUserStoreReader {

    /** logger */
    private static Log log = LogFactory.getLog(HybridUserStoreReader.class);

    protected UserStoreReader externalReader = null;

    public HybridUserStoreReader(DataSource dataSource, UserStoreReader reader,
            UserClaimsAdmin claimAdmin, UserProfileAdmin profileAdmin) {
        super(dataSource, (SQLHelper) new HybridSQLHelper(), claimAdmin, profileAdmin,
                HybridRealm.DUMMY_SYSTEM_USER);
        externalReader = reader;
    }
    
    public String[] getAllRoleNames() throws UserStoreException {
        String[] roleNames = externalReader.getAllRoleNames();
        String[] internalRoles = super.getAllRoleNames();
        if (internalRoles.length > 0) {
            List<String> totalRoles = new ArrayList<String>();
            for (String name : roleNames) {
                totalRoles.add(name);
            }
            for (String name : internalRoles) {
                totalRoles.add(name);
            }
            roleNames = totalRoles.toArray(new String[totalRoles.size()]);
        }
        return roleNames;
    }

    public String[] getAllUserNames() throws UserStoreException {
        String[] userNames = externalReader.getAllUserNames();
        return userNames;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        String[] roleNames = new String[0];
        try {
            roleNames = externalReader.getAllRoleNames();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            // do not throw exception. we need to continue
        }

        for (String name : roleNames) {
            if (name.equals(roleName)) {
                return true;
            }
        }

        roleNames = super.getAllRoleNames();
        for (String name : roleNames) {
            if (name.equals(roleName)) {
                return true;
            }
        }

        return false;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        boolean isExistingUser = false;
        try {
            isExistingUser = externalReader.isExistingUser(userName);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            // do not throw exception. we need to continue
        }
        return isExistingUser;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] userRoles = null;
        userRoles = externalReader.getUserRoles(userName);

        // get internal roles defined by external roles
        DefaultUserStoreAdmin defUSAdmin = new DefaultUserStoreAdmin(dataSource, sqlHelper,
                HybridRealm.DUMMY_SYSTEM_USER);
        String[] intnlRoles = defUSAdmin.getUserRoles(userName);
        String[] roles = null;

        if (intnlRoles == null || intnlRoles.length == 0) {
            if (userRoles == null) {
                return new String[0];
            }
            return userRoles;
        }
        roles = new String[intnlRoles.length + userRoles.length];
        int i = 0;
        for (int j = 0; j < userRoles.length; j++) {
            roles[i++] = userRoles[j];
        }
        for (int j = 0; j < intnlRoles.length; j++) {
            roles[i++] = intnlRoles[j];
        }
        return roles;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String[] roleNames = externalReader.getAllRoleNames();

        for (String name : roleNames) {
            if (name.equals(roleName)) {
                return externalReader.getUsersInRole(roleName);
            }
        }

        roleNames = super.getAllRoleNames();
        for (String name : roleNames) {
            if (name.equals(roleName)) {
                return super.getUsersInRole(roleName);
            }
        }

        return new String[0];

    }

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException {
        return externalReader.listUsers(filter, maxItemLimit);
    }

    public Map<String, String> getUserProperties(String userName, String[] propertyNames,
            String profileName) throws UserStoreException {
        return externalReader.getUserProperties(userName, propertyNames, profileName);
    }

    public String getUserPropertyValue(String userName, String propertyName, String profileName)
            throws UserStoreException {
        return externalReader.getUserPropertyValue(userName, propertyName, profileName);
    }

    public String[] getAllProfileNames() throws UserStoreException {
        String[] profiles = externalReader.getAllProfileNames();
        if (profiles == null || profiles.length == 0) {
            profiles = new String[] { UserCoreConstants.DEFAULT_PROFILE };
        }
        return profiles;
    }

    public String[] getProfileNames(String username) throws UserStoreException {
        String[] profiles = externalReader.getProfileNames(username);
        if (profiles == null || profiles.length == 0) {
            profiles = new String[] { UserCoreConstants.DEFAULT_PROFILE };
        }
        return profiles;
    }

    // ***************************** Not going to implement
    // ************************

    public byte[] getUserBinaryContent(String userName, String contentName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public byte[] getProfilePicture(String userName, String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public byte[] getUserBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

}
