package org.wso2.carbon.user.core.hybrid.actdir;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.AuthenticatorException;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.util.JNDIUtil;

/**
 * 
 * @see org.wso2.usermanager.Authenticator
 * 
 */
public class ActDirAuthenticator implements Authenticator {

    private static Log log = LogFactory.getLog(ActDirAuthenticator.class);

    private ActDirRealmConfig config = null;

    public ActDirAuthenticator(ActDirRealmConfig config) {
        this.config = config;
    }

    /**
     * @see org.wso2.usermanager.Authenticator#authenticate(String, Object)
     */
    public boolean authenticate(String userName, Object credentials) throws AuthenticatorException {

        if (userName == null || credentials == null) {
            return false;
        }
        userName = userName.trim();
        String password = (String) credentials;
        password = password.trim();

        if (userName.equals("") || password.equals("")) {
            return false;
        }

        boolean bValue = false;
        String[] userNames = new String[0];
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

        String searchFilter = "(&(objectClass=user)(sAMAccountName=" + userName + "))";
        String searchBase = config.getSearchBase();

        try {
            String name = ActDirUtil.getUserCN(config, userName);
            if (name != null) {
                bValue = this.bindAsUser(config.getContext(), name, (String) credentials);
            }

        } catch (Exception e) {
            log.error("Unable to authenticate", e);
            throw new AuthenticatorException("Failed to Authenticate");
        }
        return bValue;
    }

    /**
     * Check credentials by binding to the directory as the user
     * 
     * @param context
     *                The directory context
     * @param user
     *                The User to be authenticated
     * @param credentials
     *                Authentication credentials
     * 
     * @exception NamingException
     *                    if a directory server error occurs
     */
    protected boolean bindAsUser(DirContext context, String dn, String credentials)
            throws AuthenticatorException, NamingException, UserStoreException {
        boolean isAuthed = false;

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, config.getDriverName());
        env.put(Context.SECURITY_PRINCIPAL, dn);
        env.put(Context.SECURITY_CREDENTIALS, credentials);
        env.put(Context.PROVIDER_URL, config.getConnectionURL());
        env.put(Context.SECURITY_AUTHENTICATION, "simple");

        DirContext cxt = null;
        try {
            cxt = new InitialLdapContext(env, null);
            isAuthed = true;
        } catch (NamingException e) {
            log.error("Failed login attempt by " + dn);
            log.debug(e.getMessage(), e);
           // Don't throw exception. User is not authenticated
        } finally {
            JNDIUtil.closeContext(cxt);
        }
        return isAuthed;
    }

}
