package org.wso2.carbon.user.core.hybrid.actdir;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.UserClaimsAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.util.JNDIUtil;

public class ActDirUserStoreReader implements UserStoreReader {

    protected DataSource dataSource = null;

    private static Log log = LogFactory.getLog(ActDirUserStoreReader.class);

    private ActDirRealmConfig config = null;

    public ActDirUserStoreReader(ActDirRealmConfig config) {
        this.config = config;
    }

    public String[] getAllRoleNames() throws UserStoreException {
        if (log.isDebugEnabled()) {
            log.debug("Retrieving roles begin at ::" + System.currentTimeMillis());
        }
        String[] roleNames = new String[0];
        String searchFilter = "(objectcategory=group)";
        try {
            NamingEnumeration answer = searchAD(searchFilter, new String[] {});
            List<String> list = new ArrayList<String>();
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                String name = sr.getName().substring(3);
                int index = name.indexOf(",");
                name = name.substring(0, index);
                list.add(name);
            }

            roleNames = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        if (log.isDebugEnabled()) {
            log.debug("Retrieving roles end at ::" + System.currentTimeMillis());
        }
        return roleNames;
    }

    public String[] getAllUserNames() throws UserStoreException {
        String[] userNames = new String[0];
        String searchFilter = "(&(&(objectCategory=person)(objectClass=user)(!(objectClass=computer))))";
        String returnedAtts[] = { "sAMAccountName" };
        try {
            NamingEnumeration answer = this.searchAD(searchFilter, returnedAtts);
            List<String> list = new ArrayList<String>();
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                String name = (String) sr.getAttributes().get("sAMAccountName").get();
                list.add(name);
            }
            userNames = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return userNames;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        boolean isExistingUser = false;
        String searchFilter = "(&(objectCategory=person)(sAMAccountName=" + userName + "))";
        NamingEnumeration answer = searchAD(searchFilter, new String[] {});
        while (answer.hasMoreElements()) {
            isExistingUser = true;
            break;
        }
        return isExistingUser;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] userRoles = new String[0];
        String searchFilter = "(&(objectCategory=person)(sAMAccountName=" + userName + "))";
        int totalResults = 0;
        String returnedAtts[] = { "memberOf" };
        try {
            NamingEnumeration answer = searchAD(searchFilter, returnedAtts);
            int count = 0;
            List<String> list = new ArrayList<String>();
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();

                if (count > 0) {
                    log.error("More than one user exist with name :" + userName);
                }
                count++;
                // Print out the groups

                Attributes attrs = sr.getAttributes();
                if (attrs != null) {
                    try {
                        for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
                            Attribute attr = (Attribute) ae.next();
                            for (NamingEnumeration e = attr.getAll(); e.hasMore(); totalResults++) {
                                String value = e.next().toString();
                                int index = value.indexOf(",");
                                value = value.substring(3, index);
                                list.add(value);
                            }

                        }
                    } catch (NamingException e) {
                        log.error(e.getMessage(), e);
                    }

                }
            }
            userRoles = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return userRoles;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String[] userNames = new String[0];
        try {
            String searchFilter = "(&((objectcategory=group))(CN=" + roleName + "))";
            String returnedAtts[] = { "member" };
            NamingEnumeration answer = searchAD(searchFilter, returnedAtts);
            List<String> list = new ArrayList<String>();
            int totalResults = 0;
            int count = 0;
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                if (count > 0) {
                    log.error("More than one user exist with name :" + roleName);
                }
                count++;
                Attributes attrs = sr.getAttributes();
                if (attrs != null) {
                    try {
                        for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
                            Attribute attr = (Attribute) ae.next();
                            for (NamingEnumeration e = attr.getAll(); e.hasMore(); totalResults++) {
                                String value = e.next().toString();
                                int index = value.indexOf(",");
                                value = value.substring(3, index);
                                value = ActDirUtil.getUserLoginName(config, value);
                                list.add(value);
                            }
                        }
                    } catch (NamingException e) {
                        log.error(e.getMessage(), e);
                        throw new UserStoreException(e.getMessage(), e);
                    }

                }
            }
            userNames = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return userNames;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        boolean bFound = false;
        String[] names = this.getAllRoleNames();
        for (String name : names) {
            if (name.equals(roleName)) {
                bFound = true;
                break;
            }
        }
        return bFound;
    }

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException {
        String[] userNames = new String[0];
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        searchCtls.setCountLimit(maxItemLimit);

        if (filter.contains("?")) {
            throw new UserStoreException("invalid character for act dir serch");
        }

        String searchFilter = "(&(&(&(objectCategory=person)(objectClass=user)(!(objectClass=computer))(sAMAccountName="
                + filter + "))))";
        String returnedAtts[] = { "sAMAccountName" };
        try {
            NamingEnumeration answer = this.searchAD(searchFilter, returnedAtts);
            List<String> list = new ArrayList<String>();
            int i = 0;
            while (answer.hasMoreElements() && i < maxItemLimit) {
                SearchResult sr = (SearchResult) answer.next();
                String name = (String) sr.getAttributes().get("sAMAccountName").get();
                list.add(name);
                i++;
            }
            userNames = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return userNames;
    }

    public Map<String, String> getUserProperties(String userName, String[] propertyNames,
            String profileName) throws UserStoreException {
        try {
            Map<String, String> values = new HashMap<String, String>();
            String searchFilter = "(&(objectCategory=person)(sAMAccountName=" + userName + "))";
            NamingEnumeration answer = searchAD(searchFilter, propertyNames);
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                for (String name : propertyNames) {
                    Attributes attrs = sr.getAttributes();
                    if (attrs == null) {
                        throw new UserStoreException(
                                "Invalid attribute Ids provided. User store is not configured properly. Please contact adminstrator.");
                    }
                    if (attrs.get(name) != null) {
                        String value = (String) attrs.get(name).get();
                        values.put(name, value);
                    }
                }
            }
            return values;
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
    }

    public String getUserPropertyValue(String userName, String propertyName, String profileName)
            throws UserStoreException {
        try {
            String value = null;
            String searchFilter = "(&(objectCategory=person)(sAMAccountName=" + userName + "))";
            NamingEnumeration answer = searchAD(searchFilter, new String[] { propertyName });
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                if (sr.getAttributes().get(propertyName) != null) {
                    value = (String) sr.getAttributes().get(propertyName).get();
                }
            }
            return value;
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
    }

    // ////***************************** Not going to implement
    // ************************

    public String[] getUserPropertyNames() throws UserStoreException {
        return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        return null;
    }

    public Map getRoleProperties(String roleName) throws UserStoreException {
        return new HashMap();
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
        // TODO 1.0
        return null;
    }

    public String getDefaultUserProfileName(String userName) throws UserStoreException {
        // TODO 1.0
        return null;
    }

    public byte[] getUserBinaryContent(String userName, String contentName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getAllProfileNames() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getProfileNames(String username) throws UserStoreException {
        return null;
    }

    public byte[] getProfilePicture(String userName, String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public byte[] getUserBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String getUserClaimValue(String userName, String claim, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserClaimValues(String userName, String[] claims,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public UserClaimsAdmin getClaimsAdmin() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Claim[] getUserClaimValues(String userName, String profileName)
            throws UserStoreException {
        return null;
    }

    protected NamingEnumeration searchAD(String searchFilter, String[] returnedAtts)
            throws UserStoreException {
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        String searchBase = config.getSearchBase();
        if (returnedAtts != null && returnedAtts.length > 0) {
            searchCtls.setReturningAttributes(returnedAtts);
        }
        DirContext dirContext = null;
        try {
            dirContext = config.getContext();
            NamingEnumeration answer = dirContext.search(searchBase, searchFilter, searchCtls);
            return answer;
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            JNDIUtil.closeContext(dirContext);
        }
    }
}
