package org.wso2.carbon.user.core.hybrid.actdir;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.util.JNDIUtil;

public class ActDirUtil {

    private static Log log = LogFactory.getLog(ActDirUtil.class);

    public static String getUserCN(ActDirRealmConfig config, String loginName)
            throws UserStoreException {
        String name = null;
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

        String searchFilter = "(&(objectClass=user)(sAMAccountName=" + loginName + "))";
        String searchBase = config.getSearchBase();

        DirContext dirContext = null;

        try {
            dirContext = config.getContext();
            NamingEnumeration answer = dirContext.search(searchBase, searchFilter, searchCtls);

            int count = 0;
            SearchResult userObj = null;
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                if (count > 0) {
                    log.error("More than one user exist for the same name");
                }
                count++;
                userObj = sr;
            }
            if (userObj != null) {
                name = userObj.getNameInNamespace();
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            JNDIUtil.closeContext(dirContext);
        }

        return name;
    }

    public static String getUserLoginName(ActDirRealmConfig config, String CN)
            throws UserStoreException {
        String name = null;
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

        String searchFilter = "(&(objectClass=user)(CN=" + CN + "))";
        String searchBase = config.getSearchBase();
        String returnedAtts[] = { "sAMAccountName" };
        searchCtls.setReturningAttributes(returnedAtts);
        DirContext dirContext = null;
        try {
            dirContext = config.getContext();
            NamingEnumeration answer = dirContext.search(searchBase, searchFilter, searchCtls);
            int count = 0;
            SearchResult userObj = null;
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                if (count > 0) {
                    log.error("More than one user exist for the same name");
                }
                count++;
                name = (String) sr.getAttributes().get("sAMAccountName").get();
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            JNDIUtil.closeContext(dirContext);
        }

        return name;
    }
}
