/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.user.core.hybrid.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.AuthenticatorException;

/**
 * 
 * @see org.wso2.usermanager.Authenticator
 *
 */
public class JDBCAuthenticator implements Authenticator {

    private static Log log = LogFactory.getLog(JDBCAuthenticator.class);

    private JDBCRealmConfig config = null;

    public JDBCAuthenticator(JDBCRealmConfig config) {
        this.config = config;
    }

    /**
     * @see org.wso2.usermanager.Authenticator#authenticate(String, Object)
     */
    public boolean authenticate(String userName, Object credentials) throws AuthenticatorException {

        if (credentials == null) {
            return false;
        }

        if (userName == null) {
            return false;
        }

        boolean bValue = false;

        if (!(credentials instanceof String)) {
            throw new AuthenticatorException("Can handle only string type credentials");
        }
        Connection dbConnection =null;
        try {
            dbConnection = config.getDataSource().getConnection();
            dbConnection.setAutoCommit(false);
            String password = (String) credentials;
            String sqlstmt = config.getUserPasswordSQL();

            if (log.isDebugEnabled()) {
                log.debug(sqlstmt);
            }

            PreparedStatement getPass = dbConnection.prepareStatement(sqlstmt);
            getPass.setString(1, userName);
            
            ResultSet rs = getPass.executeQuery();

            if (rs.next() == true) {
                String value = rs.getString(1);
                if ((value != null) && (value.equals(password))) {
                    bValue = true;
                }
            }
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
            throw new AuthenticatorException("Authentication Failure");
        } finally {
            try {       
                if(dbConnection!=null){
                    dbConnection.close();
                }
            } catch (SQLException e) {
               log.error(e);
            }
        }

        return bValue;
    }

}
