package org.wso2.carbon.user.core.hybrid.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.util.DatabaseUtil;

public class JDBCUserStoreReader implements UserStoreReader {

    private static Log log = LogFactory.getLog(JDBCUserStoreReader.class);
    protected JDBCRealmConfig config = null;

    public JDBCUserStoreReader(JDBCRealmConfig config) {
        this.config = config;
    }

    public String[] getAllRoleNames() throws UserStoreException {
        String[] names = new String[0];
        String sqlstmt = config.getRoleListSQL();
        if (sqlstmt != null) {
            names = this.retriveNames(sqlstmt, null);
        }
        return names;
    }

    public String[] getAllUserNames() throws UserStoreException {
        String[] names = new String[0];
        String sqlstmt = config.getUserListSQL();
        if (sqlstmt != null) {
            names = this.retriveNames(sqlstmt, null);
        }
        return names;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        String[] roleNames = this.getAllRoleNames();
        for (String name : roleNames) {
            if (name.equals(roleName)) {
                return true;
            }
        }
        return false;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        String sqlstmt = config.getIsUserExistingSQL();
        boolean isExisting = false;
        Connection dbConnection = null;
        try {
            dbConnection = config.getDataSource().getConnection();
            dbConnection.setAutoCommit(false);
            PreparedStatement prepStmt = dbConnection.prepareStatement(sqlstmt);
            prepStmt.setString(1, userName);
            ResultSet rs = prepStmt.executeQuery();
            while (rs.next()) {
                isExisting = true;
            }

        } catch (SQLException e) {
            log.error("Using the sql statment :'" + config.getIsUserExistingSQL()
                    + "'. And error is " + e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                log.error(e.getMessage(), e);

            }
        }
        return isExisting;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] names = new String[0];
        String sqlstmt = config.getUserRoleSQL();
        if (sqlstmt != null) {
            names = this.retriveNames(sqlstmt, userName);
        }
        return names;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String sqlstmt = config.getUsersInRoleSQL();
        return this.retriveNames(sqlstmt, roleName);
    }

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException {
        String[] users = new String[0];
        Connection dbConnection = null;
        try {
            String sqlStmt = config.getUserFilterSQL();
            if (filter != null && filter.trim().length() != 0) {
                filter = filter.trim();
                filter = filter.replace("*", "%");
                filter = filter.replace("?", "_");
                filter = filter;
            } else {
                filter = "%";
            }

            dbConnection = config.getDataSource().getConnection();

            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            dbConnection.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);

            PreparedStatement prepStmt = dbConnection.prepareStatement(sqlStmt);
            prepStmt.setString(1, filter);
            ResultSet rs = prepStmt.executeQuery();

            List lst = new LinkedList();

            int i = 0;
            while (rs.next()) {
                if (i < maxItemLimit) {
                    lst.add(rs.getString(1));
                } else {
                    break;
                }
                i++;
            }
            rs.close();

            if (lst.size() > 0) {
                users = (String[]) lst.toArray(new String[lst.size()]);
            }

        } catch (SQLException e) {
            log.error("Using the sql statment :'" + config.getUserFilterSQL() + "'. And error is "
                    + e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return users;
    }

    private String[] retriveNames(String sqlstmt, String param) throws UserStoreException {
        String[] names = new String[0];

        if (sqlstmt == null) {
            return names;
        }

        Connection dbConnection = null;
        try {
            dbConnection = config.getDataSource().getConnection();
            dbConnection.setAutoCommit(false);
            PreparedStatement prepStmt = dbConnection.prepareStatement(sqlstmt);
            if (param != null) {
                prepStmt.setString(1, param);
            }
            ResultSet rs = prepStmt.executeQuery();
            List<String> lst = new ArrayList<String>();
            while (rs.next()) {
                String name = rs.getString(1);
                lst.add(name);
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
        } catch (SQLException e) {
            log.error(
                    "Using the sql statment : '" + sqlstmt + "' . And error is " + e.getMessage(),
                    e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                log.error(e.getMessage(), e);

            }
        }
        return names;

    }

    public String getUserPropertyValue(String userName, String propertyName, String profileName)
            throws UserStoreException {
        String value = null;
        Map<String, String> values = getUserProperties(userName, new String[]{propertyName},
                profileName);
        if (values != null && values.containsKey(propertyName)) {
            value = values.get(propertyName);
        }
        return value;
    }

    public Map<String, String> getUserProperties(String userName, String[] propertyNames,
            String profileName) throws UserStoreException {
        Map<String, String> values = new HashMap<String, String>();
        values = getPropertiesBySelectingAtOnce(userName, propertyNames, profileName);
        return values;
    }

    // -----------------------------------------------------------------------------------------
    public byte[] getUserBinaryContent(String userName, String contentName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getRoleProperties(String roleName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getAllProfileNames() throws UserStoreException {
        return null;
    }

    public String[] getProfileNames(String username) throws UserStoreException {
        String sqlstmt = config.getProfileNamesSQL();
        return this.retriveNames(sqlstmt, username);
    }

    public byte[] getProfilePicture(String userName, String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public byte[] getUserBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String getUserClaimValue(String userName, String claim, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserClaimValues(String userName, String[] claims,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Claim[] getUserClaimValues(String userName, String profileName)
            throws UserStoreException {
        return null;
    }

    /**
     * This method handles sqls of type select * from um_user_data where
     * username=? and profilename = ?
     * 
     * @param userName
     * @param propertyNames
     * @param profileName
     * @return
     * @throws UserStoreException
     */
    private Map<String, String> getPropertiesBySelectingAtOnce(String userName,
            String[] propertyNames, String profileName) throws UserStoreException {
        Map<String, String> values = new HashMap<String, String>();
        Connection dbConnection = null;
        PreparedStatement prepStmt = null;
        ResultSet rs = null;
        String sql = null;
        try {
            dbConnection = config.getDataSource().getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            if (profileName == null || UserCoreConstants.DEFAULT_PROFILE.equals(profileName)) {
                sql = config.getAttributesSQL();
                prepStmt = dbConnection.prepareStatement(sql);
                prepStmt.setString(1, userName);
            } else {
                sql = config.getAttrsForProfileSQL();
                prepStmt = dbConnection.prepareStatement(sql);
                prepStmt.setString(1, userName);
                prepStmt.setString(2, profileName);
            }
            // sort the array to do binary search
            Arrays.sort(propertyNames, String.CASE_INSENSITIVE_ORDER);
            rs = prepStmt.executeQuery();
            ResultSetMetaData mt = rs.getMetaData();
            int count = mt.getColumnCount();
            if (rs.next()) {
                for (int i = 1; i < count + 1; i++) {
                    String colName = mt.getColumnName(i);
                    int index = Arrays.binarySearch(propertyNames, colName,
                            String.CASE_INSENSITIVE_ORDER);
                    if (index > -1) {
                        int type = mt.getColumnType(i);
                        String value = getValueAsString(rs, i, type);
                        values.put(propertyNames[index], value);
                    }
                }
            }
        } catch (SQLException e) {
            log.error("Using the sql statment :'" + sql + "'. And error is " + e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection, rs, prepStmt);
        }
        return values;
    }

    private String getValueAsString(ResultSet rs, int index, int type) throws UserStoreException,
            SQLException {
        String strValue = null;
        switch (type) {
        case Types.INTEGER: {
            int value = rs.getInt(index);
            strValue = Integer.toString(value);
        }
            break;
        case Types.DOUBLE: {
            double value = rs.getDouble(index);
            strValue = Double.toString(value);
        }
            break;
        case Types.FLOAT: {
            float value = rs.getFloat(index);
            strValue = Float.toString(value);
        }
            break;
        case Types.BOOLEAN: {
            boolean value = rs.getBoolean(index);
            strValue = Boolean.toString(value);
        }
        case Types.VARCHAR: {
            String value = rs.getString(index);
            strValue = value;
        }
            break;
        default: {
            throw new UserStoreException("Unsupported type");
        }
        }
        return strValue;
    }
}
