package org.wso2.carbon.user.core.hybrid.ldap;

import java.security.MessageDigest;
import java.text.MessageFormat;
import java.util.Hashtable;

import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

import org.apache.axis2.databinding.types.HexBinary;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.AuthenticatorException;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.util.JNDIUtil;

/**
 * 
 * @see org.wso2.usermanager.Authenticator
 * 
 */
public class LDAPAuthenticator implements Authenticator {

    private static Log log = LogFactory.getLog(LDAPAuthenticator.class);

    private LDAPRealmConfig config = null;

    public LDAPAuthenticator(LDAPRealmConfig config) {
        this.config = config;
    }

    /**
     * @see org.wso2.usermanager.Authenticator#authenticate(String, Object)
     */
    public boolean authenticate(String userName, Object credentials) throws AuthenticatorException {

        if (userName == null || credentials == null) {
            return false;
        }

        userName = userName.trim();
        String password = (String) credentials;
        password = password.trim();

        if (userName.equals("") || password.equals("")) {
            return false;
        }

        boolean bValue = false;
        String[] userNames = new String[0];
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);

        String searchFilter = "(&(objectClass=person)("+config.getUidSearchName()+"=" + userName + "))";
        String searchBase = config.getUserContextName();

        try {
            String name = null;
            NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
                    searchCtls);
            int count = 0;
            SearchResult userObj = null;
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                if (count > 0) {
                    log.error("More than one user exist for the same name");
                }
                count++;
                userObj = sr;
            }
            if (userObj != null) {
                name = userObj.getNameInNamespace();
            }

            if (name != null) {
                bValue = this.bindAsUser(config.getContext(), name, (String) credentials);
            }
        } catch (Exception e) {
            log.error("Unable to authenticate", e);
            throw new AuthenticatorException("Authentication failure", e);
        }
        return bValue;
    }

    /**
     * Check credentials by binding to the directory as the user
     * 
     * @param context
     *            The directory context
     * @param user
     *            The User to be authenticated
     * @param credentials
     *            Authentication credentials
     * 
     * @exception NamingException
     *                if a directory server error occurs
     */
    protected boolean bindAsUser(DirContext context, String dn, String credentials)
            throws NamingException, AuthenticatorException, UserStoreException {
        boolean isAuthed = false;

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, config.getDriverName());
        env.put(Context.SECURITY_PRINCIPAL, dn);
        env.put(Context.PROVIDER_URL, config.getConnectionURL());
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_CREDENTIALS, credentials);

        LdapContext cxt = null;
        try {
            cxt = new InitialLdapContext(env, null);
            isAuthed = true;
        } catch (Exception e1) {
            if (log.isDebugEnabled()) {
                log.debug("Failed to authenticate using single hash password", e1);
                // not throwing exceptions, this is an invalid password
            }
        } finally {
            JNDIUtil.closeContext(cxt);
        }

        return isAuthed;
    }

}
