package org.wso2.carbon.user.core.hybrid.ldap;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class LDAPRealmConfig {

    private String connectionURL = null;

    private String connectionName = null;

    private String connectionPassword = null;

    private String uidSearchName = null;

    private String userContextName = null;

    private String driverName = "com.sun.jndi.ldap.LdapCtxFactory";

    private static Log log = LogFactory.getLog(LDAPRealmConfig.class);

    public LDAPRealmConfig() {

    }

    public LDAPRealmConfig(LDAPRealmConfig config) {
        connectionURL = new String(config.getConnectionURL());
        connectionName = new String(config.getConnectionName());
        connectionPassword = new String(config.getConnectionPassword());
    }

    public String getConnectionName() {
        return connectionName;
    }

    public String getConnectionURL() {
        return connectionURL;
    }

    public void setConnectionURL(String connectionURL) {
        this.connectionURL = connectionURL;
    }

    public void setConnectionName(String connectionName) {
        this.connectionName = connectionName.trim();
    }

    public String getConnectionPassword() {
        return connectionPassword;
    }

    public void setConnectionPassword(String connectionPassword) {
        this.connectionPassword = connectionPassword;
    }

    public DirContext getContext() throws NamingException {
        DirContext context = null;
        try {
            context = new InitialDirContext(getDirectoryContextEnvironment());
        } catch (NamingException e) {
            // try once again
            log.error(e.getMessage(), e);
            log.error("Creating LDAP connection for the second time");
            context = new InitialDirContext(getDirectoryContextEnvironment());
        } 
        return (context);

    }

    public String getUidSearchName() {
        return uidSearchName;
    }

    public void setUidSearchName(String uidSearchName) {
        this.uidSearchName = uidSearchName;
    }

    public String getUserContextName() {
        return userContextName;
    }

    public void setUserContextName(String userContextName) {
        this.userContextName = userContextName;
    }

    protected Hashtable getDirectoryContextEnvironment() {

        Hashtable env = new Hashtable();

        env.put(Context.INITIAL_CONTEXT_FACTORY, driverName);

        if (connectionName != null)
            env.put(Context.SECURITY_PRINCIPAL, connectionName);

        if (connectionPassword != null)
            env.put(Context.SECURITY_CREDENTIALS, connectionPassword);

        if (connectionURL != null)
            env.put(Context.PROVIDER_URL, connectionURL);

        return env;

    }

    public String getDriverName() {
        return driverName;
    }

}
