package org.wso2.carbon.user.core.hybrid.ldap;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.DefaultAuthorizer;
import org.wso2.carbon.user.core.hybrid.HybridRealmConstants;
import org.wso2.carbon.user.core.i18n.Messages;
import org.wso2.carbon.user.core.util.DatabaseUtil;

public class LDAPUserStoreAdmin extends LDAPUserStoreReader implements UserStoreAdmin{

    
    private static Log log = LogFactory.getLog(DefaultAuthorizer.class);
    
    public LDAPUserStoreAdmin(LDAPRealmConfig config) {
        super(config);
    }
    
   
    public void updateUser(String userName, Object newCredential, Object oldCredential)
            throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }

    public void updateUser(String userName, Object newCredential) throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
        
    }

    public void addUser(String userName, Object credential) throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }
    
    public void deleteUser(String userName) throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }

//default style
    public void deleteRole(String roleName) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement deleteRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_ROLE_SQL);
            deleteRoleStmt.setString(1, roleName);
            deleteRoleStmt.execute();
            dbConnection.commit();
            deleteRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection);
        }
    }

    public void addUserToRole(String userName, String roleName) throws UserStoreException {

        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

          
            int roleid = -1;//EmbedDerbyDefaultDBCreator.getRoleId(dbConnection, roleName);

            if (roleid == -1) {
                throw new UserStoreException(Messages.getMessage("nullData"));
            }

            PreparedStatement addUserToRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_USER_ROLE_SQL);
            addUserToRoleStmt.setString(1, userName);
            addUserToRoleStmt.setInt(2, roleid);
            addUserToRoleStmt.executeUpdate();
            dbConnection.commit();
            addUserToRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorAddingUserToRole"), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection);
        }
    }

    public void removeUserFromRole(String userName, String roleName) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement removeUserFromRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_USER_ROLE_SQL);
            removeUserFromRoleStmt.setString(1, userName);
            removeUserFromRoleStmt.setString(2, roleName);
            removeUserFromRoleStmt.executeUpdate();
            dbConnection.commit();
            removeUserFromRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection);
        }
    }

    public void addRole(String roleName) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement addRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_ROLE_SQL);
            addRoleStmt.setString(1, roleName);
            addRoleStmt.execute();
            dbConnection.commit();
            addRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            DatabaseUtil.closeAllConnections(dbConnection);
        }
    }

  
    public void setRoleProperties(String roleName, Map properties, String profileName) throws UserStoreException {
        throw new UserStoreException("actionNotAllowed");
    }
    
    
    
    public void updateUserProperties(String userName, Map<String, String> properties,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void updateRoleName(String roleName, String newRoleName) throws UserStoreException{
	throw new UserStoreException("actionNotAllowed");
    }
    
    public void updateUserName(String userName, String newUserName) throws UserStoreException{
	throw new UserStoreException("actionNotAllowed");
    }

    public void deleteBinaryContent(String userName, String contentName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void deleteUserProperties(String userName, String[] properties)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void deleteUserProperty(String userName, String property) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void setUserBinaryContent(String userName, String contentName, byte[] data)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void setUserProperties(String userName, Map<String, String> properties)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public void setUserProperty(String username, String attributeId, String attributeValue)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }

    public byte[] getUserBinaryContent(String userName, String contentName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String getUserPropertyValue(String userName, String propertyName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public void deleteBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void deleteUserClaimValue(String userName, String claimName, String claimValue,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void deleteUserClaimValues(String userName, String[] claims, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void deleteUserProperties(String userName, String[] properties, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void deleteUserProperty(String userName, String property, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setProfilePicture(String userName, byte[] picture, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setRoleProperties(String roleName, Map<String, String> properties)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setUserBinaryContent(String userName, String contentName, byte[] data,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setUserClaimValue(String userName, String claimURI, String claimValue,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setUserClaimValues(String userName, Map<String, String> claims, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setUserProperties(String userName, Map<String, String> properties,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public void setUserProperty(String username, String attributeId, String attributeValue,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        
    }


    public String[] getProfileNames() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public byte[] getProfilePicture(String userName, String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public byte[] getUserBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public String getUserClaimValue(String userName, String claim, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public Map<String, String> getUserClaimValues(String userName, String[] claims,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public Map<String, String> getUserProperties(String userName, String[] propertyNames,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public String getUserPropertyValue(String userName, String propertyName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }



}
