package org.wso2.carbon.user.core.hybrid.ldap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.util.JNDIUtil;

/**
 * 
 */
public class LDAPUserStoreReader implements UserStoreReader {

    protected DataSource dataSource = null;

    private static Log log = LogFactory.getLog(LDAPUserStoreReader.class);

    private LDAPRealmConfig config = null;

    public LDAPUserStoreReader(LDAPRealmConfig config) {
        this.config = config;
    }

    public String[] getAllUserNames() throws UserStoreException {
        String[] userNames = new String[0];
        String searchFilter = "(&(objectClass=person))";
        String searchBase = config.getUserContextName();

        String returnedAtts[] = { config.getUidSearchName() };
        try {
            NamingEnumeration answer = this.searchForUser(searchFilter, returnedAtts);
            List<String> list = new ArrayList<String>();
            int i = 0;
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                String name = (String) sr.getAttributes().get(config.getUidSearchName()).get();
                list.add(name);
                i++;
            }
            userNames = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return userNames;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        boolean bFound = false;

        String searchFilter = "(&(objectClass=person)("+config.getUidSearchName()+"=" + userName + "))";
        // RFC 1274
        String returnedAtts[] = { config.getUidSearchName() };
        try {
            NamingEnumeration answer = this.searchForUser(searchFilter, returnedAtts);
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                bFound = true;
            }

        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }

        return bFound;
    }

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException {
        String[] userNames = new String[0];
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        searchCtls.setCountLimit(maxItemLimit);

        if (filter.contains("?")) {
            throw new UserStoreException("invalid character for act dir serch");
        }

        String searchFilter = "(&(objectClass=person)("+config.getUidSearchName()+"=" + filter + "))";

        String searchBase = config.getUserContextName();

        String returnedAtts[] = { config.getUidSearchName() };
        searchCtls.setReturningAttributes(returnedAtts);
        DirContext dirContext  = null;
        try {
            dirContext = config.getContext();
            NamingEnumeration answer = dirContext.search(searchBase, searchFilter,
                    searchCtls);
            List<String> list = new ArrayList<String>();
            int i = 0;
            while (answer.hasMoreElements() && i < maxItemLimit) {
                SearchResult sr = (SearchResult) answer.next();
                String name = (String) sr.getAttributes().get(config.getUidSearchName()).get();
                list.add(name);
                i++;
            }
            userNames = list.toArray(new String[list.size()]);
        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        } finally {
            JNDIUtil.closeContext(dirContext);
        }
        return userNames;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        return new String[0];
    }

    public String[] getAllRoleNames() throws UserStoreException {
        return new String[0];
    }

    public Map<String, String> getUserProperties(String userName, String[] propertyNames,
            String profileName) throws UserStoreException {
        Map<String, String> values = new HashMap<String, String>();
        String searchFilter = "(&(objectClass=person)("+config.getUidSearchName()+"=" + userName + "))";
        // RFC 1274
        try {
            NamingEnumeration answer = this.searchForUser(searchFilter, propertyNames);
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                Attributes attributes = sr.getAttributes();
                if (attributes != null) {
                    for (String name : propertyNames) {
                        Attribute attribute = attributes.get(name);
                        if (attribute != null) {
                            StringBuffer attrBuffer = new StringBuffer();
                            for (NamingEnumeration attrs = attribute.getAll(); attrs.hasMore();) {
                                String attr = (String) attrs.next();
                                if (attr != null && attr.trim().length() > 0) {
                                    attrBuffer.append(attr + ",");
                                }
                            }
                            String value = attrBuffer.toString();
                            // Length needs to be more than one for a valid attribute, since we
                            // attach ",".
                            if (value != null && value.trim().length() > 1) {
                                value = value.substring(0, value.length() - 1);
                                values.put(name, value);
                            }
                        }
                    }
                }
            }

        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return values;
    }

    public String getUserPropertyValue(String userName, String propertyName, String profileName)
            throws UserStoreException {
        String value = "";
        String searchFilter = "(&(objectClass=person)("+config.getUidSearchName()+"=" + userName + "))";
        // RFC 1274
        try {
            NamingEnumeration answer = this.searchForUser(searchFilter,
                    new String[] { propertyName });
            while (answer.hasMoreElements()) {
                SearchResult sr = (SearchResult) answer.next();
                Attributes attributes = sr.getAttributes();
                if (attributes != null) {
                    Attribute attribute = attributes.get(propertyName);
                    if (attribute != null) {
                        value = (String) attribute.get();
                    }
                }
            }

        } catch (NamingException e) {
            log.error(e.getMessage(), e);
            throw new UserStoreException(e.getMessage(), e);
        }
        return value;
    }

    // ***************************** Not going to implement
    // ************************

    public String[] getUserPropertyNames() throws UserStoreException {
        return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        return null;
    }

    // default style

    public Map getRoleProperties(String roleName) throws UserStoreException {
        return new HashMap();
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        return new String[0];
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        return false;
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
        // TODO 1.0
        return null;
    }

    public String getDefaultUserProfileName(String userName) throws UserStoreException {
        // TODO 1.0
        return null;
    }

    public boolean isExistingUserProfile(String userName, String profileName)
            throws UserStoreException {
        // TODO 1.0
        return false;
    }

    public byte[] getUserBinaryContent(String userName, String contentName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getAllProfileNames() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getProfileNames(String username) throws UserStoreException {
        return null;
    }

    public byte[] getProfilePicture(String userName, String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public byte[] getUserBinaryContent(String userName, String contentName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String getUserClaimValue(String userName, String claim, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserClaimValues(String userName, String[] claims,
            String profileName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Claim[] getUserClaimValues(String userName, String profileName)
            throws UserStoreException {
        return null;
    }

    protected NamingEnumeration searchForUser(String searchFilter, String[] returnedAtts)
            throws UserStoreException {
        SearchControls searchCtls = new SearchControls();
        searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
        String searchBase = config.getUserContextName();
        if (returnedAtts != null && returnedAtts.length > 0) {
            searchCtls.setReturningAttributes(returnedAtts);
        }
        DirContext dirContext  = null;
        try {
            dirContext = config.getContext();
            NamingEnumeration answer = dirContext.search(searchBase, searchFilter,
                    searchCtls);
            return answer;
        } catch (NamingException e) {
            log.error("Search failed.", e);
            throw new UserStoreException(e.getMessage());
        } finally {
            JNDIUtil.closeContext(dirContext);
        }
    }

}
