package org.wso2.carbon.user.core.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.sql.DataSource;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.ClaimMapping;
import org.wso2.carbon.user.core.ProfileConfiguration;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.claim.ClaimDAO;
import org.wso2.carbon.user.core.claim.DefaultClaimBuilder;
import org.wso2.carbon.user.core.def.DefaultRealm;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.user.core.delegating.DelegatingRealm;
import org.wso2.carbon.user.core.model.SystemPermissions;
import org.wso2.carbon.user.core.profile.DefaultProfileBuilder;
import org.wso2.carbon.user.core.profile.ProfileConfigDAO;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

import org.wso2.carbon.utils.CarbonUtils;
import org.wso2.carbon.utils.ServerConstants;

public class Activator implements BundleActivator {
	private static final Log log = LogFactory.getLog(Activator.class);

    public void start(BundleContext bc) throws Exception {
        try {
        	String password = null;
        	String userName = null;
        	String maxWaitStr = null;
            String maxActiveStr = null;
            String maxIdleStr = null;
            String minIdleStr = null;
            String validationQuery = null;
        	
            OMElement parentElement = getConfigurationElement();

            OMElement dbelement = parentElement.getFirstChildWithName(new QName("Database"));
            String url = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.URL)).getText();
            if(url!=null){
            	if(url.contains(CarbonConstants.CARBON_HOME_PARAMETER)){
            		File carbonHomeDir;
            		carbonHomeDir = new File(CarbonUtils.getCarbonHome());
            		String path = carbonHomeDir.getPath();
            		path = path.replaceAll(Pattern.quote("\\"), "/");
            		if(carbonHomeDir.exists() && carbonHomeDir.isDirectory()){
            			url = url.replaceAll(Pattern.quote(CarbonConstants.CARBON_HOME_PARAMETER),path);
            		}
            		else{
            			 log.warn("carbon home invalid");
            			 String[] tempStrings1 = url.split(Pattern.quote(CarbonConstants.CARBON_HOME_PARAMETER));
             			 String tempUrl = tempStrings1[1];
             			 String[] tempStrings2 = tempUrl.split("/");
             			 for(int i=0; i< tempStrings2.length -1; i++){
             				url = tempStrings1[0] + tempStrings2[i]+ "/";
             			 }
             			url = url + tempStrings2[tempStrings2.length -1];
             		}
            	}
            }
            //Username & password can be empty.
            OMElement passwordElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.PASSWORD));
            if(passwordElement != null){
            	password = passwordElement.getText();
            }
            
            OMElement userNameElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.USERNAME));
            if(userNameElement != null){
            	userName = userNameElement.getText();
            }
            String driverName = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.DRIVER)).getText();
            
            //Read connection pool properties
            long maxWait = 60 * 1000;
            int maxActive = 5;
            int maxIdle = 3;
            int minIdle = 2;
            
            OMElement maxWaitElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.MAX_WAIT));
            if(maxWaitElement != null){
            	maxWaitStr = maxWaitElement.getText();
            	if(maxWaitStr != null){
            		try{
                		maxWait = Long.parseLong(maxWaitStr);            			
            		}catch(NumberFormatException e){
            			log.error("Found String("+maxWaitStr+") value for maxWait." +
            					"Should be numeric.", e);
            		}
            	}
            }
            
            OMElement maxActiveElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.MAX_ACTIVE));
            if(maxActiveElement != null){
            	maxActiveStr = maxActiveElement.getText();
            	if(maxActiveStr != null){
            		try{
                		maxActive = Integer.parseInt(maxActiveStr);            			
            		}catch(NumberFormatException e){
            			log.error("Found String("+maxActiveStr+") value for maxActive." +
            					"Should be numeric.", e);
            		}
            	}            	
            }
            
            OMElement maxIdleElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.MAX_IDLE));
            if(maxIdleElement != null){
            	maxIdleStr = maxIdleElement.getText();
            	if(maxIdleStr != null){
            		try{
                		maxIdle = Integer.parseInt(maxIdleStr);            			
            		}catch(NumberFormatException e){
            			log.error("Found String("+maxIdleStr+") value for maxIdle." +
            					"Should be numeric.", e);
            		}
            	}            	
            }
            
            OMElement minIdleElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.MIN_IDLE));
            if(minIdleElement != null){
            	minIdleStr = minIdleElement.getText();
            	if(minIdleStr != null){
            		try{
                		minIdle = Integer.parseInt(minIdleStr);            			
            		}catch(NumberFormatException e){
            			log.error("Found String("+minIdleStr+") value for minIdle." +
            					"Should be numeric.", e);
            		}
            	}            	
            }
            
            //validationQuery
            OMElement validationQueryElement = dbelement.getFirstChildWithName(
                    new QName(UserCoreConstants.UserMgtConfig.VALIDATION_QUERY));
            if(validationQueryElement != null){
            	validationQuery = validationQueryElement.getText();
            }

            BasicDataSource dataSource = new BasicDataSource();
            dataSource.setUrl(url);
            dataSource.setDriverClassName(driverName);
            dataSource.setMaxActive(maxActive);
            dataSource.setMaxIdle(maxIdle);
            dataSource.setMaxWait(maxWait);
            dataSource.setMinIdle(minIdle);
            if(validationQuery != null){
                dataSource.setValidationQuery(validationQuery);            	
            }
            
            if(userName != null){
                dataSource.setUsername(userName);            	
            }
            if(password != null){
                dataSource.setPassword(password);            	
            }

            initializeDatabase(dataSource);

            // DefaultXMLClaimBuilder will read claim-config.xml as a resource
            // from the bundle.
            DefaultClaimBuilder.setBundleContext(bc);
            // DefaultXMLProfileBuilder will read profile-config.xml as a resource
            // from the bundle.
            DefaultProfileBuilder.setBundleContext(bc);

            UserRealm realm = exposeDefaultRealmAsService(bc, url, driverName, userName, password, 
                    dataSource, parentElement);
            exposeDelegatingAcessControlAdminAsService(bc, realm);
        } catch (Throwable e) {
            String msg = "Cannot start User Manager Core bundle";
            log.fatal(msg, e);
            throw new Exception(msg, e);
        }

    }

    public void exposeDelegatingAcessControlAdminAsService(BundleContext bc, UserRealm defaultRealm) throws Exception {
        DelegatingRealm dgltRealm = new DelegatingRealm(defaultRealm);
        Dictionary dictionary = new Hashtable();
        dictionary.put(UserCoreConstants.REALM_GENRE, UserCoreConstants.DELEGATING_REALM);
        bc.registerService(UserRealm.class.getName(), dgltRealm, dictionary);
    }
    
    public void stop(BundleContext bundleContext) throws Exception {
        ServiceReference[] serviceRefs = bundleContext.getServiceReferences(UserRealm.class
                .getName(), "(RealmGenre=Default)");
        if (serviceRefs != null && serviceRefs.length > 0) {
            DefaultRealm realm = (DefaultRealm) bundleContext.getService(serviceRefs[0]);
            DataSource ds = realm.getDataSource();
            ds = null;
        }
    }

    protected OMElement getConfigurationElement() {
        try {
            String carbonHome = System.getProperty(ServerConstants.CARBON_HOME);

            InputStream inStream = null;
            if (carbonHome != null) {
                File userMgtXml = new File(carbonHome + File.separator + "conf", "user-mgt.xml");
                if (!userMgtXml.exists()) {
                    String msg = "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found.";
                    throw new FileNotFoundException(msg);
                }
                inStream = new FileInputStream(userMgtXml);
            } else {
                inStream = this.getClass().getClassLoader()
                        .getResourceAsStream("conf/user-mgt.xml");
                if (inStream == null) {
                    String msg = "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found. Please set the carbon.home";
                    throw new FileNotFoundException(msg);
                }
            }

            StAXOMBuilder builder = new StAXOMBuilder(inStream);
            return builder.getDocumentElement();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

        return null;
    }

    protected void initializeDatabase(DataSource dataSource) throws Exception {

        String value = System.getProperty("setup");
        if (value != null) {
            DatabaseCreator databaseCreator = new DatabaseCreator(dataSource);

            try {
                databaseCreator.createRegistryDatabase();
            } catch (Exception e) {
                String msg = "Error in creatig the database";
                throw new Exception(msg, e);
            }
        }
        // only derby for now
        // UMDatabaseManager manager = new UMDatabaseManager(dataSource);
        // manager.createManagementTables();

    }

    public void addInitialData(OMElement parentElement, UserRealm defRealm) throws Exception {
        OMElement roleElem = parentElement.getFirstChildWithName(new QName("AdminRole"));
        if (roleElem == null) {
            throw new Exception("System cannot startup without system admin role.");
        }

        String adminRole = roleElem.getText();
        UserStoreAdmin usadmin = defRealm.getUserStoreAdmin();
        if (!usadmin.isExistingRole(adminRole)) {
            usadmin.addRole(adminRole);
        }

        OMElement adminElem = parentElement.getFirstChildWithName(new QName("AdminUser"));
        // system should be able to start without the admin user - mashup server
        String userName = adminElem.getFirstChildWithName(new QName("UserName")).getText();
        String password = adminElem.getFirstChildWithName(new QName("Password")).getText();
        if (adminElem != null) {
            if (!usadmin.isExistingUser(userName)) {
                usadmin.addUser(userName, password);
                usadmin.addUserToRole(userName, adminRole);
                if (!usadmin.isExistingRole("everyone"))
                    usadmin.addRole("everyone");

                usadmin.addUserToRole(userName, "everyone");
            }
        }

        List<String> sysConfig = new ArrayList<String>();
        OMElement sysPermissions = parentElement
                .getFirstChildWithName(new QName("SystemPermission"));
        if (sysPermissions == null) {
            throw new Exception("System cannot startup without system permissions.");
        }

        AccessControlAdmin acadmin = defRealm.getAccessControlAdmin();
        Iterator ite = sysPermissions.getChildElements();
        while (ite.hasNext()) {
            OMElement element = (OMElement) ite.next();
            String action = element.getText();
            SystemPermissions.addSystemPermission(action);
            sysConfig.add(action);
            if (!acadmin.isRoleAuthorized(adminRole, "System", action)) {
                acadmin.authorizeRole(adminRole, "System", action);
            }
            if (!acadmin.isUserAuthorized(userName,"System", action)) {         
                acadmin.authorizeUser(adminRole, "System", action);
            }

        }

    }

    public UserRealm exposeDefaultRealmAsService(BundleContext bc, String url, String drivername,
            String username, String password, DataSource dataSource, OMElement configElement)
            throws Exception {
        UserRealm defRealm = new DefaultRealm();

        Map<String, ClaimMapping> claimMappings = null;
        Map<String, ProfileConfiguration> profileConfigs = null;
        ClaimDAO claimDAO = new ClaimDAO(dataSource,  UserCoreConstants.INTERNAL_USERSTORE);
        ProfileConfigDAO profileDAO = new ProfileConfigDAO(dataSource, UserCoreConstants.INTERNAL_USERSTORE);
        int count = claimDAO.getDialectCount(UserCoreConstants.INTERNAL_USERSTORE);
        DefaultClaimBuilder claimBuilder = new DefaultClaimBuilder();
        DefaultProfileBuilder profileBilder = new DefaultProfileBuilder();

        if (count == 0) {
            claimMappings = claimBuilder.buildClaimMappingsFromConfigFile();
            claimDAO.addCliamMappings(claimMappings.values().toArray(
                    new ClaimMapping[claimMappings.size()]));
            profileConfigs = profileBilder.buildProfileConfigurationFromConfigFile();
            profileDAO.addProfileConfig(profileConfigs.values().toArray(
                    new ProfileConfiguration[profileConfigs.size()]));
        } else {
            claimMappings = claimBuilder.buildClaimMappingsFromDatabase(dataSource, UserCoreConstants.INTERNAL_USERSTORE);
            profileConfigs = profileBilder.buildProfileConfigurationFromDatabase(dataSource, UserCoreConstants.INTERNAL_USERSTORE);
        }
        
        DefaultRealmConfig config = new DefaultRealmConfig();
        config.setDataSource(dataSource);
        config.setClaimMapping(claimMappings);
        config.setProfileConfigs(profileConfigs);
        config.setConnectionURL(url);
        config.setDriverName(drivername);
        config.setConnectionUserName(username);
        config.setConnectionPassword(password);
        config.setSysteUserName(CarbonConstants.REGISTRY_SYSTEM_USERNAME);
        config.setDigestFunction(configElement.getFirstChildWithName(
                new QName(UserCoreConstants.DIGEST_FUNCTION)).getText());
        config.setSaltedPassword(new Boolean(configElement.getFirstChildWithName(
                new QName(UserCoreConstants.SALTED_PASSWORD)).getText()));
        
        defRealm.init(config);

        this.addInitialData(configElement, defRealm);

        Dictionary dictionary = new Hashtable();
        dictionary.put(UserCoreConstants.REALM_GENRE, UserCoreConstants.DEFAULT_REALM);
        bc.registerService(UserRealm.class.getName(), defRealm, dictionary);
        return defRealm;
    }

}
