package org.wso2.carbon.user.core.profile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.wso2.carbon.user.core.ProfileConfiguration;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.utils.ServerConstants;

public class DefaultProfileBuilder {

    private static Log log = LogFactory.getLog(DefaultProfileBuilder.class);

    private static final String PROFILE_CONFIG = "profile-config.xml";

    private static BundleContext bundleContext;

    public static void setBundleContext(BundleContext bundleContext) {
        DefaultProfileBuilder.bundleContext = bundleContext;
    }

    public Map<String, ProfileConfiguration> buildProfileConfigurationFromDatabase(DataSource ds, String realmName)
            throws ProfileBuilderException {
        try {
            ProfileConfigDAO profileDAO = new ProfileConfigDAO(ds, realmName);
            Map<String, ProfileConfiguration> profileConfigs = profileDAO.loadProfileConfigs();
            return profileConfigs;
        } catch (UserStoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            throw new ProfileBuilderException();
        }
    }

    /**
     * 
     * @return
     * @throws ProfileBuilderException
     */
    public Map<String, ProfileConfiguration> buildProfileConfigurationFromConfigFile()
            throws ProfileBuilderException {
        OMElement element = null;
        String message = null;
        Iterator<OMElement> configIterator = null;
        Map<String, ProfileConfiguration> profileConfigs = null;

        try {
            element = getRootElement();
        } catch (Exception e) {
            message = "Error while reading profile configuration";
            log.error(message, e);
            throw new ProfileBuilderException(message, e);
        }

        configIterator = element
                .getChildrenWithLocalName(ProfileConstants.LOCAL_NAME_PROFILE_CONFIG);

        profileConfigs = new HashMap<String, ProfileConfiguration>();

        while (configIterator.hasNext()) {
            OMElement profileElem = configIterator.next();
            String dialectName = null;
            String profileConfigName = null;
            Iterator<OMElement> profileIter = null;
            List<String> hidden = null;
            List<String> overridden = null;
            List<String> inherited = null;
            ProfileConfiguration profileConfiguration = null;

            profileConfigName = profileElem.getAttribute(
                    new QName(ProfileConstants.ATTR_PROFILE_CONFIG_NAME)).getAttributeValue();
            dialectName = profileElem.getAttribute(new QName(ProfileConstants.ATTR_DIALECT_URI))
                    .getAttributeValue();
            profileIter = profileElem.getChildrenWithLocalName(ProfileConstants.LOCAL_NAME_CLAIM);

            hidden = new ArrayList<String>();
            overridden = new ArrayList<String>();
            inherited = new ArrayList<String>();

            while (profileIter.hasNext()) {
                OMElement claimElem = null;
                String claimURI = null;
                String behavior = null;

                claimElem = profileIter.next();
                claimURI = claimElem.getFirstChildWithName(
                        new QName(ProfileConstants.LOCAL_NAME_CLAIM_URI)).getText();
                behavior = claimElem.getFirstChildWithName(
                        new QName(ProfileConstants.LOCAL_NAME_CLAIM_BEHAVIOR)).getText();
                if (behavior.equals(UserCoreConstants.CLAIM_HIDDEN)) {
                    hidden.add(claimURI);
                } else if (behavior.equals(UserCoreConstants.CLAIM_OVERRIDEN)) {
                    overridden.add(claimURI);
                } else {
                    inherited.add(claimURI);
                }
            }
            profileConfiguration = new ProfileConfiguration(profileConfigName, hidden, overridden,
                    inherited);
            profileConfiguration.setDialectName(dialectName);
            profileConfigs.put(profileConfigName, profileConfiguration);

        }
        return profileConfigs;
    }

    /**
     * 
     * @return
     * @throws XMLStreamException
     * @throws IOException
     * @throws ProfileBuilderException
     */
    private OMElement getRootElement() throws XMLStreamException, IOException,
            ProfileBuilderException {
        String carbonHome = null;
        InputStream inStream = null;
        StAXOMBuilder builder = null;
        boolean isClaimsLoaded = false;

        carbonHome = System.getProperty(ServerConstants.CARBON_HOME);

        if (carbonHome != null) {
            File profileConfigXml = new File(carbonHome + File.separator + "conf", PROFILE_CONFIG);
            if (profileConfigXml.exists()) {
                inStream = new FileInputStream(profileConfigXml);
                isClaimsLoaded = true;
            }
        }

        if (!isClaimsLoaded) {
            if (bundleContext != null) {
                inStream = bundleContext.getBundle().getResource(PROFILE_CONFIG).openStream();
                if (inStream == null) {
                    String message = "Profile configuration not found.";
                    if (log.isDebugEnabled()) {
                        log.debug(message);
                    }
                    throw new FileNotFoundException(message);
                }
            } else {
                String message = "Profile configuration cannot be loaded from the bundle";
                if (log.isDebugEnabled()) {
                    log.debug(message);
                }
                throw new ProfileBuilderException(message);
            }
        }

        builder = new StAXOMBuilder(inStream);
        return builder.getDocumentElement();
    }

}
