package org.wso2.carbon.user.core.claim;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.Claim;
import org.wso2.carbon.user.core.ClaimMapping;
import org.wso2.carbon.user.core.ProfileConfiguration;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.profile.ProfileConfigDAO;
import org.wso2.carbon.user.core.utils.BaseTestCase;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

public class ClaimDAOTest extends BaseTestCase {
    

    private String connURL = "jdbc:h2:target/databasetest/ClaimTestDatabase";
    private String path = "target/databasetest";
    private ClaimDAO claimDAO = null;
    private List<ClaimMapping> claims = new ArrayList<ClaimMapping>();
    private ProfileConfiguration profConfig = null;
    private ProfileConfigDAO profileDAO = null;

    public void setUp() throws Exception {
        super.setUp();
        Class clazz = Class.forName("org.h2.Driver");
        BasicDataSource ds = new BasicDataSource();
        ds.setUrl(connURL);
        ds.setDriverClassName("org.h2.Driver");
        
        DatabaseCreator creator = new DatabaseCreator(ds);
        creator.createRegistryDatabase();
        
        populateData();
        claimDAO = new ClaimDAO(ds, UserCoreConstants.INTERNAL_USERSTORE);
        profileDAO = new ProfileConfigDAO(ds, UserCoreConstants.INTERNAL_USERSTORE);
    }

    public void testClaimsAndProfilePersisting() throws Exception {
        checkCliamPersistStuff();
        checkProfilePersistStuff();
        checkDeleteDialectStuff();
    }

    public void checkCliamPersistStuff() throws Exception {

        //add
        for (int i = 0; i < 3; i++) {
            claimDAO.addClaimMapping(claims.get(i));
        }
        int count = claimDAO.getDialectCount(UserCoreConstants.INTERNAL_USERSTORE);
        TestCase.assertEquals(2, count);
        List<ClaimMapping> lstActual = claimDAO.loadClaimMappings();
        ClaimMapping cm1 = lstActual.get(0);
        String claimUri = cm1.getClaim().getClaimUri();
        if ("http://wso2.org/givenname".equals(claimUri)) {
            this.assertClaimMapping(claims.get(0), cm1);
        } else if ("http://wso2.org2/givenname2".equals(claimUri)) {
            this.assertClaimMapping(claims.get(1), cm1);
        } else if ("http://wso2.org/givenname3".equals(claimUri)) {
            this.assertClaimMapping(claims.get(2), cm1);
        } else {
            TestCase.assertTrue(false);
        }
        
        //delete
        claimDAO.deleteClaimMapping(claims.get(2));
        count = claimDAO.getDialectCount(UserCoreConstants.INTERNAL_USERSTORE);
        TestCase.assertEquals(2, count);

        //update
        claims.get(1).setMappedAttribute("zorus");
        claimDAO.updateClaim(claims.get(1));
    }

    public void checkProfilePersistStuff() throws Exception {
        //test add
        profileDAO.addProfileConfig(profConfig);
        Map<String, ProfileConfiguration> map = profileDAO.loadProfileConfigs();
        TestCase.assertEquals(1, map.size());
        ProfileConfiguration gotConfig = map.get(profConfig.getProfileName());
        TestCase.assertEquals(gotConfig.getHiddenClaims().get(0), "http://wso2.org/givenname");
        TestCase.assertEquals(gotConfig.getInheritedClaims().get(0), "http://wso2.org2/givenname2");
        TestCase.assertEquals(gotConfig.getOverriddenClaims().size(), 0);
        
        //test update
        profConfig.setInheritedClaims(new ArrayList<String>());
        profConfig.addOverriddenClaim("http://wso2.org2/givenname2");
        profileDAO.updateProfileConfig(profConfig);
        map = profileDAO.loadProfileConfigs();
        gotConfig = map.get(profConfig.getProfileName());
        TestCase.assertEquals(gotConfig.getHiddenClaims().get(0), "http://wso2.org/givenname");
        TestCase.assertEquals(gotConfig.getInheritedClaims().size(),0);
        TestCase.assertEquals(gotConfig.getOverriddenClaims().get(0), "http://wso2.org2/givenname2");
        
        //test delete
        profileDAO.deleteProfileConfig(profConfig);
        map = profileDAO.loadProfileConfigs();
        TestCase.assertEquals(0, map.size());
        
        profileDAO.addProfileConfig(profConfig);
    }

    public void checkDeleteDialectStuff() throws Exception {
        claimDAO.deleteDialect("http://wso2.org2/");
    }
    
    private void populateData() throws Exception {
        Claim claim1 = new Claim();
        claim1.setClaimUri("http://wso2.org/givenname");
        claim1.setDescription("The description is nutts");
        claim1.setDialectURI("http://wso2.org/");
        claim1.setDisplayTag("Given Name");
        claim1.setRegEx("ty&*RegEx");
        claim1.setRequired(true);
        claim1.setSupportedByDefault(true);
        ClaimMapping cm1 = new ClaimMapping();
        cm1.setClaim(claim1);
        cm1.setMappedAttribute("attr1");
        claims.add(cm1);

        Claim claim2 = new Claim();
        claim2.setClaimUri("http://wso2.org2/givenname2");
        claim2.setDescription("The description is nutts2");
        claim2.setDialectURI("http://wso2.org2/");
        claim2.setDisplayTag("Given Name2");
        claim2.setRegEx("ty&*RegEx2");
        claim2.setRequired(true);
        claim2.setSupportedByDefault(true);
        ClaimMapping cm2 = new ClaimMapping();
        cm2.setClaim(claim2);
        cm2.setMappedAttribute("attr2");
        claims.add(cm2);

        Claim claim3 = new Claim();
        claim3.setClaimUri("http://wso2.org/givenname3");
        claim3.setDescription("The description is nutts3");
        claim3.setDialectURI("http://wso2.org/");
        claim3.setDisplayTag("Given Name3");
        claim3.setRegEx("ty&*RegEx3");
        claim3.setRequired(true);
        claim3.setSupportedByDefault(true);
        ClaimMapping cm3 = new ClaimMapping();
        cm3.setClaim(claim3);
        cm3.setMappedAttribute("attr3");
        claims.add(cm3);

        profConfig = new ProfileConfiguration();
        profConfig.setProfileName("HomeProfile");
        profConfig.addHiddenClaim("http://wso2.org/givenname");
        profConfig.addInheritedClaim("http://wso2.org2/givenname2");
        profConfig.setDialectName("http://wso2.org/");

    }

    private void assertClaimMapping(ClaimMapping cm1, ClaimMapping cm2) {
        Claim claim1 = cm1.getClaim();
        Claim claim2 = cm2.getClaim();
        TestCase.assertEquals(claim1.getClaimUri(), claim2.getClaimUri());
        TestCase.assertEquals(claim1.getDescription(), claim2.getDescription());
        TestCase.assertEquals(claim1.getDialectURI(), claim1.getDialectURI());
        TestCase.assertEquals(claim1.getDisplayTag(), claim2.getDisplayTag());
        TestCase.assertEquals(claim1.getRegEx(), claim2.getRegEx());
        TestCase.assertEquals(claim1.isRequired(), claim2.isRequired());
        TestCase.assertEquals(claim1.isSupportedByDefault(), claim2.isSupportedByDefault());
        TestCase.assertEquals(cm1.getMappedAttribute(), cm2.getMappedAttribute());
    }

}
