package org.wso2.carbon.user.core.def;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.utils.BaseTestCase;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

public class DefaultRealmPropertyTest extends BaseTestCase {
    
    UserRealm realm = null;

    DefaultRealmConfig config = null;

    String connURL = "jdbc:h2:target/databasetest/DefaultTestPropertyDB";
    
    public void setUp() throws Exception {
        super.setUp();
        if (realm != null) {
            return;
        }
        Class clazz = Class.forName("org.h2.Driver");
        BasicDataSource ds = new BasicDataSource();
        ds.setUrl(connURL);
        ds.setDriverClassName("org.h2.Driver");
        
        DatabaseCreator creator = new DatabaseCreator(ds);
        creator.createRegistryDatabase();
               
        realm = new DefaultRealm();
        config = (DefaultRealmConfig) realm.getRealmConfiguration();
        config.setConnectionURL(connURL);
        config.setSysteUserName("System");
        config.setDigestFunction("SHA-256");
        realm.init(config);
    }
    
    public void testPropertyManagement() throws UserStoreException{
        doBasicProperties();
        doCustomProperties();
        doAllProperties();
    }
    
    public void doBasicProperties() throws UserStoreException{
        UserStoreAdmin usAdmin = realm.getUserStoreAdmin();
        usAdmin.addUser("muthulee", "muthulee123");
        //add one
        usAdmin.setUserProperty("muthulee", DefaultRealmConstants.EMAIL, "muthulee@yahoo.com", null);
        //add many
        Map<String, String> props = new HashMap<String, String>();
        props.put(DefaultRealmConstants.FIRST_NAME , "aFIRST_NAME");
        props.put(DefaultRealmConstants.LAST_NAME , "aLAST_NAME");
        props.put(DefaultRealmConstants.BIRTH_DATE , "aBIRTH_DATE");
        props.put(DefaultRealmConstants.FULL_NAME , "aFULL_NAME");
        props.put(DefaultRealmConstants.NAME_PREFIX , "aNAME_PREFIX");
        props.put(DefaultRealmConstants.GENDER , "aGENDER");
        props.put(DefaultRealmConstants.TIME_ZONE , "aTIME_ZONE");
        props.put(DefaultRealmConstants.COMPANY_NAME , "aCOMPANY_NAME");
        props.put(DefaultRealmConstants.JOB_TITLE , "aJOB_TITLE");
        props.put(DefaultRealmConstants.PRIMARY_PHONE , "aPRIMARY_PHONE");
        props.put(DefaultRealmConstants.HOME_PHONE , "aHOME_PHONE");
        props.put(DefaultRealmConstants.WORK_PHONE , "aWORK_PHONE");
        props.put(DefaultRealmConstants.MOBILE_PHONE , "aMOBILE_PHONE");
        props.put(DefaultRealmConstants.STREET_ADDRESS , "aSTREET_ADDRESS");
        props.put(DefaultRealmConstants.CITY , "aCITY");
        props.put(DefaultRealmConstants.STATE , "aSTATE");
        props.put(DefaultRealmConstants.COUNTRY , "aCOUNTRY");
        props.put(DefaultRealmConstants.POSTAL_CODE , "aPOSTAL_CODE");
        props.put(DefaultRealmConstants.WEB_PAGE , "aWEB_PAGE");
        props.put(DefaultRealmConstants.LANGUAGE , "aLAGUAGE");
        props.put(DefaultRealmConstants.BLOG , "aBLOG");
        usAdmin.setUserProperties("muthulee", props, null);
        //edit one
        usAdmin.setUserProperty("muthulee", DefaultRealmConstants.EMAIL, "muthuleexx@yahoo.com", null);
        //edit many
        props.clear();
        props.put(DefaultRealmConstants.EMAIL, "muthulee@gmail.com");
        props.put(DefaultRealmConstants.FIRST_NAME , "muthu");
        props.put(DefaultRealmConstants.LAST_NAME , "lee");
        usAdmin.setUserProperties("muthulee", props, null);
        
        //delete one
        usAdmin.deleteUserProperty("muthulee",DefaultRealmConstants.WEB_PAGE, null);
        
        //delete many
        List<String> lst = new ArrayList<String>();
        lst.add(DefaultRealmConstants.NAME_PREFIX);
        lst.add(DefaultRealmConstants.GENDER);
        lst.add(DefaultRealmConstants.TIME_ZONE);
        lst.add(DefaultRealmConstants.COMPANY_NAME);
        usAdmin.deleteUserProperties("muthulee", (String[])lst.toArray(new String[4]), null);
        
        
        /////////////////// getters /////////////////////////////
        
        String email = usAdmin.getUserPropertyValue("muthulee", DefaultRealmConstants.EMAIL, null);
        assertEquals("muthulee@gmail.com", email);
        
        String webPage = usAdmin.getUserPropertyValue("muthulee", DefaultRealmConstants.WEB_PAGE, null);
        assertNull(webPage);
        
        String[] names = {DefaultRealmConstants.EMAIL, DefaultRealmConstants.FIRST_NAME, DefaultRealmConstants.LAST_NAME};
        Map<String, String> returned = usAdmin.getUserProperties("muthulee", names, null);
        assertEquals(3, returned.size());
        assertEquals("muthu", returned.get(DefaultRealmConstants.FIRST_NAME));
        
    
    }

    public void doCustomProperties() throws UserStoreException{
        UserStoreAdmin usAdmin = realm.getUserStoreAdmin();    
        //add one
        usAdmin.setUserProperty("muthulee", "FavoriteSong", "Now and Forever", null);
        
        //add many
        Map<String, String> props = new HashMap<String, String>();
        props.put("FavoriteMovie", "ForrestGump");
        props.put("FavoriteTV", "BigBangTheory");
        usAdmin.setUserProperties("muthulee", props, null);
        
        //edit one
        usAdmin.setUserProperty("muthulee", "FavoriteTV", "Sports", null);
        
        //edit many
        props.put("FavoriteMovie", "P.S.I Love You");
        props.put("FavoriteTV", "CSI MIAMI");
        usAdmin.setUserProperties("muthulee", props, null);
        
        //delete one
        usAdmin.deleteUserProperty("muthulee", "FavoriteSong", null);
        
        
        //add binary
        usAdmin.setUserBinaryContent("muthulee", "picture", "Jiraffe".getBytes(), null);
        //edit
        usAdmin.setUserBinaryContent("muthulee", "picture", "Broomi".getBytes(), null);
        
        /////////////////// getters /////////////////////////
        String movie = usAdmin.getUserPropertyValue("muthulee", "FavoriteMovie", null);
        assertEquals("P.S.I Love You", movie);
        
        String song = usAdmin.getUserPropertyValue("muthulee", "FavoriteSong", null);
        assertNull(song);
        
        String[] names = {"FavoriteMovie","FavoriteTV"};
        Map<String, String> returned = usAdmin.getUserProperties("muthulee", names, null);
        assertEquals(2, returned.size());
        assertEquals("P.S.I Love You", returned.get("FavoriteMovie"));
        
        //delete many
        List<String> lst = new ArrayList<String>();
        lst.add("FavoriteMovie");
        lst.add("FavoriteTV");
        usAdmin.deleteUserProperties("muthulee", (String[])lst.toArray(new String[2]), null);
        
        String moviex = usAdmin.getUserPropertyValue("muthulee", "FavoriteMovie", null);
        assertNull(moviex);
        
        byte[] picture = usAdmin.getUserBinaryContent("muthulee", "picture", null);
        String retVal = new String(picture);
        assertEquals("Broomi", retVal);
        
    }
    
    public void doAllProperties() throws UserStoreException{
        UserStoreAdmin usAdmin = realm.getUserStoreAdmin();
        usAdmin.addUser("customer", "custi123");
        
        //add many
        Map<String, String> props = new HashMap<String, String>();
        props.put(DefaultRealmConstants.FIRST_NAME , "aFIRST_NAME");
        props.put(DefaultRealmConstants.LAST_NAME , "aLAST_NAME");
        props.put("FavoriteMovie", "ForrestGump");
        props.put("FavoriteTV", "BigBangTheory");
        usAdmin.setUserProperties("customer", props, null);
        
        //edit many
        props.clear();
        props.put(DefaultRealmConstants.FIRST_NAME , "bFIRST_NAME");
        props.put(DefaultRealmConstants.LAST_NAME , "bLAST_NAME");
        props.put("FavoriteMovie", "bForrestGump");
        props.put("FavoriteTV", "bBigBangTheory");
        usAdmin.setUserProperties("customer", props, null);
        
        List<String> lst = new ArrayList<String>();
        lst.add(DefaultRealmConstants.FIRST_NAME);
        lst.add(DefaultRealmConstants.LAST_NAME);
        lst.add("FavoriteMovie");
        
        //get many
        Map<String, String> map = usAdmin.getUserProperties("customer", (String[])lst.toArray(new String[3]), null);
        assertEquals(3, map.size());
        assertEquals("bForrestGump", map.get("FavoriteMovie"));
        
        //delete many
        usAdmin.deleteUserProperties("customer", (String[])lst.toArray(new String[3]), null);
        map = usAdmin.getUserProperties("customer", (String[])lst.toArray(new String[3]), null);
        assertEquals(3, map.size());
        assertNull(map.get("FavoriteMovie"));

    }
    
}
