/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.user.core.def;

import java.util.Arrays;
import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.hybrid.HybridInternalRoleAdmin;
import org.wso2.carbon.user.core.hybrid.HybridRealm;
import org.wso2.carbon.user.core.utils.BaseTestCase;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

public class DefaultRealmTest extends BaseTestCase {

    UserRealm realm = null;

    DefaultRealmConfig config = null;

    String connURL = "jdbc:h2:target/databasetest/DefaultUserDatabase";

        
    public void setUp() throws Exception {
        super.setUp();
        if (realm != null) {
            return;
        }
        Class clazz = Class.forName("org.h2.Driver");
        BasicDataSource ds = new BasicDataSource();
        ds.setUrl(connURL);
        ds.setDriverClassName("org.h2.Driver");

        DatabaseCreator creator = new DatabaseCreator(ds);
        creator.createRegistryDatabase();
        
               
        realm = new DefaultRealm();
        config = (DefaultRealmConfig) realm.getRealmConfiguration();
        config.setConnectionURL(connURL);
        config.setSysteUserName("System");
        config.setDigestFunction("SHA-256");
        config.setSaltedPassword(true);
        realm.init(config);
    }

    public void testRealm() throws Exception {
        userStuff();
        roleStuff();
        authorizationStuff(); 
       // attributesStuff();
        roleUserStuff();
        doTestRealm();
    }

    protected void userStuff() throws Exception {
        UserStoreAdmin admin = realm.getUserStoreAdmin();
        Authenticator authen = realm.getAuthenticator();
        UserStoreReader reader = realm.getUserStoreReader();
      
        // add
        admin.addUser("dimuthu", "credential");
        admin.addUser("vajira", "credential");
        assertTrue(authen.authenticate("dimuthu", "credential"));
                
        assertTrue(reader.isExistingUser("dimuthu"));
        assertFalse(reader.isExistingUser("muhaha"));
        // update
        admin.updateUser("dimuthu", "password", "credential");
        assertFalse(authen.authenticate("dimuthu", "credential"));
        // list
        String[] names = reader.getAllUserNames();
        assertEquals(2, names.length);
        // delete
        admin.deleteUser("vajira");
        assertFalse(authen.authenticate("vajira", "credential"));

        // restore stuff for further testing
        admin.addUser("vajira", "credential");
        admin.addUser("juhia", "jooo");
             
    }


    public void roleStuff() throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();
        // add
        admin.addRole("admin");
        admin.addRole("hora");
        admin.addRole("normal");

        // list
        String[] names = reader.getAllRoleNames();
        assertEquals(3, names.length);

        // addusertorole 
        admin.addUserToRole("dimuthu", "admin");
        admin.addUserToRole("dimuthu", "hora");
        admin.addUserToRole("vajira", "admin");

        String[] userRoles = reader.getUserRoles("dimuthu");
        assertEquals(2, userRoles.length);
        
        String[] usersInRoles = reader.getUsersInRole("admin");
        assertEquals(2, usersInRoles.length);

        // delete
        admin.deleteRole("normal");
        admin.deleteRole("hora");

        // restore stuff for further testing
        //dimuthu is a admin.
        admin.addRole("normal");
        admin.addRole("hora");
        admin.addUserToRole("dimuthu", "normal");
        
    }

    public void authorizationStuff() throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();

        AccessControlAdmin acAdmin = realm.getAccessControlAdmin();
        Authorizer athzr = realm.getAuthorizer();
        Authenticator authen = realm.getAuthenticator();

        acAdmin.authorizeUser("dimuthu", "floor", "write");
        acAdmin.authorizeRole("admin", "server", "read");
        acAdmin.authorizeRole("admin", "server", "read");
        acAdmin.authorizeUser("vajira", "floor", "write");
        acAdmin.denyUser("dimuthu", "editResource", "edit");
        acAdmin.authorizeRole("admin", "editResource", "edit");
        acAdmin.denyRole("admin", "editResource", "edit");

        assertTrue(athzr.isUserAuthorized("dimuthu", "floor", "write"));
        assertTrue(athzr.isRoleAuthorized("admin", "server", "read"));
       // assertFalse(athzr.isUserAuthorized("dimuthu", "editResource",
         //       "edit"));
       // assertFalse(athzr.isRoleAuthorized("admin", "editResource",
         //       "edit"));

        String[] authorizedUsers = athzr.getAllowedUsersForResource("floor",
                "write");
        String au = Arrays.toString(authorizedUsers);
        assertTrue(au.contains("vajira"));
        assertTrue(au.contains("dimuthu"));
        
        String[] authorizedRoles = athzr.getAllowedRolesForResource(
                "server", "read");
        String ar = Arrays.toString(authorizedRoles);
        assertTrue(ar.contains("admin"));
    
        String[] deniedRoles = athzr.getDeniedRolesForResource(
                "editResource", "edit");
        assertEquals(deniedRoles[0], "admin");
    
        acAdmin.clearUserAuthorization("dimuthu", "floor", "write");

        String[] deniedUsers = athzr.getDeniedUsersForResource("editResource",
        "edit");
        assertEquals(1,deniedUsers.length);
        
        assertTrue(athzr
                .isRoleAuthorized("admin",  "server", "read"));
       
        acAdmin.clearRoleAuthorization("admin", "server", "read");
        
      //  assertFalse(athzr
        //        .isRoleAuthorized("admin",  "server", "read"));
        
        assertTrue(athzr.isUserAuthorized("vajira", "floor", "write"));
        
        acAdmin.authorizeUser("juhia", "floor", "write");
        assertTrue(authen.authenticate("dimuthu", "password"));
        //assertFalse(athzr.isRoleAuthorized("admin", "server", "read"));
        acAdmin.authorizeRole("admin", "server", "read");
        System.out.println("Basic done");

        
        acAdmin.clearResourceAuthorizations("floor");
        acAdmin.clearRoleAuthorization("admin","server", "read");
        acAdmin.clearUserAuthorization("dimuthu", "floor", "write");
      //  acAdmin.copyAuthorizations("floor", "classroom");
      //  assertTrue(athzr.isUserAuthorized("juhia", "classroom",
        //        "write"));

       // acAdmin.copyAuthorizations("server", "desktop");
        //assertTrue(athzr.isRoleAuthorized("admin", "desktop", "read"));
        
        

    }

    public void roleUserStuff() throws UserStoreException {

        UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();
        AccessControlAdmin ac = realm.getAccessControlAdmin();
        Authorizer athzr = realm.getAuthorizer();

        admin.addUser("sam", "write");
        admin.addUser("jennifer", "write");
        admin.addUser("david", "write");

        admin.addRole("cook");
        admin.addRole("waiter");
        admin.addRole("hora2");

        ac.authorizeRole("cook", "new cooker", "invoke-service");
        ac.authorizeRole("waiter", "new apron", "edit");

        ac.denyRole("hora2", "new cooker", "invoke-service");
        ac.denyRole("hora2", "new apron", "edit");

        admin.addUserToRole("jennifer", "waiter");
        admin.addUserToRole("sam", "cook");
        
        //david is a cook and a theif
        admin.addUserToRole("david", "cook");
        admin.addUserToRole("david", "hora2");

        config.setPermissionAlgo(DefaultRealmConfig.PERMISSION_BLOCK_FIRST);
        assertTrue(athzr.isUserAuthorized("jennifer", "new apron",
                "edit"));
        assertTrue(athzr.isUserAuthorized("sam", "new cooker",
                "invoke-service"));

     //   assertFalse(athzr.isUserAuthorized("david", "new cooker",
       //         "invoke-service"));
        
        //david is good now
        admin.removeUserFromRole("david", "hora2");
        assertTrue(athzr.isUserAuthorized("david", "new cooker",
        "invoke-service"));
        
    }
    
    public void doTestRealm() throws Exception {
        AccessControlAdmin authManager = realm.getAccessControlAdmin();
        UserStoreAdmin usAdmin = realm.getUserStoreAdmin();
        usAdmin.addRole("everyone");
        usAdmin.addUser("sameera", "password");
        usAdmin.addUser("dimuthug", "password");
        usAdmin.addUserToRole("sameera", "everyone");
        usAdmin.addUserToRole("dimuthug", "everyone");
        usAdmin.addRole("bizzrole");
        usAdmin.addUserToRole("sameera", "bizzrole");
        
        authManager.authorizeRole("everyone", "/", "read");
        authManager.authorizeRole("everyone", "/top", "read");
        //authManager.authorizeRole("everyone", "/top/wso2", "read");
        authManager.denyRole("everyone", "/top/wso2/bizzness", "read");
        authManager.authorizeRole("bizzrole", "/top/wso2/bizzness", "read");
        
        assertEquals(1, authManager.getAllowedRolesForResource("/top/wso2/bizzness", "read").length);
        assertEquals(1, authManager.getDeniedRolesForResource("/top/wso2/bizzness", "read").length);
        assertFalse(authManager.isRoleAuthorized("everyone", "/top/wso2/bizzness", "read"));
        assertFalse(authManager.isUserAuthorized("dimuthu", "/top/wso2/bizzness", "read"));
    }

}
