package org.wso2.carbon.user.core.def;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

public class OracleDBTest {
    
    
    public static void doOracleStuff() throws Exception{
	
	BasicDataSource ds = new BasicDataSource();
	ds.setUrl("jdbc:oracle:thin:@10.100.1.88:1521:wsas");
	ds.setUsername("system");
	ds.setPassword("root");
	ds.setDriverClassName("oracle.jdbc.driver.OracleDriver");
	
    DatabaseCreator creator = new DatabaseCreator(ds);
    creator.createRegistryDatabase();
	ds = null;
	
	UserRealm realm = new DefaultRealm();
	DefaultRealmConfig config = new DefaultRealmConfig();
	config.setConnectionUserName("system");
	config.setConnectionPassword("root");
	config.setConnectionURL("jdbc:oracle:thin:@10.100.1.88:1521:wsas");
	config.setDriverName("oracle.jdbc.driver.OracleDriver");
	realm.init(config);
	
	if(!realm.getUserStoreReader().isExistingRole("vajira"))
	    realm.getUserStoreAdmin().deleteRole("vajira");
	
	regression(realm);
    }
       
    public static void main(String[] args) throws Exception {
	doOracleStuff();
	System.out.println("successful..........");
    }
    
    
    public static void regression(UserRealm realm) throws Exception {
        userStuff(realm);
        roleStuff(realm);
        authorizationStuff(realm);
        attributesStuff(realm);
        roleUserStuff(realm);
    }

    protected static void userStuff(UserRealm realm) throws Exception {
        UserStoreAdmin admin = realm.getUserStoreAdmin();
        Authenticator authen = realm.getAuthenticator();
        UserStoreReader reader = realm.getUserStoreReader();
      
        // add
        admin.addUser("dimuthu", "credential");
        admin.addUser("vajira", "credential");
        TestCase.assertTrue(authen.authenticate("dimuthu", "credential"));
                
        TestCase.assertTrue(reader.isExistingUser("dimuthu"));
        TestCase.assertFalse(reader.isExistingUser("muhaha"));
        // update
        admin.updateUser("dimuthu", "password", "credential");
        TestCase.assertFalse(authen.authenticate("dimuthu", "credential"));
        // list
        String[] names = reader.getAllUserNames();
        TestCase.assertEquals(2, names.length);
        // delete
        admin.deleteUser("vajira");
        TestCase.assertFalse(authen.authenticate("vajira", "credential"));

        // restore stuff for further testing
        admin.addUser("vajira", "credential");
        admin.addUser("juhia", "jooo");
             
    }

    public static void attributesStuff(UserRealm realm) throws Exception {
  /*      UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();

        Map props = new HashMap();
        props.put("address", "59C");
        props.put("telephone", "9870");
        admin.setUserProperties("dimuthu", props);
        
        String[] names = reader.getUserNamesWithPropertyValue("telephone", "9870");
        TestCase.assertEquals("dimuthu", names[0]);
        
        String[] nam = admin.getUserNamesWithPropertyValue(null, "9870");
        TestCase.assertEquals("dimuthu", nam[0]);

        Map stored = reader.getUserProperties("dimuthu");
        TestCase.assertEquals(props.size(), stored.size());
        TestCase.assertEquals("59C", stored.get("address"));

        props.put("address", "121/2");
        props.put("telephone", null);
        admin.setUserProperties("dimuthu", props);
        stored = reader.getUserProperties("dimuthu");
        TestCase.assertEquals(1, stored.size());
        TestCase.assertEquals("121/2", stored.get("address"));

        props.put("telephone", "34545");
        admin.setUserProperties("vajira", props);

        String[] attrNames = reader.getUserPropertyNames();
        TestCase.assertEquals(2, attrNames.length);
        
        */

    }

    public static void roleStuff(UserRealm realm) throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();
        // add
        admin.addRole("admin");
        admin.addRole("hora");
        admin.addRole("normal");

        // list
        String[] names = reader.getAllRoleNames();
        TestCase.assertEquals(3, names.length);

        // addusertorole 
        admin.addUserToRole("dimuthu", "admin");
        admin.addUserToRole("dimuthu", "hora");
        admin.addUserToRole("vajira", "admin");

        String[] userRoles = reader.getUserRoles("dimuthu");
        TestCase.assertEquals(2, userRoles.length);
        
        String[] usersInRoles = reader.getUsersInRole("admin");
        TestCase.assertEquals(2, usersInRoles.length);

        // delete
        admin.deleteRole("normal");
        admin.deleteRole("hora");

        // restore stuff for further testing
        //dimuthu is a admin.
        admin.addRole("normal");
        admin.addRole("hora");
        admin.addUserToRole("dimuthu", "normal");
        
    }

    public static void authorizationStuff(UserRealm realm) throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();

        AccessControlAdmin acAdmin = realm.getAccessControlAdmin();
        Authorizer athzr = realm.getAuthorizer();
        Authenticator authen = realm.getAuthenticator();

        acAdmin.authorizeUser("dimuthu", "floor", "dance");
        acAdmin.authorizeRole("admin", "server", "login");
        acAdmin.authorizeUser("vajira", "floor", "dance");
        acAdmin.denyUser("dimuthu", "denyResource", "deny");
        acAdmin.authorizeRole("admin", "denyResource", "deny");
        acAdmin.denyRole("admin", "denyResource", "deny");

        TestCase
                .assertTrue(athzr.isUserAuthorized("dimuthu", "floor", "dance"));
        TestCase.assertTrue(athzr.isRoleAuthorized("admin", "server", "login"));
        TestCase.assertFalse(athzr.isUserAuthorized("dimuthu", "denyResource",
                "deny"));
        TestCase.assertFalse(athzr.isRoleAuthorized("admin", "denyResource",
                "deny"));

        String[] authorizedUsers = athzr.getAllowedUsersForResource("floor",
                "dance");
        String au = Arrays.toString(authorizedUsers);
        TestCase.assertTrue(au.contains("vajira"));
        TestCase.assertTrue(au.contains("dimuthu"));
        
        String[] authorizedRoles = athzr.getAllowedRolesForResource(
                "server", "login");
        String ar = Arrays.toString(authorizedRoles);
        TestCase.assertTrue(ar.contains("admin"));
    
        String[] deniedRoles = athzr.getDeniedRolesForResource(
                "denyResource", "deny");
        TestCase.assertEquals(deniedRoles[0], "admin");
    
        acAdmin.clearUserAuthorization("dimuthu", "floor", "dance");

        String[] deniedUsers = athzr.getDeniedUsersForResource("denyResource",
        "deny");
        TestCase.assertEquals(1,deniedUsers.length);
        
        TestCase.assertTrue(athzr
                .isRoleAuthorized("admin",  "server", "login"));
       
        acAdmin.clearRoleAuthorization("admin", "server", "login");
        
        TestCase.assertFalse(athzr
                .isRoleAuthorized("admin",  "server", "login"));
        
        TestCase.assertTrue(athzr.isUserAuthorized("vajira", "floor", "dance"));
        
        acAdmin.authorizeUser("juhia", "floor", "dance");
        TestCase.assertTrue(authen.authenticate("dimuthu", "password"));
        TestCase
                .assertFalse(athzr.isUserAuthorized("admin", "server", "login"));
        acAdmin.authorizeRole("admin", "server", "login");
        System.out.println("Basic done");

        acAdmin.copyAuthorizations("floor", "classroom");
        TestCase.assertTrue(athzr.isUserAuthorized("juhia", "classroom",
                "dance"));

        acAdmin.copyAuthorizations("server", "desktop");
        TestCase
                .assertTrue(athzr.isRoleAuthorized("admin", "desktop", "login"));
        
        

    }

    public static void roleUserStuff(UserRealm realm) throws UserStoreException {

        UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();
        AccessControlAdmin ac = realm.getAccessControlAdmin();
        Authorizer athzr = realm.getAuthorizer();

        admin.addUser("sam", "boo");
        admin.addUser("jennifer", "boo");
        admin.addUser("david", "boo");

        admin.addRole("cook");
        admin.addRole("waiter");
        admin.addRole("hora2");

        ac.authorizeRole("cook", "new cooker", "cook stuff");
        ac.authorizeRole("waiter", "new apron", "wear");

        ac.denyRole("hora2", "new cooker", "cook stuff");
        ac.denyRole("hora2", "new apron", "wear");

        admin.addUserToRole("jennifer", "waiter");
        admin.addUserToRole("sam", "cook");
        
        //david is a cook and a theif
        admin.addUserToRole("david", "cook");
        admin.addUserToRole("david", "hora2");

       
        TestCase.assertTrue(athzr.isUserAuthorized("jennifer", "new apron",
                "wear"));
        TestCase.assertTrue(athzr.isUserAuthorized("sam", "new cooker",
                "cook stuff"));

        TestCase.assertFalse(athzr.isUserAuthorized("david", "new cooker",
                "cook stuff"));
        
        //david is good now
        admin.removeUserFromRole("david", "hora2");
        TestCase.assertTrue(athzr.isUserAuthorized("david", "new cooker",
        "cook stuff"));
        
        
        
    }
}
