package org.wso2.carbon.user.core.hybrid;

import java.sql.SQLException;
import java.util.Arrays;

import junit.framework.TestCase;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirAuthenticator;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirRealmConfig;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirUserStoreAdmin;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirUserStoreReader;

public class ActiveDirectoryTest {
    
    private static UserRealm realm = null;
    
    public void testActDir(){
        //dummy method - the test cannot be automated
    }

    public static void main(String[] args){
    
        try {
            String connectionName = "cn=Administrator,cn=users,dc=wso2,dc=lk";
            String connectionPassword = "admin123";
            String connectionUrl = "ldap://10.100.1.122:389";
            String searchBase = "dc=wso2,dc=lk";
            String connURL = "jdbc:h2:target/databasetest/ActDirTestDatabase";

            BasicDataSource ds = new BasicDataSource();
            ds.setUrl(connURL);
            ds.setUsername("roshan");
            ds.setUsername("jimmy");
            ds.setDriverClassName("org.h2.Driver");

            DatabaseCreator creator = new DatabaseCreator(ds);
            creator.createRegistryDatabase();

            ActDirRealmConfig aconfig = new ActDirRealmConfig();
            aconfig.setConnectionName(connectionName);
            aconfig.setConnectionPassword(connectionPassword);
            aconfig.setConnectionURL(connectionUrl);
            aconfig.setSearchBase(searchBase);
            
            Authenticator authenticator = new ActDirAuthenticator(aconfig);
            UserStoreReader reader = new ActDirUserStoreReader(aconfig);
            UserStoreReader admin = new ActDirUserStoreAdmin(aconfig);
            
            BasicDataSource dataSource = new BasicDataSource();
            dataSource.setUrl(connURL);
            dataSource.setUsername("roshan");
            dataSource.setUsername("jimmy");
            dataSource.setDriverClassName("org.h2.Driver");

            realm = new HybridRealm();
            HybridRealmConfig config = (HybridRealmConfig) realm.getRealmConfiguration();
            config.setAuthenticator(authenticator);
            config.setDataSource(dataSource);
            config.setUserStoreReader(reader);

            realm.init(config);
            
            doTestRealm();
        } catch (SQLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }
    
    
    public static void doTestRealm() throws Exception {
        authenticateStuff();
     //   roleStuff();
     //   authorizationStuff();
       // roleUserStuff();
       // internalRolesStuff();
    }

    public static void authenticateStuff() throws Exception {
       Authenticator admin = realm.getAuthenticator();
       admin.authenticate("prabath", "1qaz2wsx$"); 
    }
    
    public static void roleStuff() throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();
        // list
        String[] names = reader.getAllRoleNames();
        System.out.println("Role Names are :: " + Arrays.toString(names));
                
        String[] usernames = reader.getAllUserNames();
        System.out.println("User Names are :: " + Arrays.toString(usernames));
                
        String[] usersInRole = reader.getUsersInRole("Developers");
        System.out.println("Users in dev role are ::"+Arrays.toString(usersInRole));
        
        
        String[] usersRoles = reader.getUserRoles("sameera");
        System.out.println("Roles of Sameera::"+Arrays.toString(usersRoles));
        
    }

    public static void authorizationStuff() throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();

        AccessControlAdmin acAdmin = realm.getAccessControlAdmin();
        Authorizer athzr = realm.getAuthorizer();
        Authenticator authen = realm.getAuthenticator();

        acAdmin.authorizeUser("dimuthu", "floor", "dance");
        acAdmin.authorizeRole("admin", "server", "login");
        acAdmin.authorizeUser("vajira", "floor", "dance");
        acAdmin.denyUser("dimuthu", "denyResource", "deny");
        acAdmin.authorizeRole("admin", "denyResource", "deny");
        acAdmin.denyRole("admin", "denyResource", "deny");

        TestCase.assertTrue(athzr.isUserAuthorized("dimuthu", "floor", "dance"));
        TestCase.assertTrue(athzr.isRoleAuthorized("admin", "server", "login"));
        TestCase.assertFalse(athzr.isUserAuthorized("dimuthu", "denyResource", "deny"));
        TestCase.assertFalse(athzr.isRoleAuthorized("admin", "denyResource", "deny"));

        String[] authorizedUsers = athzr.getAllowedUsersForResource("floor", "dance");
        String au = Arrays.toString(authorizedUsers);
        TestCase.assertTrue(au.contains("vajira"));
        TestCase.assertTrue(au.contains("dimuthu"));

        String[] authorizedRoles = athzr.getAllowedRolesForResource("server", "login");
        String ar = Arrays.toString(authorizedRoles);
        TestCase.assertTrue(ar.contains("admin"));

        String[] deniedRoles = athzr.getDeniedRolesForResource("denyResource", "deny");
        TestCase.assertEquals(deniedRoles[0], "admin");

        acAdmin.clearUserAuthorization("dimuthu", "floor", "dance");

        String[] deniedUsers = athzr.getDeniedUsersForResource("denyResource", "deny");
        TestCase.assertEquals(1, deniedUsers.length);

        TestCase.assertTrue(athzr.isRoleAuthorized("admin", "server", "login"));

        acAdmin.clearRoleAuthorization("admin", "server", "login");

        TestCase.assertFalse(athzr.isRoleAuthorized("admin", "server", "login"));

        TestCase.assertTrue(athzr.isUserAuthorized("vajira", "floor", "dance"));

        acAdmin.authorizeUser("juhia", "floor", "dance");
        TestCase.assertFalse(athzr.isUserAuthorized("admin", "server", "login"));
        acAdmin.authorizeRole("admin", "server", "login");
        System.out.println("Basic done");

  //      acAdmin.copyAuthorizations("floor", "classroom");
  //      TestCase.assertTrue(athzr.isUserAuthorized("juhia", "classroom", "dance"));

  //      acAdmin.copyAuthorizations("server", "desktop");
  //      TestCase.assertTrue(athzr.isRoleAuthorized("admin", "desktop", "login"));

    }
    
    public static void internalRolesStuff() throws Exception{
        HybridInternalRoleAdmin admin  = ((HybridRealm)realm).getInternalRoleAdmin();
        admin.addInternalRole("ex-admin");
        admin.addInternalRole("ex-dev");
               
        admin.addUserToInternalRole("deep","ex-admin");
        admin.addUserToInternalRole("chamith", "ex-admin");
        admin.addUserToInternalRole("deep", "ex-dev");
        
        //after adding 2 roles
        String[] allRoles = admin.getInternalRoles();
        TestCase.assertEquals(2, allRoles.length);
        
        String[] roles = admin.getInternalRolesOfUser("deep");
        TestCase.assertEquals(2, roles.length);
        
        String[] users = admin.getUsersInInternalRole("ex-admin");
        TestCase.assertEquals(2, users.length);
        
        //removinging one
        admin.removeUserFromInternalRole("deep", "ex-dev");
        roles = admin.getInternalRolesOfUser("deep");
        TestCase.assertEquals(1, roles.length);
        
    }
}
