package org.wso2.carbon.user.core.hybrid;

import java.util.Arrays;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPAuthenticator;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPRealmConfig;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPUserStoreAdmin;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPUserStoreReader;

public class LDAPRealmTest {

   private static UserRealm realm = null;
    
    public void testActDir(){
        //dummy method - the test cannot be automated
    }
    
    public static void main(String[] args) throws Exception{
        
        String connectionName = "cn=admin,dc=wso2,dc=com";
        String connectionPassword = "secret";
        String connectionUrl = "ldap://localhost:389";
        String searchBase = "dc=wso2,dc=com";
        String connURL = "jdbc:h2:target/databasetest/LDAPTestDatabase";

        BasicDataSource ds = new BasicDataSource();
        ds.setUrl(connURL);
        ds.setUsername("roshan");
        ds.setUsername("jimmy");
        ds.setDriverClassName("org.h2.Driver");
        
        DatabaseCreator creator = new DatabaseCreator(ds);
        creator.createRegistryDatabase();

        LDAPRealmConfig aconfig = new LDAPRealmConfig();
        aconfig.setConnectionName(connectionName);
        aconfig.setConnectionPassword(connectionPassword);
        aconfig.setConnectionURL(connectionUrl);
        aconfig.setUidSearchName("uid");
        aconfig.setUserContextName("ou=people,dc=wso2,dc=com");
        
        Authenticator authenticator = new LDAPAuthenticator(aconfig);
        UserStoreReader reader = new LDAPUserStoreReader(aconfig);
        UserStoreReader admin = new LDAPUserStoreAdmin(aconfig);
        
        BasicDataSource dataSource = new BasicDataSource();
        dataSource.setUrl(connURL);
        dataSource.setUsername("roshan");
        dataSource.setUsername("jimmy");
        dataSource.setDriverClassName("org.h2.Driver");

        realm = new HybridRealm();
        HybridRealmConfig config = (HybridRealmConfig) realm.getRealmConfiguration();
        config.setAuthenticator(authenticator);
        config.setDataSource(dataSource);
        config.setUserStoreReader(reader);

        realm.init(config);
        
        doTestRealm();
    }
    
    
    public static void doTestRealm() throws Exception {
        authenticateStuff();
        roleStuff();
    }

    public static void authenticateStuff() throws Exception {
       Authenticator admin = realm.getAuthenticator();
       boolean value = admin.authenticate("lionel", "alice");
       System.out.println(value);
    }

    public static void roleStuff() throws Exception {

        UserStoreAdmin admin = realm.getUserStoreAdmin();
        UserStoreReader reader = realm.getUserStoreReader();
        // list
        String[] usernames = reader.getAllUserNames();
        System.out.println("User Names are :: " + Arrays.toString(usernames));        
    }

    
}
