package org.wso2.carbon.user.core.performance;

import java.io.File;
import java.util.Date;

import org.apache.commons.dbcp.BasicDataSource;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.utils.BaseTestCase;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;
import org.wso2.carbon.user.core.def.DefaultRealm;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;

public class UserMgtProfileTest extends BaseTestCase {

    public static String CONN_URL = "jdbc:h2:target/databasetest/DefaultUserDatabase";
    public static String RESOURCE_VAL = "resource";
    public static String USER_VAL = "user";
    public static String ROLE_VAL = "role";
    public static int RESOURCE_COUNT = 300;
    public static int USER_COUNT = 10;
    public static int ROLE_COUNT = 4;

    public void testProfileIsAuthorized() throws Exception {
	UserRealm realm = new DefaultRealm();
	DefaultRealmConfig config = (DefaultRealmConfig) realm.getRealmConfiguration();
	config.setConnectionURL(CONN_URL);
	config.setDriverName("org.h2.Driver");
	config.setPermissionAlgo(DefaultRealmConfig.PERMISSION_BLOCK_FIRST);
	realm.init(config);

	AccessControlAdmin acAdmin = realm.getAccessControlAdmin();
	long start = 0;
	for (int i = 0; i < RESOURCE_COUNT; i++) {
	    //long start = new Date().getTime();
	    if(i==1){
		start = System.nanoTime();
	    }
	    acAdmin.isUserAuthorized("user2", RESOURCE_VAL + i, "write");
	    //long end = new Date().getTime();
	    //System.out.println(end-start);
	    
	}
	long end = System.nanoTime();
	long average = (end - start) / RESOURCE_COUNT-1;
	System.out.println("Average time per is user authorization :: " + average);

    }

    public void _testProfileCopyAuthorization() throws Exception {
	AccessControlAdmin acAdmin;
	long average;
	UserRealm realm = new DefaultRealm();
	DefaultRealmConfig config = (DefaultRealmConfig) realm.getRealmConfiguration();
	config.setConnectionURL(CONN_URL);
	config.setDriverName("org.h2.Driver");
	realm.init(config);
	acAdmin = realm.getAccessControlAdmin();
	try {
	   
	    long start = new Date().getTime();
	    for (int i = 0; i < RESOURCE_COUNT; i++) {
		acAdmin.copyAuthorizations(RESOURCE_VAL+i, "resourcex"+i);
	    }
	    long end = new Date().getTime();

	    average = (end - start) / RESOURCE_COUNT;

	    System.out.println("Average time per copy authorization :: "+average);
	} catch (RuntimeException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	} finally {
	    for (int i = 0; i < RESOURCE_COUNT; i++) {
		acAdmin.clearResourceAuthorizations("resourcex"+i);
	    }
	}

    }

    public void setUp() throws Exception {
	String fileName = "target/DefaultUserDatabase";
	File file = new File(fileName);
	if (file.exists()) {
	    return;
	}

	Class clazz = Class.forName("org.h2.Driver");
	BasicDataSource ds = new BasicDataSource();
	ds.setUrl(CONN_URL);
	ds.setDriverClassName("org.h2.Driver");

    DatabaseCreator creator = new DatabaseCreator(ds);
    creator.createRegistryDatabase();

	UserRealm realm = new DefaultRealm();
	DefaultRealmConfig config = (DefaultRealmConfig) realm.getRealmConfiguration();
	config.setConnectionURL(CONN_URL);
	realm.init(config);

	UserStoreAdmin admin = realm.getUserStoreAdmin();
	for (int i = 0; i < ROLE_COUNT; i++) {
	    admin.addRole(ROLE_VAL + i);
	}

	for (int i = 0; i < USER_COUNT; i++) {
	    String user = USER_VAL + i;
	    admin.addUser(user, "password");
	    if (i % 2 == 0) {
		admin.addUserToRole(user, "role0");
		admin.addUserToRole(user, "role1");
		admin.addUserToRole(user, "role2");
		admin.addUserToRole(user, "role3");
	    }
	}

	AccessControlAdmin acAdmin = realm.getAccessControlAdmin();
	for (int i = 0; i < RESOURCE_COUNT; i++) {
	    acAdmin.authorizeRole("role1", RESOURCE_VAL + i, "read"+i);
	    acAdmin.authorizeRole("role2", RESOURCE_VAL + i, "write"+i);
	}

    }

}
