package org.wso2.carbon.user.ui.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.stub.AddExternalUserStoreConfig;
import org.wso2.carbon.user.stub.AddRoleToInternalStore;
import org.wso2.carbon.user.stub.AddSpecialInternalRole;
import org.wso2.carbon.user.stub.AddSystemPermissionsToRole;
import org.wso2.carbon.user.stub.AddUserToInternalStore;
import org.wso2.carbon.user.stub.ChangePassword;
import org.wso2.carbon.user.stub.DeleteRoleFromInternalStore;
import org.wso2.carbon.user.stub.DeleteSpecialInternalRole;
import org.wso2.carbon.user.stub.DeleteUserFromInternalStore;
import org.wso2.carbon.user.stub.GetAllRolesNamesResponse;
import org.wso2.carbon.user.stub.GetExternalGroupsResponse;
import org.wso2.carbon.user.stub.GetExternalUserStoreConfigResponse;
import org.wso2.carbon.user.stub.GetExternalUsersResponse;
import org.wso2.carbon.user.stub.GetInternalRolesResponse;
import org.wso2.carbon.user.stub.GetInternalUsersResponse;
import org.wso2.carbon.user.stub.GetSpecialInternalRolesResponse;
import org.wso2.carbon.user.stub.GetSystemPermissionsOfExternalRole;
import org.wso2.carbon.user.stub.GetSystemPermissionsOfExternalRoleResponse;
import org.wso2.carbon.user.stub.GetSystemPermissionsOfRole;
import org.wso2.carbon.user.stub.GetSystemPermissionsOfRoleResponse;
import org.wso2.carbon.user.stub.GetSystemPermissionsOfSpecialInternalRole;
import org.wso2.carbon.user.stub.GetSystemPermissionsOfSpecialInternalRoleResponse;
import org.wso2.carbon.user.stub.GetSystemPermissionsResponse;
import org.wso2.carbon.user.stub.GetUserRoles;
import org.wso2.carbon.user.stub.GetUserRolesResponse;
import org.wso2.carbon.user.stub.GetUsersInExternalRole;
import org.wso2.carbon.user.stub.GetUsersInExternalRoleResponse;
import org.wso2.carbon.user.stub.GetUsersInRole;
import org.wso2.carbon.user.stub.GetUsersInRoleResponse;
import org.wso2.carbon.user.stub.GetUsersOfSpecialInternalRole;
import org.wso2.carbon.user.stub.GetUsersOfSpecialInternalRoleResponse;
import org.wso2.carbon.user.stub.HasExternalUserStoreResponse;
import org.wso2.carbon.user.stub.ListExternalUsers;
import org.wso2.carbon.user.stub.ListExternalUsersResponse;
import org.wso2.carbon.user.stub.ListInternalUsers;
import org.wso2.carbon.user.stub.ListInternalUsersResponse;
import org.wso2.carbon.user.stub.TestConnectionResponse;
import org.wso2.carbon.user.stub.UpdateExternalUserStoreConfig;
import org.wso2.carbon.user.stub.UpdateRoleName;
import org.wso2.carbon.user.stub.UpdateSystemPermissionsOfExternalRole;
import org.wso2.carbon.user.stub.UpdateSystemPermissionsOfRole;
import org.wso2.carbon.user.stub.UpdateSystemPermissionsOfSpecialInternalRole;
import org.wso2.carbon.user.stub.UpdateUserName;
import org.wso2.carbon.user.stub.UpdateUsersOfRole;
import org.wso2.carbon.user.stub.UpdateUsersOfSpecialInternalRole;
import org.wso2.carbon.user.stub.UserManagementServiceStub;
import org.wso2.carbon.user.stub.xsd.StoreProperty;

public class UserMgtClient {

    public static final int MAX_USER_COUNT = 500;
    private UserManagementServiceStub stub = null;
    private String serviceEndPoint = null;

    private static Log log = LogFactory.getLog(UserMgtClient.class);

    public UserMgtClient(String cookie, String url, ConfigurationContext configContext)
            throws java.lang.Exception {
        try {

            this.serviceEndPoint = url + "UserManagementService";
            stub = new UserManagementServiceStub(configContext, serviceEndPoint);

            ServiceClient client = stub._getServiceClient();
            Options option = client.getOptions();
            option.setManageSession(true);
            option.setProperty(org.apache.axis2.transport.http.HTTPConstants.COOKIE_STRING, cookie);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getInternalUsers() throws java.lang.Exception {
        try {
            GetInternalUsersResponse response = stub.getInternalUsers();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }
    
    public String[] listInternalUsers(String filter) throws java.lang.Exception {
	try {
            ListInternalUsers request = new ListInternalUsers();
            request.setFilter(filter);
            ListInternalUsersResponse response = stub.listInternalUsers(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getInternalRoles() throws java.lang.Exception {
        try {
            GetInternalRolesResponse response = stub.getInternalRoles();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void deleteUserFromInternalStore(String name) throws java.lang.Exception {
        try {
            DeleteUserFromInternalStore request = new DeleteUserFromInternalStore();
            request.setUserName(name);
            stub.deleteUserFromInternalStore(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void addUserToInternalStore(UserBean userBean) throws java.lang.Exception {
        try {
            AddUserToInternalStore request = new AddUserToInternalStore();
            request.setUserName(userBean.getUsername());
            request.setPassword(userBean.getPassword());
            request.setRoles(userBean.getUserRoles());
            stub.addUserToInternalStore(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void changePassword(String name, String newPassword) throws java.lang.Exception {
        try {
            ChangePassword request = new ChangePassword();
            request.setUserName(name);
            request.setNewPassword(newPassword);
            stub.changePassword(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void addRoleToInternalStore(RoleBean roleBean) throws java.lang.Exception {
        try {
            AddRoleToInternalStore request = new AddRoleToInternalStore();
            request.setRoleName(roleBean.getRoleName());
            if(roleBean.getSelectedUsers().length == 0){
        	request.setUserNames(new String[]{null});
            }else{
        	request.setUserNames(roleBean.getSelectedUsers());
            }
            request.setPermissions(roleBean.getSelectedPermissions());
            stub.addRoleToInternalStore(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void deleteRoleFromInternalStore(String roleName) throws java.lang.Exception {
        try {
            DeleteRoleFromInternalStore request = new DeleteRoleFromInternalStore();
            request.setRoleName(roleName);
            stub.deleteRoleFromInternalStore(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void updateUsersOfRole(RoleBean roleBean) throws java.lang.Exception {
        try {
            UpdateUsersOfRole request = new UpdateUsersOfRole();
            request.setRoleName(roleBean.getRoleName());
            request.setUsersSelected(roleBean.getSelectedUsers());
            request.setUsersShown(roleBean.getShownUsers());
            stub.updateUsersOfRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void addRolesToResource(String[] roleNames, String resource, String action) {

    }

    public void updateRolesofResource(String[] roleNames, String resource, String action) {

    }

    public String[] getUsersInRole(String roleName) throws java.lang.Exception {
        try {
            GetUsersInRole request = new GetUsersInRole();
            request.setRoleName(roleName);
            GetUsersInRoleResponse response = stub.getUsersInRole(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getSystemPermissions() throws java.lang.Exception {
        try {
            GetSystemPermissionsResponse response = stub.getSystemPermissions();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void addSystemPermissionsToRole(String roleName, String[] actions)
            throws java.lang.Exception {
        try {
            AddSystemPermissionsToRole request = new AddSystemPermissionsToRole();
            request.setRoleName(roleName);
            request.setActions(actions);
            stub.addSystemPermissionsToRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }
    
    public String[] getUsersInExternalRole(String roleName) throws java.lang.Exception {
	try {
	    GetUsersInExternalRole request = new GetUsersInExternalRole();
            request.setRoleName(roleName);
            GetUsersInExternalRoleResponse response = stub.getUsersInExternalRole(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getSystemPermissionsOfRole(String roleName) throws java.lang.Exception {
        try {
            GetSystemPermissionsOfRole request = new GetSystemPermissionsOfRole();
            request.setRoleName(roleName);
            GetSystemPermissionsOfRoleResponse response = stub.getSystemPermissionsOfRole(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void updateSystemPermissionsOfRole(RoleBean bean) throws java.lang.Exception {
        try {
            UpdateSystemPermissionsOfRole request = new UpdateSystemPermissionsOfRole();
            request.setRoleName(bean.getRoleName());
            request.setActions(bean.getSelectedPermissions());
            stub.updateSystemPermissionsOfRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    // ***************************** External User Store
    // ******************************* //

    public boolean hasExternalUserStore() throws java.lang.Exception {
        try {
            HasExternalUserStoreResponse response = stub.hasExternalUserStore();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getExternalUsers() throws java.lang.Exception {
        try {
            GetExternalUsersResponse response = stub.getExternalUsers();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] listExternalUsers(String filter) throws java.lang.Exception {
	try {
            ListExternalUsers request = new ListExternalUsers();
            request.setFilter(filter);
            ListExternalUsersResponse response = stub.listExternalUsers(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }
    
    public String[] getExternalGroups() throws java.lang.Exception {
        try {
            GetExternalGroupsResponse response = stub.getExternalGroups();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void addExternalUserStoreConfig(UserStoreConfigBean config) throws java.lang.Exception {
        try {
            AddExternalUserStoreConfig request = new AddExternalUserStoreConfig();
            request.setRealmType(config.getRealmType());
            request.setConfigs(this.getStorePropertyFromConfig(config));
            stub.addExternalUserStoreConfig(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }
    public void updateExternalUserStoreConfig(UserStoreConfigBean config) throws java.lang.Exception {
        try {
            StoreProperty[] props = this.getStorePropertyFromConfig(config);
            UpdateExternalUserStoreConfig request = new UpdateExternalUserStoreConfig();
            request.setProps(props);
            stub.updateExternalUserStoreConfig(request);
        } catch (java.lang.Exception e) {
            e.printStackTrace();
            log.error(e);
            throw e;
        }
    }

    public Map<String, String> getExternalUserStoreConfig() throws java.lang.Exception {
        Map<String, String> mapValues = new HashMap<String, String>();
        try {
            GetExternalUserStoreConfigResponse response = stub.getExternalUserStoreConfig();
            StoreProperty[] props = response.get_return();
            for(StoreProperty prop : props){
        	if(prop.getValue() == null){
        	    mapValues.put(prop.getName(), "");
        	}else{
        	    mapValues.put(prop.getName(), prop.getValue());
        	}
            }
            return mapValues;
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void deleteExternalUserStore() throws java.lang.Exception {
        try {
            stub.deleteExternalUserStore();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public boolean testConnection() throws java.lang.Exception {
        try {
            TestConnectionResponse response = stub.testConnection();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void updateSystemPermissionsOfExternalRole(RoleBean bean)
            throws java.lang.Exception {
        try {
            UpdateSystemPermissionsOfExternalRole request = new UpdateSystemPermissionsOfExternalRole();
            request.setRoleName(bean.getRoleName());
            request.setPermissions(bean.getSelectedPermissions());
            stub.updateSystemPermissionsOfExternalRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getSystemPermissionsOfExternalRole(String roleName) throws java.lang.Exception {
        try {
            GetSystemPermissionsOfExternalRole request = new GetSystemPermissionsOfExternalRole();
            request.setRoleName(roleName);
            GetSystemPermissionsOfExternalRoleResponse response = stub
                    .getSystemPermissionsOfExternalRole(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }
    
    public void updateUserName(String userName, String newUserName) throws java.lang.Exception{
        try {
            UpdateUserName request = new UpdateUserName();
            request.setUserName(userName);
            request.setNewUserName(newUserName);
            stub.updateUserName(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }	
    }
    
    public void updateRoleName(String roleName, String newRoleName) throws java.lang.Exception{
	try {
            UpdateRoleName request = new UpdateRoleName();
            request.setRoleName(roleName);
            request.setNewRoleName(newRoleName);
            stub.updateRoleName(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }	
    }

    // ******************************* Roles with External users

    public void addSpecialInternalRole(RoleBean roleBean) throws java.lang.Exception {
        try {
            AddSpecialInternalRole request = new AddSpecialInternalRole();
            request.setRoleName(roleBean.getRoleName());
            request.setPermissions(roleBean.getSelectedPermissions());
            request.setUserNames(roleBean.getSelectedUsers());
            stub.addSpecialInternalRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void deleteSpecialInternalRole(String roleName) throws java.lang.Exception {
        try {
            DeleteSpecialInternalRole request = new DeleteSpecialInternalRole();
            request.setRoleName(roleName);
            stub.deleteSpecialInternalRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }

    }    
    
    public String[] getSpecialInternalRoles() throws java.lang.Exception {
        try {
            GetSpecialInternalRolesResponse request = stub.getSpecialInternalRoles();
            return request.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void updateSystemPermissionsOfSpecialInternalRole(RoleBean bean)
            throws java.lang.Exception {
        try {
            UpdateSystemPermissionsOfSpecialInternalRole request = new UpdateSystemPermissionsOfSpecialInternalRole();
            request.setRoleName(bean.getRoleName());
            request.setPermissions(bean.getSelectedPermissions());
            stub.updateSystemPermissionsOfSpecialInternalRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public void updateUsersOfSpecialInternalRole(RoleBean bean) throws java.lang.Exception {
        try {
            UpdateUsersOfSpecialInternalRole request = new UpdateUsersOfSpecialInternalRole();
            request.setRoleName(bean.getRoleName());
            request.setUsersSelected(bean.getSelectedUsers());
            request.setUsersShown(bean.getShownUsers());
            stub.updateUsersOfSpecialInternalRole(request);
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getUsersOfSpecialInternalRole(String roleName) throws java.lang.Exception {
        try {
            GetUsersOfSpecialInternalRole request = new GetUsersOfSpecialInternalRole();
            request.setRoleName(roleName);
            GetUsersOfSpecialInternalRoleResponse response = stub
                    .getUsersOfSpecialInternalRole(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getSystemPermissionsOfSpecialInternalRole(String roleName)
            throws java.lang.Exception {
        try {
            GetSystemPermissionsOfSpecialInternalRole request = new GetSystemPermissionsOfSpecialInternalRole();
            request.setRoleName(roleName);
            GetSystemPermissionsOfSpecialInternalRoleResponse response = stub
                    .getSystemPermissionsOfSpecialInternalRole(request);
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }

    public String[] getUserRoles(String userName) throws java.lang.Exception {
        try {
            GetUserRoles request = new GetUserRoles();
            request.setUserName(userName);
            GetUserRolesResponse response = stub.getUserRoles(request);
            return response.get_return();
        } catch (Exception e) {
            log.error(e);
            throw e;
        }
        
    }
    // //// ******************************** gneral

    public String[] getAllRolesNames() throws java.lang.Exception {
        try {
            GetAllRolesNamesResponse response = stub.getAllRolesNames();
            return response.get_return();
        } catch (java.lang.Exception e) {
            log.error(e);
            throw e;
        }
    }
    
    
    // //// ******************************************** private methods
    private StoreProperty[] getStorePropertyFromConfig(UserStoreConfigBean config) {
        List<StoreProperty> lst = new ArrayList<StoreProperty>();
        String value = config.getConnectionName();
        addProperty(lst, "ConnectionName", value);
        value = config.getConnectionPassword();
        addProperty(lst, "ConnectionPassword", value);
        value = config.getConnectionURL();
        addProperty(lst, "ConnectionURL", value);
        String realmType = config.getRealmType();
        // jdbc
        if ("jdbc".equals(realmType)) {
            value = config.getRoleListSQL();
            addProperty(lst, "RoleListSQL", value);
            value = config.getUserListSQL();
            addProperty(lst, "UserListSQL", value);
            value = config.getUserRoleSQL();
            addProperty(lst, "UserRoleSQL", value);
            value = config.getUserPasswordSQL();
            addProperty(lst, "UserPasswordSQL", value);
            value = config.getDriverName();
            addProperty(lst, "DriverName", value);
            value = config.getUsersInRoleSQL();
            addProperty(lst, "UsersInRoleSQL", value);
            value = config.getIsUserExistingSQL();
            addProperty(lst, "IsUserExistingSQL", value);
            value = config.getUserFilterSQL();
            addProperty(lst, "UserFilterSQL", value);
            value = config.getAttrsSQL();
            addProperty(lst, "AttrsSQL", value);
            value = config.getAttrsForProfileSQL();
            addProperty(lst, "AttrsForProfileSQL", value);
            value = config.getProfileNamesSQL();
            addProperty(lst, "ProfileNamesSQL", value);
        } else if("ldap".equals(realmType)){
            value = config.getUserContextName();
            addProperty(lst, "UserContextName", value);
            value  = config.getUIDSearchName();
            addProperty(lst, "UIDSearchName", value);
        } else if("act-dir".equals(realmType)){
            value = config.getSearchBase();
            addProperty(lst, "SearchBase", value);
        }

        return (StoreProperty[]) lst.toArray(new StoreProperty[lst.size()]);
    }

    private void addProperty(List<StoreProperty> lst, String name, String value) {
        StoreProperty prop = new StoreProperty();
        prop.setName(name);
        prop.setValue(value);
        lst.add(prop);
    }

}
