package org.wso2.carbon.user.mgt;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.i18n.Messages;

public class UMDatabaseManager {

    private DataSource dataSource = null;

    private final String CREATE_TABLE = "CREATE TABLE USER_MGT_PROPERTIES(ID INTEGER GENERATED BY DEFAULT AS IDENTITY, "
            + "PROPERTY_NAME VARCHAR(255) NOT NULL, "
            + "PROPERTY_VALUE VARCHAR(255), "
            + "PRIMARY KEY (ID))";
    private final String SET_PROPERTY = "INSERT INTO USER_MGT_PROPERTIES(PROPERTY_NAME, PROPERTY_VALUE) VALUES (?,?)";
    private final String GET_PROPERTY = "SELECT PROPERTY_VALUE FROM USER_MGT_PROPERTIES WHERE PROPERTY_NAME=?";
    private final String GET_PROPERTIES = "SELECT PROPERTY_NAME, PROPERTY_VALUE FROM USER_MGT_PROPERTIES";
    private final String DELETE_PROPERTIES = "DELETE * FROM USER_MGT_PROPERTIES";
    public UMDatabaseManager(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public String getProperty(String name) throws SQLException {
        String value = null;
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            PreparedStatement stmt = dbConnection.prepareStatement(GET_PROPERTY);
            stmt.setString(1, name);
            stmt.executeQuery();
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                value = rs.getString(1);
            }
        } finally {
            if (dbConnection != null) {
                dbConnection.commit();
                dbConnection.close();
            }
        }
        return value;
    }

    public Map<String, String> getExternalStoreProperties() throws SQLException {
        Connection dbConnection = null;
        Map<String, String> map = new HashMap<String, String>();
        try {
            dbConnection = dataSource.getConnection();
            PreparedStatement stmt = dbConnection.prepareStatement(GET_PROPERTIES);
            ResultSet rs = stmt.executeQuery();
            while (rs.next()) {
                String name = rs.getString(1);
                String value = rs.getString(2);
                map.put(name, value);
            }
        } finally {
            if (dbConnection != null) {
                dbConnection.commit();
                dbConnection.close();
            }
        }
        return map;
    }

    public void deleteAllProperties() throws SQLException {
        Connection dbConnection = null;
        Map<String, String> map = new HashMap<String, String>();
        try {
            dbConnection = dataSource.getConnection();
            PreparedStatement stmt = dbConnection.prepareStatement(DELETE_PROPERTIES);
            stmt.executeUpdate();
        } finally {
            if (dbConnection != null) {
                dbConnection.commit();
                dbConnection.close();
            }
        }
    }
    
    public void setProperty(String name, String value) throws SQLException {
        Connection dbConnection = null;
        try {
            dataSource.getConnection();
            dbConnection = dataSource.getConnection();
            dbConnection.setAutoCommit(false);
            PreparedStatement stmt = dbConnection.prepareStatement(SET_PROPERTY);
            stmt.setString(1, name);
            stmt.setString(2, value);
            stmt.executeUpdate();
            stmt.executeUpdate();
        } finally {
            if (dbConnection != null) {
                dbConnection.commit();
                dbConnection.close();
            }
        }
    }

    public void createManagementTables() throws SQLException {
        Connection dbConnection = dataSource.getConnection();
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "USER_MGT_PROPERTIES", null);
        Statement stmt = dbConnection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                ResultSet.CONCUR_UPDATABLE);

        if (rs.next() == false) {

            stmt.executeUpdate(CREATE_TABLE);

            dbConnection.commit();
        } else {
            // TODO
        }
        dbConnection.close();
    }
    

}
