package org.wso2.carbon.user.mgt;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTracker;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.ClaimMapping;
import org.wso2.carbon.user.core.ProfileConfiguration;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.claim.ClaimDAO;
import org.wso2.carbon.user.core.claim.DefaultClaimBuilder;
import org.wso2.carbon.user.core.hybrid.HybridRealm;
import org.wso2.carbon.user.core.hybrid.HybridRealmConfig;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirAuthenticator;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirRealmConfig;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirUserStoreAdmin;
import org.wso2.carbon.user.core.hybrid.actdir.ActDirUserStoreReader;
import org.wso2.carbon.user.core.hybrid.jdbc.JDBCAuthenticator;
import org.wso2.carbon.user.core.hybrid.jdbc.JDBCRealmConfig;
import org.wso2.carbon.user.core.hybrid.jdbc.JDBCUserStoreAdmin;
import org.wso2.carbon.user.core.hybrid.jdbc.JDBCUserStoreReader;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPAuthenticator;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPRealmConfig;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPUserStoreAdmin;
import org.wso2.carbon.user.core.hybrid.ldap.LDAPUserStoreReader;
import org.wso2.carbon.user.core.profile.DefaultProfileBuilder;
import org.wso2.carbon.user.core.profile.ProfileConfigDAO;
import org.wso2.carbon.user.mgt.service.UserMgtException;

public class UserMgtContext {

    private static Log log = LogFactory.getLog(UserMgtContext.class);

    private static BundleContext bundleContext = null;
    private static String connectionURL = null;
    private static String connectionName = null;
    private static String connectionPass = null;
    private static String driverName = null;
    private static List<String> systemPermissions = null;
    private static RegistryService registryService = null;
    private static UserRealm defaultRealm = null;
    private static ServiceTracker hybridTracker = null;
    private static ServiceRegistration hybridRealmRegistration = null;

    public static Registry getRegistry() throws UserMgtException {
        if (registryService == null) {
            throw new UserMgtException("Registry is null");
        }
        try {
            return registryService.getSystemRegistry();
        } catch (Exception e) {
            e.printStackTrace();
            throw new UserMgtException();
        }
    }

    public static UserRealm getDefaultRealm() throws UserMgtException {
        if (defaultRealm == null) {
            throw new UserMgtException();
        }
        return defaultRealm;
    }

    public static UserRealm getHybridRealm() throws UserMgtException {
        UserRealm realm = null;
        if (hybridTracker != null) {
            realm = (UserRealm) hybridTracker.getService();
        }
        return realm;
    }

    public synchronized static void registerHybridRealmService() throws Exception {
        HybridRealm realm = createHybridRealm();
        if (realm != null && hybridRealmRegistration == null) {
            Dictionary dictionary = new Hashtable();
            dictionary.put(UserCoreConstants.REALM_GENRE, UserCoreConstants.HYBRID_REALM);
            hybridRealmRegistration = bundleContext.registerService(UserRealm.class.getName(),
                    realm, dictionary);
            ServiceReference[] refs = bundleContext.getServiceReferences(UserRealm.class.getName(),
                    "(RealmGenre=Hybrid)");
            hybridTracker = new ServiceTracker(bundleContext, refs[0], null);
            hybridTracker.open();
        }
    }

    public synchronized static void unregisterHybridRealmService() {
        if (hybridRealmRegistration != null) {
            hybridRealmRegistration.unregister();
            hybridRealmRegistration = null;
        }

    }

    public static void init(BundleContext bc, String url, String username, String password,
            String driver) throws Exception {
        bundleContext = bc;
        connectionURL = url;
        connectionName = username;
        connectionPass = password;
        driverName = driver;

        HybridRealm realm = createHybridRealm();
        if (realm != null) {
            Dictionary dictionary = new Hashtable();
            dictionary.put(UserCoreConstants.REALM_GENRE, UserCoreConstants.HYBRID_REALM);
            hybridRealmRegistration = bundleContext.registerService(UserRealm.class.getName(),
                    realm, dictionary);
            ServiceReference[] refs = bundleContext.getServiceReferences(UserRealm.class.getName(),
                    "(RealmGenre=Hybrid)");
            hybridTracker = new ServiceTracker(bundleContext, refs[0], null);
            hybridTracker.open();
        }
    }

    public static BundleContext getBundleContext() {
        return bundleContext;
    }

    public static String getConnectionURL() {
        return connectionURL;
    }

    public static String getConnectionName() {
        return connectionName;
    }

    public static String getConnectionPass() {
        return connectionPass;
    }

    public static String getDriverName() {
        return driverName;
    }

    public static List<String> getSystemPermissions() {
        return systemPermissions;
    }

    public static String[] getSystemPermissionsAsArray() {
        String[] array = systemPermissions.toArray(new String[systemPermissions.size()]);
        return array;
    }

    public static void setSystemPermissions(List<String> systemPermissions) {
        UserMgtContext.systemPermissions = systemPermissions;
    }

    private static HybridRealm createHybridRealm() throws UserMgtException {
        HybridRealm realm = new HybridRealm();
        try {

            Registry registry = getRegistry();

            if (!registry.resourceExists(UserMgtConstants.EXTERNAL_USERSTORE)) {
                return null;
            }

            Resource resource = registry.get(UserMgtConstants.EXTERNAL_USERSTORE);

            String realmName = resource.getProperty("RealmType");

            Authenticator authenticator = null;
            UserStoreReader reader = null;
            UserStoreAdmin admin = null;

            if (realmName.equals("jdbc")) {
                JDBCRealmConfig config = new JDBCRealmConfig();
                config.setConnectionPassword(resource.getProperty("ConnectionPassword"));
                config.setConnectionName(resource.getProperty("ConnectionName"));
                config.setConnectionURL(resource.getProperty("ConnectionURL"));
                config.setDriverName(resource.getProperty("DriverName"));
                config.setRoleListSQL(resource.getProperty("RoleListSQL"));
                config.setUserListSQL(resource.getProperty("UserListSQL"));
                config.setUserPasswordSQL(resource.getProperty("UserPasswordSQL"));
                config.setUserRoleSQL(resource.getProperty("UserRoleSQL"));
                config.setUsersInRoleSQL(resource.getProperty("UsersInRoleSQL"));
                config.setIsUserExistingSQL(resource.getProperty("IsUserExistingSQL"));
                config.setUserFilterSQL(resource.getProperty("UserFilterSQL"));
                config.setAttributesSQL(resource.getProperty("AttrsSQL"));
                config.setAttrsForProfileSQL(resource.getProperty("AttrsForProfileSQL"));
                config.setProfileNamesSQL(resource.getProperty("ProfileNamesSQL"));
                authenticator = new JDBCAuthenticator(config);
                reader = new JDBCUserStoreReader(config);
                admin = new JDBCUserStoreAdmin(config);
            } else if (realmName.equals("ldap")) {
                LDAPRealmConfig config = new LDAPRealmConfig();
                config.setConnectionPassword(resource.getProperty("ConnectionPassword"));
                config.setConnectionName(resource.getProperty("ConnectionName"));
                config.setConnectionURL(resource.getProperty("ConnectionURL"));
                config.setUserContextName(resource.getProperty("UserContextName"));
                config.setUidSearchName(resource.getProperty("UIDSearchName"));

                authenticator = new LDAPAuthenticator(config);
                reader = new LDAPUserStoreReader(config);
                admin = new LDAPUserStoreAdmin(config);

            } else if (realmName.equals("act-dir")) {
                ActDirRealmConfig config = new ActDirRealmConfig();
                config.setConnectionPassword(resource.getProperty("ConnectionPassword"));
                config.setConnectionName(resource.getProperty("ConnectionName"));
                config.setConnectionURL(resource.getProperty("ConnectionURL"));
                config.setSearchBase(resource.getProperty("SearchBase"));

                authenticator = new ActDirAuthenticator(config);
                reader = new ActDirUserStoreReader(config);
                admin = new ActDirUserStoreAdmin(config);
            }

            BasicDataSource dataSource = new BasicDataSource();
            dataSource.setUrl(connectionURL);
            dataSource.setUsername(connectionName);
            dataSource.setPassword(connectionPass);
            dataSource.setDriverClassName(driverName);

            realm = new HybridRealm();
            HybridRealmConfig hconfig = (HybridRealmConfig) realm.getRealmConfiguration();
            hconfig.setAuthenticator(authenticator);
            hconfig.setDataSource(dataSource);
            hconfig.setUserStoreReader(reader);
            hconfig.setUserStoreAdmin(admin);
            hconfig.setClaimMapping(getClaimsMappingForHybridUserStore(dataSource));
            hconfig.setProfileConfigs(getProfileConfigurationForHybridUserStore(dataSource));
            realm.init(hconfig);
            return realm;
        } catch (Exception e) {
            log.error(e);
            throw new UserMgtException();
        }
    }

    public static void setRegistryService(RegistryService registryService) {
        UserMgtContext.registryService = registryService;
    }

    public static void setDefaultRealm(UserRealm defaultRealm) {
        UserMgtContext.defaultRealm = defaultRealm;
    }

    private static Map<String, ClaimMapping> getClaimsMappingForHybridUserStore(
            DataSource dataSource) throws UserMgtException {
        Map<String, ClaimMapping> claimMappings = null;
        try {
            ClaimDAO claimDAO = new ClaimDAO(dataSource, UserCoreConstants.EXTERNAL_USERSTORE);
            int count = claimDAO.getDialectCount(UserCoreConstants.EXTERNAL_USERSTORE);
            DefaultClaimBuilder claimBuilder = new DefaultClaimBuilder();
            if (count == 0) {
                claimMappings = claimBuilder.buildClaimMappingsFromConfigFile();
                claimDAO.addCliamMappings(claimMappings.values().toArray(
                        new ClaimMapping[claimMappings.size()]));
            } else {
                claimMappings = claimBuilder.buildClaimMappingsFromDatabase(dataSource,
                        UserCoreConstants.EXTERNAL_USERSTORE);

            }
            return claimMappings;
        } catch (Exception e) {
            log.error("Error occurred while creating claim admin", e);
            throw new UserMgtException("Error occurred while creating claim admin", e);
        }
    }

    private static Map<String, ProfileConfiguration> getProfileConfigurationForHybridUserStore(
            DataSource dataSource) throws UserMgtException {
        Map<String, ProfileConfiguration> profileConfigs = null;
        try {
            ClaimDAO claimDAO = new ClaimDAO(dataSource, UserCoreConstants.EXTERNAL_USERSTORE);
            int count = claimDAO.getDialectCount(UserCoreConstants.EXTERNAL_USERSTORE);
            ProfileConfigDAO profileDAO = new ProfileConfigDAO(dataSource, UserCoreConstants.EXTERNAL_USERSTORE);
            DefaultClaimBuilder claimBuilder = new DefaultClaimBuilder();
            DefaultProfileBuilder profileBilder = new DefaultProfileBuilder();

            if (count == 0) {

                profileConfigs = profileBilder.buildProfileConfigurationFromConfigFile();
                profileDAO.addProfileConfig(profileConfigs.values().toArray(
                        new ProfileConfiguration[profileConfigs.size()]));
            } else {
                profileConfigs = profileBilder.buildProfileConfigurationFromDatabase(dataSource, UserCoreConstants.EXTERNAL_USERSTORE);
            }
        } catch (Exception e) {
            log.error("Error occurred while creating profile admin", e);
            throw new UserMgtException("Error occurred while creating profile admin", e);
        }
        return profileConfigs;
    }

}
