package org.wso2.carbon.user.mgt.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.hybrid.HybridRealm;
import org.wso2.carbon.user.mgt.UserMgtContext;

public class UserMgtInitializer implements BundleActivator {

    private ServiceRegistration serviceRegistration = null;
    
    public void start(BundleContext bc) throws Exception {

        OMElement parentElement = getConfigurationElement();

        OMElement dbelement = parentElement.getFirstChildWithName(new QName("Database"));

        String url = dbelement
                .getFirstChildWithName(new QName(UserCoreConstants.UserMgtConfig.URL)).getText();
        String password = dbelement.getFirstChildWithName(
                new QName(UserCoreConstants.UserMgtConfig.PASSWORD)).getText();
        String userName = dbelement.getFirstChildWithName(
                new QName(UserCoreConstants.UserMgtConfig.USERNAME)).getText();
        String driverName = dbelement.getFirstChildWithName(
                new QName(UserCoreConstants.UserMgtConfig.DRIVER)).getText();
        String dialect = dbelement.getFirstChildWithName(
                new QName(UserCoreConstants.UserMgtConfig.DIALECT)).getText();

        UserMgtContext.init(bc, url, userName, password, driverName);
        addInitialData(parentElement);
    }

    public void stop(BundleContext arg0) throws Exception {
        HybridRealm realm = (HybridRealm) UserMgtContext.getHybridRealm();
        if (realm != null) {
        }

    }

    public void addInitialData(OMElement parentElement) throws Exception {

        List<String> sysConfig = new ArrayList<String>();
        OMElement sysPermissions = parentElement
                .getFirstChildWithName(new QName("SystemPermission"));
        if (sysPermissions == null) {
            throw new Exception("System cannot startup without system permissions.");
        }

        Iterator ite = sysPermissions.getChildElements();
        while (ite.hasNext()) {
            OMElement element = (OMElement) ite.next();
            String action = element.getText();
            sysConfig.add(action);
        }

        UserMgtContext.setSystemPermissions(sysConfig);

    }

    private OMElement getConfigurationElement() {
        try {
            String carbonHome = System.getProperty("carbon.home");
            if (carbonHome == null) {
                carbonHome = System.getenv("CARBON_HOME");
            }

            InputStream inStream = null;
            if (carbonHome != null) {
                File userMgtXml = new File(carbonHome + File.separator + "conf", "user-mgt.xml");
                if (!userMgtXml.exists()) {
                    String msg = "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found.";
                    throw new FileNotFoundException(msg);
                }
                inStream = new FileInputStream(userMgtXml);
            } else {
                inStream = this.getClass().getClassLoader()
                        .getResourceAsStream("conf/user-mgt.xml");
                if (inStream == null) {
                    String msg = "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found. Please set the carbon.home";
                    throw new FileNotFoundException(msg);
                }
            }

            StAXOMBuilder builder = new StAXOMBuilder(inStream);
            return builder.getDocumentElement();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

        return null;
    }

}