/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.user.mgt.internal;

import org.osgi.service.component.ComponentContext;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.mgt.UserMgtContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @scr.component name="usermgt.component"" immediate="true"
 * @scr.reference name="registry.service" interface="org.wso2.carbon.registry.core.service.RegistryService" cardinality="1..1" policy="dynamic"  bind="setRegistryService" unbind="unsetRegistryService"
 * @scr.reference name="user.realm.default"
 *                interface="org.wso2.carbon.user.core.UserRealm"
 *                cardinality="1..1" policy="dynamic"
 *                bind="setUserRealmDefault"
 *                unbind="unsetUserRealmDefault"
 */
public class UserMgtServiceInjector {
    private static Log log = LogFactory.getLog(UserMgtServiceInjector.class);    
    private ComponentContext ctxt;

    public UserMgtServiceInjector(){
    }

    protected void activate(ComponentContext ctxt) {
        log.debug("User Mgt bundle is activated ");
        UserMgtInitializer userMgtInitializer = new UserMgtInitializer();
        try {
            userMgtInitializer.start(ctxt.getBundleContext());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }
    protected void deactivate(ComponentContext ctxt) {
	    log.debug("User Mgt bundle is deactivated ");
    }

    protected void setRegistryService(RegistryService registryService) {
        if (log.isDebugEnabled()) {
            log.info("Setting the Registry Service");
        }
        UserMgtContext.setRegistryService(registryService);
    }
    
    protected void setUserRealmDefault(UserRealm realm) {
        if (log.isDebugEnabled()) {
            log.info("Setting the UserRealm");
        }
        UserMgtContext.setDefaultRealm(realm);
    }

    protected void unsetUserRealmDefault(UserRealm realm) {
        if (log.isDebugEnabled()) {
            log.info("Unsetting the UserRealm");
        }
        UserMgtContext.setDefaultRealm(null);
        
    }
    protected void unsetRegistryService(RegistryService registryService) {
        if (log.isDebugEnabled()) {
            log.info("Unsetting the Registry Service");
        }
        UserMgtContext.setRegistryService(null);
    }
    
    
}
